@@ -1,7 +1,6 @@
 // Multiple live iterators on the same Map can cope with removing entries.
 
-load(libdir + "eqArrayHelper.js");
-load(libdir + "asserts.js");
+load(libdir + "iteration.js");
 
 // Make a map.
 var map = Map();
@@ -13,9 +12,9 @@ for (var j = 0; j < SIZE; j++)
 var NITERS = 5;
 var iters = [];
 for (var i = 0; i < NITERS; i++) {
-    var iter = map.iterator();
-    assertEqArray(iter.next(), [0, 0]);
-    assertEqArray(iter.next(), [1, 1]);
+    var iter = map[std_iterator]();
+    assertIteratorNext(iter, [0, 0]);
+    assertIteratorNext(iter, [1, 1]);
     iters[i] = iter;
 }
 
@@ -27,6 +26,6 @@ for (var j = 0; j < SIZE; j += 2)
 for (var i = 0; i < NITERS; i++) {
     var iter = iters[i];
     for (var j = 3; j < SIZE; j += 2)
-        assertEqArray(iter.next(), [j, j]);
-    assertThrowsValue(function () { iter.next(); }, StopIteration);
+        assertIteratorNext(iter, [j, j]);
+    assertIteratorDone(iter, undefined);
 }
