@@ -5,28 +5,27 @@
 
 
 /**
-   File Name:          15.7.3.6-4.js
-   ECMA Section:       15.7.3.6 Number.POSITIVE_INFINITY
+   File Name:          15.7.3.5-3.js
+   ECMA Section:       15.7.3.5 Number.NEGATIVE_INFINITY
    Description:        All value properties of the Number object should have
    the attributes [DontEnum, DontDelete, ReadOnly]
 
-   this test checks the DontEnum attribute of Number.POSITIVE_INFINITY
+   this test checks the ReadOnly attribute of Number.NEGATIVE_INFINITY
 
    Author:             christine@netscape.com
    Date:               16 september 1997
 */
-var SECTION = "15.7.3.6-4";
+
+var SECTION = "15.7.3.5-3";
 var VERSION = "ECMA_1";
 startTest();
-var TITLE   = "Number.POSITIVE_INFINITY";
+var TITLE   = "Number.NEGATIVE_INFINITY";
 
 writeHeaderToLog( SECTION + " "+TITLE);
 
 new TestCase( SECTION,
-	      "var string = ''; for ( prop in Number ) { string += ( prop == 'POSITIVE_INFINITY' ) ? prop : '' } string;",
-	      "",
-	      eval("var string = ''; for ( prop in Number ) { string += ( prop == 'POSITIVE_INFINITY' ) ? prop : '' } string;")
-  );
-
+	      "Number.NEGATIVE_INFINITY=0; Number.NEGATIVE_INFINITY",
+	      -Infinity,
+	      eval("Number.NEGATIVE_INFINITY=0; Number.NEGATIVE_INFINITY") );
 
 test();
