@@ -4,11 +4,12 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 //-----------------------------------------------------------------------------
-var BUGNUMBER = 355829;
-var summary = 'js_ValueToObject should return the original object if OBJ_DEFAULT_VALUE returns a primitive value';
-var actual = '';
-var expect = '';
+var BUGNUMBER = 360969;
+var summary = '2^17: local var';
+var actual = 'No Crash';
+var expect = 'No Crash';
 
+var global = this;
 
 //-----------------------------------------------------------------------------
 test();
@@ -19,11 +20,22 @@ function test()
   enterFunc ('test');
   printBugNumber(BUGNUMBER);
   printStatus (summary);
+ 
+  var start = new Date();
+  var p;
+  var i;
+  var limit = 2 << 16;
 
-  var a = [ { valueOf: function() { return null; } } ];
-  a.toLocaleString();
+  for (var i = 0; i < limit; i++)
+  {
+    eval('var pv;');
+  }
 
   reportCompare(expect, actual, summary);
 
+  var stop = new Date();
+
+  print('Elapsed time: ' + Math.floor((stop - start)/1000) + ' seconds');
+
   exitFunc ('test');
 }
