@@ -4,8 +4,8 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 //-----------------------------------------------------------------------------
-var BUGNUMBER = 465980;
-var summary = 'Do not crash @ InitArrayElements';
+var BUGNUMBER = 352742;
+var summary = 'eval("return") in toString';
 var actual = '';
 var expect = '';
 
@@ -20,14 +20,11 @@ function test()
   printBugNumber(BUGNUMBER);
   printStatus (summary);
 
-  try
-  {
-    var a = new Array(4294967294);
-    a.push("foo", "bar");
-  }
-  catch(ex)
-  {
-  }
+  expect = 3;
+  var j = ({toString: function() { eval("return"); }});
+  actual = 2;
+  try { "" + j; } catch(e){}
+  actual = 3;
 
   reportCompare(expect, actual, summary);
 
