@@ -4,17 +4,17 @@
 /// "Use Terms").   Any redistribution of this code must retain the above 
 /// copyright and this notice and otherwise comply with the Use Terms.
 /**
- * @path ch07/7.3/7.3-10.js
- * @description 7.3 - ES5 recognizes the character <PS> (\u2029) as a NonEscapeCharacter
+ * @path ch07/7.6/7.6-2.js
+ * @description 7.6 - SyntaxError expected: reserved words used as Identifier Names in UTF8: true (true)
  */
 
 
 function testcase() {
-        try {
-            eval("var prop = \\u2029;");
-            return false;
-        } catch (e) {
-            return e instanceof SyntaxError;
-        }
+            try {
+                eval("var tr\u0075e = 123;");  
+                return false;
+            } catch (e) {
+                return e instanceof SyntaxError;  
+            }
     }
 runTestCase(testcase);
