@@ -4,21 +4,23 @@
 /// "Use Terms").   Any redistribution of this code must retain the above 
 /// copyright and this notice and otherwise comply with the Use Terms.
 /**
- * @path ch07/7.8/7.8.4/7.8.4-6-s.js
+ * @path ch07/7.8/7.8.4/7.8.4-17-s.js
  * @description An OctalEscapeSequence is not allowed in a String under Strict Mode
  * @onlyStrict
  */
 
 
+
+
 function testcase()
 {
   try 
   {
-    eval('"use strict"; var x = "\\01z";');
+    eval('"use strict"; var x = "\\411";');
     return false;
   }
   catch (e) {
     return (e instanceof SyntaxError);
   }
- }
-runTestCase(testcase);
+}
+runTestCase(testcase);
\ No newline at end of file
