@@ -4,14 +4,19 @@
 /// "Use Terms").   Any redistribution of this code must retain the above 
 /// copyright and this notice and otherwise comply with the Use Terms.
 /**
- * @path ch10/10.4/10.4.3/10.4.3-1-19-s.js
- * @description Strict Mode - checking 'this' (indirect eval used within strict mode)
- * @onlyStrict
+ * @path ch10/10.1/10.1.1/10.1.1-14-s.js
+ * @description Strict Mode - The call to eval function is contained in a Strict Mode block
+ * @noStrict
  */
-    
+
+
 function testcase() {
-"use strict";
-var my_eval = eval;
-return my_eval("this") === fnGlobalObject();
-}
-runTestCase(testcase);
\ No newline at end of file
+        'use strict';
+        try {
+            eval("var public = 1;");
+            return false;
+        } catch (e) {
+            return true;
+        }
+    }
+runTestCase(testcase);
