@@ -5,8 +5,8 @@
  * No matter how control leaves the embedded 'Statement',
  * the scope chain is always restored to its former state
  *
- * @path ch12/12.10/S12.10_A3.5_T2.js
- * @description Using "with" statement within "for-in" statement, leading to completion by exception
+ * @path ch12/12.10/S12.10_A3.6_T2.js
+ * @description Using "with" statement within another "with" statement, leading to completion by exception
  * @noStrict
  */
 
@@ -20,9 +20,16 @@ var myObj = {
     valueOf : function(){return 'obj_valueOf';}
 }
 
+var theirObj = {
+    p1: true, 
+    value: 'theirObj_value',
+    valueOf : function(){return 'thr_valueOf';}
+}
+
+
 try {
-    for(var prop in myObj){
-        with(myObj){
+    with(myObj){
+        with(theirObj){
             p1 = 'x1';
             throw value;
         }
@@ -33,25 +40,26 @@ try {
 
 //////////////////////////////////////////////////////////////////////////////
 //CHECK#1
-if(result !== 1){
-  $ERROR('#1: result === 1. Actual:  result ==='+ result  );
+if(p1 !== 1){
+  $ERROR('#1: p1 === 1. Actual:  p1 ==='+ p1  );
 }
 //
 //////////////////////////////////////////////////////////////////////////////
 
 //////////////////////////////////////////////////////////////////////////////
 //CHECK#2
-if(p1 !== 1){
-  $ERROR('#2: p1 === 1. Actual:  p1 ==='+ p1  );
+if(myObj.p1 !== "a"){
+  $ERROR('#2: myObj.p1 === "a". Actual:  myObj.p1 ==='+ myObj.p1  );
 }
 //
 //////////////////////////////////////////////////////////////////////////////
 
 //////////////////////////////////////////////////////////////////////////////
 //CHECK#3
-if(myObj.p1 !== "x1"){
-  $ERROR('#3: myObj.p1 === "x1". Actual:  myObj.p1 ==='+ myObj.p1  );
+if(theirObj.p1 !== "x1"){
+  $ERROR('#3: theirObj.p1 === "x1". Actual:  theirObj.p1 ==='+ theirObj.p1  );
 }
 //
 //////////////////////////////////////////////////////////////////////////////
 
+
