@@ -4,20 +4,16 @@
 /// "Use Terms").   Any redistribution of this code must retain the above 
 /// copyright and this notice and otherwise comply with the Use Terms.
 /**
- * @path ch12/12.14/12.14.1/12.14.1-4-s.js
- * @description Strict Mode - SyntaxError isn't thrown if a TryStatement with a Catch occurs within strict code and the Identifier of the Catch production is EVAL
+ * @path ch12/12.2/12.2.1/12.2.1-9-s.js
+ * @description Strict Mode: an indirect eval declaring a var named 'eval' does not throw
  * @onlyStrict
  */
 
 
 function testcase() {
-        "use strict";
-
-        try {
-            throw new Error("...");
-            return false;
-        } catch (EVAL) {
-            return EVAL instanceof Error;
-        }
-    }
+  'use strict';
+  var s = eval;
+  s('var eval;');
+  return true;
+ }
 runTestCase(testcase);
