@@ -253,7 +253,7 @@ void CLOSTexture::ConstructTexture(int unit)
 
 	glGenTextures(1, &m_Texture);
 
-	// Initialise texture with SoD colour, for the areas we don't
+	// Initialise texture with SoD color, for the areas we don't
 	// overwrite with glTexSubImage2D later
 	u8* texData = new u8[m_TextureSize * m_TextureSize * 4];
 	memset(texData, 0x00, m_TextureSize * m_TextureSize * 4);
@@ -342,7 +342,7 @@ void CLOSTexture::RecomputeTexture(int unit)
 	if (!cmpRangeManager)
 		return;
 
-	ICmpRangeManager::CLosQuerier los(cmpRangeManager->GetLosQuerier(g_Game->GetPlayerID()));
+	ICmpRangeManager::CLosQuerier los(cmpRangeManager->GetLosQuerier(g_Game->GetSimulation2()->GetSimContext().GetCurrentDisplayedPlayer()));
 
 	GenerateBitmap(los, &losData[0], m_MapSize, m_MapSize, pitch);
 
