@@ -32,7 +32,11 @@ CLightEnv::CLightEnv()
 	m_LightingModel("standard"),
 	m_SunColor(1.5, 1.5, 1.5),
 	m_TerrainAmbientColor(0x50/255.f, 0x60/255.f, 0x85/255.f),
-	m_UnitsAmbientColor(0x80/255.f, 0x80/255.f, 0x80/255.f)
+	m_UnitsAmbientColor(0x80/255.f, 0x80/255.f, 0x80/255.f),
+	m_FogColor(0xCC/255.f, 0xCC/255.f, 0xE5/255.f),
+	m_FogFactor(0.000f),
+	m_FogMax(0.5f),
+	m_Brightness(0.0f), m_Contrast(1.0f), m_Saturation(0.99f), m_Bloom(0.1999f)
 {
 	CalculateSunDirection();
 }
