@@ -1,4 +1,4 @@
-/* Copyright (C) 2013 Wildfire Games.
+/* Copyright (C) 2016 Wildfire Games.
  * This file is part of 0 A.D.
  *
  * 0 A.D. is free software: you can redistribute it and/or modify
@@ -20,7 +20,7 @@
 #include "MapReader.h"
 
 #include "graphics/Camera.h"
-#include "graphics/CinemaTrack.h"
+#include "graphics/CinemaManager.h"
 #include "graphics/Entity.h"
 #include "graphics/GameView.h"
 #include "graphics/MapGenerator.h"
@@ -60,7 +60,7 @@ CMapReader::CMapReader()
 	// Maps that don't override the default probably want the old lighting model
 	//m_LightEnv.SetLightingModel("old");
 	//pPostproc->SetPostEffect(L"default");
-	
+
 }
 
 // LoadMap: try to load the map from given file; reinitialise the scene to new data if successful
@@ -112,7 +112,7 @@ void CMapReader::LoadMap(const VfsPath& pathname, JSRuntime* rt,  JS::HandleValu
 	// delete all existing entities
 	if (pSimulation2)
 		pSimulation2->ResetState();
-	
+
 	// reset post effects
 	if (pPostproc)
 		pPostproc->SetPostEffect(L"default");
@@ -296,7 +296,7 @@ int CMapReader::ApplyTerrainData()
 
 						mp.Tex = m_TerrainTextures[tileptr->m_Tex1Index];
 						mp.Priority = tileptr->m_Priority;
-	
+
 						tileptr++;
 					}
 				}
@@ -324,7 +324,7 @@ int CMapReader::ApplyData()
 	{
 		// Default to global camera (with constraints)
 		pGameView->ResetCameraTarget(pGameView->GetCamera()->GetFocus());
-	
+
 		// TODO: Starting rotation?
 		CmpPtr<ICmpPlayer> cmpPlayer(*pSimContext, cmpPlayerManager->GetPlayerByID(m_PlayerID));
 		if (cmpPlayer && cmpPlayer->HasStartingCamera())
@@ -358,7 +358,7 @@ PSRETURN CMapSummaryReader::LoadMap(const VfsPath& pathname)
 	VfsPath filename_xml = pathname.ChangeExtension(L".xml");
 
 	CXeromyces xmb_file;
-	if (xmb_file.Load(g_VFS, filename_xml) != PSRETURN_OK)
+	if (xmb_file.Load(g_VFS, filename_xml, "scenario") != PSRETURN_OK)
 		return PSRETURN_File_ReadFailed;
 
 	// Define all the relevant elements used in the XML file
@@ -388,7 +388,7 @@ void CMapSummaryReader::GetMapSettings(ScriptInterface& scriptInterface, JS::Mut
 {
 	JSContext* cx = scriptInterface.GetContext();
 	JSAutoRequest rq(cx);
-	
+
 	scriptInterface.Eval("({})", ret);
 	if (m_ScriptSettings.empty())
 		return;
@@ -410,7 +410,7 @@ class CXMLReader
 	NONCOPYABLE(CXMLReader);
 public:
 	CXMLReader(const VfsPath& xml_filename, CMapReader& mapReader)
-		: m_MapReader(mapReader)
+		: m_MapReader(mapReader), nodes(NULL, 0, NULL)
 	{
 		Init(xml_filename);
 	}
@@ -435,7 +435,6 @@ private:
 	int el_actor;
 	int at_x, at_y, at_z;
 	int at_group, at_group2;
-	int at_id;
 	int at_angle;
 	int at_uid;
 	int at_seed;
@@ -443,8 +442,8 @@ private:
 	XMBElementList nodes; // children of root
 
 	// loop counters
-	int node_idx;
-	int entity_idx;
+	size_t node_idx;
+	size_t entity_idx;
 
 	// # entities+nonentities processed and total (for progress calc)
 	int completed_jobs, total_jobs;
@@ -457,7 +456,7 @@ private:
 	void ReadTerrain(XMBElement parent);
 	void ReadEnvironment(XMBElement parent);
 	void ReadCamera(XMBElement parent);
-	void ReadCinema(XMBElement parent);
+	void ReadPaths(XMBElement parent);
 	void ReadTriggers(XMBElement parent);
 	int ReadEntities(XMBElement parent, double end_time);
 };
@@ -468,7 +467,7 @@ void CXMLReader::Init(const VfsPath& xml_filename)
 	// must only assign once, so do it here
 	node_idx = entity_idx = 0;
 
-	if (xmb_file.Load(g_VFS, xml_filename) != PSRETURN_OK)
+	if (xmb_file.Load(g_VFS, xml_filename, "scenario") != PSRETURN_OK)
 		throw PSERROR_File_ReadFailed();
 
 	// define the elements and attributes that are frequently used in the XML file,
@@ -501,8 +500,8 @@ void CXMLReader::Init(const VfsPath& xml_filename)
 	// (used when calculating progress)
 	completed_jobs = 0;
 	total_jobs = 0;
-	for (int i = 0; i < nodes.Count; i++)
-		total_jobs += nodes.Item(i).GetChildNodes().Count;
+	for (XMBElement node : nodes)
+		total_jobs += node.GetChildNodes().size();
 
 	// Find the maximum entity ID, so we can safely allocate new IDs without conflicts
 
@@ -595,15 +594,15 @@ void CXMLReader::ReadEnvironment(XMBElement parent)
 	EL(lightingmodel);
 	EL(posteffect);
 	EL(skyset);
-	EL(suncolour);
+	EL(suncolor);
 	EL(sunelevation);
 	EL(sunrotation);
-	EL(terrainambientcolour);
-	EL(unitsambientcolour);
+	EL(terrainambientcolor);
+	EL(unitsambientcolor);
 	EL(water);
 	EL(waterbody);
 	EL(type);
-	EL(colour);
+	EL(color);
 	EL(tint);
 	EL(height);
 	EL(shininess);	// for compatibility
@@ -613,7 +612,7 @@ void CXMLReader::ReadEnvironment(XMBElement parent)
 	EL(reflectiontint);	// for compatibility
 	EL(reflectiontintstrength);	// for compatibility
 	EL(fog);
-	EL(fogcolour);
+	EL(fogcolor);
 	EL(fogfactor);
 	EL(fogthickness);
 	EL(postproc);
@@ -640,7 +639,7 @@ void CXMLReader::ReadEnvironment(XMBElement parent)
 			if (m_MapReader.pSkyMan)
 				m_MapReader.pSkyMan->SetSkySet(element.GetText().FromUTF8());
 		}
-		else if (element_name == el_suncolour)
+		else if (element_name == el_suncolor)
 		{
 			m_MapReader.m_LightEnv.m_SunColor = RGBColor(
 				attrs.GetNamedItem(at_r).ToFloat(),
@@ -655,14 +654,14 @@ void CXMLReader::ReadEnvironment(XMBElement parent)
 		{
 			m_MapReader.m_LightEnv.m_Rotation = attrs.GetNamedItem(at_angle).ToFloat();
 		}
-		else if (element_name == el_terrainambientcolour)
+		else if (element_name == el_terrainambientcolor)
 		{
 			m_MapReader.m_LightEnv.m_TerrainAmbientColor = RGBColor(
 				attrs.GetNamedItem(at_r).ToFloat(),
 				attrs.GetNamedItem(at_g).ToFloat(),
 				attrs.GetNamedItem(at_b).ToFloat());
 		}
-		else if (element_name == el_unitsambientcolour)
+		else if (element_name == el_unitsambientcolor)
 		{
 			m_MapReader.m_LightEnv.m_UnitsAmbientColor = RGBColor(
 				attrs.GetNamedItem(at_r).ToFloat(),
@@ -674,7 +673,7 @@ void CXMLReader::ReadEnvironment(XMBElement parent)
 			XERO_ITER_EL(element, fog)
 			{
 				int element_name = fog.GetNodeName();
-				if (element_name == el_fogcolour)
+				if (element_name == el_fogcolor)
 				{
 					XMBAttributeList attrs = fog.GetAttributes();
 					m_MapReader.m_LightEnv.m_FogColor = RGBColor(
@@ -740,7 +739,7 @@ void CXMLReader::ReadEnvironment(XMBElement parent)
 					// graphics are disabled
 					if (!m_MapReader.pWaterMan)
 						continue;
-					
+
 					if (element_name == el_type)
 					{
 						if (waterelement.GetText() == "default")
@@ -752,7 +751,7 @@ void CXMLReader::ReadEnvironment(XMBElement parent)
 					{
 						// deprecated.
 					}
-#define READ_COLOUR(el, out) \
+#define READ_COLOR(el, out) \
 					else if (element_name == el) \
 					{ \
 						XMBAttributeList attrs = waterelement.GetAttributes(); \
@@ -769,14 +768,14 @@ void CXMLReader::ReadEnvironment(XMBElement parent)
 						out = waterelement.GetText().ToFloat(); \
 					} \
 
-					READ_COLOUR(el_colour, m_MapReader.pWaterMan->m_WaterColor)
-					READ_COLOUR(el_tint, m_MapReader.pWaterMan->m_WaterTint)
+					READ_COLOR(el_color, m_MapReader.pWaterMan->m_WaterColor)
+					READ_COLOR(el_tint, m_MapReader.pWaterMan->m_WaterTint)
 					READ_FLOAT(el_waviness, m_MapReader.pWaterMan->m_Waviness)
 					READ_FLOAT(el_murkiness, m_MapReader.pWaterMan->m_Murkiness)
 					READ_FLOAT(el_windangle, m_MapReader.pWaterMan->m_WindAngle)
 
 #undef READ_FLOAT
-#undef READ_COLOUR
+#undef READ_COLOR
 
 					else
 						debug_warn(L"Invalid map XML data");
@@ -840,120 +839,115 @@ void CXMLReader::ReadCamera(XMBElement parent)
 	}
 }
 
-void CXMLReader::ReadCinema(XMBElement parent)
+void CXMLReader::ReadPaths(XMBElement parent)
 {
 	#define EL(x) int el_##x = xmb_file.GetElementID(#x)
 	#define AT(x) int at_##x = xmb_file.GetAttributeID(#x)
 
 	EL(path);
 	EL(rotation);
-	EL(distortion);
 	EL(node);
 	EL(position);
-	EL(time);
+	EL(target);
 	AT(name);
 	AT(timescale);
+	AT(orientation);
 	AT(mode);
 	AT(style);
-	AT(growth);
-	AT(switch);
 	AT(x);
 	AT(y);
 	AT(z);
+	AT(deltatime);
 
 #undef EL
 #undef AT
-	
-	std::map<CStrW, CCinemaPath> pathList;
+
 	XERO_ITER_EL(parent, element)
 	{
 		int elementName = element.GetNodeName();
-			
-		if ( elementName == el_path )
+
+		if (elementName == el_path)
 		{
-			XMBAttributeList attrs = element.GetAttributes();
-			CStrW name(attrs.GetNamedItem(at_name).FromUTF8());
-			float timescale = attrs.GetNamedItem(at_timescale).ToFloat();
 			CCinemaData pathData;
-			pathData.m_Timescale = timescale;
-			TNSpline spline, backwardSpline;
+			XMBAttributeList attrs = element.GetAttributes();
+			CStrW pathName(attrs.GetNamedItem(at_name).FromUTF8());
+			pathData.m_Timescale = fixed::FromString(attrs.GetNamedItem(at_timescale));
+			TNSpline pathSpline, targetSpline;
+			fixed lastTargetTime = fixed::Zero();
+
+			pathData.m_Name = pathName;
+			pathData.m_Orientation = attrs.GetNamedItem(at_orientation).FromUTF8();
+			pathData.m_Mode = attrs.GetNamedItem(at_mode).FromUTF8();
+			pathData.m_Style = attrs.GetNamedItem(at_style).FromUTF8();
 
 			XERO_ITER_EL(element, pathChild)
 			{
 				elementName = pathChild.GetNodeName();
 				attrs = pathChild.GetAttributes();
 
-				//Load distortion attributes
-				if ( elementName == el_distortion )
-				{
-						pathData.m_Mode = attrs.GetNamedItem(at_mode).ToInt();
-						pathData.m_Style = attrs.GetNamedItem(at_style).ToInt();
-						pathData.m_Growth = attrs.GetNamedItem(at_growth).ToInt();
-						pathData.m_Switch = attrs.GetNamedItem(at_switch).ToInt();
-				}
-				
-				//Load node data used for spline
-				else if ( elementName == el_node )
+				// Load node data used for spline
+				if (elementName == el_node)
 				{
+					bool positionDeclared = false;
 					SplineData data;
+					data.Distance = fixed::FromString(attrs.GetNamedItem(at_deltatime));
+					lastTargetTime += fixed::FromString(attrs.GetNamedItem(at_deltatime));
 					XERO_ITER_EL(pathChild, nodeChild)
 					{
 						elementName = nodeChild.GetNodeName();
 						attrs = nodeChild.GetAttributes();
-						
-						//Fix?:  assumes that time is last element
-						if ( elementName == el_position )
+
+						if (elementName == el_position)
 						{
-							data.Position.X = attrs.GetNamedItem(at_x).ToFloat();
-							data.Position.Y = attrs.GetNamedItem(at_y).ToFloat();
-							data.Position.Z = attrs.GetNamedItem(at_z).ToFloat();
-							continue;
+							data.Position.X = fixed::FromString(attrs.GetNamedItem(at_x));
+							data.Position.Y = fixed::FromString(attrs.GetNamedItem(at_y));
+							data.Position.Z = fixed::FromString(attrs.GetNamedItem(at_z));
+							positionDeclared = true;
 						}
-						else if ( elementName == el_rotation )
+						else if (elementName == el_rotation)
 						{
-							data.Rotation.X = attrs.GetNamedItem(at_x).ToFloat();
-							data.Rotation.Y = attrs.GetNamedItem(at_y).ToFloat();
-							data.Rotation.Z = attrs.GetNamedItem(at_z).ToFloat();
-							continue;
+							data.Rotation.X = fixed::FromString(attrs.GetNamedItem(at_x));
+							data.Rotation.Y = fixed::FromString(attrs.GetNamedItem(at_y));
+							data.Rotation.Z = fixed::FromString(attrs.GetNamedItem(at_z));
 						}
-						else if ( elementName == el_time )
-							data.Distance = nodeChild.GetText().ToFloat();
-						else 
-							debug_warn(L"Invalid cinematic element for node child");
-					
-						backwardSpline.AddNode(data.Position, data.Rotation, data.Distance);
+						else if (elementName == el_target)
+						{
+							CFixedVector3D targetPosition;
+							targetPosition.X = fixed::FromString(attrs.GetNamedItem(at_x));
+							targetPosition.Y = fixed::FromString(attrs.GetNamedItem(at_y));
+							targetPosition.Z = fixed::FromString(attrs.GetNamedItem(at_z));
+
+							targetSpline.AddNode(targetPosition, CFixedVector3D(), lastTargetTime);
+							lastTargetTime = fixed::Zero();
+						}
+						else
+							LOGWARNING("Invalid cinematic element for node child");
 					}
+
+					// Skip the node if no position
+					if (positionDeclared)
+						pathSpline.AddNode(data.Position, data.Rotation, data.Distance);
 				}
 				else
-					debug_warn(L"Invalid cinematic element for path child");
-				
-				
+					LOGWARNING("Invalid cinematic element for path child");
 			}
 
-			//Construct cinema path with data gathered
-			CCinemaPath temp(pathData, backwardSpline);
-			const std::vector<SplineData>& nodes = temp.GetAllNodes();
-			if ( nodes.empty() )
+			// Construct cinema path with data gathered
+			CCinemaPath path(pathData, pathSpline, targetSpline);
+			if (path.Empty())
 			{
-				debug_warn(L"Failure loading cinematics");
+				LOGWARNING("Path with name '%s' is empty", pathName.ToUTF8());
 				return;
 			}
-					
-			for ( std::vector<SplineData>::const_reverse_iterator it = nodes.rbegin(); 
-															it != nodes.rend(); ++it )
-			{
-				spline.AddNode(it->Position, it->Rotation, it->Distance);
-			}
-				
-			CCinemaPath path(pathData, spline);
-			pathList[name] = path;	
+
+			if (!m_MapReader.pCinema->HasPath(pathName))
+				m_MapReader.pCinema->AddPath(pathName, path);
+			else
+				LOGWARNING("Path with name '%s' already exists", pathName.ToUTF8());
 		}
 		else
-			ENSURE("Invalid cinema child");
+			LOGWARNING("Invalid path child with name '%s'", element.GetText());
 	}
-
-	if (m_MapReader.pCinema)
-		m_MapReader.pCinema->SetAllPaths(pathList);
 }
 
 void CXMLReader::ReadTriggers(XMBElement UNUSED(parent))
@@ -968,12 +962,12 @@ int CXMLReader::ReadEntities(XMBElement parent, double end_time)
 	CSimulation2& sim = *m_MapReader.pSimulation2;
 	CmpPtr<ICmpPlayerManager> cmpPlayerManager(sim, SYSTEM_ENTITY);
 
-	while (entity_idx < entities.Count)
+	while (entity_idx < entities.size())
 	{
 		// all new state at this scope and below doesn't need to be
 		// wrapped, since we only yield after a complete iteration.
 
-		XMBElement entity = entities.Item(entity_idx++);
+		XMBElement entity = entities[entity_idx++];
 		ENSURE(entity.GetNodeName() == el_entity);
 
 		XMBAttributeList attrs = entity.GetAttributes();
@@ -1101,9 +1095,8 @@ int CXMLReader::ReadEntities(XMBElement parent, double end_time)
 
 void CXMLReader::ReadXML()
 {
-	for (int i = 0; i < nodes.Count; ++i)
+	for (XMBElement node : nodes)
 	{
-		XMBElement node = nodes.Item(i);
 		CStr name = xmb_file.GetElementString(node.GetNodeName());
 		if (name == "Terrain")
 		{
@@ -1127,7 +1120,7 @@ void CXMLReader::ReadXML()
 		}
 		else if (name == "Paths")
 		{
-			ReadCinema(node);
+			ReadPaths(node);
 		}
 		else if (name == "Triggers")
 		{
@@ -1154,9 +1147,9 @@ int CXMLReader::ProgressiveReadEntities()
 
 	int ret;
 
-	while (node_idx < nodes.Count)
+	while (node_idx < nodes.size())
 	{
-		XMBElement node = nodes.Item(node_idx);
+		XMBElement node = nodes[node_idx];
 		CStr name = xmb_file.GetElementString(node.GetNodeName());
 		if (name == "Entities")
 		{
@@ -1260,20 +1253,20 @@ int CMapReader::GenerateMap()
 {
 	JSContext* cx = pSimulation2->GetScriptInterface().GetContext();
 	JSAutoRequest rq(cx);
-	
+
 	if (!m_MapGen)
 	{
 		// Initialize map generator
 		m_MapGen = new CMapGenerator();
 
 		VfsPath scriptPath;
-		
+
 		if (m_ScriptFile.length())
 			scriptPath = L"maps/random/"+m_ScriptFile;
 
 		// Stringify settings to pass across threads
 		std::string scriptSettings = pSimulation2->GetScriptInterface().StringifyJSON(&m_ScriptSettings.get());
-		
+
 		// Try to generate map
 		m_MapGen->GenerateMap(scriptPath, scriptSettings);
 	}
@@ -1291,9 +1284,9 @@ int CMapReader::GenerateMap()
 		shared_ptr<ScriptInterface::StructuredClone> results = m_MapGen->GetResults();
 
 		// Parse data into simulation context
-		JS::RootedValue data(cx); 
+		JS::RootedValue data(cx);
 		pSimulation2->GetScriptInterface().ReadStructuredClone(results, &data);
-		
+
 		if (data.isUndefined())
 		{
 			// RMS failed - return to main menu
@@ -1312,7 +1305,7 @@ int CMapReader::GenerateMap()
 		// to allow more CPU for the map generator thread
 		SDL_Delay(100);
 	}
-	
+
 	// return progress
 	return progress;
 };
@@ -1323,7 +1316,7 @@ int CMapReader::ParseTerrain()
 	TIMER(L"ParseTerrain");
 	JSContext* cx = pSimulation2->GetScriptInterface().GetContext();
 	JSAutoRequest rq(cx);
-	
+
 	// parse terrain from map data
 	//	an error here should stop the loading process
 #define GET_TERRAIN_PROPERTY(val, prop, out)\
@@ -1376,7 +1369,7 @@ int CMapReader::ParseTerrain()
 		{
 			size_t patchY = y / PATCH_SIZE;
 			size_t offY = y % PATCH_SIZE;
-			
+
 			STileDesc tile;
 			tile.m_Tex1Index = tileIndex[y*size + x];
 			tile.m_Tex2Index = 0xFFFF;
@@ -1411,7 +1404,7 @@ int CMapReader::ParseEntities()
 
 	size_t entity_idx = 0;
 	size_t num_entities = entities.size();
-	
+
 	Entity currEnt;
 
 	while (entity_idx < num_entities)
@@ -1488,18 +1481,18 @@ int CMapReader::ParseEnvironment()
 		pSkyMan->SetSkySet(skySet);
 
 	CColor sunColor;
-	GET_ENVIRONMENT_PROPERTY(envObj, SunColour, sunColor)
+	GET_ENVIRONMENT_PROPERTY(envObj, SunColor, sunColor)
 	m_LightEnv.m_SunColor = RGBColor(sunColor.r, sunColor.g, sunColor.b);
 
 	GET_ENVIRONMENT_PROPERTY(envObj, SunElevation, m_LightEnv.m_Elevation)
 	GET_ENVIRONMENT_PROPERTY(envObj, SunRotation, m_LightEnv.m_Rotation)
-	
+
 	CColor terrainAmbientColor;
-	GET_ENVIRONMENT_PROPERTY(envObj, TerrainAmbientColour, terrainAmbientColor)
+	GET_ENVIRONMENT_PROPERTY(envObj, TerrainAmbientColor, terrainAmbientColor)
 	m_LightEnv.m_TerrainAmbientColor = RGBColor(terrainAmbientColor.r, terrainAmbientColor.g, terrainAmbientColor.b);
 
 	CColor unitsAmbientColor;
-	GET_ENVIRONMENT_PROPERTY(envObj, UnitsAmbientColour, unitsAmbientColor)
+	GET_ENVIRONMENT_PROPERTY(envObj, UnitsAmbientColor, unitsAmbientColor)
 	m_LightEnv.m_UnitsAmbientColor = RGBColor(unitsAmbientColor.r, unitsAmbientColor.g, unitsAmbientColor.b);
 
 	// Water properties
@@ -1523,7 +1516,7 @@ int CMapReader::ParseEnvironment()
 		GET_ENVIRONMENT_PROPERTY(waterBodyObj, Type, pWaterMan->m_WaterType)
 		if (pWaterMan->m_WaterType == L"default")
 			pWaterMan->m_WaterType = L"ocean";
-		GET_ENVIRONMENT_PROPERTY(waterBodyObj, Colour, pWaterMan->m_WaterColor)
+		GET_ENVIRONMENT_PROPERTY(waterBodyObj, Color, pWaterMan->m_WaterColor)
 		GET_ENVIRONMENT_PROPERTY(waterBodyObj, Tint, pWaterMan->m_WaterTint)
 		GET_ENVIRONMENT_PROPERTY(waterBodyObj, Waviness, pWaterMan->m_Waviness)
 		GET_ENVIRONMENT_PROPERTY(waterBodyObj, Murkiness, pWaterMan->m_Murkiness)
