@@ -1,4 +1,4 @@
-/* Copyright (C) 2014 Wildfire Games.
+/* Copyright (C) 2016 Wildfire Games.
  * This file is part of 0 A.D.
  *
  * 0 A.D. is free software: you can redistribute it and/or modify
@@ -18,7 +18,7 @@
 #include "precompiled.h"
 
 #include "Camera.h"
-#include "CinemaTrack.h"
+#include "CinemaManager.h"
 #include "GameView.h"
 #include "LightEnv.h"
 #include "MapReader.h"
@@ -95,7 +95,7 @@ static u16 GetEntryIndex(const CTerrainTextureEntry* entry, const std::vector<CT
 }
 
 ///////////////////////////////////////////////////////////////////////////////////////////////////
-// EnumTerrainTextures: build lists of textures used by map, and tile descriptions for 
+// EnumTerrainTextures: build lists of textures used by map, and tile descriptions for
 // each tile on the terrain
 void CMapWriter::EnumTerrainTextures(CTerrain *pTerrain,
 									 std::vector<CStr>& textures,
@@ -103,7 +103,7 @@ void CMapWriter::EnumTerrainTextures(CTerrain *pTerrain,
 {
 	// the list of all handles in use
 	std::vector<CTerrainTextureEntry*> entries;
-	
+
 	// resize tile array to required size
 	tiles.resize(SQR(pTerrain->GetVerticesPerSide()-1));
 	STileDesc* tileptr=&tiles[0];
@@ -160,24 +160,24 @@ void CMapWriter::PackTerrain(CFilePacker& packer, CTerrain* pTerrain)
 	// pack map size
 	const ssize_t mapsize = pTerrain->GetPatchesPerSide();
 	packer.PackSize(mapsize);
-	
+
 	// pack heightmap
-	packer.PackRaw(pTerrain->GetHeightMap(),sizeof(u16)*SQR(pTerrain->GetVerticesPerSide()));	
+	packer.PackRaw(pTerrain->GetHeightMap(),sizeof(u16)*SQR(pTerrain->GetVerticesPerSide()));
 
 	// the list of textures used by map
 	std::vector<CStr> terrainTextures;
 	// descriptions of each tile
 	std::vector<STileDesc> tiles;
-	
+
 	// build lists by scanning through the terrain
 	EnumTerrainTextures(pTerrain, terrainTextures, tiles);
-	
+
 	// pack texture names
 	const size_t numTextures = terrainTextures.size();
 	packer.PackSize(numTextures);
 	for (size_t i=0;i<numTextures;i++)
 		packer.PackString(terrainTextures[i]);
-	
+
 	// pack tile data
 	packer.PackRaw(&tiles[0],sizeof(STileDesc)*tiles.size());
 }
@@ -208,7 +208,7 @@ void CMapWriter::WriteXML(const VfsPath& filename,
 
 			XML_Setting("SkySet", pSkyMan->GetSkySet());
 			{
-				XML_Element("SunColour");
+				XML_Element("SunColor");
 				XML_Attribute("r", pLightEnv->m_SunColor.X); // yes, it's X/Y/Z...
 				XML_Attribute("g", pLightEnv->m_SunColor.Y);
 				XML_Attribute("b", pLightEnv->m_SunColor.Z);
@@ -222,13 +222,13 @@ void CMapWriter::WriteXML(const VfsPath& filename,
 				XML_Attribute("angle", pLightEnv->m_Rotation);
 			}
 			{
-				XML_Element("TerrainAmbientColour");
+				XML_Element("TerrainAmbientColor");
 				XML_Attribute("r", pLightEnv->m_TerrainAmbientColor.X);
 				XML_Attribute("g", pLightEnv->m_TerrainAmbientColor.Y);
 				XML_Attribute("b", pLightEnv->m_TerrainAmbientColor.Z);
 			}
 			{
-				XML_Element("UnitsAmbientColour");
+				XML_Element("UnitsAmbientColor");
 				XML_Attribute("r", pLightEnv->m_UnitsAmbientColor.X);
 				XML_Attribute("g", pLightEnv->m_UnitsAmbientColor.Y);
 				XML_Attribute("b", pLightEnv->m_UnitsAmbientColor.Z);
@@ -238,7 +238,7 @@ void CMapWriter::WriteXML(const VfsPath& filename,
 				XML_Setting("FogFactor", pLightEnv->m_FogFactor);
 				XML_Setting("FogThickness", pLightEnv->m_FogMax);
 				{
-					XML_Element("FogColour");
+					XML_Element("FogColor");
 					XML_Attribute("r", pLightEnv->m_FogColor.X);
 					XML_Attribute("g", pLightEnv->m_FogColor.Y);
 					XML_Attribute("b", pLightEnv->m_FogColor.Z);
@@ -253,7 +253,7 @@ void CMapWriter::WriteXML(const VfsPath& filename,
 					ENSURE(cmpWaterManager);
 					XML_Setting("Type", pWaterMan->m_WaterType);
 					{
-						XML_Element("Colour");
+						XML_Element("Color");
 						XML_Attribute("r", pWaterMan->m_WaterColor.r);
 						XML_Attribute("g", pWaterMan->m_WaterColor.g);
 						XML_Attribute("b", pWaterMan->m_WaterColor.b);
@@ -270,7 +270,7 @@ void CMapWriter::WriteXML(const VfsPath& filename,
 					XML_Setting("WindAngle", pWaterMan->m_WindAngle);
 				}
 			}
-			
+
 			{
 				XML_Element("Postproc");
 				{
@@ -347,7 +347,10 @@ void CMapWriter::WriteXML(const VfsPath& filename,
 				CmpPtr<ICmpPosition> cmpPosition(sim, ent);
 				if (cmpPosition)
 				{
-					CFixedVector3D pos = cmpPosition->GetPosition();
+					CFixedVector3D pos;
+					if (cmpPosition->IsInWorld())
+						pos = cmpPosition->GetPosition();
+
 					CFixedVector3D rot = cmpPosition->GetRotation();
 					{
 						XML_Element("Position");
@@ -399,49 +402,64 @@ void CMapWriter::WriteXML(const VfsPath& filename,
 
 		const std::map<CStrW, CCinemaPath>& paths = pCinema->GetAllPaths();
 		std::map<CStrW, CCinemaPath>::const_iterator it = paths.begin();
-		
+
 		{
 			XML_Element("Paths");
 
 			for ( ; it != paths.end(); ++it )
 			{
-				CStrW name = it->first;
-				float timescale = it->second.GetTimescale();
-				
-				XML_Element("Path");
-				XML_Attribute("name", name);
-				XML_Attribute("timescale", timescale);
-
+				fixed timescale = it->second.GetTimescale();
 				const std::vector<SplineData>& nodes = it->second.GetAllNodes();
+				const std::vector<SplineData>& target_nodes = it->second.getTargetSpline().GetAllNodes();
 				const CCinemaData* data = it->second.GetData();
 				
-				{
-					XML_Element("Distortion");
-					XML_Attribute("mode", data->m_Mode);
-					XML_Attribute("style", data->m_Style);
-					XML_Attribute("growth", data->m_Growth);
-					XML_Attribute("switch", data->m_Switch);
-				}
+				XML_Element("Path");
+				XML_Attribute("name", data->m_Name);
+				XML_Attribute("timescale", timescale);
+				XML_Attribute("orientation", data->m_Orientation);
+				XML_Attribute("mode", data->m_Mode);
+				XML_Attribute("style", data->m_Style);
 
-				for ( ssize_t j=nodes.size()-1; j >= 0; --j )
+				fixed last_target = fixed::Zero();
+				for (size_t i = 0, j = 0; i < nodes.size(); ++i)
 				{
 					XML_Element("Node");
-					
+					fixed distance = i > 0 ? nodes[i - 1].Distance : fixed::Zero();
+					last_target += distance;
+
+					XML_Attribute("deltatime", distance);
+
 					{
 						XML_Element("Position");
-						XML_Attribute("x", nodes[j].Position.X);
-						XML_Attribute("y", nodes[j].Position.Y);
-						XML_Attribute("z", nodes[j].Position.Z);
+						XML_Attribute("x", nodes[i].Position.X);
+						XML_Attribute("y", nodes[i].Position.Y);
+						XML_Attribute("z", nodes[i].Position.Z);
 					}
 
 					{
 						XML_Element("Rotation");
-						XML_Attribute("x", nodes[j].Rotation.X);
-						XML_Attribute("y", nodes[j].Rotation.Y);
-						XML_Attribute("z", nodes[j].Rotation.Z);
+						XML_Attribute("x", nodes[i].Rotation.X);
+						XML_Attribute("y", nodes[i].Rotation.Y);
+						XML_Attribute("z", nodes[i].Rotation.Z);
 					}
 
-					XML_Setting("Time", nodes[j].Distance);
+					if (j >= target_nodes.size())
+						continue;
+
+					fixed target_distance = j > 0 ? target_nodes[j - 1].Distance : fixed::Zero();
+
+					if (target_distance > last_target)
+						continue;
+
+					{
+						XML_Element("Target");
+						XML_Attribute("x", target_nodes[j].Position.X);
+						XML_Attribute("y", target_nodes[j].Position.Y);
+						XML_Attribute("z", target_nodes[j].Position.Z);
+					}
+
+					last_target = fixed::Zero();
+					++j;
 				}
 			}
 		}
