@@ -1,4 +1,4 @@
-/* Copyright (C) 2013 Wildfire Games.
+/* Copyright (C) 2015 Wildfire Games.
  * This file is part of 0 A.D.
  *
  * 0 A.D. is free software: you can redistribute it and/or modify
@@ -44,7 +44,7 @@ bool CObjectBase::Load(const VfsPath& pathname)
 	m_UsedFiles.insert(pathname);
 
 	CXeromyces XeroFile;
-	if (XeroFile.Load(g_VFS, pathname) != PSRETURN_OK)
+	if (XeroFile.Load(g_VFS, pathname, "actor") != PSRETURN_OK)
 		return false;
 
 	// Define all the elements used in the XML file
@@ -63,7 +63,7 @@ bool CObjectBase::Load(const VfsPath& pathname)
 	EL(mesh);
 	EL(texture);
 	EL(textures);
-	EL(colour);
+	EL(color);
 	EL(decal);
 	EL(particles);
 	AT(file);
@@ -109,9 +109,7 @@ bool CObjectBase::Load(const VfsPath& pathname)
 		XERO_ITER_EL(root, child)
 		{
 			if (child.GetNodeName() == el_group)
-			{
-				variantGroupSizes.push_back(child.GetChildNodes().Count);
-			}
+				variantGroupSizes.push_back(child.GetChildNodes().size());
 		}
 
 		m_VariantGroups.resize(variantGroupSizes.size());
@@ -164,7 +162,7 @@ bool CObjectBase::Load(const VfsPath& pathname)
 								if (se.Name == at_file)
 									samp.m_SamplerFile = VfsPath("art/textures/skins") / se.Value.FromUTF8();
 								else if (se.Name == at_name)
-									samp.m_SamplerName = se.Value;
+									samp.m_SamplerName = CStrIntern(se.Value);
 							}
 							currentVariant->m_Samplers.push_back(samp);
 						}
@@ -190,7 +188,7 @@ bool CObjectBase::Load(const VfsPath& pathname)
 						// so remember the relevant particle file as a dependency for this actor
 						m_UsedFiles.insert(file);
 					}
-					else if (option_name == el_colour)
+					else if (option_name == el_color)
 					{
 						currentVariant->m_Color = option.GetText();
 					}
@@ -459,9 +457,9 @@ const CObjectBase::Variation CObjectBase::BuildVariation(const std::vector<u8>&
 		
 		// Same for samplers, though perhaps not strictly necessary:
 		for (std::vector<CObjectBase::Samp>::iterator it = var.m_Samplers.begin(); it != var.m_Samplers.end(); ++it)
-			variation.samplers.erase(it->m_SamplerName);
+			variation.samplers.erase(it->m_SamplerName.string());
 		for (std::vector<CObjectBase::Samp>::iterator it = var.m_Samplers.begin(); it != var.m_Samplers.end(); ++it)
-			variation.samplers.insert(make_pair(it->m_SamplerName, *it));
+			variation.samplers.insert(make_pair(it->m_SamplerName.string(), *it));
 	}
 
 	return variation;
