@@ -1,4 +1,4 @@
-/* Copyright (C) 2013 Wildfire Games.
+/* Copyright (C) 2016 Wildfire Games.
  * This file is part of 0 A.D.
  *
  * 0 A.D. is free software: you can redistribute it and/or modify
@@ -44,11 +44,10 @@ CObjectEntry::CObjectEntry(CObjectBase* base, CSimulation2& simulation) :
 {
 }
 
-template<typename T, typename S> static void delete_pair_2nd(std::pair<T,S> v) { delete v.second; }
-
 CObjectEntry::~CObjectEntry()
 {
-	std::for_each(m_Animations.begin(), m_Animations.end(), delete_pair_2nd<CStr, CSkeletonAnim*>);
+	for (const std::pair<CStr, CSkeletonAnim*>& anim : m_Animations)
+		delete anim.second;
 
 	delete m_Model;
 }
@@ -73,7 +72,7 @@ bool CObjectEntry::BuildVariation(const std::vector<std::set<CStr> >& selections
 		str << variation.color;
 		int r, g, b;
 		if (! (str >> r >> g >> b)) // Any trailing data is ignored
-			LOGERROR("Actor '%s' has invalid RGB colour '%s'", utf8_from_wstring(m_Base->m_ShortName), variation.color);
+			LOGERROR("Actor '%s' has invalid RGB color '%s'", utf8_from_wstring(m_Base->m_ShortName), variation.color);
 		else
 			m_Color = CColor(r/255.0f, g/255.0f, b/255.0f, 1.0f);
 	}
@@ -82,15 +81,14 @@ bool CObjectEntry::BuildVariation(const std::vector<std::set<CStr> >& selections
 	{
 		CMaterial material = g_Renderer.GetMaterialManager().LoadMaterial(m_Base->m_Material);
 		
-		std::vector<CObjectBase::Samp>::iterator samp;
-		for (samp = m_Samplers.begin(); samp != m_Samplers.end(); ++samp)
+		for (const CObjectBase::Samp& samp : m_Samplers)
 		{
-			CTextureProperties textureProps(samp->m_SamplerFile);
+			CTextureProperties textureProps(samp.m_SamplerFile);
 			textureProps.SetWrap(GL_CLAMP_TO_BORDER);
 			CTexturePtr texture = g_Renderer.GetTextureManager().CreateTexture(textureProps);
 			// TODO: Should check which renderpath is selected and only preload the necessary textures.
 			texture->Prefetch(); 
-			material.AddSampler(CMaterial::TextureSampler(samp->m_SamplerName, texture));
+			material.AddSampler(CMaterial::TextureSampler(samp.m_SamplerName, texture));
 		}
 
 		SDecal decal(material,
@@ -131,24 +129,28 @@ bool CObjectEntry::BuildVariation(const std::vector<std::set<CStr> >& selections
 	model->GetMaterial().AddStaticUniform("objectColor", CVector4D(m_Color.r, m_Color.g, m_Color.b, m_Color.a));
 	model->InitModel(modeldef);
 	
-	if (m_Samplers.size() == 0)
-	{
+	if (m_Samplers.empty())
 		LOGERROR("Actor '%s' has no textures.", utf8_from_wstring(m_Base->m_ShortName));
-	}
 	
-	std::vector<CObjectBase::Samp>::iterator samp;
-	for (samp = m_Samplers.begin(); samp != m_Samplers.end(); ++samp)
+	for (const CObjectBase::Samp& samp : m_Samplers)
 	{
-		CTextureProperties textureProps(samp->m_SamplerFile);
+		CTextureProperties textureProps(samp.m_SamplerFile);
 		textureProps.SetWrap(GL_CLAMP_TO_EDGE);
 		CTexturePtr texture = g_Renderer.GetTextureManager().CreateTexture(textureProps);
 		// if we've loaded this model we're probably going to render it soon, so prefetch its texture. 
 		// All textures are prefetched even in the fixed pipeline, including the normal maps etc.
 		// TODO: Should check which renderpath is selected and only preload the necessary textures.
 		texture->Prefetch(); 
-		model->GetMaterial().AddSampler(CMaterial::TextureSampler(samp->m_SamplerName, texture));
+		model->GetMaterial().AddSampler(CMaterial::TextureSampler(samp.m_SamplerName, texture));
 	}
 
+	for (const CStrIntern& requSampName : model->GetMaterial().GetRequiredSampler())
+	{
+		if (std::find_if(m_Samplers.begin(), m_Samplers.end(),
+		                 [&](const CObjectBase::Samp& sampler) { return sampler.m_SamplerName == requSampName; }) == m_Samplers.end())
+			LOGERROR("Actor %s: required texture sampler %s not found (material %s)", utf8_from_wstring(m_Base->m_ShortName), requSampName.string().c_str(), m_Base->m_Material.string8().c_str());
+	}
+	
 	// calculate initial object space bounds, based on vertex positions
 	model->CalcStaticObjectBounds();
 
@@ -157,13 +159,7 @@ bool CObjectEntry::BuildVariation(const std::vector<std::set<CStr> >& selections
 	{
 		CStr name = it->first.LowerCase();
 
-		// TODO: Use consistent names everywhere, then remove this translation section.
-		// (It's just mapping the names used in actors onto the names used by code.)
-		if (name == "attack") name = "melee";
-		else if (name == "chop") name = "gather";
-		else if (name == "decay") name = "corpse";
-
-		if (! it->second.m_FileName.empty())
+		if (!it->second.m_FileName.empty())
 		{
 			CSkeletonAnim* anim = model->BuildAnimation(it->second.m_FileName, name, it->second.m_Speed, it->second.m_ActionPos, it->second.m_ActionPos2, it->second.m_SoundPos);
 			if (anim)
@@ -195,10 +191,8 @@ bool CObjectEntry::BuildVariation(const std::vector<std::set<CStr> >& selections
 
 	// build props - TODO, RC - need to fix up bounds here
 	// TODO: Make sure random variations get handled correctly when a prop fails
-	for (size_t p = 0; p < props.size(); p++)
+	for (const CObjectBase::Prop& prop : props)
 	{
-		const CObjectBase::Prop& prop = props[p];
-
 		// Pluck out the special attachpoint 'projectile'
 		if (prop.m_PropPointName == "projectile")
 		{
