@@ -18,11 +18,11 @@
 #ifndef INCLUDED_SHADERPROGRAM
 #define INCLUDED_SHADERPROGRAM
 
+#include "graphics/ShaderProgramPtr.h"
 #include "graphics/Texture.h"
 #include "lib/ogl.h"
 #include "lib/file/vfs/vfs_path.h"
 #include "lib/res/handle.h"
-#include "ps/CStr.h"
 
 #include <map>
 
@@ -205,6 +205,4 @@ protected:
 	int m_ValidStreams; // which streams have been specified via VertexPointer etc since the last Bind
 };
 
-typedef shared_ptr<CShaderProgram> CShaderProgramPtr;
-
 #endif // INCLUDED_SHADERPROGRAM
