@@ -1,4 +1,4 @@
-/* Copyright (C) 2013 Wildfire Games.
+/* Copyright (C) 2015 Wildfire Games.
  * This file is part of 0 A.D.
  *
  * 0 A.D. is free software: you can redistribute it and/or modify
@@ -45,7 +45,7 @@ CTerrainTextureEntry::CTerrainTextureEntry(CTerrainPropertiesPtr properties, con
 	ENSURE(properties);
 	
 	CXeromyces XeroFile;
-	if (XeroFile.Load(g_VFS, path) != PSRETURN_OK)
+	if (XeroFile.Load(g_VFS, path, "terrain_texture") != PSRETURN_OK)
 	{
 		LOGERROR("Terrain xml not found (%s)", path.string8());
 		return;
@@ -99,7 +99,7 @@ CTerrainTextureEntry::CTerrainTextureEntry(CTerrainPropertiesPtr properties, con
 					else if (se.Name == at_name)
 						name = se.Value;
 				}
-				samplers.push_back(make_pair(name, path));
+				samplers.emplace_back(name, path);
 			}
 		
 		}
@@ -174,7 +174,7 @@ CTerrainTextureEntry::~CTerrainTextureEntry()
 		(*it)->RemoveTerrain(this);
 }
 
-// BuildBaseColor: calculate the root colour of the texture, used for coloring minimap, and store
+// BuildBaseColor: calculate the root color of the texture, used for coloring minimap, and store
 // in m_BaseColor member
 void CTerrainTextureEntry::BuildBaseColor()
 {
@@ -186,10 +186,10 @@ void CTerrainTextureEntry::BuildBaseColor()
 		return;
 	}
 
-	// Use the texture colour if available
+	// Use the texture color if available
 	if (GetTexture()->TryLoad())
 	{
-		m_BaseColor = GetTexture()->GetBaseColour();
+		m_BaseColor = GetTexture()->GetBaseColor();
 		m_BaseColorValid = true;
 	}
 }
