@@ -24,10 +24,9 @@
 #include "graphics/ShaderProgram.h"
 #include "lib/ogl.h"
 #include "ps/CStrIntern.h"
+#include "ps/GameSetup/Config.h"
 #include "renderer/Renderer.h"
 
-extern int g_xres, g_yres;
-
 CTextRenderer::CTextRenderer(const CShaderProgramPtr& shader) :
 	m_Shader(shader)
 {
@@ -38,12 +37,15 @@ CTextRenderer::CTextRenderer(const CShaderProgramPtr& shader) :
 
 void CTextRenderer::ResetTransform()
 {
+	float xres = g_xres * g_GuiScale;
+	float yres = g_yres * g_GuiScale;
+
 	m_Transform.SetIdentity();
 	m_Transform.Scale(1.0f, -1.f, 1.0f);
-	m_Transform.Translate(0.0f, (float)g_yres, -1000.0f);
+	m_Transform.Translate(0.0f, yres, -1000.0f);
 
 	CMatrix3D proj;
-	proj.SetOrtho(0.f, (float)g_xres, 0.f, (float)g_yres, -1.f, 1000.f);
+	proj.SetOrtho(0.f, xres, 0.f, yres, -1.f, 1000.f);
 	m_Transform = proj * m_Transform;
 	m_Dirty = true;
 }
@@ -106,7 +108,7 @@ void CTextRenderer::PrintfAdvance(const wchar_t* fmt, ...)
 	va_end(args);
 
 	if (ret < 0)
-		debug_printf(L"CTextRenderer::Printf vswprintf failed (buffer size exceeded?) - return value %d, errno %d\n", ret, errno);
+		debug_printf("CTextRenderer::Printf vswprintf failed (buffer size exceeded?) - return value %d, errno %d\n", ret, errno);
 
 	PutAdvance(buf);
 }
@@ -122,7 +124,7 @@ void CTextRenderer::PrintfAt(float x, float y, const wchar_t* fmt, ...)
 	va_end(args);
 
 	if (ret < 0)
-		debug_printf(L"CTextRenderer::PrintfAt vswprintf failed (buffer size exceeded?) - return value %d, errno %d\n", ret, errno);
+		debug_printf("CTextRenderer::PrintfAt vswprintf failed (buffer size exceeded?) - return value %d, errno %d\n", ret, errno);
 
 	Put(x, y, buf);
 }
@@ -144,6 +146,14 @@ void CTextRenderer::Put(float x, float y, const wchar_t* buf)
 	PutString(x, y, new std::wstring(buf), true);
 }
 
+void CTextRenderer::Put(float x, float y, const char* buf)
+{
+	if (buf[0] == 0)
+		return; // empty string; don't bother storing
+
+	PutString(x, y, new std::wstring(wstring_from_utf8(buf)), true);
+}
+
 void CTextRenderer::Put(float x, float y, const std::wstring* buf)
 {
 	if (buf->empty())
