@@ -1,4 +1,4 @@
-/* Copyright (C) 2014 Wildfire Games.
+/* Copyright (C) 2015 Wildfire Games.
  * This file is part of 0 A.D.
  *
  * 0 A.D. is free software: you can redistribute it and/or modify
@@ -16,26 +16,20 @@
  */
 
 /*
-GUI text, handles text stuff
-
---Overview--
-
-	Mainly contains struct SGUIText and friends.
-	 Actual text processing is made in CGUI::GenerateText()
-
---More info--
-
-	Check GUI.h
-
-*/
+ * GUI text, handles text stuff
+ *
+ * --Overview--
+ * Mainly contains struct SGUIText and friends.
+ * Actual text processing is made in CGUI::GenerateText()
+ *
+ * --More info--
+ * Check GUI.h
+ *
+ */
 
 #ifndef INCLUDED_GUITEXT
 #define INCLUDED_GUITEXT
 
-
-//--------------------------------------------------------
-//  Includes / Compiler directives
-//--------------------------------------------------------
 #include <list>
 
 #include "CGUISprite.h"
@@ -43,13 +37,9 @@ GUI text, handles text stuff
 
 class CGUI;
 
-//--------------------------------------------------------
-//  Declarations
-//--------------------------------------------------------
-
 /**
  * An SGUIText object is a parsed string, divided into
- * text-rendering components. Each component, being a 
+ * text-rendering components. Each component, being a
  * call to the Renderer. For instance, if you by tags
  * change the color, then the GUI will have to make
  * individual calls saying it want that color on the
@@ -99,7 +89,7 @@ struct SGUIText
 	 */
 	struct STextCall
 	{
-		STextCall() : 
+		STextCall() :
 			m_UseCustomColor(false),
 			m_Bold(false), m_Italic(false), m_Underlined(false),
 			m_pSpriteCall(NULL) {}
@@ -220,7 +210,7 @@ public:
 			 * @return True if m_TagType was set.
 			 */
 			bool SetTagType(const CStrW& tagtype);
-			TagType GetTagType(const CStrW& tagtype);
+			TagType GetTagType(const CStrW& tagtype) const;
 
 
 			/**
@@ -258,8 +248,8 @@ public:
 	struct SFeedback
 	{
 		// Constants
-		static const int Left=0;
-		static const int Right=1;
+		static const int Left = 0;
+		static const int Right = 1;
 
 		/**
 		 * Reset all member data.
@@ -319,12 +309,7 @@ public:
 	 *		  to make several GenerateTextCall in different phases,
 	 *		  it avoids duplicates.
 	 */
-	void GenerateTextCall(const CGUI *pGUI,
-						  SFeedback &Feedback,
-						  CStrIntern DefaultFont,
-						  const int &from, const int &to,
-						  const bool FirstLine,
-						  const IGUIObject *pObject=NULL) const;
+	void GenerateTextCall(const CGUI* pGUI, SFeedback& Feedback, CStrIntern DefaultFont, const int& from, const int& to, const bool FirstLine, const IGUIObject* pObject = NULL) const;
 
 	/**
 	 * Words
@@ -348,4 +333,4 @@ private:
 	CStrW m_OriginalString;
 };
 
-#endif
+#endif // INCLUDED_GUITEXT
