@@ -1,4 +1,4 @@
-/* Copyright (C) 2014 Wildfire Games.
+/* Copyright (C) 2016 Wildfire Games.
  * This file is part of 0 A.D.
  *
  * 0 A.D. is free software: you can redistribute it and/or modify
@@ -30,14 +30,16 @@
 #include "graphics/TerritoryTexture.h"
 #include "gui/GUI.h"
 #include "gui/GUIManager.h"
-#include "lib/ogl.h"
-#include "lib/external_libraries/libsdl.h"
 #include "lib/bits.h"
+#include "lib/external_libraries/libsdl.h"
+#include "lib/ogl.h"
 #include "lib/timer.h"
 #include "ps/ConfigDB.h"
+#include "ps/Filesystem.h"
 #include "ps/Game.h"
 #include "ps/Profile.h"
 #include "ps/World.h"
+#include "ps/XML/Xeromyces.h"
 #include "renderer/Renderer.h"
 #include "renderer/WaterManager.h"
 #include "scriptinterface/ScriptInterface.h"
@@ -70,9 +72,12 @@ CMiniMap::CMiniMap() :
 	m_Clicking = false;
 	m_MouseHovering = false;
 
+	// Register Relax NG validator
+	CXeromyces::AddValidator(g_VFS, "pathfinder", "simulation/data/pathfinder.rng");
+
 	// Get the maximum height for unit passage in water.
 	CParamNode externalParamNode;
-	CParamNode::LoadXML(externalParamNode, L"simulation/data/pathfinder.xml");
+	CParamNode::LoadXML(externalParamNode, L"simulation/data/pathfinder.xml", "pathfinder");
 	const CParamNode pathingSettings = externalParamNode.GetChild("Pathfinder").GetChild("PassabilityClasses");
 	if (pathingSettings.GetChild("default").IsOk() && pathingSettings.GetChild("default").GetChild("MaxWaterDepth").IsOk())
 		m_ShallowPassageHeight = pathingSettings.GetChild("default").GetChild("MaxWaterDepth").ToFloat();
@@ -82,11 +87,11 @@ CMiniMap::CMiniMap() :
 	m_AttributePos.type = GL_FLOAT;
 	m_AttributePos.elems = 2;
 	m_VertexArray.AddAttribute(&m_AttributePos);
-	
+
 	m_AttributeColor.type = GL_UNSIGNED_BYTE;
 	m_AttributeColor.elems = 4;
 	m_VertexArray.AddAttribute(&m_AttributeColor);
-	
+
 	m_VertexArray.SetNumVertices(MAX_ENTITIES_DRAWN);
 	m_VertexArray.Layout();
 
@@ -101,7 +106,7 @@ CMiniMap::CMiniMap() :
 
 	VertexArrayIterator<float[2]> attrPos = m_AttributePos.GetIterator<float[2]>();
 	VertexArrayIterator<u8[4]> attrColor = m_AttributeColor.GetIterator<u8[4]>();
-	for (u16 i = 0; i < MAX_ENTITIES_DRAWN; i++)
+	for (u16 i = 0; i < MAX_ENTITIES_DRAWN; ++i)
 	{
 		(*attrColor)[0] = 0;
 		(*attrColor)[1] = 0;
@@ -133,60 +138,44 @@ CMiniMap::~CMiniMap()
 	Destroy();
 }
 
-void CMiniMap::HandleMessage(SGUIMessage &Message)
+void CMiniMap::HandleMessage(SGUIMessage& Message)
 {
-	switch(Message.type)
+	switch (Message.type)
 	{
 	case GUIM_MOUSE_PRESS_LEFT:
+		if (m_MouseHovering)
 		{
-			if (m_MouseHovering)
-			{
-				SetCameraPos();
-				m_Clicking = true;
-			}
-			break;
+			SetCameraPos();
+			m_Clicking = true;
 		}
+		break;
 	case GUIM_MOUSE_RELEASE_LEFT:
-		{
-			if(m_MouseHovering && m_Clicking)
-				SetCameraPos();
-			m_Clicking = false;
-			break;
-		}
+		if (m_MouseHovering && m_Clicking)
+			SetCameraPos();
+		m_Clicking = false;
+		break;
 	case GUIM_MOUSE_DBLCLICK_LEFT:
-		{
-			if(m_MouseHovering && m_Clicking)
-				SetCameraPos();
-			m_Clicking = false;
-			break;
-		}
+		if (m_MouseHovering && m_Clicking)
+			SetCameraPos();
+		m_Clicking = false;
+		break;
 	case GUIM_MOUSE_ENTER:
-		{
-			m_MouseHovering = true;
-			break;
-		}
+		m_MouseHovering = true;
+		break;
 	case GUIM_MOUSE_LEAVE:
-		{
-			m_Clicking = false;
-			m_MouseHovering = false;
-			break;
-		}
+		m_Clicking = false;
+		m_MouseHovering = false;
+		break;
 	case GUIM_MOUSE_RELEASE_RIGHT:
-		{
-			CMiniMap::FireWorldClickEvent(SDL_BUTTON_RIGHT, 1);
-			break;
-		}
+		CMiniMap::FireWorldClickEvent(SDL_BUTTON_RIGHT, 1);
+		break;
 	case GUIM_MOUSE_DBLCLICK_RIGHT:
-		{
-			CMiniMap::FireWorldClickEvent(SDL_BUTTON_RIGHT, 2);
-			break;
-		}
+		CMiniMap::FireWorldClickEvent(SDL_BUTTON_RIGHT, 2);
+		break;
 	case GUIM_MOUSE_MOTION:
-		{
-			if (m_MouseHovering && m_Clicking)
-				SetCameraPos();
-			break;
-		}
+		if (m_MouseHovering && m_Clicking)
+			SetCameraPos();
+		break;
 	case GUIM_MOUSE_WHEEL_DOWN:
 	case GUIM_MOUSE_WHEEL_UP:
 		Message.Skip();
@@ -194,7 +183,7 @@ void CMiniMap::HandleMessage(SGUIMessage &Message)
 
 	default:
 		break;
-	}	// switch
+	}
 }
 
 bool CMiniMap::MouseOver()
@@ -244,11 +233,11 @@ float CMiniMap::GetAngle()
 	return -atan2(cameraIn.X, cameraIn.Z);
 }
 
-void CMiniMap::FireWorldClickEvent(int button, int clicks)
+void CMiniMap::FireWorldClickEvent(int UNUSED(button), int UNUSED(clicks))
 {
 	JSContext* cx = g_GUI->GetActiveGUI()->GetScriptInterface()->GetContext();
 	JSAutoRequest rq(cx);
-	
+
 	float x, z;
 	GetMouseWorldCoordinates(x, z);
 
@@ -257,9 +246,6 @@ void CMiniMap::FireWorldClickEvent(int button, int clicks)
 	g_GUI->GetActiveGUI()->GetScriptInterface()->SetProperty(coords, "x", x, false);
 	g_GUI->GetActiveGUI()->GetScriptInterface()->SetProperty(coords, "z", z, false);
 	ScriptEvent("worldclick", coords);
-
-	UNUSED2(button);
-	UNUSED2(clicks);
 }
 
 // This sets up and draws the rectangle on the minimap
@@ -267,10 +253,8 @@ void CMiniMap::FireWorldClickEvent(int button, int clicks)
 void CMiniMap::DrawViewRect(CMatrix3D transform)
 {
 	// Compute the camera frustum intersected with a fixed-height plane.
-	// TODO: Currently we hard-code the height, so this'll be dodgy when maps aren't the
-	// expected height - how can we make it better without the view rect wobbling in
-	// size while the player scrolls?
-	float h = 16384.f * HEIGHT_SCALE;
+	// Use the water height as a fixed base height, which should be the lowest we can go
+	float h = g_Renderer.GetWaterManager()->m_WaterHeight;
 	const float width = m_CachedActualSize.GetWidth();
 	const float height = m_CachedActualSize.GetHeight();
 	const float invTileMapSize = 1.0f / float(TERRAIN_TILE_SIZE * m_MapSize);
@@ -282,7 +266,8 @@ void CMiniMap::DrawViewRect(CMatrix3D transform)
 	hitPt[3] = m_Camera->GetWorldCoordinates(0, 0, h);
 
 	float ViewRect[4][2];
-	for (int i = 0; i < 4; i++) {
+	for (int i = 0; i < 4; ++i)
+	{
 		// convert to minimap space
 		ViewRect[i][0] = (width * hitPt[i].X * invTileMapSize);
 		ViewRect[i][1] = (height * hitPt[i].Z * invTileMapSize);
@@ -299,7 +284,7 @@ void CMiniMap::DrawViewRect(CMatrix3D transform)
 	glScissor(
 		m_CachedActualSize.left / g_GuiScale,
 		g_Renderer.GetHeight() - m_CachedActualSize.bottom / g_GuiScale,
-		width / g_GuiScale, 
+		width / g_GuiScale,
 		height / g_GuiScale);
 	glEnable(GL_SCISSOR_TEST);
 	glLineWidth(2.0f);
@@ -421,7 +406,7 @@ void CMiniMap::Draw()
 	const double cur_time = timer_Time();
 	const bool doUpdate = cur_time - last_time > 0.5;
 	if (doUpdate)
-	{	
+	{
 		last_time = cur_time;
 		if (m_TerrainDirty)
 			RebuildTerrainTexture();
@@ -463,7 +448,7 @@ void CMiniMap::Draw()
 	CTerritoryTexture& territoryTexture = g_Game->GetView()->GetTerritoryTexture();
 
 	shader->BindTexture(str_baseTex, territoryTexture.GetTexture());
-	const CMatrix3D *territoryTransform = territoryTexture.GetMinimapTextureMatrix();
+	const CMatrix3D* territoryTransform = territoryTexture.GetMinimapTextureMatrix();
 	shader->Uniform(str_transform, baseTransform);
 	shader->Uniform(str_textureTransform, *territoryTransform);
 
@@ -482,7 +467,7 @@ void CMiniMap::Draw()
 
 	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
 
-	const CMatrix3D *losTransform = losTexture.GetMinimapTextureMatrix();
+	const CMatrix3D* losTransform = losTexture.GetMinimapTextureMatrix();
 	shader->Uniform(str_transform, baseTransform);
 	shader->Uniform(str_textureTransform, *losTransform);
 
@@ -545,7 +530,7 @@ void CMiniMap::Draw()
 			ICmpMinimap* cmpMinimap = static_cast<ICmpMinimap*>(it->second);
 			if (cmpMinimap->GetRenderData(v.r, v.g, v.b, posX, posZ))
 			{
-				ICmpRangeManager::ELosVisibility vis = cmpRangeManager->GetLosVisibility(it->first, g_Game->GetPlayerID());
+				ICmpRangeManager::ELosVisibility vis = cmpRangeManager->GetLosVisibility(it->first, g_Game->GetSimulation2()->GetSimContext().GetCurrentDisplayedPlayer());
 				if (vis != ICmpRangeManager::VIS_HIDDEN)
 				{
 					v.a = 255;
@@ -656,10 +641,10 @@ void CMiniMap::RebuildTerrainTexture()
 
 	m_TerrainDirty = false;
 
-	for(u32 j = 0; j < h; j++)
+	for (u32 j = 0; j < h; ++j)
 	{
-		u32 *dataPtr = m_TerrainData + ((y + j) * (m_MapSize - 1)) + x;
-		for(u32 i = 0; i < w; i++)
+		u32* dataPtr = m_TerrainData + ((y + j) * (m_MapSize - 1)) + x;
+		for (u32 i = 0; i < w; ++i)
 		{
 			float avgHeight = ( m_Terrain->GetVertexGroundLevel((int)i, (int)j)
 					+ m_Terrain->GetVertexGroundLevel((int)i+1, (int)j)
@@ -684,11 +669,11 @@ void CMiniMap::RebuildTerrainTexture()
 
 				u32 color = 0xFFFFFFFF;
 
-				CMiniPatch *mp = m_Terrain->GetTile(x + i, y + j);
-				if(mp)
+				CMiniPatch* mp = m_Terrain->GetTile(x + i, y + j);
+				if (mp)
 				{
-					CTerrainTextureEntry *tex = mp->GetTextureEntry();
-					if(tex)
+					CTerrainTextureEntry* tex = mp->GetTextureEntry();
+					if (tex)
 					{
 						// If the texture can't be loaded yet, set the dirty flags
 						// so we'll try regenerating the terrain texture again soon
@@ -711,12 +696,11 @@ void CMiniMap::RebuildTerrainTexture()
 
 void CMiniMap::Destroy()
 {
-	if(m_TerrainTexture)
+	if (m_TerrainTexture)
 	{
 		glDeleteTextures(1, &m_TerrainTexture);
 		m_TerrainTexture = 0;
 	}
 
-	delete[] m_TerrainData;
-	m_TerrainData = 0;
+	SAFE_ARRAY_DELETE(m_TerrainData);
 }
