@@ -87,6 +87,17 @@
 #endif
 
 
+/**
+ * mark a function as noreturn for static analyzer purposes.
+ * currently only for clang-analyzer.
+ */
+#if __has_feature(attribute_analyzer_noreturn)
+# define ANALYZER_NORETURN __attribute__((analyzer_noreturn))
+#else
+# define ANALYZER_NORETURN
+#endif
+
+
 /**
  * "unreachable code" helpers
  *
