@@ -1,4 +1,4 @@
-/* Copyright (c) 2012 Wildfire Games
+/* Copyright (c) 2015 Wildfire Games
  *
  * Permission is hereby granted, free of charge, to any person obtaining
  * a copy of this software and associated documentation files (the
@@ -56,20 +56,6 @@
 #define CONFIG2_CACHE_READ_ONLY 1
 #endif
 
-// enable the wsdl emulator in Windows builds.
-//
-// NOTE: the official SDL distribution has two problems on Windows:
-// - it specifies "/defaultlib:msvcrt.lib". this is troublesome because
-//   multiple heaps are active; errors result when allocated blocks are
-//   (for reasons unknown) passed to a different heap to be freed.
-//   one workaround is to add "/nodefaultlib:msvcrt.lib" to the linker
-//   command line in debug configurations.
-// - it doesn't support color hardware mouse cursors and clashes with
-//   cursor.cpp's efforts by resetting the mouse cursor after movement.
-#ifndef CONFIG2_WSDL
-# define CONFIG2_WSDL 1
-#endif
-
 #ifndef CONFIG2_FILE_ENABLE_AIO
 // work around a bug introduced in Linux 2.6.38
 // (http://www.wildfiregames.com/forum/index.php?showtopic=14561&view=findpost&p=217710)
@@ -111,4 +97,14 @@
 # define CONFIG2_NVTT 1
 #endif
 
+// allow use of lobby
+#ifndef CONFIG2_LOBBY
+# define CONFIG2_LOBBY 1
+#endif
+
+// allow use of miniupnpc
+#ifndef CONFIG2_MINIUPNPC
+# define CONFIG2_MINIUPNPC 1
+#endif
+
 #endif	// #ifndef INCLUDED_CONFIG2
