@@ -1,4 +1,4 @@
-/* Copyright (c) 2010 Wildfire Games
+/* Copyright (c) 2015 Wildfire Games
  *
  * Permission is hereby granted, free of charge, to any person obtaining
  * a copy of this software and associated documentation files (the
@@ -130,7 +130,7 @@ Status GetDirectoryEntries(const OsPath& path, CFileInfos* files, DirectoryNames
 }
 
 
-Status CreateDirectories(const OsPath& path, mode_t mode)
+Status CreateDirectories(const OsPath& path, mode_t mode, bool breakpoint)
 {
 	if(path.empty())
 		return INFO::OK;
@@ -146,15 +146,18 @@ Status CreateDirectories(const OsPath& path, mode_t mode)
 	// If we were passed a path ending with '/', strip the '/' now so that
 	// we can consistently use Parent to find parent directory names
 	if(path.IsDirectory())
-		return CreateDirectories(path.Parent(), mode);
+		return CreateDirectories(path.Parent(), mode, breakpoint);
 
 	RETURN_STATUS_IF_ERR(CreateDirectories(path.Parent(), mode));
 
 	errno = 0;
 	if(wmkdir(path, mode) != 0)
 	{
-		debug_printf("CreateDirectories: failed to mkdir %s (mode %d)", path.string8().c_str(), mode);
-		WARN_RETURN(StatusFromErrno());
+		debug_printf("CreateDirectories: failed to mkdir %s (mode %d)\n", path.string8().c_str(), mode);
+		if (breakpoint)
+			WARN_RETURN(StatusFromErrno());
+		else
+			return StatusFromErrno();
 	}
 
 	return INFO::OK;
