@@ -1,4 +1,4 @@
-/* Copyright (c) 2013 Wildfire Games
+/* Copyright (c) 2014 Wildfire Games
  *
  * Permission is hereby granted, free of charge, to any person obtaining
  * a copy of this software and associated documentation files (the
@@ -43,11 +43,13 @@ static const StatusDefinition vfsStatusDefinitions[] = {
 STATUS_ADD_DEFINITIONS(vfsStatusDefinitions);
 
 static pthread_mutex_t vfs_mutex = PTHREAD_MUTEX_INITIALIZER;
+namespace {
 struct ScopedLock
 {
 	ScopedLock() { pthread_mutex_lock(&vfs_mutex); }
 	~ScopedLock() { pthread_mutex_unlock(&vfs_mutex); }
 };
+} // namespace
 
 class VFS : public IVFS
 {
@@ -77,7 +79,7 @@ public:
 		return INFO::OK;
 	}
 
-	virtual Status GetFileInfo(const VfsPath& pathname, FileInfo* pfileInfo) const
+	virtual Status GetFileInfo(const VfsPath& pathname, CFileInfo* pfileInfo) const
 	{
 		ScopedLock s;
 		VfsDirectory* directory; VfsFile* file;
@@ -85,7 +87,7 @@ public:
 		if(!pfileInfo)	// just indicate if the file exists without raising warnings.
 			return ret;
 		WARN_RETURN_STATUS_IF_ERR(ret);
-		*pfileInfo = FileInfo(file->Name(), file->Size(), file->MTime());
+		*pfileInfo = CFileInfo(file->Name(), file->Size(), file->MTime());
 		return INFO::OK;
 	}
 
@@ -98,11 +100,11 @@ public:
 		return INFO::OK;
 	}
 
-	virtual Status GetDirectoryEntries(const VfsPath& path, FileInfos* fileInfos, DirectoryNames* subdirectoryNames) const
+	virtual Status GetDirectoryEntries(const VfsPath& path, CFileInfos* fileInfos, DirectoryNames* subdirectoryNames) const
 	{
 		ScopedLock s;
 		VfsDirectory* directory;
-		WARN_RETURN_STATUS_IF_ERR(vfs_Lookup(path, &m_rootDirectory, directory, 0));
+		RETURN_STATUS_IF_ERR(vfs_Lookup(path, &m_rootDirectory, directory, 0));
 
 		if(fileInfos)
 		{
@@ -112,7 +114,7 @@ public:
 			for(VfsDirectory::VfsFiles::const_iterator it = files.begin(); it != files.end(); ++it)
 			{
 				const VfsFile& file = it->second;
-				fileInfos->push_back(FileInfo(file.Name(), file.Size(), file.MTime()));
+				fileInfos->push_back(CFileInfo(file.Name(), file.Size(), file.MTime()));
 			}
 		}
 
@@ -132,7 +134,12 @@ public:
 	{
 		ScopedLock s;
 		VfsDirectory* directory;
-		WARN_RETURN_STATUS_IF_ERR(vfs_Lookup(pathname, &m_rootDirectory, directory, 0, VFS_LOOKUP_ADD|VFS_LOOKUP_CREATE|VFS_LOOKUP_CREATE_ALWAYS));
+		Status st;
+		st = vfs_Lookup(pathname, &m_rootDirectory, directory, 0, VFS_LOOKUP_ADD|VFS_LOOKUP_CREATE|VFS_LOOKUP_CREATE_ALWAYS);
+		if (st == ERR::FILE_ACCESS)
+			return ERR::FILE_ACCESS;
+
+		WARN_RETURN_STATUS_IF_ERR(st);
 
 		const PRealDirectory& realDirectory = directory->AssociatedDirectory();
 		const OsPath name = pathname.Filename();
@@ -163,8 +170,8 @@ public:
 			s.~ScopedLock();
 			return CreateFile(pathname, fileContents, size);
 		}
-		else
-			WARN_RETURN_STATUS_IF_ERR(st);
+
+		WARN_RETURN_STATUS_IF_ERR(st);
 
 		RealDirectory realDirectory(file->Loader()->Path(), file->Priority(), directory->AssociatedDirectory()->Flags());
 		RETURN_STATUS_IF_ERR(realDirectory.Store(pathname.Filename(), fileContents, size));
