@@ -1,4 +1,4 @@
-/* Copyright (C) 2014 Wildfire Games.
+/* Copyright (C) 2016 Wildfire Games.
  * This file is part of 0 A.D.
  *
  * 0 A.D. is free software: you can redistribute it and/or modify
@@ -24,6 +24,7 @@
 #include "lib/utf8.h"
 #include "ps/CLogger.h"
 #include "ps/ConfigDB.h"
+#include "ps/Pyrogenesis.h"
 #include "scriptinterface/ScriptInterface.h"
 
 //debug
@@ -67,7 +68,7 @@ IXmppClient* IXmppClient::create(const std::string& sUsername, const std::string
  * @param regOpt If we are just registering or not.
  */
 XmppClient::XmppClient(const std::string& sUsername, const std::string& sPassword, const std::string& sRoom, const std::string& sNick, const int historyRequestSize, bool regOpt)
-	: m_client(NULL), m_mucRoom(NULL), m_registration(NULL), m_username(sUsername), m_password(sPassword), m_nick(sNick)
+	: m_client(NULL), m_mucRoom(NULL), m_registration(NULL), m_username(sUsername), m_password(sPassword), m_nick(sNick), m_initialLoadComplete(false)
 {
 	// Read lobby configuration from default.cfg
 	std::string sServer;
@@ -81,7 +82,7 @@ XmppClient::XmppClient(const std::string& sUsername, const std::string& sPasswor
 
 	// If we are connecting, use the full jid and a password
 	// If we are registering, only use the server name
-	if(!regOpt)
+	if (!regOpt)
 		m_client = new glooxwrapper::Client(clientJid, sPassword);
 	else
 		m_client = new glooxwrapper::Client(sServer);
@@ -97,18 +98,18 @@ XmppClient::XmppClient(const std::string& sUsername, const std::string& sPasswor
 
 	m_client->registerConnectionListener(this);
 	m_client->setPresence(gloox::Presence::Available, -1);
-	m_client->disco()->setVersion("Pyrogenesis", "0.0.18");
+	m_client->disco()->setVersion("Pyrogenesis", engine_version);
 	m_client->disco()->setIdentity("client", "bot");
 	m_client->setCompression(false);
 
 	m_client->registerStanzaExtension(new GameListQuery());
-	m_client->registerIqHandler(this, ExtGameListQuery);
+	m_client->registerIqHandler(this, EXTGAMELISTQUERY);
 
 	m_client->registerStanzaExtension(new BoardListQuery());
-	m_client->registerIqHandler(this, ExtBoardListQuery);
+	m_client->registerIqHandler(this, EXTBOARDLISTQUERY);
 
 	m_client->registerStanzaExtension(new ProfileQuery());
-	m_client->registerIqHandler(this, ExtProfileQuery);
+	m_client->registerIqHandler(this, EXTPROFILEQUERY);
 
 	m_client->registerMessageHandler(this);
 
@@ -144,17 +145,18 @@ XmppClient::~XmppClient()
 
 	delete m_client;
 
-	for (std::vector<const glooxwrapper::Tag*>::const_iterator it = m_GameList.begin(); it != m_GameList.end(); ++it)
-		glooxwrapper::Tag::free(*it);
-	for (std::vector<const glooxwrapper::Tag*>::const_iterator it = m_BoardList.begin(); it != m_BoardList.end(); ++it)
-		glooxwrapper::Tag::free(*it);
-	for (std::vector<const glooxwrapper::Tag*>::const_iterator it = m_Profile.begin(); it != m_Profile.end(); ++it)
-		glooxwrapper::Tag::free(*it);
+	for (const glooxwrapper::Tag* const& t : m_GameList)
+		glooxwrapper::Tag::free(t);
+	for (const glooxwrapper::Tag* const& t : m_BoardList)
+		glooxwrapper::Tag::free(t);
+	for (const glooxwrapper::Tag* const& t : m_Profile)
+		glooxwrapper::Tag::free(t);
 }
 
 /// Network
 void XmppClient::connect()
 {
+	m_initialLoadComplete = false;
 	m_client->connect(false);
 }
 
@@ -187,7 +189,7 @@ void XmppClient::onConnect()
 {
 	if (m_mucRoom)
 	{
-		CreateSimpleMessage("system", "connected");
+		CreateGUIMessage("system", "connected");
 		m_mucRoom->join();
 	}
 
@@ -206,21 +208,18 @@ void XmppClient::onDisconnect(gloox::ConnectionError error)
 		m_mucRoom->leave();
 
 	// Clear game, board and player lists.
-	for (std::vector<const glooxwrapper::Tag*>::const_iterator it = m_GameList.begin(); it != m_GameList.end(); ++it)
-		glooxwrapper::Tag::free(*it);
-	for (std::vector<const glooxwrapper::Tag*>::const_iterator it = m_BoardList.begin(); it != m_BoardList.end(); ++it)
-		glooxwrapper::Tag::free(*it);
-	for (std::vector<const glooxwrapper::Tag*>::const_iterator it = m_Profile.begin(); it != m_Profile.end(); ++it)
-		glooxwrapper::Tag::free(*it);
+	for (const glooxwrapper::Tag* const& t : m_GameList)
+		glooxwrapper::Tag::free(t);
+	for (const glooxwrapper::Tag* const& t : m_BoardList)
+		glooxwrapper::Tag::free(t);
+	for (const glooxwrapper::Tag* const& t : m_Profile)
+		glooxwrapper::Tag::free(t);
 	m_BoardList.clear();
 	m_GameList.clear();
 	m_PlayerMap.clear();
 	m_Profile.clear();
 
-	if(error == gloox::ConnAuthenticationFailed)
-		CreateSimpleMessage("system", g_L10n.Translate("Authentication failed"), "error");
-	else
-		CreateSimpleMessage("system", "disconnected");
+	CreateGUIMessage("system", "disconnected", ConnectionErrorToString(error));
 }
 
 /**
@@ -246,8 +245,7 @@ bool XmppClient::onTLSConnect(const glooxwrapper::CertInfo& info)
  */
 void XmppClient::handleMUCError(glooxwrapper::MUCRoom*, gloox::StanzaError err)
 {
-	std::string msg = StanzaErrorToString(err);
-	CreateSimpleMessage("system", msg, "error");
+	CreateGUIMessage("system", "error", StanzaErrorToString(err));
 }
 
 /*****************************************************
@@ -332,15 +330,15 @@ void XmppClient::SendIqGameReport(ScriptInterface& scriptInterface, JS::HandleVa
 	// Iterate through all the properties reported and add them to the stanza.
 	std::vector<std::string> properties;
 	scriptInterface.EnumeratePropertyNamesWithPrefix(data, "", properties);
-	for (std::vector<int>::size_type i = 0; i != properties.size(); i++)
+	for (const std::string& p : properties)
 	{
 		std::wstring value;
-		scriptInterface.GetProperty(data, properties[i].c_str(), value);
-		report->addAttribute(properties[i], utf8_from_wstring(value));
+		scriptInterface.GetProperty(data, p.c_str(), value);
+		report->addAttribute(p, utf8_from_wstring(value));
 	}
 
 	// Add stanza to IQ
-	game->m_GameReport.push_back(report);
+	game->m_GameReport.emplace_back(report);
 
 	// Send IQ
 	glooxwrapper::IQ iq(gloox::IQ::Set, xpartamuppJid);
@@ -368,15 +366,15 @@ void XmppClient::SendIqRegisterGame(ScriptInterface& scriptInterface, JS::Handle
 	// Iterate through all the properties reported and add them to the stanza.
 	std::vector<std::string> properties;
 	scriptInterface.EnumeratePropertyNamesWithPrefix(data, "", properties);
-	for (std::vector<int>::size_type i = 0; i != properties.size(); i++)
+	for (const std::string& p : properties)
 	{
 		std::wstring value;
-		scriptInterface.GetProperty(data, properties[i].c_str(), value);
-		game->addAttribute(properties[i], utf8_from_wstring(value));
+		scriptInterface.GetProperty(data, p.c_str(), value);
+		game->addAttribute(p, utf8_from_wstring(value));
 	}
 
 	// Push the stanza onto the IQ
-	g->m_GameList.push_back(game);
+	g->m_GameList.emplace_back(game);
 
 	// Send IQ
 	glooxwrapper::IQ iq(gloox::IQ::Set, xpartamuppJid);
@@ -395,12 +393,12 @@ void XmppClient::SendIqUnregisterGame()
 	// Send IQ
 	GameListQuery* g = new GameListQuery();
 	g->m_Command = "unregister";
-	g->m_GameList.push_back(glooxwrapper::Tag::allocate( "game" ));
+	g->m_GameList.emplace_back(glooxwrapper::Tag::allocate("game"));
 
 	glooxwrapper::IQ iq( gloox::IQ::Set, xpartamuppJid );
-	iq.addExtension( g );
+	iq.addExtension(g);
 	DbgXMPP("SendIqUnregisterGame [" << tag_xml(iq) << "]");
-	m_client->send( iq );
+	m_client->send(iq);
 }
 
 /**
@@ -420,10 +418,10 @@ void XmppClient::SendIqChangeStateGame(const std::string& nbp, const std::string
 	glooxwrapper::Tag* game = glooxwrapper::Tag::allocate("game");
 	game->addAttribute("nbp", nbp);
 	game->addAttribute("players", players);
-	g->m_GameList.push_back(game);
+	g->m_GameList.emplace_back(game);
 
 	glooxwrapper::IQ iq(gloox::IQ::Set, xpartamuppJid);
-	iq.addExtension( g );
+	iq.addExtension(g);
 	DbgXMPP("SendIqChangeStateGame [" << tag_xml(iq) << "]");
 	m_client->send(iq);
 }
@@ -443,28 +441,9 @@ void XmppClient::handleRegistrationFields(const glooxwrapper::JID&, int fields,
 void XmppClient::handleRegistrationResult(const glooxwrapper::JID&, gloox::RegistrationResult result)
 {
 	if (result == gloox::RegistrationSuccess)
-	{
-		CreateSimpleMessage("system", "registered");
-	}
+		CreateGUIMessage("system", "registered");
 	else
-	{
-	std::string msg;
-#define CASE(X, Y) case gloox::X: msg = Y; break
-		switch(result)
-		{
-		CASE(RegistrationNotAcceptable, g_L10n.Translate("Registration not acceptable"));
-		CASE(RegistrationConflict, g_L10n.Translate("Registration conflict"));
-		CASE(RegistrationNotAuthorized, g_L10n.Translate("Registration not authorized"));
-		CASE(RegistrationBadRequest, g_L10n.Translate("Registration bad request"));
-		CASE(RegistrationForbidden, g_L10n.Translate("Registration forbidden"));
-		CASE(RegistrationRequired, g_L10n.Translate("Registration required"));
-		CASE(RegistrationUnexpectedRequest, g_L10n.Translate("Registration unexpected request"));
-		CASE(RegistrationNotAllowed, g_L10n.Translate("Registration not allowed"));
-		default: msg = g_L10n.Translate("Registration unknown error");
-		}
-#undef CASE
-		CreateSimpleMessage("system", msg, "error");
-	}
+		CreateGUIMessage("system", "error", RegistrationResultToString(result));
 	disconnect();
 }
 
@@ -496,18 +475,18 @@ void XmppClient::GUIGetPlayerList(ScriptInterface& scriptInterface, JS::MutableH
 {
 	JSContext* cx = scriptInterface.GetContext();
 	JSAutoRequest rq(cx);
-	
+
 	scriptInterface.Eval("([])", ret);
 
 	// Convert the internal data structure to a Javascript object.
-	for (std::map<std::string, std::vector<std::string> >::const_iterator it = m_PlayerMap.begin(); it != m_PlayerMap.end(); ++it)
+	for (const std::pair<std::string, std::vector<std::string> >& p : m_PlayerMap)
 	{
 		JS::RootedValue player(cx);
 		scriptInterface.Eval("({})", &player);
-		scriptInterface.SetProperty(player, "name", wstring_from_utf8(it->first));
-		scriptInterface.SetProperty(player, "presence", wstring_from_utf8(it->second[0]));
-		scriptInterface.SetProperty(player, "rating", wstring_from_utf8(it->second[1]));
-		scriptInterface.SetProperty(player, "role", wstring_from_utf8(it->second[2]));
+		scriptInterface.SetProperty(player, "name", wstring_from_utf8(p.first));
+		scriptInterface.SetProperty(player, "presence", wstring_from_utf8(p.second[0]));
+		scriptInterface.SetProperty(player, "rating", wstring_from_utf8(p.second[1]));
+		scriptInterface.SetProperty(player, "role", wstring_from_utf8(p.second[2]));
 		scriptInterface.CallFunctionVoid(ret, "push", player);
 	}
 }
@@ -521,16 +500,16 @@ void XmppClient::GUIGetGameList(ScriptInterface& scriptInterface, JS::MutableHan
 {
 	JSContext* cx = scriptInterface.GetContext();
 	JSAutoRequest rq(cx);
-	
+
 	scriptInterface.Eval("([])", ret);
-	for(std::vector<const glooxwrapper::Tag*>::const_iterator it = m_GameList.begin(); it != m_GameList.end(); ++it)
+	const char* stats[] = { "name", "ip", "state", "nbp", "tnbp", "players", "mapName", "niceMapName", "mapSize", "mapType", "victoryCondition" };
+	for(const glooxwrapper::Tag* const& t : m_GameList)
 	{
 		JS::RootedValue game(cx);
 		scriptInterface.Eval("({})", &game);
 
-		const char* stats[] = { "name", "ip", "state", "nbp", "tnbp", "players", "mapName", "niceMapName", "mapSize", "mapType", "victoryCondition" };
 		for (size_t i = 0; i < ARRAY_SIZE(stats); ++i)
-			scriptInterface.SetProperty(game, stats[i], wstring_from_utf8((*it)->findAttribute(stats[i]).to_string()));
+			scriptInterface.SetProperty(game, stats[i], wstring_from_utf8(t->findAttribute(stats[i]).to_string()));
 
 		scriptInterface.CallFunctionVoid(ret, "push", game);
 	}
@@ -547,14 +526,14 @@ void XmppClient::GUIGetBoardList(ScriptInterface& scriptInterface, JS::MutableHa
 	JSAutoRequest rq(cx);
 
 	scriptInterface.Eval("([])", ret);
-	for(std::vector<const glooxwrapper::Tag*>::const_iterator it = m_BoardList.begin(); it != m_BoardList.end(); ++it)
+	const char* attributes[] = { "name", "rank", "rating" };
+	for(const glooxwrapper::Tag* const& t : m_BoardList)
 	{
 		JS::RootedValue board(cx);
 		scriptInterface.Eval("({})", &board);
 
-		const char* attributes[] = { "name", "rank", "rating" };
 		for (size_t i = 0; i < ARRAY_SIZE(attributes); ++i)
-			scriptInterface.SetProperty(board, attributes[i], wstring_from_utf8((*it)->findAttribute(attributes[i]).to_string()));
+			scriptInterface.SetProperty(board, attributes[i], wstring_from_utf8(t->findAttribute(attributes[i]).to_string()));
 
 		scriptInterface.CallFunctionVoid(ret, "push", board);
 	}
@@ -572,13 +551,13 @@ void XmppClient::GUIGetProfile(ScriptInterface& scriptInterface, JS::MutableHand
 
 	scriptInterface.Eval("([])", ret);
 	const char* stats[] = { "player", "rating", "totalGamesPlayed", "highestRating", "wins", "losses", "rank" };
-	for (std::vector<const glooxwrapper::Tag*>::const_iterator it = m_Profile.begin(); it != m_Profile.end(); ++it)
+	for (const glooxwrapper::Tag* const& t : m_Profile)
 	{
 		JS::RootedValue profile(cx);
 		scriptInterface.Eval("({})", &profile);
 
 		for (size_t i = 0; i < ARRAY_SIZE(stats); ++i)
-			scriptInterface.SetProperty(profile, stats[i], wstring_from_utf8((*it)->findAttribute(stats[i]).to_string()));
+			scriptInterface.SetProperty(profile, stats[i], wstring_from_utf8(t->findAttribute(stats[i]).to_string()));
 
 		scriptInterface.CallFunctionVoid(ret, "push", profile);
 	}
@@ -611,11 +590,11 @@ void XmppClient::GuiPollMessage(ScriptInterface& scriptInterface, JS::MutableHan
 		scriptInterface.SetProperty(ret, "text", message.text);
 	if (!message.level.empty())
 		scriptInterface.SetProperty(ret, "level", message.level);
-	if (!message.message.empty())
-		scriptInterface.SetProperty(ret, "message", message.message);
 	if (!message.data.empty())
 		scriptInterface.SetProperty(ret, "data", message.data);
-	
+	if (!message.datetime.empty())
+		scriptInterface.SetProperty(ret, "datetime", message.datetime);
+
 	m_GuiMessageQueue.pop_front();
 }
 
@@ -634,25 +613,56 @@ void XmppClient::SendMUCMessage(const std::string& message)
  */
 void XmppClient::PushGuiMessage(XmppClient::GUIMessage message)
 {
-	m_GuiMessageQueue.push_back(message);
+	m_GuiMessageQueue.push_back(std::move(message));
 }
 
 /**
- * Handle a standard MUC textual message.
+ * Clears all presence updates from the message queue.
+ * Used when rejoining the lobby, since we don't need to handle past presence changes.
+ */
+void XmppClient::ClearPresenceUpdates()
+{
+	m_GuiMessageQueue.erase(
+		std::remove_if(m_GuiMessageQueue.begin(), m_GuiMessageQueue.end(),
+			[](XmppClient::GUIMessage& message)
+			{
+				return message.type == L"chat" && message.level == L"presence";
+			}
+	), m_GuiMessageQueue.end());
+}
+
+/**
+ * Used in order to update the GUI only once when multiple updates are queued.
+ */
+int XmppClient::GetMucMessageCount()
+{
+	return std::count_if(m_GuiMessageQueue.begin(), m_GuiMessageQueue.end(),
+		[](XmppClient::GUIMessage& message)
+		{
+			return message.type == L"chat";
+		});
+}
+
+/**
+ * Handle a room message.
  */
 void XmppClient::handleMUCMessage(glooxwrapper::MUCRoom*, const glooxwrapper::Message& msg, bool)
 {
 	DbgXMPP(msg.from().resource() << " said " << msg.body());
 
 	GUIMessage message;
-	message.type = L"mucmessage";
+	message.type = L"chat";
+	message.level = L"room-message";
 	message.from = wstring_from_utf8(msg.from().resource().to_string());
 	message.text = wstring_from_utf8(msg.body().to_string());
+	if (msg.when())
+		// See http://xmpp.org/extensions/xep-0082.html#sect-idp285136 for format
+		message.datetime = msg.when()->stamp().to_string();
 	PushGuiMessage(message);
 }
 
 /**
- * Handle a standard textual message.
+ * Handle a private message.
  */
 void XmppClient::handleMessage(const glooxwrapper::Message& msg, glooxwrapper::MessageSession *)
 {
@@ -660,8 +670,13 @@ void XmppClient::handleMessage(const glooxwrapper::Message& msg, glooxwrapper::M
 	  << ", message " << msg.body() << ", thread id " << msg.thread());
 
 	GUIMessage message;
+	message.type = L"chat";
+	message.level = L"private-message";
 	message.from = wstring_from_utf8(msg.from().username().to_string());
-	message.message = wstring_from_utf8(msg.body().to_string());
+	message.text = wstring_from_utf8(msg.body().to_string());
+	if (msg.when())
+		//See http://xmpp.org/extensions/xep-0082.html#sect-idp285136 for format
+		message.datetime = msg.when()->stamp().to_string();
 	PushGuiMessage(message);
 }
 
@@ -672,72 +687,70 @@ bool XmppClient::handleIq(const glooxwrapper::IQ& iq)
 {
 	DbgXMPP("handleIq [" << tag_xml(iq) << "]");
 
-	if(iq.subtype() == gloox::IQ::Result)
+	if (iq.subtype() == gloox::IQ::Result)
 	{
-		const GameListQuery* gq = iq.findExtension<GameListQuery>( ExtGameListQuery );
-		const BoardListQuery* bq = iq.findExtension<BoardListQuery>( ExtBoardListQuery );
-		const ProfileQuery* pq = iq.findExtension<ProfileQuery>( ExtProfileQuery );
-		if(gq)
+		const GameListQuery* gq = iq.findExtension<GameListQuery>(EXTGAMELISTQUERY);
+		const BoardListQuery* bq = iq.findExtension<BoardListQuery>(EXTBOARDLISTQUERY);
+		const ProfileQuery* pq = iq.findExtension<ProfileQuery>(EXTPROFILEQUERY);
+		if (gq)
 		{
-			for(std::vector<const glooxwrapper::Tag*>::const_iterator it = m_GameList.begin(); it != m_GameList.end(); ++it )
-				glooxwrapper::Tag::free(*it);
+			for (const glooxwrapper::Tag* const& t : m_GameList)
+				glooxwrapper::Tag::free(t);
 			m_GameList.clear();
 
-			for(std::vector<const glooxwrapper::Tag*>::const_iterator it = gq->m_GameList.begin(); it != gq->m_GameList.end(); ++it)
-				m_GameList.push_back( (*it)->clone() );
+			for (const glooxwrapper::Tag* const& t : gq->m_GameList)
+				m_GameList.emplace_back(t->clone());
 
-			CreateSimpleMessage("system", "gamelist updated", "internal");
+			CreateGUIMessage("game", "gamelist");
 		}
-		if(bq)
+		if (bq)
 		{
 			if (bq->m_Command == "boardlist")
 			{
-				for(std::vector<const glooxwrapper::Tag*>::const_iterator it = m_BoardList.begin(); it != m_BoardList.end(); ++it )
-					glooxwrapper::Tag::free(*it);
+				for (const glooxwrapper::Tag* const& t : m_BoardList)
+					glooxwrapper::Tag::free(t);
 				m_BoardList.clear();
 
-				for(std::vector<const glooxwrapper::Tag*>::const_iterator it = bq->m_StanzaBoardList.begin(); it != bq->m_StanzaBoardList.end(); ++it)
-					m_BoardList.push_back((*it)->clone());
+				for (const glooxwrapper::Tag* const& t : bq->m_StanzaBoardList)
+					m_BoardList.emplace_back(t->clone());
 
-				CreateSimpleMessage("system", "boardlist updated", "internal");
+				CreateGUIMessage("game", "leaderboard");
 			}
 			else if (bq->m_Command == "ratinglist")
 			{
-				for(std::vector<const glooxwrapper::Tag*>::const_iterator it = bq->m_StanzaBoardList.begin(); it != bq->m_StanzaBoardList.end(); ++it)
+				for (const glooxwrapper::Tag* const& t : bq->m_StanzaBoardList)
 				{
-					std::string name = (*it)->findAttribute("name").to_string();
+					std::string name = t->findAttribute("name").to_string();
 					if (m_PlayerMap.find(name) != m_PlayerMap.end())
-						m_PlayerMap[name][1] = (*it)->findAttribute("rating").to_string();
+						m_PlayerMap[name][1] = t->findAttribute("rating").to_string();
 				}
 
-				CreateSimpleMessage("system", "ratinglist updated", "internal");
+				CreateGUIMessage("game", "ratinglist");
 			}
 		}
 		if (pq)
 		{
-			for (std::vector<const glooxwrapper::Tag*>::const_iterator it = m_Profile.begin(); it != m_Profile.end(); ++it)
-				glooxwrapper::Tag::free(*it);
+			for (const glooxwrapper::Tag* const& t : m_Profile)
+				glooxwrapper::Tag::free(t);
 			m_Profile.clear();
 
-			for (std::vector<const glooxwrapper::Tag*>::const_iterator it = pq->m_StanzaProfile.begin(); it != pq->m_StanzaProfile.end(); ++it)
-				m_Profile.push_back((*it)->clone());
+			for (const glooxwrapper::Tag* const& t : pq->m_StanzaProfile)
+				m_Profile.emplace_back(t->clone());
 
-			CreateSimpleMessage("system", "profile updated", "internal");
+			CreateGUIMessage("game", "profile");
 		}
 	}
 	else if (iq.subtype() == gloox::IQ::Error)
 	{
 		gloox::StanzaError err = iq.error_error();
-		std::string msg = StanzaErrorToString(err);
-		CreateSimpleMessage("system", msg, "error");
+		CreateGUIMessage("system", "error", StanzaErrorToString(err));
 	}
 	else
 	{
-		CreateSimpleMessage("system", g_L10n.Translate("unknown subtype (see logs)"), "error");
+		CreateGUIMessage("system", "error", g_L10n.Translate("unknown subtype (see logs)"));
 		std::string tag = tag_name(iq);
 		LOGMESSAGE("unknown subtype '%s'", tag.c_str());
 	}
-
 	return true;
 }
 
@@ -749,7 +762,7 @@ bool XmppClient::handleIq(const glooxwrapper::IQ& iq)
  * @param text Body of the message
  * @param data Optional field, used for auxiliary data
  */
-void XmppClient::CreateSimpleMessage(const std::string& type, const std::string& text, const std::string& level, const std::string& data)
+void XmppClient::CreateGUIMessage(const std::string& type, const std::string& level, const std::string& text, const std::string& data)
 {
 	GUIMessage message;
 	message.type = wstring_from_utf8(type);
@@ -783,20 +796,29 @@ void XmppClient::handleMUCParticipantPresence(glooxwrapper::MUCRoom*, const gloo
 			m_PlayerMap[newNick].resize(3);
 			m_PlayerMap[newNick][0] = presenceString;
 			m_PlayerMap[newNick][2] = roleString;
-			CreateSimpleMessage("muc", nick, "nick", participant.newNick.to_string());
+			CreateGUIMessage("chat", "nick", nick, participant.newNick.to_string());
 		}
 		else
-			CreateSimpleMessage("muc", nick, "leave");
+			CreateGUIMessage("chat", "leave", nick);
 
 		DbgXMPP(nick << " left the room");
 		m_PlayerMap.erase(nick);
 	}
 	else
 	{
-		if (m_PlayerMap.find(nick) == m_PlayerMap.end())
-			CreateSimpleMessage("muc", nick, "join");
+		/* During the initialization process, we recieve join messages for everyone
+		 * currently in the room. We don't want to display these, so we filter them
+		 * out. We will always be the last to join during initialization.
+		 */
+		if (!m_initialLoadComplete)
+		{
+			if (m_mucRoom->nick().to_string() == nick)
+				m_initialLoadComplete = true;
+		}
+		else if (m_PlayerMap.find(nick) == m_PlayerMap.end())
+			CreateGUIMessage("chat", "join", nick);
 		else
-			CreateSimpleMessage("muc", nick, "presence");
+			CreateGUIMessage("chat", "presence", nick);
 
 		DbgXMPP(nick << " is in the room, presence : " << (int)presenceType);
 		m_PlayerMap[nick].resize(3);
@@ -811,7 +833,7 @@ void XmppClient::handleMUCParticipantPresence(glooxwrapper::MUCRoom*, const gloo
 void XmppClient::handleMUCSubject(glooxwrapper::MUCRoom*, const glooxwrapper::string& UNUSED(nick), const glooxwrapper::string& subject)
 {
 	m_Subject = subject.c_str();
-	CreateSimpleMessage("muc", m_Subject, "subject");
+	CreateGUIMessage("chat", "subject", m_Subject);
 }
 
 /**
@@ -878,7 +900,6 @@ void XmppClient::SetPresence(const std::string& presence)
 	else IF("chat", Chat);
 	else IF("away", Away);
 	else IF("playing", DND);
-	else IF("gone", XA);
 	else IF("offline", Unavailable);
 	// The others are not to be set
 #undef IF
@@ -932,7 +953,7 @@ void XmppClient::GetPresenceString(const gloox::Presence::PresenceType p, std::s
 	CASE(Chat, "chat");
 	CASE(Away, "away");
 	CASE(DND, "playing");
-	CASE(XA, "gone");
+	CASE(XA, "away");
 	CASE(Unavailable, "offline");
 	CASE(Probe, "probe");
 	CASE(Error, "error");
@@ -954,7 +975,7 @@ void XmppClient::GetRoleString(const gloox::MUCRoomRole r, std::string& role) co
 {
 	switch(r)
 	{
-#define CASE(x,y) case gloox::x: role = y; break
+#define CASE(X, Y) case gloox::X: role = Y; break
 	CASE(RoleNone, "none");
 	CASE(RoleVisitor, "visitor");
 	CASE(RoleParticipant, "participant");
@@ -969,42 +990,112 @@ void XmppClient::GetRoleString(const gloox::MUCRoomRole r, std::string& role) co
 
 /**
  * Convert a gloox stanza error type to string.
+ * Keep in sync with Gloox documentation
  *
  * @param err Error to be converted
  * @return Converted error string
  */
-std::string XmppClient::StanzaErrorToString(gloox::StanzaError err)
+std::string XmppClient::StanzaErrorToString(gloox::StanzaError err) const
 {
-	std::string msg;
 #define CASE(X, Y) case gloox::X: return Y
+#define DEBUG_CASE(X, Y) case gloox::X: return g_L10n.Translate("Error") + " (" + Y + ")"
 	switch (err)
 	{
-	CASE(StanzaErrorBadRequest, g_L10n.Translate("Bad request"));
-	CASE(StanzaErrorConflict, g_L10n.Translate("Player name already in use"));
-	CASE(StanzaErrorFeatureNotImplemented, g_L10n.Translate("Feature not implemented"));
+	CASE(StanzaErrorUndefined, g_L10n.Translate("No error"));
+	DEBUG_CASE(StanzaErrorBadRequest, "Server recieved malformed XML");
+	CASE(StanzaErrorConflict, g_L10n.Translate("Player already logged in"));
+	DEBUG_CASE(StanzaErrorFeatureNotImplemented, "Server does not implement requested feature");
 	CASE(StanzaErrorForbidden, g_L10n.Translate("Forbidden"));
-	CASE(StanzaErrorGone, g_L10n.Translate("Recipient or server gone"));
+	DEBUG_CASE(StanzaErrorGone, "Unable to find message receipiant");
 	CASE(StanzaErrorInternalServerError, g_L10n.Translate("Internal server error"));
-	CASE(StanzaErrorItemNotFound, g_L10n.Translate("Item not found"));
-	CASE(StanzaErrorJidMalformed, g_L10n.Translate("Jid malformed"));
-	CASE(StanzaErrorNotAcceptable, g_L10n.Translate("Not acceptable"));
+	DEBUG_CASE(StanzaErrorItemNotFound, "Message receipiant does not exist");
+	DEBUG_CASE(StanzaErrorJidMalformed, "JID (XMPP address) malformed");
+	DEBUG_CASE(StanzaErrorNotAcceptable, "Receipiant refused message. Possible policy issue");
 	CASE(StanzaErrorNotAllowed, g_L10n.Translate("Not allowed"));
 	CASE(StanzaErrorNotAuthorized, g_L10n.Translate("Not authorized"));
-	CASE(StanzaErrorNotModified, g_L10n.Translate("Not modified"));
-	CASE(StanzaErrorPaymentRequired, g_L10n.Translate("Payment required"));
-	CASE(StanzaErrorRecipientUnavailable, g_L10n.Translate("Recipient unavailable"));
-	CASE(StanzaErrorRedirect, g_L10n.Translate("Redirect"));
+	DEBUG_CASE(StanzaErrorNotModified, "Requested item has not changed since last request");
+	DEBUG_CASE(StanzaErrorPaymentRequired, "This server requires payment");
+	CASE(StanzaErrorRecipientUnavailable, g_L10n.Translate("Recipient temporarily unavailable"));
+	DEBUG_CASE(StanzaErrorRedirect, "Request redirected");
 	CASE(StanzaErrorRegistrationRequired, g_L10n.Translate("Registration required"));
-	CASE(StanzaErrorRemoteServerNotFound, g_L10n.Translate("Remote server not found"));
-	CASE(StanzaErrorRemoteServerTimeout, g_L10n.Translate("Remote server timeout"));
-	CASE(StanzaErrorResourceConstraint, g_L10n.Translate("Resource constraint"));
+	DEBUG_CASE(StanzaErrorRemoteServerNotFound, "Remote server not found");
+	DEBUG_CASE(StanzaErrorRemoteServerTimeout, "Remote server timed out");
+	DEBUG_CASE(StanzaErrorResourceConstraint, "The recipient is unable to process the message due to resource constraints");
 	CASE(StanzaErrorServiceUnavailable, g_L10n.Translate("Service unavailable"));
-	CASE(StanzaErrorSubscribtionRequired, g_L10n.Translate("Subscription Required"));
-	CASE(StanzaErrorUndefinedCondition, g_L10n.Translate("Undefined condition"));
-	CASE(StanzaErrorUnexpectedRequest, g_L10n.Translate("Unexpected request"));
-	CASE(StanzaErrorUnknownSender, g_L10n.Translate("Unknown sender"));
+	DEBUG_CASE(StanzaErrorSubscribtionRequired, "Service requires subscription");
+	DEBUG_CASE(StanzaErrorUnexpectedRequest, "Attempt to send from invalid stanza address");
+	DEBUG_CASE(StanzaErrorUnknownSender, "Invalid 'from' address");
 	default:
-		return g_L10n.Translate("Error undefined");
+		return g_L10n.Translate("Unknown error");
 	}
+#undef DEBUG_CASE
+#undef CASE
+}
+
+/**
+ * Convert a gloox connection error enum to string
+ * Keep in sync with Gloox documentation
+ * 
+ * @param err Error to be converted
+ * @return Converted error string
+ */
+std::string XmppClient::ConnectionErrorToString(gloox::ConnectionError err) const
+{
+#define CASE(X, Y) case gloox::X: return Y
+#define DEBUG_CASE(X, Y) case gloox::X: return g_L10n.Translate("Error") + " (" + Y + ")"
+	switch (err)
+	{
+	CASE(ConnNoError, g_L10n.Translate("No error"));
+	CASE(ConnStreamError, g_L10n.Translate("Stream error"));
+	CASE(ConnStreamVersionError, g_L10n.Translate("The incoming stream version is unsupported"));
+	CASE(ConnStreamClosed, g_L10n.Translate("The stream has been closed by the server"));
+	DEBUG_CASE(ConnProxyAuthRequired, "The HTTP/SOCKS5 proxy requires authentication");
+	DEBUG_CASE(ConnProxyAuthFailed, "HTTP/SOCKS5 proxy authentication failed");
+	DEBUG_CASE(ConnProxyNoSupportedAuth, "The HTTP/SOCKS5 proxy requires an unsupported authentication mechanism");
+	CASE(ConnIoError, g_L10n.Translate("An I/O error occured"));
+	DEBUG_CASE(ConnParseError, "An XML parse error occured");
+	CASE(ConnConnectionRefused, g_L10n.Translate("The connection was refused by the server"));
+	CASE(ConnDnsError, g_L10n.Translate("Resolving the server's hostname failed"));
+	CASE(ConnOutOfMemory, g_L10n.Translate("This system is out of memory"));
+	DEBUG_CASE(ConnNoSupportedAuth, "The authentication mechanisms the server offered are not supported or no authentication mechanisms were available");
+	CASE(ConnTlsFailed, g_L10n.Translate("The server's certificate could not be verified or the TLS handshake did not complete successfully"));
+	CASE(ConnTlsNotAvailable, g_L10n.Translate("The server did not offer required TLS encryption"));
+	DEBUG_CASE(ConnCompressionFailed, "Negotiation/initializing compression failed");
+	CASE(ConnAuthenticationFailed, g_L10n.Translate("Authentication failed. Incorrect password or account does not exist"));
+	CASE(ConnUserDisconnected, g_L10n.Translate("The user or system requested a disconnect"));
+	CASE(ConnNotConnected, g_L10n.Translate("There is no active connection"));
+	default:
+		return g_L10n.Translate("Unknown error");
+	}
+#undef DEBUG_CASE
+#undef CASE
+}
+
+/**
+ * Convert a gloox registration result enum to string
+ * Keep in sync with Gloox documentation
+ * 
+ * @param err Enum to be converted
+ * @return Converted string
+ */
+std::string XmppClient::RegistrationResultToString(gloox::RegistrationResult res) const
+{
+#define CASE(X, Y) case gloox::X: return Y
+#define DEBUG_CASE(X, Y) case gloox::X: return g_L10n.Translate("Error") + " (" + Y + ")"
+	switch (res)
+	{
+	CASE(RegistrationSuccess, g_L10n.Translate("Success"));
+	CASE(RegistrationNotAcceptable, g_L10n.Translate("Not all necessary information provided"));
+	CASE(RegistrationConflict, g_L10n.Translate("Username already exists"));
+	DEBUG_CASE(RegistrationNotAuthorized, "Account removal timeout or insufficiently secure channel for password change");
+	DEBUG_CASE(RegistrationBadRequest, "Server recieved incomplete request");
+	DEBUG_CASE(RegistrationForbidden, "This client has insufficient permissions to remove an account");
+	DEBUG_CASE(RegistrationRequired, "Account cannot be removed as it does not exist");
+	DEBUG_CASE(RegistrationUnexpectedRequest, "This client is unregistered with the server");
+	DEBUG_CASE(RegistrationNotAllowed, "Server does not permit password changes");
+	default:
+		return g_L10n.Translate("Unknown error");
+	}
+#undef DEBUG_CASE
 #undef CASE
 }
