@@ -383,6 +383,17 @@ void glooxwrapper::Client::setPresence(gloox::Presence::PresenceType pres, int p
 }
 
 
+glooxwrapper::DelayedDelivery::DelayedDelivery(const gloox::DelayedDelivery* wrapped)
+{
+	m_Wrapped = wrapped;
+}
+
+const glooxwrapper::string glooxwrapper::DelayedDelivery::stamp() const
+{
+	return m_Wrapped->stamp();
+}
+
+
 glooxwrapper::Disco::Disco(gloox::Disco* wrapped)
 {
 	m_Wrapped = wrapped;
@@ -513,6 +524,14 @@ glooxwrapper::string glooxwrapper::Message::thread() const
 	return m_Wrapped->thread();
 }
 
+const glooxwrapper::DelayedDelivery* glooxwrapper::Message::when() const
+{
+	const gloox::DelayedDelivery* wrapped = m_Wrapped->when();
+	if (wrapped == 0)
+		return 0;
+	return new glooxwrapper::DelayedDelivery(wrapped);
+}
+
 
 glooxwrapper::MUCRoom::MUCRoom(Client* parent, const JID& nick, MUCRoomHandler* mrh, MUCRoomConfigHandler* UNUSED(mrch))
 {
@@ -713,9 +732,8 @@ const glooxwrapper::Tag* glooxwrapper::Tag::findTag_clone(const string& expressi
 
 glooxwrapper::ConstTagList glooxwrapper::Tag::findTagList_clone(const string& expression) const
 {
-	gloox::ConstTagList tagList = m_Wrapped->findTagList(expression.to_string());
 	glooxwrapper::ConstTagList tagListWrapper;
-	for (gloox::ConstTagList::iterator it = tagList.begin(); it != tagList.end(); ++it)
-		tagListWrapper.push_back(new glooxwrapper::Tag(const_cast<gloox::Tag*>(*it), false));
+	for (const gloox::Tag* const& t : m_Wrapped->findTagList(expression.to_string()))
+		tagListWrapper.push_back(new glooxwrapper::Tag(const_cast<gloox::Tag*>(t), false));
 	return tagListWrapper;
 }
