@@ -1,4 +1,4 @@
-/* Copyright (C) 2014 Wildfire Games.
+/* Copyright (C) 2016 Wildfire Games.
  * This file is part of 0 A.D.
  *
  * 0 A.D. is free software: you can redistribute it and/or modify
@@ -43,6 +43,7 @@ that of Atlas depending on commandline parameters.
 #include "ps/ArchiveBuilder.h"
 #include "ps/CConsole.h"
 #include "ps/CLogger.h"
+#include "ps/ConfigDB.h"
 #include "ps/Filesystem.h"
 #include "ps/Game.h"
 #include "ps/Globals.h"
@@ -95,7 +96,6 @@ static InReaction MainInputHandler(const SDL_Event_* ev)
 {
 	switch(ev->ev.type)
 	{
-#if SDL_VERSION_ATLEAST(2, 0, 0)
 	case SDL_WINDOWEVENT:
 		switch(ev->ev.window.event)
 		{
@@ -113,24 +113,6 @@ static InReaction MainInputHandler(const SDL_Event_* ev)
 			g_VideoMode.UpdatePosition(ev->ev.window.data1, ev->ev.window.data2);
 		}
 		break;
-#else
-	case SDL_ACTIVEEVENT:
-		if (ev->ev.active.state & SDL_APPMOUSEFOCUS)
-		{
-			// Tell renderer not to render cursor if mouse focus is lost
-			//	this restores system cursor, until/if focus is regained
-			if (!ev->ev.active.gain)
-				RenderCursor(false);
-			else
-				RenderCursor(true);
-		}
-		break;
-
-	case SDL_VIDEORESIZE:
-		g_ResizedW = ev->ev.resize.w;
-		g_ResizedH = ev->ev.resize.h;
-		break;
-#endif
 
 	case SDL_QUIT:
 		kill_mainloop();
@@ -155,12 +137,7 @@ static InReaction MainInputHandler(const SDL_Event_* ev)
 		}
 		else if (hotkey == "togglefullscreen")
 		{
-#if !OS_MACOSX || SDL_VERSION_ATLEAST(2, 0, 0)
-			// Fullscreen toggling is broken on OS X w/ SDL 1.2, see http://trac.wildfiregames.com/ticket/741
 			g_VideoMode.ToggleFullscreen();
-#else
-			LOGWARNING("Toggling fullscreen and resizing are disabled on OS X due to a known bug. Please use the config file to change display settings.");
-#endif
 			return IN_HANDLED;
 		}
 		else if (hotkey == "profile2.toggle")
@@ -305,9 +282,20 @@ static void Frame()
 		SDL_Delay(10);
 	}
 
-	// TODO: throttling: limit update and render frequency to the minimum.
-	// this is mostly relevant for "inactive" state, so that other windows
-	// get enough CPU time, but it's always nice for power+thermal management.
+	// Throttling: limit update and render frequency to the minimum to 50 FPS
+	// in the "inactive" state, so that other windows get enough CPU time, 
+	// (and it's always nice for power+thermal management).
+	// TODO: when the game performance is high enough, implementing a limit for
+	// in-game framerate might be sensible.
+	const float maxFPSMenu = 50.0;
+	bool limit_fps = false;
+	CFG_GET_VAL("gui.menu.limitfps", limit_fps);
+	if (limit_fps && (!g_Game || !g_Game->IsGameStarted()))
+	{
+		float remainingFrameTime = (1000.0 / maxFPSMenu) - realTimeSinceLastFrame;
+		if (remainingFrameTime > 0)
+			SDL_Delay(remainingFrameTime);
+	}
 
 
 	// this scans for changed files/directories and reloads them, thus
@@ -367,11 +355,7 @@ static void Frame()
 		Render();
 
 		PROFILE3("swap buffers");
-#if SDL_VERSION_ATLEAST(2, 0, 0)
 		SDL_GL_SwapWindow(g_VideoMode.GetWindow());
-#else
-		SDL_GL_SwapBuffers();
-#endif
 	}
 	ogl_WarnIfError();
 
@@ -432,20 +416,42 @@ static void RunGameOrAtlas(int argc, const char* argv[])
 
 	g_args = args;
 
+	if (args.Has("version") || args.Has("-version"))
+	{
+		debug_printf("Pyrogenesis %s\n", engine_version);
+		return;
+	}
+
 	// We need to initialise libxml2 in the main thread before
 	// any thread uses it. So initialise it here before we
 	// might run Atlas.
 	CXeromyces::Startup();
 
-	// run Atlas (if requested via args)
-	bool ran_atlas = ATLAS_RunIfOnCmdLine(args, false);
 	// Atlas handles the whole init/shutdown/etc sequence by itself;
-	// when we get here, it has exited and we're done.
-	if(ran_atlas)
+	if (ATLAS_RunIfOnCmdLine(args, false))
 		return;
 
+	const bool isReplay = args.Has("replay");
+	const bool isVisualReplay = args.Has("replay-visual");
+	const std::string replayFile = isReplay ? args.Get("replay") : (isVisualReplay ? args.Get("replay-visual") : "");
+
+	// Ensure the replay file exists
+	if (isReplay || isVisualReplay)
+	{
+		if (!FileExists(OsPath(replayFile)))
+		{
+			debug_printf("ERROR: The requested replay file '%s' does not exist!\n", replayFile.c_str());
+			return;
+		}
+		if (DirectoryExists(OsPath(replayFile)))
+		{
+			debug_printf("ERROR: The requested replay file '%s' is a directory!\n", replayFile.c_str());
+			return;
+		}
+	}
+
 	// run non-visual simulation replay if requested
-	if (args.Has("replay"))
+	if (isReplay)
 	{
 		Paths paths(args);
 		g_VFS = CreateVfs(20 * MiB);
@@ -454,8 +460,8 @@ static void RunGameOrAtlas(int argc, const char* argv[])
 
 		{
 			CReplayPlayer replay;
-			replay.Load(args.Get("replay"));
-			replay.Replay(args.Has("serializationtest"));
+			replay.Load(replayFile);
+			replay.Replay(args.Has("serializationtest"), args.Has("ooslog"));
 		}
 
 		g_VFS.reset();
