@@ -1,4 +1,4 @@
-/* Copyright (C) 2011 Wildfire Games.
+/* Copyright (C) 2016 Wildfire Games.
  * This file is part of 0 A.D.
  *
  * 0 A.D. is free software: you can redistribute it and/or modify
@@ -68,10 +68,10 @@ public:
 	CNetClient(CGame* game);
 
 	virtual ~CNetClient();
-	
+
 	/**
 	 * We assume that adding a tracing function that's only called
-	 * during GC is better for performance than using a 
+	 * during GC is better for performance than using a
 	 * PersistentRooted<T> where each value needs to be added to
 	 * the root set.
 	 */
@@ -79,15 +79,21 @@ public:
 	{
 		reinterpret_cast<CNetClient*>(data)->TraceMember(trc);
 	}
-	
+
 	void TraceMember(JSTracer *trc);
-	
+
 	/**
 	 * Set the user's name that will be displayed to all players.
 	 * This must not be called after the connection setup.
 	 */
 	void SetUserName(const CStrW& username);
 
+	/**
+	 * Returns the GUID of the local client.
+	 * Used for distinguishing observers.
+	 */
+	CStr GetGUID() const { return m_GUID; }
+
 	/**
 	 * Set up a connection to the remote networked server.
 	 * @param server IP address or host name to connect to
@@ -108,6 +114,11 @@ public:
 	 */
 	void Poll();
 
+	/**
+	 * Locally triggers a GUI message if the connection to the server is being lost or has bad latency.
+	 */
+	void CheckServerConnection();
+
 	/**
 	 * Flush any queued outgoing network messages.
 	 * This should be called soon after sending a group of messages that may be batched together.
@@ -177,9 +188,15 @@ public:
 	void LoadFinished();
 
 	void SendChatMessage(const std::wstring& text);
-	
+
 	void SendReadyMessage(const int status);
 
+	/**
+	 * Call when the client has rejoined a running match and finished
+	 * the loading screen.
+	 */
+	void SendRejoinedMessage();
+
 private:
 	// Net message / FSM transition handlers
 	static bool OnConnect(void* context, CFsmEvent* event);
@@ -194,6 +211,10 @@ private:
 	static bool OnGameStart(void* context, CFsmEvent* event);
 	static bool OnJoinSyncStart(void* context, CFsmEvent* event);
 	static bool OnJoinSyncEndCommandBatch(void* context, CFsmEvent* event);
+	static bool OnRejoined(void* context, CFsmEvent* event);
+	static bool OnKicked(void* context, CFsmEvent* event);
+	static bool OnClientTimeout(void* context, CFsmEvent* event);
+	static bool OnClientPerformance(void* context, CFsmEvent* event);
 	static bool OnLoadedGame(void* context, CFsmEvent* event);
 
 	/**
@@ -232,6 +253,9 @@ private:
 
 	/// Serialized game state received when joining an in-progress game
 	std::string m_JoinSyncBuffer;
+
+	/// Time when the server was last checked for timeouts and bad latency
+	std::time_t m_LastConnectionCheck;
 };
 
 /// Global network client for the standard game
