@@ -1,4 +1,4 @@
-/* Copyright (C) 2011 Wildfire Games.
+/* Copyright (C) 2016 Wildfire Games.
  * This file is part of 0 A.D.
  *
  * 0 A.D. is free software: you can redistribute it and/or modify
@@ -60,9 +60,14 @@ typedef std::map<CStr, PlayerAssignment> PlayerAssignmentMap; // map from GUID -
 enum NetDisconnectReason
 {
 	NDR_UNKNOWN = 0,
-	NDR_UNEXPECTED_SHUTDOWN,
+	NDR_SERVER_SHUTDOWN,
 	NDR_INCORRECT_PROTOCOL_VERSION,
-	NDR_SERVER_ALREADY_IN_GAME
+	NDR_SERVER_LOADING,
+	NDR_SERVER_ALREADY_IN_GAME,
+	NDR_KICKED,
+	NDR_BANNED,
+	NDR_PLAYERNAME_IN_USE,
+	NDR_SERVER_FULL
 };
 
 class CNetHost
