@@ -1,4 +1,4 @@
-/* Copyright (C) 2011 Wildfire Games.
+/* Copyright (C) 2016 Wildfire Games.
  * This file is part of 0 A.D.
  *
  * 0 A.D. is free software: you can redistribute it and/or modify
@@ -131,6 +131,22 @@ CNetMessage* CNetMessageFactory::CreateMessage(const void* pData,
 		pNewMessage = new CJoinSyncStartMessage;
 		break;
 
+	case NMT_REJOINED:
+		pNewMessage = new CRejoinedMessage;
+		break;
+
+	case NMT_KICKED:
+		pNewMessage = new CKickedMessage;
+		break;
+
+	case NMT_CLIENT_TIMEOUT:
+		pNewMessage = new CClientTimeoutMessage;
+		break;
+
+	case NMT_CLIENT_PERFORMANCE:
+		pNewMessage = new CClientPerformanceMessage;
+		break;
+
 	case NMT_LOADED_GAME:
 		pNewMessage = new CLoadedGameMessage;
 		break;
@@ -174,7 +190,7 @@ CNetMessage* CNetMessageFactory::CreateMessage(const void* pData,
 	case NMT_CHAT:
 		pNewMessage = new CChatMessage;
 		break;
-	
+
 	case NMT_READY:
 		pNewMessage = new CReadyMessage;
 		break;
