@@ -1,4 +1,4 @@
-/* Copyright (C) 2014 Wildfire Games.
+/* Copyright (C) 2016 Wildfire Games.
  * This file is part of 0 A.D.
  *
  * 0 A.D. is free software: you can redistribute it and/or modify
@@ -39,9 +39,16 @@
 #include <miniupnpc/upnperrors.h>
 #endif
 
+/**
+ * Number of peers to allocate for the enet host.
+ * Limited by ENET_PROTOCOL_MAXIMUM_PEER_ID (4096).
+ *
+ * At most 8 players, 16 observers and 1 temporary connection to send the "server full" disconnect-reason.
+ */
+#define MAX_CLIENTS 25
+
 #define	DEFAULT_SERVER_NAME			L"Unnamed Server"
 #define DEFAULT_WELCOME_MESSAGE		L"Welcome"
-#define MAX_CLIENTS					8
 
 static const int CHANNEL_COUNT = 1;
 
@@ -121,7 +128,8 @@ CNetServerWorker::CNetServerWorker(int autostartPlayers) :
 	m_AutostartPlayers(autostartPlayers),
 	m_Shutdown(false),
 	m_ScriptInterface(NULL),
-	m_NextHostID(1), m_Host(NULL), m_Stats(NULL)
+	m_NextHostID(1), m_Host(NULL), m_HostGUID(), m_Stats(NULL),
+	m_LastConnectionCheck(0)
 {
 	m_State = SERVER_STATE_UNCONNECTED;
 
@@ -151,7 +159,7 @@ CNetServerWorker::~CNetServerWorker()
 
 	for (size_t i = 0; i < m_Sessions.size(); ++i)
 	{
-		m_Sessions[i]->DisconnectNow(NDR_UNEXPECTED_SHUTDOWN);
+		m_Sessions[i]->DisconnectNow(NDR_SERVER_SHUTDOWN);
 		delete m_Sessions[i];
 	}
 
@@ -236,7 +244,11 @@ void* CNetServerWorker::SetupUPnP(void*)
 		ret = 1;
 	}
 	// No cached URL, or it did not respond. Try getting a valid UPnP device for 10 seconds.
+#if defined(MINIUPNPC_API_VERSION) && MINIUPNPC_API_VERSION >= 14
+	else if ((devlist = upnpDiscover(10000, 0, 0, 0, 0, 2, 0)) != NULL)
+#else
 	else if ((devlist = upnpDiscover(10000, 0, 0, 0, 0, 0)) != NULL)
+#endif
 	{
 		ret = UPNP_GetValidIGD(devlist, &urls, &data, internalIPAddress, sizeof(internalIPAddress));
 		allocatedUrls = ret != 0; // urls is allocated on non-zero return values
@@ -305,13 +317,13 @@ void* CNetServerWorker::SetupUPnP(void*)
 
 	// Cache root descriptor URL to try to avoid discovery next time.
 	g_ConfigDB.SetValueString(CFG_USER, "network.upnprootdescurl", urls.controlURL);
-	g_ConfigDB.WriteFile(CFG_USER);
+	g_ConfigDB.WriteValueToFile(CFG_USER, "network.upnprootdescurl", urls.controlURL);
 	LOGMESSAGE("Net server: cached UPnP root descriptor URL as %s", urls.controlURL);
 
 	// Make sure everything is properly freed.
 	if (allocatedUrls)
 		FreeUPNPUrls(&urls);
-    
+
 	freeUPNPDevlist(devlist);
 
 	return NULL;
@@ -362,7 +374,7 @@ void CNetServerWorker::Run()
 {
 	// The script runtime uses the profiler and therefore the thread must be registered before the runtime is created
 	g_Profiler2.RegisterCurrentThread("Net server");
-	
+
 	// To avoid the need for JS_SetContextThread, we create and use and destroy
 	// the script interface entirely within this network thread
 	m_ScriptInterface = new ScriptInterface("Engine", "Net server", ScriptInterface::CreateRuntime(g_ScriptRuntime));
@@ -380,7 +392,7 @@ void CNetServerWorker::Run()
 		// Update profiler stats
 		m_Stats->LatchHostState(m_Host);
 	}
-	
+
 	// Clear roots before deleting their context
 	m_GameAttributes.clear();
 	m_SavedCommands.clear();
@@ -393,9 +405,9 @@ bool CNetServerWorker::RunStep()
 	// Check for messages from the game thread.
 	// (Do as little work as possible while the mutex is held open,
 	// to avoid performance problems and deadlocks.)
-	
+
 	m_ScriptInterface->GetRuntime()->MaybeIncrementalGC(0.5f);
-	
+
 	JSContext* cx = m_ScriptInterface->GetContext();
 	JSAutoRequest rq(cx);
 
@@ -447,6 +459,8 @@ bool CNetServerWorker::RunStep()
 	for (size_t i = 0; i < m_Sessions.size(); ++i)
 		m_Sessions[i]->GetFileTransferer().Poll();
 
+	CheckClientConnections();
+
 	// Process network events:
 
 	ENetEvent event;
@@ -479,6 +493,12 @@ bool CNetServerWorker::RunStep()
 
 		CNetServerSession* session = new CNetServerSession(*this, event.peer);
 
+		// Prevent the local client of the host from timing out too quickly
+#if (ENET_VERSION >= ENET_VERSION_CREATE(1, 3, 4))
+		if (session->GetIPAddress() == "127.0.0.1")
+			enet_peer_timeout(event.peer, 0, MAXIMUM_HOST_TIMEOUT, MAXIMUM_HOST_TIMEOUT);
+#endif
+
 		m_Sessions.push_back(session);
 
 		SetupSession(session);
@@ -545,6 +565,55 @@ bool CNetServerWorker::RunStep()
 	return true;
 }
 
+void CNetServerWorker::CheckClientConnections()
+{
+	if (m_State == SERVER_STATE_LOADING)
+		return;
+
+	// Send messages at most once per second
+	std::time_t now = std::time(nullptr);
+	if (now <= m_LastConnectionCheck)
+		return;
+
+	m_LastConnectionCheck = now;
+
+	for (size_t i = 0; i < m_Sessions.size(); ++i)
+	{
+		u32 lastReceived = m_Sessions[i]->GetLastReceivedTime();
+		u32 meanRTT = m_Sessions[i]->GetMeanRTT();
+
+		CNetMessage* message = nullptr;
+
+		// Report if we didn't hear from the client since few seconds
+		if (lastReceived > NETWORK_WARNING_TIMEOUT)
+		{
+			CClientTimeoutMessage* msg = new CClientTimeoutMessage();
+			msg->m_GUID = m_Sessions[i]->GetGUID();
+			msg->m_LastReceivedTime = lastReceived;
+			message = msg;
+		}
+		// Report if the client has bad ping
+		else if (meanRTT > DEFAULT_TURN_LENGTH_MP)
+		{
+			CClientPerformanceMessage* msg = new CClientPerformanceMessage();
+			CClientPerformanceMessage::S_m_Clients client;
+			client.m_GUID = m_Sessions[i]->GetGUID();
+			client.m_MeanRTT = meanRTT;
+			msg->m_Clients.push_back(client);
+			message = msg;
+		}
+
+		// Send to all clients except the affected one
+		// (since that will show the locally triggered warning instead)
+		if (message)
+			for (size_t j = 0; j < m_Sessions.size(); ++j)
+				if (i != j)
+					m_Sessions[j]->SendMessage(message);
+
+		SAFE_DELETE(message);
+	}
+}
+
 void CNetServerWorker::HandleMessageReceive(const CNetMessage* message, CNetServerSession* session)
 {
 	// Handle non-FSM messages first
@@ -593,6 +662,7 @@ void CNetServerWorker::SetupSession(CNetServerSession* session)
 	session->AddTransition(NSS_JOIN_SYNCING, (uint)NMT_CONNECTION_LOST, NSS_UNCONNECTED, (void*)&OnDisconnect, context);
 	session->AddTransition(NSS_JOIN_SYNCING, (uint)NMT_LOADED_GAME, NSS_INGAME, (void*)&OnJoinSyncingLoadedGame, context);
 
+	session->AddTransition(NSS_INGAME, (uint)NMT_REJOINED, NSS_INGAME, (void*)&OnRejoined, context);
 	session->AddTransition(NSS_INGAME, (uint)NMT_CONNECTION_LOST, NSS_UNCONNECTED, (void*)&OnDisconnect, context);
 	session->AddTransition(NSS_INGAME, (uint)NMT_CHAT, NSS_INGAME, (void*)&OnChat, context);
 	session->AddTransition(NSS_INGAME, (uint)NMT_SIMULATION_COMMAND, NSS_INGAME, (void*)&OnInGame, context);
@@ -605,6 +675,12 @@ void CNetServerWorker::SetupSession(CNetServerSession* session)
 
 bool CNetServerWorker::HandleConnect(CNetServerSession* session)
 {
+	if (std::find(m_BannedIPs.begin(), m_BannedIPs.end(), session->GetIPAddress()) != m_BannedIPs.end())
+	{
+		session->Disconnect(NDR_BANNED);
+		return false;
+	}
+
 	CSrvHandshakeMessage handshake;
 	handshake.m_Magic = PS_PROTOCOL_MAGIC;
 	handshake.m_ProtocolVersion = PS_PROTOCOL_VERSION;
@@ -616,6 +692,9 @@ void CNetServerWorker::OnUserJoin(CNetServerSession* session)
 {
 	AddPlayer(session->GetGUID(), session->GetUserName());
 
+	if (m_HostGUID.empty() && session->GetIPAddress() == "127.0.0.1")
+		m_HostGUID = session->GetGUID();
+
 	CGameSetupMessage gameSetupMessage(GetScriptInterface());
 	gameSetupMessage.m_Data = m_GameAttributes.get();
 	session->SendMessage(&gameSetupMessage);
@@ -708,6 +787,40 @@ void CNetServerWorker::ClearAllPlayerReady()
 	SendPlayerAssignments();
 }
 
+bool CNetServerWorker::KickPlayer(const CStrW& playerName, const bool ban)
+{
+	// Find the user with that name
+	std::vector<CNetServerSession*>::iterator it = std::find_if(m_Sessions.begin(), m_Sessions.end(),
+		[&](CNetServerSession* session) { return session->GetUserName() == playerName; });
+
+	// and return if no one or the host has that name
+	if (it == m_Sessions.end() || (*it)->GetGUID() == m_HostGUID)
+		return false;
+
+	if (ban)
+	{
+		// Remember name
+		if (std::find(m_BannedPlayers.begin(), m_BannedPlayers.end(), playerName) == m_BannedPlayers.end())
+			m_BannedPlayers.push_back(playerName);
+
+		// Remember IP address
+		CStr ipAddress = (*it)->GetIPAddress();
+		if (!ipAddress.empty() && std::find(m_BannedIPs.begin(), m_BannedIPs.end(), ipAddress) == m_BannedIPs.end())
+			m_BannedIPs.push_back(ipAddress);
+	}
+
+	// Disconnect that user
+	(*it)->Disconnect(ban ? NDR_BANNED : NDR_KICKED);
+
+	// Send message notifying other clients
+	CKickedMessage kickedMessage;
+	kickedMessage.m_Name = playerName;
+	kickedMessage.m_Ban = ban;
+	Broadcast(&kickedMessage);
+
+	return true;
+}
+
 void CNetServerWorker::AssignPlayer(int playerID, const CStr& guid)
 {
 	// Remove anyone who's already assigned to this player
@@ -788,25 +901,75 @@ bool CNetServerWorker::OnAuthenticate(void* context, CFsmEvent* event)
 	CNetServerSession* session = (CNetServerSession*)context;
 	CNetServerWorker& server = session->GetServer();
 
-	CAuthenticateMessage* message = (CAuthenticateMessage*)event->GetParamRef();
+	// Prohibit joins while the game is loading
+	if (server.m_State == SERVER_STATE_LOADING)
+	{
+		LOGMESSAGE("Refused connection while the game is loading");
+		session->Disconnect(NDR_SERVER_LOADING);
+		return true;
+	}
 
-	CStrW username = server.DeduplicatePlayerName(SanitisePlayerName(message->m_Name));
+	CAuthenticateMessage* message = (CAuthenticateMessage*)event->GetParamRef();
+	CStrW username = SanitisePlayerName(message->m_Name);
+
+	// Either deduplicate or prohibit join if name is in use
+	bool duplicatePlayernames = false;
+	CFG_GET_VAL("network.duplicateplayernames", duplicatePlayernames);
+	if (duplicatePlayernames)
+		username = server.DeduplicatePlayerName(username);
+	else if (std::find_if(
+			server.m_Sessions.begin(), server.m_Sessions.end(),
+			[&username] (const CNetServerSession* session)
+			{ return session->GetUserName() == username; })
+		!= server.m_Sessions.end())
+	{
+		session->Disconnect(NDR_PLAYERNAME_IN_USE);
+		return true;
+	}
+
+	// Disconnect banned usernames
+	if (std::find(server.m_BannedPlayers.begin(), server.m_BannedPlayers.end(), username) != server.m_BannedPlayers.end())
+	{
+		session->Disconnect(NDR_BANNED);
+		return true;
+	}
+
+	// Optionally allow observers to join after the game has started
+	bool observerLateJoin = false;
+	CFG_GET_VAL("network.lateobserverjoins", observerLateJoin);
+
+	int maxObservers = 0;
+	CFG_GET_VAL("network.observerlimit", maxObservers);
 
 	bool isRejoining = false;
-
-	if (server.m_State != SERVER_STATE_PREGAME)
+	bool serverFull = false;
+	if (server.m_State == SERVER_STATE_PREGAME)
 	{
-// 		isRejoining = true; // uncomment this to test rejoining even if the player wasn't connected previously
-
-		// Search for an old disconnected player of the same name
+		// Don't check for maxObservers in the gamesetup, as we don't know yet who will be assigned
+		serverFull = server.m_Sessions.size() >= MAX_CLIENTS;
+	}
+	else
+	{
+		isRejoining = observerLateJoin;
+		bool isObserver = true;
+		int disconnectedPlayers = 0;
+		int connectedPlayers = 0;
 		// (TODO: if GUIDs were stable, we should use them instead)
 		for (PlayerAssignmentMap::iterator it = server.m_PlayerAssignments.begin(); it != server.m_PlayerAssignments.end(); ++it)
 		{
 			if (!it->second.m_Enabled && it->second.m_Name == username)
 			{
+				isObserver = it->second.m_PlayerID == -1;
 				isRejoining = true;
-				break;
 			}
+
+			if (it->second.m_PlayerID == -1)
+				continue;
+
+			if (it->second.m_Enabled)
+				++connectedPlayers;
+			else
+				++disconnectedPlayers;
 		}
 
 		// Players who weren't already in the game are not allowed to join now that it's started
@@ -816,6 +979,17 @@ bool CNetServerWorker::OnAuthenticate(void* context, CFsmEvent* event)
 			session->Disconnect(NDR_SERVER_ALREADY_IN_GAME);
 			return true;
 		}
+
+		// Ensure all players will be able to rejoin
+		serverFull = isObserver && (
+			(int) server.m_Sessions.size() - connectedPlayers > maxObservers ||
+			(int) server.m_Sessions.size() + disconnectedPlayers >= MAX_CLIENTS);
+	}
+
+	if (serverFull)
+	{
+		session->Disconnect(NDR_SERVER_FULL);
+		return true;
 	}
 
 	// TODO: check server password etc?
@@ -864,6 +1038,23 @@ bool CNetServerWorker::OnInGame(void* context, CFsmEvent* event)
 	{
 		CSimulationMessage* simMessage = static_cast<CSimulationMessage*> (message);
 
+		// Ignore messages sent by one player on behalf of another player
+		// unless cheating is enabled
+		bool cheatsEnabled = false;
+		ScriptInterface& scriptInterface = server.GetScriptInterface();
+		JSContext* cx = scriptInterface.GetContext();
+		JSAutoRequest rq(cx);
+		JS::RootedValue settings(cx);
+		scriptInterface.GetProperty(server.m_GameAttributes.get(), "settings", &settings);
+		if (scriptInterface.HasProperty(settings, "CheatsEnabled"))
+			scriptInterface.GetProperty(settings, "CheatsEnabled", cheatsEnabled);
+
+		PlayerAssignmentMap::iterator it = server.m_PlayerAssignments.find(session->GetGUID());
+		// When cheating is disabled, fail if the player the message claims to
+		// represent does not exist or does not match the sender's player name
+		if (!cheatsEnabled && (it == server.m_PlayerAssignments.end() || it->second.m_PlayerID != simMessage->m_Player))
+			return true;
+
 		// Send it back to all clients immediately
 		server.Broadcast(simMessage);
 
@@ -872,14 +1063,12 @@ bool CNetServerWorker::OnInGame(void* context, CFsmEvent* event)
 			server.m_SavedCommands.resize(simMessage->m_Turn + 1);
 		server.m_SavedCommands[simMessage->m_Turn].push_back(*simMessage);
 
-		// TODO: we should do some validation of ownership (clients can't send commands on behalf of opposing players)
-
 		// TODO: we shouldn't send the message back to the client that first sent it
 	}
 	else if (message->GetType() == (uint)NMT_SYNC_CHECK)
 	{
 		CSyncCheckMessage* syncMessage = static_cast<CSyncCheckMessage*> (message);
-		server.m_ServerTurnManager->NotifyFinishedClientUpdate(session->GetHostID(), syncMessage->m_Turn, syncMessage->m_Hash);
+		server.m_ServerTurnManager->NotifyFinishedClientUpdate(session->GetHostID(), session->GetUserName(), syncMessage->m_Turn, syncMessage->m_Hash);
 	}
 	else if (message->GetType() == (uint)NMT_END_COMMAND_BATCH)
 	{
@@ -944,7 +1133,7 @@ bool CNetServerWorker::OnJoinSyncingLoadedGame(void* context, CFsmEvent* event)
 	// The simulation may have progressed since then, so send any subsequent
 	// commands to them and set them as an active player so they can participate
 	// in all future turns.
-	// 
+	//
 	// (TODO: if it takes a long time for them to receive and execute all these
 	// commands, the other players will get frozen for that time and may be unhappy;
 	// we could try repeating this process a few times until the client converges
@@ -988,6 +1177,23 @@ bool CNetServerWorker::OnJoinSyncingLoadedGame(void* context, CFsmEvent* event)
 	return true;
 }
 
+bool CNetServerWorker::OnRejoined(void* context, CFsmEvent* event)
+{
+	// A client has finished rejoining and the loading screen disappeared.
+	ENSURE(event->GetType() == (uint)NMT_REJOINED);
+
+	CNetServerSession* session = (CNetServerSession*)context;
+	CNetServerWorker& server = session->GetServer();
+
+	CRejoinedMessage* message = (CRejoinedMessage*)event->GetParamRef();
+
+	message->m_GUID = session->GetGUID();
+
+	server.Broadcast(message);
+
+	return true;
+}
+
 bool CNetServerWorker::OnDisconnect(void* context, CFsmEvent* event)
 {
 	ENSURE(event->GetType() == (uint)NMT_CONNECTION_LOST);
@@ -1026,6 +1232,14 @@ void CNetServerWorker::StartGame()
 
 	// Send the final setup state to all clients
 	UpdateGameAttributes(&m_GameAttributes.get());
+
+	// Remove players and observers that are not present when the game starts
+	for (PlayerAssignmentMap::iterator it = m_PlayerAssignments.begin(); it != m_PlayerAssignments.end();)
+		if (it->second.m_Enabled)
+			++it;
+		else
+			it = m_PlayerAssignments.erase(it);
+
 	SendPlayerAssignments();
 
 	CGameStartMessage gameStart;
@@ -1109,16 +1323,22 @@ bool CNetServer::SetupConnection()
 	return m_Worker->SetupConnection();
 }
 
+bool CNetServer::KickPlayer(const CStrW& playerName, const bool ban)
+{
+	CScopeLock lock(m_Worker->m_WorkerMutex);
+	return m_Worker->KickPlayer(playerName, ban);
+}
+
 void CNetServer::AssignPlayer(int playerID, const CStr& guid)
 {
 	CScopeLock lock(m_Worker->m_WorkerMutex);
-	m_Worker->m_AssignPlayerQueue.push_back(std::make_pair(playerID, guid));
+	m_Worker->m_AssignPlayerQueue.emplace_back(playerID, guid);
 }
 
 void CNetServer::SetPlayerReady(const CStr& guid, int ready)
 {
 	CScopeLock lock(m_Worker->m_WorkerMutex);
-	m_Worker->m_PlayerReadyQueue.push_back(std::make_pair(guid, ready));
+	m_Worker->m_PlayerReadyQueue.emplace_back(guid, ready);
 }
 
 void CNetServer::ClearAllPlayerReady()
