@@ -1,4 +1,4 @@
-/* Copyright (C) 2013 Wildfire Games.
+/* Copyright (C) 2016 Wildfire Games.
  * This file is part of 0 A.D.
  *
  * 0 A.D. is free software: you can redistribute it and/or modify
@@ -122,7 +122,7 @@ public:
 	 * The changes will be asynchronously propagated to all clients.
 	 */
 	void AssignPlayer(int playerID, const CStr& guid);
-	
+
 	/**
 	 * Call from the GUI to update the player readiness.
 	 * The changes will be asynchronously propagated to all clients.
@@ -134,7 +134,12 @@ public:
 	 * The changes will be asynchronously propagated to all clients.
 	 */
 	void ClearAllPlayerReady();
-	
+
+	/**
+	 * Disconnects a player from gamesetup or session.
+	 */
+	bool KickPlayer(const CStrW& playerName, const bool ban);
+
 	/**
 	 * Call from the GUI to asynchronously notify all clients that they should start loading the game.
 	 */
@@ -182,6 +187,11 @@ public:
 	 */
 	bool SendMessage(ENetPeer* peer, const CNetMessage* message);
 
+	/**
+	 * Disconnects a player from gamesetup or session.
+	 */
+	bool KickPlayer(const CStrW& playerName, const bool ban);
+
 	/**
 	 * Send a message to all clients who have completed the full connection process
 	 * (i.e. are in the pre-game or in-game states).
@@ -245,7 +255,7 @@ private:
 
 	void AddPlayer(const CStr& guid, const CStrW& name);
 	void RemovePlayer(const CStr& guid);
-	void SetPlayerReady(const CStr& guid, const int ready); 
+	void SetPlayerReady(const CStr& guid, const int ready);
 	void SendPlayerAssignments();
 	void ClearAllPlayerReady();
 
@@ -262,6 +272,7 @@ private:
 	static bool OnReady(void* context, CFsmEvent* event);
 	static bool OnLoadedGame(void* context, CFsmEvent* event);
 	static bool OnJoinSyncingLoadedGame(void* context, CFsmEvent* event);
+	static bool OnRejoined(void* context, CFsmEvent* event);
 	static bool OnDisconnect(void* context, CFsmEvent* event);
 
 	void CheckGameLoadStatus(CNetServerSession* changedSession);
@@ -270,6 +281,10 @@ private:
 
 	void HandleMessageReceive(const CNetMessage* message, CNetServerSession* session);
 
+	/**
+	 * Send a network warning if the connection to a client is being lost or has bad latency.
+	 */
+	void CheckClientConnections();
 
 	/**
 	 * Internal script context for (de)serializing script messages,
@@ -298,10 +313,15 @@ private:
 	CStrW m_ServerName;
 	CStrW m_WelcomeMessage;
 
+	std::vector<CStr> m_BannedIPs;
+	std::vector<CStrW> m_BannedPlayers;
+
 	u32 m_NextHostID;
 
 	CNetServerTurnManager* m_ServerTurnManager;
 
+	CStr m_HostGUID;
+
 	/**
 	 * A copy of all simulation commands received so far, indexed by
 	 * turn number, to simplify support for rejoining etc.
@@ -315,6 +335,11 @@ private:
 	 */
 	std::string m_JoinSyncFile;
 
+	/**
+	 *  Time when the clients connections were last checked for timeouts and latency.
+	 */
+	std::time_t m_LastConnectionCheck;
+
 private:
 	// Thread-related stuff:
 
