@@ -1,4 +1,4 @@
-/* Copyright (C) 2011 Wildfire Games.
+/* Copyright (C) 2016 Wildfire Games.
  * This file is part of 0 A.D.
  *
  * 0 A.D. is free software: you can redistribute it and/or modify
@@ -25,6 +25,10 @@
 #include "ps/CLogger.h"
 #include "scriptinterface/ScriptInterface.h"
 
+const u32 NETWORK_WARNING_TIMEOUT = 4000;
+
+const u32 MAXIMUM_HOST_TIMEOUT = std::numeric_limits<u32>::max();
+
 static const int CHANNEL_COUNT = 1;
 
 CNetClientSession::CNetClientSession(CNetClient& client) :
@@ -39,7 +43,7 @@ CNetClientSession::~CNetClientSession()
 	if (m_Host && m_Server)
 	{
 		// Disconnect immediately (we can't wait for acks)
-		enet_peer_disconnect_now(m_Server, NDR_UNEXPECTED_SHUTDOWN);
+		enet_peer_disconnect_now(m_Server, NDR_SERVER_SHUTDOWN);
 		enet_host_destroy(m_Host);
 
 		m_Host = NULL;
@@ -71,6 +75,12 @@ bool CNetClientSession::Connect(u16 port, const CStr& server)
 	m_Host = host;
 	m_Server = peer;
 
+	// Prevent the local client of the host from timing out too quickly.
+#if (ENET_VERSION >= ENET_VERSION_CREATE(1, 3, 4))
+	if (GetIPAddress() == "127.0.0.1")
+		enet_peer_timeout(peer, 1, MAXIMUM_HOST_TIMEOUT, MAXIMUM_HOST_TIMEOUT);
+#endif
+
 	m_Stats = new CNetStatsTable(m_Server);
 	if (CProfileViewer::IsInitialised())
 		g_ProfileViewer.AddRootTable(m_Stats);
@@ -168,6 +178,31 @@ bool CNetClientSession::SendMessage(const CNetMessage* message)
 	return CNetHost::SendMessage(message, m_Server, "server");
 }
 
+CStr CNetClientSession::GetIPAddress() const
+{
+	char ipAddress[256] = "";
+	if (enet_address_get_host_ip(&m_Server->address, ipAddress, ARRAY_SIZE(ipAddress)) < 0)
+		LOGMESSAGE("Could not get IP address of the server!");
+
+	return ipAddress;
+}
+
+u32 CNetClientSession::GetLastReceivedTime() const
+{
+	if (!m_Server)
+		return 0;
+
+	return enet_time_get() - m_Server->lastReceiveTime;
+}
+
+u32 CNetClientSession::GetMeanRTT() const
+{
+	if (!m_Server)
+		return 0;
+
+	return m_Server->roundTripTime;
+}
+
 
 
 CNetServerSession::CNetServerSession(CNetServerWorker& server, ENetPeer* peer) :
@@ -175,6 +210,31 @@ CNetServerSession::CNetServerSession(CNetServerWorker& server, ENetPeer* peer) :
 {
 }
 
+CStr CNetServerSession::GetIPAddress() const
+{
+	char ipAddress[256] = "";
+	if (enet_address_get_host_ip(&m_Peer->address, ipAddress, ARRAY_SIZE(ipAddress)) < 0)
+		LOGMESSAGE("Could not get IP address of a client!");
+
+	return ipAddress;
+}
+
+u32 CNetServerSession::GetLastReceivedTime() const
+{
+	if (!m_Peer)
+		return 0;
+
+	return enet_time_get() - m_Peer->lastReceiveTime;
+}
+
+u32 CNetServerSession::GetMeanRTT() const
+{
+	if (!m_Peer)
+		return 0;
+
+	return m_Peer->roundTripTime;
+}
+
 void CNetServerSession::Disconnect(u32 reason)
 {
 	Update((uint)NMT_CONNECTION_LOST, NULL);
