@@ -1,4 +1,4 @@
-/* Copyright (C) 2012 Wildfire Games.
+/* Copyright (C) 2016 Wildfire Games.
  * This file is part of 0 A.D.
  *
  * 0 A.D. is free software: you can redistribute it and/or modify
@@ -18,6 +18,7 @@
 #include "precompiled.h"
 
 #include "NetTurnManager.h"
+#include "NetMessage.h"
 
 #include "network/NetServer.h"
 #include "network/NetClient.h"
@@ -37,8 +38,8 @@
 #include <fstream>
 #include <iomanip>
 
-static const int DEFAULT_TURN_LENGTH_MP = 500;
-static const int DEFAULT_TURN_LENGTH_SP = 200;
+const u32 DEFAULT_TURN_LENGTH_MP = 500;
+const u32 DEFAULT_TURN_LENGTH_SP = 200;
 
 static const int COMMAND_DELAY = 2;
 
@@ -60,7 +61,7 @@ static std::string Hexify(const std::string& s)
 CNetTurnManager::CNetTurnManager(CSimulation2& simulation, u32 defaultTurnLength, int clientId, IReplayLogger& replay) :
 	m_Simulation2(simulation), m_CurrentTurn(0), m_ReadyTurn(1), m_TurnLength(defaultTurnLength), m_DeltaSimTime(0),
 	m_PlayerId(-1), m_ClientId(clientId), m_HasSyncError(false), m_Replay(replay),
-	m_TimeWarpNumTurns(0)
+	m_TimeWarpNumTurns(0), m_FinalTurn(std::numeric_limits<u32>::max())
 {
 	// When we are on turn n, we schedule new commands for n+2.
 	// We know that all other clients have finished scheduling commands for n (else we couldn't have got here).
@@ -90,6 +91,9 @@ bool CNetTurnManager::WillUpdate(float simFrameLength)
 {
 	// Keep this in sync with the return value of Update()
 
+	if (m_CurrentTurn > m_FinalTurn)
+		return false;
+
 	if (m_DeltaSimTime + simFrameLength < 0)
 		return false;
 
@@ -101,8 +105,16 @@ bool CNetTurnManager::WillUpdate(float simFrameLength)
 
 bool CNetTurnManager::Update(float simFrameLength, size_t maxTurns)
 {
+	if (m_CurrentTurn > m_FinalTurn)
+		return false;
+
 	m_DeltaSimTime += simFrameLength;
 
+	// If the game becomes laggy, m_DeltaSimTime increases progressively.
+	// The engine will fast forward accordingly to catch up.
+	// To keep the game playable, stop fast forwarding after 2 turn lengths.
+	m_DeltaSimTime = std::min(m_DeltaSimTime, 2.0f * m_TurnLength / 1000.0f);
+
 	// If we haven't reached the next turn yet, do nothing
 	if (m_DeltaSimTime < 0)
 		return false;
@@ -139,7 +151,8 @@ bool CNetTurnManager::Update(float simFrameLength, size_t maxTurns)
 
 		NotifyFinishedOwnCommands(m_CurrentTurn + COMMAND_DELAY);
 
-		m_CurrentTurn += 1; // increase the turn number now, so Update can send new commands for a subsequent turn
+		// Increase now, so Update can send new commands for a subsequent turn
+		++m_CurrentTurn;
 
 		// Clean up any destroyed entities since the last turn (e.g. placement previews
 		// or rally point flags generated by the GUI). (Must do this before the time warp
@@ -195,7 +208,7 @@ bool CNetTurnManager::UpdateFastForward()
 		// (This is similar but doesn't call any Notify functions or update DeltaTime,
 		// it just updates the simulation state)
 
-		m_CurrentTurn += 1;
+		++m_CurrentTurn;
 
 		m_Simulation2.FlushDestroyedEntities();
 
@@ -218,33 +231,52 @@ bool CNetTurnManager::UpdateFastForward()
 	return true;
 }
 
-void CNetTurnManager::OnSyncError(u32 turn, const std::string& expectedHash)
+void CNetTurnManager::OnSyncError(u32 turn, const CStr& expectedHash, std::vector<CSyncErrorMessage::S_m_PlayerNames>& playerNames)
 {
 	NETTURN_LOG((L"OnSyncError(%d, %hs)\n", turn, Hexify(expectedHash).c_str()));
 
 	// Only complain the first time
 	if (m_HasSyncError)
 		return;
-	m_HasSyncError = true;
 
 	bool quick = !TurnNeedsFullHash(turn);
 	std::string hash;
-	bool ok = m_Simulation2.ComputeStateHash(hash, quick);
-	ENSURE(ok);
+	ENSURE(m_Simulation2.ComputeStateHash(hash, quick));
 
-	OsPath path = psLogDir()/"oos_dump.txt";
+	OsPath path = psLogDir() / "oos_dump.txt";
 	std::ofstream file (OsString(path).c_str(), std::ofstream::out | std::ofstream::trunc);
 	m_Simulation2.DumpDebugState(file);
 	file.close();
 
+	hash = Hexify(hash);
+	const std::string& expectedHashHex = Hexify(expectedHash);
+
+	DisplayOOSError(turn, hash, expectedHashHex, false, &playerNames, &path);
+}
+
+void CNetTurnManager::DisplayOOSError(u32 turn, const CStr& hash, const CStr& expectedHash, bool isReplay, std::vector<CSyncErrorMessage::S_m_PlayerNames>* playerNames = NULL, OsPath* path = NULL)
+{
+	m_HasSyncError = true;
+
 	std::stringstream msg;
-	msg << "Out of sync on turn " << turn << ": expected hash " << Hexify(expectedHash) << "\n\n";
-	msg << "Current state: turn " << m_CurrentTurn << ", hash " << Hexify(hash) << "\n\n";
-	msg << "Dumping current state to " << utf8_from_wstring(path.string());
+	msg << "Out of sync on turn " << turn;
+
+	if (playerNames)
+		for (size_t i = 0; i < playerNames->size(); ++i)
+			msg << (i == 0 ? "\nPlayers: " : ", ") << utf8_from_wstring((*playerNames)[i].m_Name);
+
+	if (isReplay)
+		msg << "\n\n" << "The current game state is different from the original game state.";
+	else
+		msg << "\n\n" << "Your game state is " << (expectedHash == hash ? "identical to" : "different from") << " the hosts game state.";
+
+	if (path)
+		msg << "\n\n" << "Dumping current state to " << CStr(path->string8()).EscapeToPrintableASCII();
+
+	LOGERROR("%s", msg.str());
+
 	if (g_GUI)
 		g_GUI->DisplayMessageBox(600, 350, L"Sync error", wstring_from_utf8(msg.str()));
-	else
-		LOGERROR("%s", msg.str());
 }
 
 void CNetTurnManager::Interpolate(float simFrameLength, float realFrameLength)
@@ -253,6 +285,11 @@ void CNetTurnManager::Interpolate(float simFrameLength, float realFrameLength)
 	// we need to save the previous turn length?
 
 	float offset = clamp(m_DeltaSimTime / (m_TurnLength / 1000.f) + 1.0, 0.0, 1.0);
+
+	// Stop animations while still updating the selection highlight
+	if (m_CurrentTurn > m_FinalTurn)
+		simFrameLength = 0;
+
 	m_Simulation2.Interpolate(simFrameLength, offset, realFrameLength);
 }
 
@@ -266,8 +303,8 @@ void CNetTurnManager::AddCommand(int client, int player, JS::HandleValue data, u
 		return;
 	}
 
-	SimulationCommand cmd(player, m_Simulation2.GetScriptInterface().GetContext(), data);
-	m_QueuedCommands[turn - (m_CurrentTurn+1)][client].emplace_back(std::move(cmd));
+	m_Simulation2.GetScriptInterface().FreezeObject(data, true);
+	m_QueuedCommands[turn - (m_CurrentTurn+1)][client].emplace_back(player, m_Simulation2.GetScriptInterface().GetContext(), data);
 }
 
 void CNetTurnManager::FinishedAllCommands(u32 turn, u32 turnLength)
@@ -320,10 +357,9 @@ void CNetTurnManager::RewindTimeWarp()
 void CNetTurnManager::QuickSave()
 {
 	TIMER(L"QuickSave");
-	
+
 	std::stringstream stream;
-	bool ok = m_Simulation2.SerializeState(stream);
-	if (!ok)
+	if (!m_Simulation2.SerializeState(stream))
 	{
 		LOGERROR("Failed to quicksave game");
 		return;
@@ -350,8 +386,7 @@ void CNetTurnManager::QuickLoad()
 	}
 
 	std::stringstream stream(m_QuickSaveState);
-	bool ok = m_Simulation2.DeserializeState(stream);
-	if (!ok)
+	if (!m_Simulation2.DeserializeState(stream))
 	{
 		LOGERROR("Failed to quickload game");
 		return;
@@ -402,14 +437,17 @@ void CNetClientTurnManager::NotifyFinishedUpdate(u32 turn)
 	std::string hash;
 	{
 		PROFILE3("state hash check");
-		bool ok = m_Simulation2.ComputeStateHash(hash, quick);
-		ENSURE(ok);
+		ENSURE(m_Simulation2.ComputeStateHash(hash, quick));
 	}
 
 	NETTURN_LOG((L"NotifyFinishedUpdate(%d, %hs)\n", turn, Hexify(hash).c_str()));
 
 	m_Replay.Hash(hash, quick);
 
+	// Don't send the hash if OOS
+	if (m_HasSyncError)
+		return;
+
 	// Send message to the server
 	CSyncCheckMessage msg;
 	msg.m_Turn = turn;
@@ -452,8 +490,7 @@ void CNetLocalTurnManager::NotifyFinishedUpdate(u32 UNUSED(turn))
 	std::string hash;
 	{
 		PROFILE3("state hash check");
-		bool ok = m_Simulation2.ComputeStateHash(hash);
-		ENSURE(ok);
+		ENSURE(m_Simulation2.ComputeStateHash(hash));
 	}
 	m_Replay.Hash(hash);
 #endif
@@ -464,11 +501,82 @@ void CNetLocalTurnManager::OnSimulationMessage(CSimulationMessage* UNUSED(msg))
 	debug_warn(L"This should never be called");
 }
 
+CNetReplayTurnManager::CNetReplayTurnManager(CSimulation2& simulation, IReplayLogger& replay) :
+	CNetLocalTurnManager(simulation, replay)
+{
+}
 
+void CNetReplayTurnManager::StoreReplayCommand(u32 turn, int player, const std::string& command)
+{
+	// Using the pair we make sure that commands per turn will be processed in the correct order
+	m_ReplayCommands[turn].emplace_back(player, command);
+}
 
+void CNetReplayTurnManager::StoreReplayHash(u32 turn, const std::string& hash, bool quick)
+{
+	m_ReplayHash[turn] = std::make_pair(hash, quick);
+}
+
+void CNetReplayTurnManager::StoreReplayTurnLength(u32 turn, u32 turnLength)
+{
+	m_ReplayTurnLengths[turn] = turnLength;
+
+	// Initialize turn length
+	if (turn == 0)
+		m_TurnLength = m_ReplayTurnLengths[0];
+}
+
+void CNetReplayTurnManager::StoreFinalReplayTurn(u32 turn)
+{
+	m_FinalTurn = turn;
+}
+
+void CNetReplayTurnManager::NotifyFinishedUpdate(u32 turn)
+{
+	if (turn == 1 && m_FinalTurn == 0)
+		g_GUI->SendEventToAll("ReplayFinished");
+
+	if (turn > m_FinalTurn)
+		return;
+
+	DoTurn(turn);
+
+	// Compare hash if it exists in the replay and if we didn't have an OOS already
+	if (m_HasSyncError || m_ReplayHash.find(turn) == m_ReplayHash.end())
+		return;
+
+	std::string expectedHash = m_ReplayHash[turn].first;
+	bool quickHash = m_ReplayHash[turn].second;
+
+	// Compute hash
+	std::string hash;
+	ENSURE(m_Simulation2.ComputeStateHash(hash, quickHash));
+	hash = Hexify(hash);
+
+	if (hash != expectedHash)
+		DisplayOOSError(turn, hash, expectedHash, true);
+}
+
+void CNetReplayTurnManager::DoTurn(u32 turn)
+{
+	debug_printf("Executing turn %u of %u\n", turn, m_FinalTurn);
+
+	m_TurnLength = m_ReplayTurnLengths[turn];
+
+	// Simulate commands for that turn
+	for (const std::pair<player_id_t, std::string>& pair : m_ReplayCommands[turn])
+	{
+		JS::RootedValue command(m_Simulation2.GetScriptInterface().GetContext());
+		m_Simulation2.GetScriptInterface().ParseJSON(pair.second, &command);
+		AddCommand(m_ClientId, pair.first, command, m_CurrentTurn + 1);
+	}
+
+	if (turn == m_FinalTurn)
+		g_GUI->SendEventToAll("ReplayFinished");
+}
 
 CNetServerTurnManager::CNetServerTurnManager(CNetServerWorker& server) :
-	m_NetServer(server), m_ReadyTurn(1), m_TurnLength(DEFAULT_TURN_LENGTH_MP)
+	m_NetServer(server), m_ReadyTurn(1), m_TurnLength(DEFAULT_TURN_LENGTH_MP), m_HasSyncError(false)
 {
 	// The first turn we will actually execute is number 2,
 	// so store dummy values into the saved lengths list
@@ -517,12 +625,17 @@ void CNetServerTurnManager::CheckClientsReady()
 	m_SavedTurnLengths.push_back(m_TurnLength);
 }
 
-void CNetServerTurnManager::NotifyFinishedClientUpdate(int client, u32 turn, const std::string& hash)
+void CNetServerTurnManager::NotifyFinishedClientUpdate(int client, const CStrW& playername, u32 turn, const CStr& hash)
 {
 	// Clients must advance one turn at a time
 	ENSURE(turn == m_ClientsSimulated[client] + 1);
 	m_ClientsSimulated[client] = turn;
 
+	// Check for OOS only if in sync
+	if (m_HasSyncError)
+		return;
+
+	m_ClientPlayernames[client] = playername;
 	m_ClientStateHashes[turn][client] = hash;
 
 	// Find the newest turn which we know all clients have simulated
@@ -542,22 +655,34 @@ void CNetServerTurnManager::NotifyFinishedClientUpdate(int client, u32 turn, con
 		// Assume the host is correct (maybe we should choose the most common instead to help debugging)
 		std::string expected = it->second.begin()->second;
 
+		// Find all players that are OOS on that turn
+		std::vector<CStrW> OOSPlayerNames;
 		for (std::map<int, std::string>::iterator cit = it->second.begin(); cit != it->second.end(); ++cit)
 		{
 			NETTURN_LOG((L"sync check %d: %d = %hs\n", it->first, cit->first, Hexify(cit->second).c_str()));
 			if (cit->second != expected)
 			{
 				// Oh no, out of sync
-
-				// Tell everyone about it
-				CSyncErrorMessage msg;
-				msg.m_Turn = it->first;
-				msg.m_HashExpected = expected;
-				m_NetServer.Broadcast(&msg);
-
-				break;
+				m_HasSyncError = true;
+				OOSPlayerNames.push_back(m_ClientPlayernames[cit->first]);
 			}
 		}
+
+		// Tell everyone about it
+		if (m_HasSyncError)
+		{
+			CSyncErrorMessage msg;
+			msg.m_Turn = it->first;
+			msg.m_HashExpected = expected;
+			for (const CStrW& playername : OOSPlayerNames)
+			{
+				CSyncErrorMessage::S_m_PlayerNames h;
+				h.m_Name = playername;
+				msg.m_PlayerNames.push_back(h);
+			}
+			m_NetServer.Broadcast(&msg);
+			break;
+		}
 	}
 
 	// Delete the saved hashes for all turns that we've already verified
