@@ -1,4 +1,4 @@
-/* Copyright (C) 2012 Wildfire Games.
+/* Copyright (C) 2016 Wildfire Games.
  * This file is part of 0 A.D.
  *
  * 0 A.D. is free software: you can redistribute it and/or modify
@@ -19,9 +19,15 @@
 #define INCLUDED_NETTURNMANAGER
 
 #include "simulation2/helpers/SimulationCommand.h"
+#include "lib/os_path.h"
+#include "NetMessage.h"
 
 #include <list>
 #include <map>
+#include <vector>
+
+extern const u32 DEFAULT_TURN_LENGTH_MP;
+extern const u32 DEFAULT_TURN_LENGTH_SP;
 
 class CNetServerWorker;
 class CNetClient;
@@ -71,7 +77,7 @@ public:
 	 * Advance the simulation by a certain time. If this brings us past the current
 	 * turn length, the next turns are processed and the function returns true.
 	 * Otherwise, nothing happens and it returns false.
-	 * 
+	 *
 	 * @param simFrameLength Length of the previous frame, in simulation seconds
 	 * @param maxTurns Maximum number of turns to simulate at once
 	 */
@@ -105,7 +111,12 @@ public:
 	/**
 	 * Called when there has been an out-of-sync error.
 	 */
-	virtual void OnSyncError(u32 turn, const std::string& expectedHash);
+	virtual void OnSyncError(u32 turn, const CStr& expectedHash, std::vector<CSyncErrorMessage::S_m_PlayerNames>& playerNames);
+
+	/**
+	 * Shows a message box when an out of sync error has been detected in the session or visual replay.
+	 */
+	virtual void DisplayOOSError(u32 turn, const CStr& hash, const CStr& expectedHash, bool isReplay, std::vector<CSyncErrorMessage::S_m_PlayerNames>* playerNames, OsPath* path);
 
 	/**
 	 * Called by simulation code, to add a new command to be distributed to all clients and executed soon.
@@ -182,6 +193,9 @@ protected:
 
 	IReplayLogger& m_Replay;
 
+	// The number of the last turn that is allowed to be executed (used for replays)
+	u32 m_FinalTurn;
+
 private:
 	size_t m_TimeWarpNumTurns; // 0 if disabled
 	std::list<std::string> m_TimeWarpStates;
@@ -189,6 +203,7 @@ private:
 	std::string m_QuickSaveMetadata;
 };
 
+
 /**
  * Implementation of CNetTurnManager for network clients.
  */
@@ -200,7 +215,7 @@ public:
 	virtual void OnSimulationMessage(CSimulationMessage* msg);
 
 	virtual void PostCommand(JS::HandleValue data);
-	
+
 	/**
 	 * Notifiy the server that all commands are sent to prepare the connection for termination.
 	 */
@@ -233,6 +248,38 @@ protected:
 };
 
 
+
+/**
+ * Implementation of CNetTurnManager for replay games.
+ */
+class CNetReplayTurnManager : public CNetLocalTurnManager
+{
+public:
+	CNetReplayTurnManager(CSimulation2& simulation, IReplayLogger& replay);
+
+	void StoreReplayCommand(u32 turn, int player, const std::string& command);
+
+	void StoreReplayTurnLength(u32 turn, u32 turnLength);
+
+	void StoreReplayHash(u32 turn, const std::string& hash, bool quick);
+
+	void StoreFinalReplayTurn(u32 turn);
+
+
+protected:
+	virtual void NotifyFinishedUpdate(u32 turn);
+
+	void DoTurn(u32 turn);
+
+	// Contains the commands of every player on each turn
+	std::map<u32, std::vector<std::pair<player_id_t, std::string> > > m_ReplayCommands;
+
+	// Contains the length of every turn
+	std::map<u32, u32> m_ReplayTurnLengths;
+
+	// Contains all replay hash values and weather or not the quick hash method was used
+	std::map<u32, std::pair<std::string, bool> > m_ReplayHash;
+};
 /**
  * The server-side counterpart to CNetClientTurnManager.
  * Records the turn state of each client, and sends turn advancement messages
@@ -249,7 +296,7 @@ public:
 
 	void NotifyFinishedClientCommands(int client, u32 turn);
 
-	void NotifyFinishedClientUpdate(int client, u32 turn, const std::string& hash);
+	void NotifyFinishedClientUpdate(int client, const CStrW& playername, u32 turn, const CStr& hash);
 
 	/**
 	 * Inform the turn manager of a new client who will be sending commands.
@@ -292,6 +339,9 @@ protected:
 	// Map of turn -> {Client ID -> state hash}; old indexes <= min(m_ClientsSimulated) are deleted
 	std::map<u32, std::map<int, std::string> > m_ClientStateHashes;
 
+	// Map of client ID -> playername
+	std::map<u32, CStrW> m_ClientPlayernames;
+
 	// Current turn length
 	u32 m_TurnLength;
 
@@ -299,6 +349,8 @@ protected:
 	std::vector<u32> m_SavedTurnLengths;
 
 	CNetServerWorker& m_NetServer;
+
+	bool m_HasSyncError;
 };
 
 #endif // INCLUDED_NETTURNMANAGER
