@@ -1,4 +1,4 @@
-/* Copyright (C) 2012 Wildfire Games.
+/* Copyright (C) 2014 Wildfire Games.
  * This file is part of 0 A.D.
  *
  * 0 A.D. is free software: you can redistribute it and/or modify
@@ -77,7 +77,7 @@ public:
 
 		for (size_t i = 0; ; ++i)
 		{
-//			debug_printf(L".");
+//			debug_printf(".");
 			for (size_t j = 0; j < clients.size(); ++j)
 				clients[j]->Poll();
 
@@ -99,7 +99,7 @@ public:
 	{
 		for (size_t i = 0; ; ++i)
 		{
-//			debug_printf(L".");
+//			debug_printf(".");
 			server.Poll();
 			for (size_t j = 0; j < clients.size(); ++j)
 				clients[j]->Poll();
@@ -135,6 +135,9 @@ public:
 		// and prints a load of debug output so you can see if anything funny's going on
 
 		ScriptInterface scriptInterface("Engine", "Test", g_ScriptRuntime);
+		JSContext* cx = scriptInterface.GetContext();
+		JSAutoRequest rq(cx);
+		
 		TestStdoutLogger logger;
 
 		std::vector<CNetClient*> clients;
@@ -145,9 +148,9 @@ public:
 
 		CNetServer server;
 
-		CScriptValRooted attrs;
-		scriptInterface.Eval("({mapType:'scenario',map:'_default',thing:'example'})", attrs);
-		server.UpdateGameAttributes(attrs.get(), scriptInterface);
+		JS::RootedValue attrs(cx);
+		scriptInterface.Eval("({mapType:'scenario',map:'maps/scenarios/Saharan Oases',mapPath:'maps/scenarios/',thing:'example'})", &attrs);
+		server.UpdateGameAttributes(&attrs, scriptInterface);
 
 		CNetClient client1(&client1Game);
 		CNetClient client2(&client2Game);
@@ -158,7 +161,7 @@ public:
 		clients.push_back(&client3);
 
 		connect(server, clients);
-		debug_printf(L"%ls", client1.TestReadGuiMessages().c_str());
+		debug_printf("%s", client1.TestReadGuiMessages().c_str());
 
 		server.StartGame();
 		SDL_Delay(100);
@@ -172,14 +175,14 @@ public:
 		wait(clients, 100);
 
 		{
-			CScriptValRooted cmd;
-			client1.GetScriptInterface().Eval("({type:'debug-print', message:'[>>> client1 test sim command]\\n'})", cmd);
+			JS::RootedValue cmd(cx);
+			client1.GetScriptInterface().Eval("({type:'debug-print', message:'[>>> client1 test sim command]\\n'})", &cmd);
 			client1Game.GetTurnManager()->PostCommand(cmd);
 		}
 
 		{
-			CScriptValRooted cmd;
-			client2.GetScriptInterface().Eval("({type:'debug-print', message:'[>>> client2 test sim command]\\n'})", cmd);
+			JS::RootedValue cmd(cx);
+			client2.GetScriptInterface().Eval("({type:'debug-print', message:'[>>> client2 test sim command]\\n'})", &cmd);
 			client2Game.GetTurnManager()->PostCommand(cmd);
 		}
 
@@ -197,6 +200,9 @@ public:
 	void test_rejoin_DISABLED()
 	{
 		ScriptInterface scriptInterface("Engine", "Test", g_ScriptRuntime);
+		JSContext* cx = scriptInterface.GetContext();
+		JSAutoRequest rq(cx);
+		
 		TestStdoutLogger logger;
 
 		std::vector<CNetClient*> clients;
@@ -207,9 +213,9 @@ public:
 
 		CNetServer server;
 
-		CScriptValRooted attrs;
-		scriptInterface.Eval("({mapType:'scenario',map:'_default',thing:'example'})", attrs);
-		server.UpdateGameAttributes(attrs.get(), scriptInterface);
+		JS::RootedValue attrs(cx);
+		scriptInterface.Eval("({mapType:'scenario',map:'maps/scenarios/Saharan Oases',mapPath:'maps/scenarios/',thing:'example'})", &attrs);
+		server.UpdateGameAttributes(&attrs, scriptInterface);
 
 		CNetClient client1(&client1Game);
 		CNetClient client2(&client2Game);
@@ -224,7 +230,7 @@ public:
 		clients.push_back(&client3);
 
 		connect(server, clients);
-		debug_printf(L"%ls", client1.TestReadGuiMessages().c_str());
+		debug_printf("%s", client1.TestReadGuiMessages().c_str());
 
 		server.StartGame();
 		SDL_Delay(100);
@@ -238,8 +244,8 @@ public:
 		wait(clients, 100);
 
 		{
-			CScriptValRooted cmd;
-			client1.GetScriptInterface().Eval("({type:'debug-print', message:'[>>> client1 test sim command 1]\\n'})", cmd);
+			JS::RootedValue cmd(cx);
+			client1.GetScriptInterface().Eval("({type:'debug-print', message:'[>>> client1 test sim command 1]\\n'})", &cmd);
 			client1Game.GetTurnManager()->PostCommand(cmd);
 		}
 
@@ -250,17 +256,17 @@ public:
 		wait(clients, 100);
 
 		{
-			CScriptValRooted cmd;
-			client1.GetScriptInterface().Eval("({type:'debug-print', message:'[>>> client1 test sim command 2]\\n'})", cmd);
+			JS::RootedValue cmd(cx);
+			client1.GetScriptInterface().Eval("({type:'debug-print', message:'[>>> client1 test sim command 2]\\n'})", &cmd);
 			client1Game.GetTurnManager()->PostCommand(cmd);
 		}
 
-		debug_printf(L"==== Disconnecting client 2\n");
+		debug_printf("==== Disconnecting client 2\n");
 
 		client2.DestroyConnection();
 		clients.erase(clients.begin()+1);
 
-		debug_printf(L"==== Connecting client 2B\n");
+		debug_printf("==== Connecting client 2B\n");
 
 		CGame client2BGame(true);
 		CNetClient client2B(&client2BGame);
@@ -271,7 +277,7 @@ public:
 
 		for (size_t i = 0; ; ++i)
 		{
-			debug_printf(L"[%u]\n", client2B.GetCurrState());
+			debug_printf("[%u]\n", client2B.GetCurrState());
 			client2B.Poll();
 			if (client2B.GetCurrState() == NCS_PREGAME)
 				break;
@@ -305,8 +311,8 @@ public:
 		// CNetTurnManager::TurnNeedsFullHash to always return true)
 
 		{
-			CScriptValRooted cmd;
-			client1.GetScriptInterface().Eval("({type:'debug-print', message:'[>>> client1 test sim command 3]\\n'})", cmd);
+			JS::RootedValue cmd(cx);
+			client1.GetScriptInterface().Eval("({type:'debug-print', message:'[>>> client1 test sim command 3]\\n'})", &cmd);
 			client1Game.GetTurnManager()->PostCommand(cmd);
 		}
 
@@ -318,8 +324,8 @@ public:
 		wait(clients, 100);
 
 		{
-			CScriptValRooted cmd;
-			client1.GetScriptInterface().Eval("({type:'debug-print', message:'[>>> client1 test sim command 4]\\n'})", cmd);
+			JS::RootedValue cmd(cx);
+			client1.GetScriptInterface().Eval("({type:'debug-print', message:'[>>> client1 test sim command 4]\\n'})", &cmd);
 			client1Game.GetTurnManager()->PostCommand(cmd);
 		}
 
