@@ -1,4 +1,4 @@
-/* Copyright (C) 2015 Wildfire Games.
+/* Copyright (C) 2016 Wildfire Games.
  * This file is part of 0 A.D.
  *
  * 0 A.D. is free software: you can redistribute it and/or modify
@@ -35,9 +35,9 @@
 #include "ps/CStr.h"
 #include "ps/Loader.h"
 #include "ps/LoaderThunks.h"
-#include "ps/Overlay.h"
 #include "ps/Profile.h"
 #include "ps/Replay.h"
+#include "ps/Shapes.h"
 #include "ps/World.h"
 #include "ps/GameSetup/GameSetup.h"
 #include "renderer/Renderer.h"
@@ -63,7 +63,7 @@ CGame *g_Game=NULL;
  * Constructor
  *
  **/
-CGame::CGame(bool disableGraphics):
+CGame::CGame(bool disableGraphics, bool replayLog):
 	m_World(new CWorld(this)),
 	m_Simulation2(new CSimulation2(&m_World->GetUnitManager(), g_ScriptRuntime, m_World->GetTerrain())),
 	m_GameView(disableGraphics ? NULL : new CGameView(this)),
@@ -71,10 +71,15 @@ CGame::CGame(bool disableGraphics):
 	m_Paused(false),
 	m_SimRate(1.0f),
 	m_PlayerID(-1),
-	m_IsSavedGame(false)
+	m_IsSavedGame(false),
+	m_IsVisualReplay(false),
+	m_ReplayStream(NULL)
 {
-	m_ReplayLogger = new CReplayLogger(m_Simulation2->GetScriptInterface());
 	// TODO: should use CDummyReplayLogger unless activated by cmd-line arg, perhaps?
+	if (replayLog)
+		m_ReplayLogger = new CReplayLogger(m_Simulation2->GetScriptInterface());
+	else
+		m_ReplayLogger = new CDummyReplayLogger();
 
 	// Need to set the CObjectManager references after various objects have
 	// been initialised, so do it here rather than via the initialisers above.
@@ -101,6 +106,7 @@ CGame::~CGame()
 	delete m_Simulation2;
 	delete m_World;
 	delete m_ReplayLogger;
+	delete m_ReplayStream;
 }
 
 void CGame::SetTurnManager(CNetTurnManager* turnManager)
@@ -114,6 +120,76 @@ void CGame::SetTurnManager(CNetTurnManager* turnManager)
 		m_TurnManager->SetPlayerID(m_PlayerID);
 }
 
+int CGame::LoadVisualReplayData()
+{
+	ENSURE(m_IsVisualReplay);
+	ENSURE(!m_ReplayPath.empty());
+	ENSURE(m_ReplayStream);
+
+	CNetReplayTurnManager* replayTurnMgr = static_cast<CNetReplayTurnManager*>(GetTurnManager());
+
+	u32 currentTurn = 0;
+	std::string type;
+	while ((*m_ReplayStream >> type).good())
+	{
+		if (type == "turn")
+		{
+			u32 turn = 0;
+			u32 turnLength = 0;
+			*m_ReplayStream >> turn >> turnLength;
+			ENSURE(turn == currentTurn && "You tried to replay a commands.txt file of a rejoined client. Please use the host's file.");
+			replayTurnMgr->StoreReplayTurnLength(currentTurn, turnLength);
+		}
+		else if (type == "cmd")
+		{
+			player_id_t player;
+			*m_ReplayStream >> player;
+
+			std::string line;
+			std::getline(*m_ReplayStream, line);
+			replayTurnMgr->StoreReplayCommand(currentTurn, player, line);
+		}
+		else if (type == "hash" || type == "hash-quick")
+		{
+			bool quick = (type == "hash-quick");
+			std::string replayHash;
+			*m_ReplayStream >> replayHash;
+			replayTurnMgr->StoreReplayHash(currentTurn, replayHash, quick);
+		}
+		else if (type == "end")
+			++currentTurn;
+		else
+			CancelLoad(L"Failed to load replay data (unrecognized content)");
+	}
+	SAFE_DELETE(m_ReplayStream);
+	m_FinalReplayTurn = currentTurn > 0 ? currentTurn - 1 : 0;
+	replayTurnMgr->StoreFinalReplayTurn(m_FinalReplayTurn);
+	return 0;
+}
+
+bool CGame::StartVisualReplay(const std::string& replayPath)
+{
+	debug_printf("Starting to replay %s\n", replayPath.c_str());
+
+	m_IsVisualReplay = true;
+	ScriptInterface& scriptInterface = m_Simulation2->GetScriptInterface();
+
+	SetTurnManager(new CNetReplayTurnManager(*m_Simulation2, GetReplayLogger()));
+
+	m_ReplayPath = replayPath;
+	m_ReplayStream = new std::ifstream(m_ReplayPath.c_str());
+
+	std::string type;
+	ENSURE((*m_ReplayStream >> type).good() && type == "start");
+
+	std::string line;
+	std::getline(*m_ReplayStream, line);
+	JS::RootedValue attribs(scriptInterface.GetContext());
+	scriptInterface.ParseJSON(line, &attribs);
+	StartGame(&attribs, "");
+
+	return true;
+}
 
 /**
  * Initializes the game with the set of attributes provided.
@@ -176,6 +252,9 @@ void CGame::RegisterInit(const JS::HandleValue attribs, const std::string& saved
 	if (m_IsSavedGame)
 		RegMemFun(this, &CGame::LoadInitialState, L"Loading game", 1000);
 
+	if (m_IsVisualReplay)
+		RegMemFun(this, &CGame::LoadVisualReplayData, L"Loading visual replay data", 1000);
+
 	LDR_EndRegistering();
 }
 
@@ -235,6 +314,9 @@ PSRETURN CGame::ReallyStartGame()
 	if (CRenderer::IsInitialised())
 		Render();
 
+	if (g_NetClient)
+		g_NetClient->LoadFinished();
+
 	// Call the reallyStartGame GUI function, but only if it exists
 	if (g_GUI && g_GUI->HasPages())
 	{
@@ -243,9 +325,6 @@ PSRETURN CGame::ReallyStartGame()
 			g_GUI->GetActiveGUI()->GetScriptInterface()->CallFunctionVoid(global, "reallyStartGame");
 	}
 
-	if (g_NetClient)
-		g_NetClient->LoadFinished();
-
 	debug_printf("GAME STARTED, ALL INIT COMPLETE\n");
 
 	// The call tree we've built for pregame probably isn't useful in-game.
@@ -263,16 +342,20 @@ int CGame::GetPlayerID()
 	return m_PlayerID;
 }
 
-void CGame::SetPlayerID(int playerID)
+void CGame::SetPlayerID(player_id_t playerID)
 {
 	m_PlayerID = playerID;
 	if (m_TurnManager)
 		m_TurnManager->SetPlayerID(m_PlayerID);
+
+	m_Simulation2->SetCurrentDisplayedPlayer(g_Game->GetPlayerID());
 }
 
 void CGame::StartGame(JS::MutableHandleValue attribs, const std::string& savedState)
 {
-	m_ReplayLogger->StartGame(attribs);
+	if (m_ReplayLogger)
+		m_ReplayLogger->StartGame(attribs);
+
 	RegisterInit(attribs, savedState);
 }
 
@@ -280,17 +363,13 @@ void CGame::StartGame(JS::MutableHandleValue attribs, const std::string& savedSt
 // so that it has more control over the update rate. The game might want to
 // do the same, and then doInterpolate should be redundant and removed.
 
-bool CGame::Update(const double deltaRealTime, bool doInterpolate)
+void CGame::Update(const double deltaRealTime, bool doInterpolate)
 {
-	if (m_Paused)
-		return true;
-
-	if (!m_TurnManager)
-		return true;
+	if (m_Paused || !m_TurnManager)
+		return;
 
 	const double deltaSimTime = deltaRealTime * m_SimRate;
 	
-	bool ok = true;
 	if (deltaSimTime)
 	{
 		// To avoid confusing the profiler, we need to trigger the new turn
@@ -313,7 +392,7 @@ bool CGame::Update(const double deltaRealTime, bool doInterpolate)
 
 			GetView()->GetLOSTexture().MakeDirty();
 		}
-		
+
 		if (CRenderer::IsInitialised())
 			g_Renderer.GetTimeManager().Update(deltaSimTime);
 	}
@@ -325,8 +404,6 @@ bool CGame::Update(const double deltaRealTime, bool doInterpolate)
 		if ( g_SoundManager )
 			g_SoundManager->IdleTask();
 	}
-
-	return ok;
 }
 
 void CGame::Interpolate(float simFrameLength, float realFrameLength)
@@ -340,32 +417,32 @@ void CGame::Interpolate(float simFrameLength, float realFrameLength)
 
 static CColor BrokenColor(0.3f, 0.3f, 0.3f, 1.0f);
 
-void CGame::CachePlayerColours()
+void CGame::CachePlayerColors()
 {
-	m_PlayerColours.clear();
+	m_PlayerColors.clear();
 
 	CmpPtr<ICmpPlayerManager> cmpPlayerManager(*m_Simulation2, SYSTEM_ENTITY);
 	if (!cmpPlayerManager)
 		return;
 
 	int numPlayers = cmpPlayerManager->GetNumPlayers();
-	m_PlayerColours.resize(numPlayers);
+	m_PlayerColors.resize(numPlayers);
 
 	for (int i = 0; i < numPlayers; ++i)
 	{
 		CmpPtr<ICmpPlayer> cmpPlayer(*m_Simulation2, cmpPlayerManager->GetPlayerByID(i));
 		if (!cmpPlayer)
-			m_PlayerColours[i] = BrokenColor;
+			m_PlayerColors[i] = BrokenColor;
 		else
-			m_PlayerColours[i] = cmpPlayer->GetColour();
+			m_PlayerColors[i] = cmpPlayer->GetColor();
 	}
 }
 
 
-CColor CGame::GetPlayerColour(int player) const
+CColor CGame::GetPlayerColor(player_id_t player) const
 {
-	if (player < 0 || player >= (int)m_PlayerColours.size())
+	if (player < 0 || player >= (int)m_PlayerColors.size())
 		return BrokenColor;
 
-	return m_PlayerColours[player];
+	return m_PlayerColors[player];
 }
