@@ -1,4 +1,4 @@
-/* Copyright (C) 2013 Wildfire Games.
+/* Copyright (C) 2016 Wildfire Games.
  * This file is part of 0 A.D.
  *
  * 0 A.D. is free software: you can redistribute it and/or modify
@@ -22,6 +22,7 @@
 #include <vector>
 
 #include "scriptinterface/ScriptVal.h"
+#include "simulation2/helpers/Player.h"
 
 class CWorld;
 class CSimulation2;
@@ -60,12 +61,12 @@ class CGame
 	 **/
 	float m_SimRate;
 
-	int m_PlayerID;
+	player_id_t m_PlayerID;
 
 	CNetTurnManager* m_TurnManager;
 
 public:
-	CGame(bool disableGraphics = false);
+	CGame(bool disableGraphics = false, bool replayLog = true);
 	~CGame();
 
 	/**
@@ -76,6 +77,8 @@ public:
 	void StartGame(JS::MutableHandleValue attribs, const std::string& savedState);
 	PSRETURN ReallyStartGame();
 
+	bool StartVisualReplay(const std::string& replayPath);
+
 	/**
 	 * Periodic heartbeat that controls the process. performs all per-frame updates.
 	 * Simulation update is called and game status update is called.
@@ -85,22 +88,22 @@ public:
 	 * @return bool false if it can't keep up with the desired simulation rate
 	 *	indicating that you might want to render less frequently.
 	 */
-	bool Update(const double deltaRealTime, bool doInterpolate = true);
+	void Update(const double deltaRealTime, bool doInterpolate = true);
 
 	void Interpolate(float simFrameLength, float realFrameLength);
 
 	int GetPlayerID();
-	void SetPlayerID(int playerID);
+	void SetPlayerID(player_id_t playerID);
 
 	/**
-	 * Retrieving player colours from scripts is slow, so this updates an
-	 * internal cache of all players' colours.
+	 * Retrieving player colors from scripts is slow, so this updates an
+	 * internal cache of all players' colors.
 	 * Call this just before rendering, so it will always have the latest
-	 * colours.
+	 * colors.
 	 */
-	void CachePlayerColours();
+	void CachePlayerColors();
 
-	CColor GetPlayerColour(int player) const;
+	CColor GetPlayerColor(player_id_t player) const;
 
 	/**
 	 * Get m_GameStarted.
@@ -112,6 +115,14 @@ public:
 		return m_GameStarted;
 	}
 
+	/**
+	 * Get m_IsVisualReplay.
+	 *
+	 * @return bool the value of m_IsVisualReplay.
+	 **/
+	inline bool IsVisualReplay() const
+	{	return m_IsVisualReplay; }
+
 	/**
 	 * Get the pointer to the game world object.
 	 *
@@ -147,7 +158,7 @@ public:
 	inline void SetSimRate(float simRate)
 	{	if (isfinite(simRate)) m_SimRate = std::max(simRate, 0.0f); }
 
-	inline float GetSimRate()
+	inline float GetSimRate() const
 	{	return m_SimRate; }
 
 	/**
@@ -166,11 +177,17 @@ private:
 	void RegisterInit(const JS::HandleValue attribs, const std::string& savedState);
 	IReplayLogger* m_ReplayLogger;
 
-	std::vector<CColor> m_PlayerColours;
+	std::vector<CColor> m_PlayerColors;
 
 	int LoadInitialState();
 	std::string m_InitialSavedState; // valid between RegisterInit and LoadInitialState
 	bool m_IsSavedGame; // true if loading a saved game; false for a new game
+
+	int LoadVisualReplayData();
+	std::string m_ReplayPath;
+	bool m_IsVisualReplay;
+	std::istream* m_ReplayStream;
+	u32 m_FinalReplayTurn;
 };
 
 extern CGame *g_Game;
