@@ -1,4 +1,4 @@
-/* Copyright (C) 2012 Wildfire Games.
+/* Copyright (C) 2015 Wildfire Games.
  * This file is part of 0 A.D.
  *
  * 0 A.D. is free software: you can redistribute it and/or modify
@@ -72,6 +72,13 @@ extern bool g_Silhouettes;
 // flag to switch on sky rendering
 extern bool g_ShowSky;
 
+// Prefer GLSL shaders over ARB shaders
+extern bool g_PreferGLSL;
+// Use screen-space postprocessing filters (HDR, bloom, DOF, etc)
+extern bool g_PostProc;
+// Use smooth LOS interpolation
+extern bool g_SmoothLOS;
+
 extern float g_Gamma;
 // name of configured render path (depending on OpenGL extensions, this may not be
 // the render path that is actually in use right now)
@@ -84,7 +91,6 @@ extern bool g_VSync;
 extern bool g_Quickstart;
 extern bool g_DisableAudio;
 
-extern bool g_JSDebuggerEnabled;
 extern bool g_ScriptProfilingEnabled;
 
 extern CStrW g_CursorName;
