@@ -1,4 +1,4 @@
-/* Copyright (C) 2015 Wildfire Games.
+/* Copyright (C) 2016 Wildfire Games.
  * This file is part of 0 A.D.
  *
  * 0 A.D. is free software: you can redistribute it and/or modify
@@ -35,7 +35,8 @@
 #include "lib/sysdep/os/win/wversion.h"
 #endif
 
-#include "graphics/CinemaTrack.h"
+#include "graphics/CinemaManager.h"
+#include "graphics/CinemaPath.h"
 #include "graphics/FontMetrics.h"
 #include "graphics/GameView.h"
 #include "graphics/LightEnv.h"
@@ -66,7 +67,6 @@
 #include "ps/Joystick.h"
 #include "ps/Loader.h"
 #include "ps/Mod.h"
-#include "ps/Overlay.h"
 #include "ps/Profile.h"
 #include "ps/ProfileViewer.h"
 #include "ps/Profiler2.h"
@@ -81,7 +81,6 @@
 #include "renderer/Renderer.h"
 #include "renderer/VertexBufferManager.h"
 #include "renderer/ModelRenderer.h"
-#include "scriptinterface/DebuggingServer.h"
 #include "scriptinterface/ScriptInterface.h"
 #include "scriptinterface/ScriptStats.h"
 #include "simulation2/Simulation2.h"
@@ -229,6 +228,11 @@ void Render()
 
 	g_Renderer.RenderTextOverlays();
 
+	if (g_Game && g_Game->IsGameStarted())
+		g_Game->GetView()->GetCinema()->Render();
+
+	ogl_WarnIfError();
+
 	if (g_DoRenderGui)
 		g_GUI->Draw();
 
@@ -546,10 +550,6 @@ static void InitPs(bool setup_gui, const CStrW& gui_page, ScriptInterface* srcSc
 
 static void InitInput()
 {
-#if !SDL_VERSION_ATLEAST(2, 0, 0)
-	SDL_EnableKeyRepeat(SDL_DEFAULT_REPEAT_DELAY, SDL_DEFAULT_REPEAT_INTERVAL);
-#endif
-
 	g_Joystick.Initialise();
 
 	// register input handlers
@@ -573,6 +573,8 @@ static void InitInput()
 	in_add_handler(gui_handler);
 
 	in_add_handler(touch_input_handler);
+	
+	in_add_handler(cinema_manager_handler);
 
 	// must be registered after (called before) the GUI which relies on these globals
 	in_add_handler(GlobalsInputHandler);
@@ -603,21 +605,39 @@ static void InitRenderer()
 	new CRenderer;
 
 	// set renderer options from command line options - NOVBO must be set before opening the renderer
+	// and init them in the ConfigDB when needed
 	g_Renderer.SetOptionBool(CRenderer::OPT_NOVBO, g_NoGLVBO);
 	g_Renderer.SetOptionBool(CRenderer::OPT_SHADOWS, g_Shadows);
+	g_ConfigDB.SetValueBool(CFG_SYSTEM, "shadows", g_Shadows);
 
 	g_Renderer.SetOptionBool(CRenderer::OPT_WATERUGLY, g_WaterUgly);
+	g_ConfigDB.SetValueBool(CFG_SYSTEM, "waterugly", g_WaterUgly);
 	g_Renderer.SetOptionBool(CRenderer::OPT_WATERFANCYEFFECTS, g_WaterFancyEffects);
+	g_ConfigDB.SetValueBool(CFG_SYSTEM, "waterfancyeffects", g_WaterFancyEffects);
 	g_Renderer.SetOptionBool(CRenderer::OPT_WATERREALDEPTH, g_WaterRealDepth);
+	g_ConfigDB.SetValueBool(CFG_SYSTEM, "waterrealdepth", g_WaterRealDepth);
 	g_Renderer.SetOptionBool(CRenderer::OPT_WATERREFLECTION, g_WaterReflection);
+	g_ConfigDB.SetValueBool(CFG_SYSTEM, "waterreflection", g_WaterReflection);
 	g_Renderer.SetOptionBool(CRenderer::OPT_WATERREFRACTION, g_WaterRefraction);
+	g_ConfigDB.SetValueBool(CFG_SYSTEM, "waterrefraction", g_WaterRefraction);
 	g_Renderer.SetOptionBool(CRenderer::OPT_SHADOWSONWATER, g_WaterShadows);
+	g_ConfigDB.SetValueBool(CFG_SYSTEM, "watershadows", g_WaterShadows);
 	
 	g_Renderer.SetRenderPath(CRenderer::GetRenderPathByName(g_RenderPath));
 	g_Renderer.SetOptionBool(CRenderer::OPT_SHADOWPCF, g_ShadowPCF);
+	g_ConfigDB.SetValueBool(CFG_SYSTEM, "shadowpcf", g_ShadowPCF);
 	g_Renderer.SetOptionBool(CRenderer::OPT_PARTICLES, g_Particles);
+	g_ConfigDB.SetValueBool(CFG_SYSTEM, "particles", g_Particles);
 	g_Renderer.SetOptionBool(CRenderer::OPT_SILHOUETTES, g_Silhouettes);
+	g_ConfigDB.SetValueBool(CFG_SYSTEM, "silhouettes", g_Silhouettes);
 	g_Renderer.SetOptionBool(CRenderer::OPT_SHOWSKY, g_ShowSky);
+	g_ConfigDB.SetValueBool(CFG_SYSTEM, "showsky", g_ShowSky);
+	g_Renderer.SetOptionBool(CRenderer::OPT_PREFERGLSL, g_PreferGLSL);
+	g_ConfigDB.SetValueBool(CFG_SYSTEM, "preferglsl", g_PreferGLSL);
+	g_Renderer.SetOptionBool(CRenderer::OPT_POSTPROC, g_PostProc);
+	g_ConfigDB.SetValueBool(CFG_SYSTEM, "postproc", g_PostProc);
+	g_Renderer.SetOptionBool(CRenderer::OPT_SMOOTHLOS, g_SmoothLOS);
+	g_ConfigDB.SetValueBool(CFG_SYSTEM, "smoothlos", g_SmoothLOS);
 
 	// create terrain related stuff
 	new CTerrainTextureManager;
@@ -661,11 +681,7 @@ static void InitSDL()
 	}
 	atexit(SDL_Quit);
 
-#if !SDL_VERSION_ATLEAST(2, 0, 0)
-	SDL_EnableUNICODE(1);
-#endif
-
-#if OS_MACOSX && SDL_VERSION_ATLEAST(2, 0, 0)
+#if OS_MACOSX
 	// Some Mac mice only have one button, so they can't right-click
 	// but SDL2 can emulate that with Ctrl+Click
 	bool macMouse = false;
@@ -733,12 +749,6 @@ void Shutdown(int flags)
 	g_UserReporter.Deinitialize();
 	TIMER_END(L"shutdown UserReporter");
 
-
-	// JS debugger temporarily disabled during the SpiderMonkey upgrade (check trac ticket #2348 for details)
-	//TIMER_BEGIN(L"shutdown DebuggingServer (if active)");
-	//delete g_DebuggingServer;
-	//TIMER_END(L"shutdown DebuggingServer (if active)");
-
 	delete &g_L10n;
 
 from_config:
@@ -773,12 +783,12 @@ from_config:
 
 		CNetHost::Deinitialize();
 
-		SAFE_DELETE(g_ScriptStatsTable);
-
 		// should be last, since the above use them
 		SAFE_DELETE(g_Logger);
 		delete &g_Profiler;
 		delete &g_ProfileViewer;
+
+		SAFE_DELETE(g_ScriptStatsTable);
 	TIMER_END(L"shutdown misc");
 }
 
@@ -882,6 +892,11 @@ void EarlyInit()
 
 bool Autostart(const CmdLineArgs& args);
 
+/**
+ * Returns true if the user has intended to start a visual replay from command line.
+ */
+bool AutostartVisualReplay(const std::string& replayFile);
+
 bool Init(const CmdLineArgs& args, int flags)
 {
 	h_mgr_init();
@@ -928,12 +943,6 @@ bool Init(const CmdLineArgs& args, int flags)
 		exit(0);
 	}
 
-	// override ah_translate with our i18n code.
-	AppHooks hooks = {0};
-	hooks.translate = psTranslate;
-	hooks.translate_free = psTranslateFree;
-	app_hooks_update(&hooks);
-
 	CNetHost::Initialize();
 
 #if CONFIG2_AUDIO
@@ -963,11 +972,6 @@ bool Init(const CmdLineArgs& args, int flags)
 
 	new L10n;
 
-	// before scripting 
-	// JS debugger temporarily disabled during the SpiderMonkey upgrade (check trac ticket #2348 for details)
-	//if (g_JSDebuggerEnabled)
-	//	g_DebuggingServer = new CDebuggingServer();
-
 	// Optionally start profiler HTTP output automatically
 	// (By default it's only enabled by a hotkey, for security/performance)
 	bool profilerHTTPEnable = false;
@@ -992,10 +996,6 @@ void InitGraphics(const CmdLineArgs& args, int flags)
 
 		if (!g_VideoMode.InitSDL())
 			throw PSERROR_System_VmodeFailed(); // abort startup
-
-#if !SDL_VERSION_ATLEAST(2, 0, 0)
-		SDL_WM_SetCaption("0 A.D.", "0 A.D.");
-#endif
 	}
 
 	RunHardwareDetection();
@@ -1072,9 +1072,13 @@ void InitGraphics(const CmdLineArgs& args, int flags)
 
 	ogl_WarnIfError();
 
+	// TODO: Is this the best place for this?
+	if (VfsDirectoryExists(L"maps/"))
+		CXeromyces::AddValidator(g_VFS, "map", "maps/scenario.rng");
+
 	try
 	{
-		if (!Autostart(args))
+		if (!AutostartVisualReplay(args.Get("replay-visual")) && !Autostart(args))
 		{
 			const bool setup_gui = ((flags & INIT_NO_GUI) == 0);
 			// We only want to display the splash screen at startup
@@ -1146,20 +1150,17 @@ CStr8 LoadSettingsOfScenarioMap(const VfsPath &mapPath)
 	XMBElement mapElement = mapFile.GetRoot();
 
 	// Select the ScriptSettings node in the map file...
-	for (int i = 0; pathToSettings[i][0]; i++)
+	for (int i = 0; pathToSettings[i][0]; ++i)
 	{
 		int childId = mapFile.GetElementID(pathToSettings[i]);
 
-		XMBElementList children = mapElement.GetChildNodes();
-		for (int childIndex = 0; childIndex < children.Count; childIndex++)
-		{
-			XMBElement child = children.Item(childIndex);
-			if (child.GetNodeName() == childId)
-			{
-				mapElement = child;
-				break;
-			}
-		}
+		XMBElementList nodes = mapElement.GetChildNodes();
+		auto it = std::find_if(nodes.begin(), nodes.end(), [&childId](const XMBElement& child) {
+			return child.GetNodeName() == childId;
+		});
+
+		if (it != nodes.end())
+			mapElement = *it;
 	}
 	// ... they contain a JSON document to initialize the game setup
 	// screen
@@ -1474,6 +1475,26 @@ bool Autostart(const CmdLineArgs& args)
 	return true;
 }
 
+bool AutostartVisualReplay(const std::string& replayFile)
+{
+	if (!FileExists(OsPath(replayFile)))
+		return false;
+
+	g_Game = new CGame(false, false);
+	g_Game->SetPlayerID(-1);
+	g_Game->StartVisualReplay(replayFile);
+
+	// TODO: Non progressive load can fail - need a decent way to handle this
+	LDR_NonprogressiveLoad();
+
+	ENSURE(g_Game->ReallyStartGame() == PSRETURN_OK);
+
+	ScriptInterface& scriptInterface = g_Game->GetSimulation2()->GetScriptInterface();
+
+	InitPs(true, L"page_session.xml", &scriptInterface, JS::UndefinedHandleValue);
+	return true;
+}
+
 void CancelLoad(const CStrW& message)
 {
 	shared_ptr<ScriptInterface> pScriptInterface = g_GUI->GetActiveGUI()->GetScriptInterface();
