@@ -1,4 +1,4 @@
-/* Copyright (C) 2014 Wildfire Games.
+/* Copyright (C) 2015 Wildfire Games.
  * This file is part of 0 A.D.
  *
  * 0 A.D. is free software: you can redistribute it and/or modify
@@ -149,7 +149,7 @@ void SetDisableShadowPCF(ScriptInterface::CxPrivate* UNUSED(pCxPrivate), bool di
 
 void SetDisableAllWater(ScriptInterface::CxPrivate* UNUSED(pCxPrivate), bool disabled)
 {
-	g_WaterUgly = true;
+	g_WaterUgly = disabled;
 	if (!IsOverridden("waterfancyeffects"))
 		g_WaterFancyEffects = !disabled;
 	if (!IsOverridden("waterrealdepth"))
@@ -158,7 +158,7 @@ void SetDisableAllWater(ScriptInterface::CxPrivate* UNUSED(pCxPrivate), bool dis
 		g_WaterRefraction = !disabled;
 	if (!IsOverridden("waterreflection"))
 		g_WaterReflection = !disabled;
-	if (!IsOverridden("shadowsonwater"))
+	if (!IsOverridden("watershadows"))
 		g_WaterShadows = !disabled;
 }
 
@@ -168,11 +168,29 @@ void SetDisableFancyWater(ScriptInterface::CxPrivate* UNUSED(pCxPrivate), bool d
 		g_WaterFancyEffects = !disabled;
 	if (!IsOverridden("waterrealdepth"))
 		g_WaterRealDepth = !disabled;
-	if (!IsOverridden("shadowsonwater"))
+	if (!IsOverridden("watershadows"))
 		g_WaterShadows = !disabled;
 }
 
-void SetRenderPath(ScriptInterface::CxPrivate* UNUSED(pCxPrivate), std::string renderpath)
+void SetEnableGLSL(ScriptInterface::CxPrivate* UNUSED(pCxPrivate), bool enabled)
+{
+	if (!IsOverridden("preferglsl"))
+		g_PreferGLSL = enabled;
+}
+
+void SetEnablePostProc(ScriptInterface::CxPrivate* UNUSED(pCxPrivate), bool enabled)
+{
+	if (!IsOverridden("postproc"))
+		g_PostProc = enabled;
+}
+
+void SetEnableSmoothLOS(ScriptInterface::CxPrivate* UNUSED(pCxPrivate), bool enabled)
+{
+	if (!IsOverridden("smoothlos"))
+		g_SmoothLOS = enabled;
+}
+
+void SetRenderPath(ScriptInterface::CxPrivate* UNUSED(pCxPrivate), const std::string& renderpath)
 {
 	g_RenderPath = renderpath;
 }
@@ -191,6 +209,9 @@ void RunHardwareDetection()
 	scriptInterface.RegisterFunction<void, bool, &SetDisableShadowPCF>("SetDisableShadowPCF");
 	scriptInterface.RegisterFunction<void, bool, &SetDisableAllWater>("SetDisableAllWater");
 	scriptInterface.RegisterFunction<void, bool, &SetDisableFancyWater>("SetDisableFancyWater");
+	scriptInterface.RegisterFunction<void, bool, &SetEnableGLSL>("SetEnableGLSL");
+	scriptInterface.RegisterFunction<void, bool, &SetEnablePostProc>("SetEnablePostProc");
+	scriptInterface.RegisterFunction<void, bool, &SetEnableSmoothLOS>("SetEnableSmoothLOS");
 	scriptInterface.RegisterFunction<void, std::string, &SetRenderPath>("SetRenderPath");
 
 	// Load the detection script:
@@ -223,6 +244,7 @@ void RunHardwareDetection()
 	scriptInterface.SetProperty(settings, "arch_ia32", ARCH_IA32);
 	scriptInterface.SetProperty(settings, "arch_amd64", ARCH_AMD64);
 	scriptInterface.SetProperty(settings, "arch_arm", ARCH_ARM);
+	scriptInterface.SetProperty(settings, "arch_aarch64", ARCH_AARCH64);
 
 #ifdef NDEBUG
 	scriptInterface.SetProperty(settings, "build_debug", 0);
@@ -707,18 +729,10 @@ static void ReportGLLimits(ScriptInterface& scriptInterface, JS::HandleValue set
 
 	SDL_SysWMinfo wminfo;
 	SDL_VERSION(&wminfo.version);
-#if SDL_VERSION_ATLEAST(2, 0, 0)
 	const int ret = SDL_GetWindowWMInfo(g_VideoMode.GetWindow(), &wminfo);
-#else
-	const int ret = SDL_GetWMInfo(&wminfo);
-#endif
 	if (ret && wminfo.subsystem == SDL_SYSWM_X11)
 	{
-#if SDL_VERSION_ATLEAST(2, 0, 0)
 		Display* dpy = wminfo.info.x11.display;
-#else
-		Display* dpy = wminfo.info.x11.gfxdisplay;
-#endif
 		int scrnum = DefaultScreen(dpy);
 
 		const char* glxexts = glXQueryExtensionsString(dpy, scrnum);
