@@ -1,4 +1,4 @@
-/* Copyright (C) 2014 Wildfire Games.
+/* Copyright (C) 2016 Wildfire Games.
  * This file is part of 0 A.D.
  *
  * 0 A.D. is free software: you can redistribute it and/or modify
@@ -26,22 +26,22 @@
 #include "lib/tex/tex.h"
 #include "ps/Game.h"
 #include "ps/Loader.h"
+#include "ps/Mod.h"
 #include "ps/Profile.h"
 #include "ps/ProfileViewer.h"
+#include "ps/Pyrogenesis.h"
+#include "ps/Util.h"
+#include "ps/VisualReplay.h"
 #include "scriptinterface/ScriptInterface.h"
 #include "scriptinterface/ScriptStats.h"
 #include "simulation2/Simulation2.h"
 #include "simulation2/helpers/SimulationCommand.h"
 
+#include <ctime>
 #include <sstream>
 #include <fstream>
 #include <iomanip>
 
-#if MSC_VERSION
-#include <process.h>
-#define getpid _getpid // use the non-deprecated function name
-#endif
-
 static std::string Hexify(const std::string& s)
 {
 	std::stringstream str;
@@ -52,22 +52,8 @@ static std::string Hexify(const std::string& s)
 }
 
 CReplayLogger::CReplayLogger(ScriptInterface& scriptInterface) :
-	m_ScriptInterface(scriptInterface)
+	m_ScriptInterface(scriptInterface), m_Stream(NULL)
 {
-	// Construct the directory name based on the PID, to be relatively unique.
-	// Append "-1", "-2" etc if we run multiple matches in a single session,
-	// to avoid accidentally overwriting earlier logs.
-
-	std::wstringstream name;
-	name << getpid();
-
-	static int run = -1;
-	if (++run)
-		name << "-" << run;
-
-	OsPath path = psLogDir() / L"sim_log" / name.str() / L"commands.txt";
-	CreateDirectories(path.Parent(), 0700);
-	m_Stream = new std::ofstream(OsString(path).c_str(), std::ofstream::out | std::ofstream::trunc);
 }
 
 CReplayLogger::~CReplayLogger()
@@ -77,6 +63,17 @@ CReplayLogger::~CReplayLogger()
 
 void CReplayLogger::StartGame(JS::MutableHandleValue attribs)
 {
+	// Add timestamp, since the file-modification-date can change
+	m_ScriptInterface.SetProperty(attribs, "timestamp", std::to_string(std::time(nullptr)));
+
+	// Add engine version and currently loaded mods for sanity checks when replaying
+	m_ScriptInterface.SetProperty(attribs, "engine_version", CStr(engine_version));
+	m_ScriptInterface.SetProperty(attribs, "mods", g_modsLoaded);
+
+	m_Directory = getDateIndexSubdirectory(VisualReplay::GetDirectoryName());
+	debug_printf("Writing replay to %s\n", m_Directory.string8().c_str());
+
+	m_Stream = new std::ofstream(OsString(m_Directory / L"commands.txt").c_str(), std::ofstream::out | std::ofstream::trunc);
 	*m_Stream << "start " << m_ScriptInterface.StringifyJSON(attribs, false) << "\n";
 }
 
@@ -84,7 +81,7 @@ void CReplayLogger::Turn(u32 n, u32 turnLength, std::vector<SimulationCommand>&
 {
 	JSContext* cx = m_ScriptInterface.GetContext();
 	JSAutoRequest rq(cx);
-	
+
 	*m_Stream << "turn " << n << " " << turnLength << "\n";
 	for (size_t i = 0; i < commands.size(); ++i)
 	{
@@ -102,6 +99,11 @@ void CReplayLogger::Hash(const std::string& hash, bool quick)
 		*m_Stream << "hash " << Hexify(hash) << "\n";
 }
 
+OsPath CReplayLogger::GetDirectory() const
+{
+	return m_Directory;
+}
+
 ////////////////////////////////////////////////////////////////
 
 CReplayPlayer::CReplayPlayer() :
@@ -122,7 +124,7 @@ void CReplayPlayer::Load(const std::string& path)
 	ENSURE(m_Stream->good());
 }
 
-void CReplayPlayer::Replay(bool serializationtest)
+void CReplayPlayer::Replay(bool serializationtest, bool ooslog)
 {
 	ENSURE(m_Stream);
 
@@ -130,18 +132,16 @@ void CReplayPlayer::Replay(bool serializationtest)
 	new CProfileManager;
 	g_ScriptStatsTable = new CScriptStatsTable;
 	g_ProfileViewer.AddRootTable(g_ScriptStatsTable);
-	
+
 	const int runtimeSize = 384 * 1024 * 1024;
 	const int heapGrowthBytesGCTrigger = 20 * 1024 * 1024;
 	g_ScriptRuntime = ScriptInterface::CreateRuntime(shared_ptr<ScriptRuntime>(), runtimeSize, heapGrowthBytesGCTrigger);
 
-	CGame game(true);
-	g_Game = &game;
+	g_Game = new CGame(true, false);
 	if (serializationtest)
-		game.GetSimulation2()->EnableSerializationTest();
-		
-	JSContext* cx = game.GetSimulation2()->GetScriptInterface().GetContext();
-	JSAutoRequest rq(cx);
+		g_Game->GetSimulation2()->EnableSerializationTest();
+	if (ooslog)
+		g_Game->GetSimulation2()->EnableOOSLog();
 
 	// Need some stuff for terrain movement costs:
 	// (TODO: this ought to be independent of any graphics code)
@@ -155,30 +155,31 @@ void CReplayPlayer::Replay(bool serializationtest)
 	u32 turn = 0;
 	u32 turnLength = 0;
 
+	{
+	JSContext* cx = g_Game->GetSimulation2()->GetScriptInterface().GetContext();
+	JSAutoRequest rq(cx);
 	std::string type;
 	while ((*m_Stream >> type).good())
 	{
-//		if (turn >= 1400) break;
-
 		if (type == "start")
 		{
 			std::string line;
 			std::getline(*m_Stream, line);
 			JS::RootedValue attribs(cx);
-			ENSURE(game.GetSimulation2()->GetScriptInterface().ParseJSON(line, &attribs));
+			ENSURE(g_Game->GetSimulation2()->GetScriptInterface().ParseJSON(line, &attribs));
 
-			game.StartGame(&attribs, "");
+			g_Game->StartGame(&attribs, "");
 
 			// TODO: Non progressive load can fail - need a decent way to handle this
 			LDR_NonprogressiveLoad();
 
-			PSRETURN ret = game.ReallyStartGame();
+			PSRETURN ret = g_Game->ReallyStartGame();
 			ENSURE(ret == PSRETURN_OK);
 		}
 		else if (type == "turn")
 		{
 			*m_Stream >> turn >> turnLength;
-			debug_printf("Turn %u (%u)... ", turn, turnLength);
+			debug_printf("Turn %u (%u)...\n", turn, turnLength);
 		}
 		else if (type == "cmd")
 		{
@@ -188,8 +189,8 @@ void CReplayPlayer::Replay(bool serializationtest)
 			std::string line;
 			std::getline(*m_Stream, line);
 			JS::RootedValue data(cx);
-			game.GetSimulation2()->GetScriptInterface().ParseJSON(line, &data);
-
+			g_Game->GetSimulation2()->GetScriptInterface().ParseJSON(line, &data);
+			g_Game->GetSimulation2()->GetScriptInterface().FreezeObject(data, true);
 			commands.emplace_back(SimulationCommand(player, cx, data));
 		}
 		else if (type == "hash" || type == "hash-quick")
@@ -199,18 +200,16 @@ void CReplayPlayer::Replay(bool serializationtest)
 
 			bool quick = (type == "hash-quick");
 
-//			if (turn >= 1300)
-//			if (turn >= 0)
 			if (turn % 100 == 0)
 			{
 				std::string hash;
-				bool ok = game.GetSimulation2()->ComputeStateHash(hash, quick);
+				bool ok = g_Game->GetSimulation2()->ComputeStateHash(hash, quick);
 				ENSURE(ok);
 				std::string hexHash = Hexify(hash);
 				if (hexHash == replayHash)
-					debug_printf("hash ok (%s)", hexHash.c_str());
+					debug_printf("hash ok (%s)\n", hexHash.c_str());
 				else
-					debug_printf("HASH MISMATCH (%s != %s)", hexHash.c_str(), replayHash.c_str());
+					debug_printf("HASH MISMATCH (%s != %s)\n", hexHash.c_str(), replayHash.c_str());
 			}
 		}
 		else if (type == "end")
@@ -221,22 +220,12 @@ void CReplayPlayer::Replay(bool serializationtest)
 				g_Profiler2.IncrementFrameNumber();
 				PROFILE2_ATTR("%d", g_Profiler2.GetFrameNumber());
 
-				game.GetSimulation2()->Update(turnLength, commands);
+				g_Game->GetSimulation2()->Update(turnLength, commands);
 				commands.clear();
 			}
 
-//			std::string hash;
-//			bool ok = game.GetSimulation2()->ComputeStateHash(hash, true);
-//			ENSURE(ok);
-//			debug_printf("%s", Hexify(hash).c_str());
-
-			debug_printf("\n");
-
 			g_Profiler.Frame();
 
-//			if (turn % 1000 == 0)
-//				JS_GC(game.GetSimulation2()->GetScriptInterface().GetContext());
-
 			if (turn % 20 == 0)
 				g_ProfileViewer.SaveToFile();
 		}
@@ -245,25 +234,28 @@ void CReplayPlayer::Replay(bool serializationtest)
 			debug_printf("Unrecognised replay token %s\n", type.c_str());
 		}
 	}
+	}
+
+	SAFE_DELETE(m_Stream);
 
 	g_Profiler2.SaveToFile();
 
 	std::string hash;
-	bool ok = game.GetSimulation2()->ComputeStateHash(hash, false);
+	bool ok = g_Game->GetSimulation2()->ComputeStateHash(hash, false);
 	ENSURE(ok);
 	debug_printf("# Final state: %s\n", Hexify(hash).c_str());
-
 	timer_DisplayClientTotals();
 
+	SAFE_DELETE(g_Game);
+
 	// Must be explicitly destructed here to avoid callbacks from the JSAPI trying to use g_Profiler2 when
 	// it's already destructed.
 	g_ScriptRuntime.reset();
-	
+
 	// Clean up
 	delete &g_TexMan;
 
 	delete &g_Profiler;
 	delete &g_ProfileViewer;
-
-	g_Game = NULL;
+	SAFE_DELETE(g_ScriptStatsTable);
 }
