@@ -1,4 +1,4 @@
-/* Copyright (C) 2014 Wildfire Games.
+/* Copyright (C) 2015 Wildfire Games.
  * This file is part of 0 A.D.
  *
  * 0 A.D. is free software: you can redistribute it and/or modify
@@ -47,6 +47,11 @@ public:
 	 * Optional hash of simulation state (for sync checking).
 	 */
 	virtual void Hash(const std::string& hash, bool quick) = 0;
+
+	/**
+	 * Remember the directory containing the commands.txt file, so that we can save additional files to it.
+	 */
+	virtual OsPath GetDirectory() const = 0;
 };
 
 /**
@@ -56,8 +61,9 @@ class CDummyReplayLogger : public IReplayLogger
 {
 public:
 	virtual void StartGame(JS::MutableHandleValue UNUSED(attribs)) { }
-	virtual void Turn(u32 UNUSED(n), u32 UNUSED(turnLength), const std::vector<SimulationCommand>& UNUSED(commands)) { }
+	virtual void Turn(u32 UNUSED(n), u32 UNUSED(turnLength), std::vector<SimulationCommand>& UNUSED(commands)) { }
 	virtual void Hash(const std::string& UNUSED(hash), bool UNUSED(quick)) { }
+	virtual OsPath GetDirectory() const { return OsPath(); }
 };
 
 /**
@@ -73,10 +79,12 @@ public:
 	virtual void StartGame(JS::MutableHandleValue attribs);
 	virtual void Turn(u32 n, u32 turnLength, std::vector<SimulationCommand>& commands);
 	virtual void Hash(const std::string& hash, bool quick);
+	virtual OsPath GetDirectory() const;
 
 private:
 	ScriptInterface& m_ScriptInterface;
 	std::ostream* m_Stream;
+	OsPath m_Directory;
 };
 
 /**
@@ -89,7 +97,7 @@ public:
 	~CReplayPlayer();
 
 	void Load(const std::string& path);
-	void Replay(bool serializationtest);
+	void Replay(bool serializationtest, bool ooslog);
 
 private:
 	std::istream* m_Stream;
