@@ -19,10 +19,10 @@
 
 #include "TemplateLoader.h"
 
+#include "lib/utf8.h"
 #include "ps/CLogger.h"
 #include "ps/Filesystem.h"
 #include "ps/XML/Xeromyces.h"
-#include "lib/utf8.h"
 
 static const wchar_t TEMPLATE_ROOT[] = L"simulation/templates/";
 static const wchar_t ACTOR_ROOT[] = L"art/actors/";
@@ -206,7 +206,7 @@ static Status AddActorToTemplates(const VfsPath& pathname, const CFileInfo& UNUS
 	return INFO::OK;
 }
 
-bool CTemplateLoader::TemplateExists(const std::string& templateName)
+bool CTemplateLoader::TemplateExists(const std::string& templateName) const
 {
 	size_t pos = templateName.rfind('|');
 	std::string baseName(pos != std::string::npos ? templateName.substr(pos+1) : templateName);
@@ -264,15 +264,14 @@ std::vector<std::string> CTemplateLoader::FindPlaceableTemplates(const std::stri
 				if (vfs::GetPathnames(g_VFS, templatePath / (directoryPath + "/"), fileFilter.c_str(), filenames) != INFO::OK)
 					continue;
 				
-				for (VfsPaths::iterator it = filenames.begin(); it != filenames.end(); ++it)
+				for (const VfsPath& filename : filenames)
 				{
-					VfsPath filename = *it;
 					// Strip the .xml extension
 					VfsPath pathstem = filename.ChangeExtension(L"");
 					// Strip the root from the path
 					std::wstring name = pathstem.string().substr(ARRAY_SIZE(TEMPLATE_ROOT) - 1);
 
-					templates.push_back(std::string(name.begin(), name.end()));
+					templates.emplace_back(name.begin(), name.end());
 				}
 				
 			}
@@ -407,7 +406,7 @@ void CTemplateLoader::CopyPreviewSubset(CParamNode& out, const CParamNode& in, b
 			CParamNode::LoadXMLString(out, "<Entity><VisualActor><DisableShadows/></VisualActor></Entity>");
 
 		// Previews should always be visible in fog-of-war/etc
-		CParamNode::LoadXMLString(out, "<Entity><Visibility><RetainInFog>false</RetainInFog><AlwaysVisible>true</AlwaysVisible><Preview>true</Preview></Visibility></Entity>");
+		CParamNode::LoadXMLString(out, "<Entity><Visibility><AlwaysVisible>true</AlwaysVisible><Preview>true</Preview></Visibility></Entity>");
 	}
 
 	if (corpse)
@@ -420,8 +419,8 @@ void CTemplateLoader::CopyPreviewSubset(CParamNode& out, const CParamNode& in, b
 		if (out.GetChild("Entity").GetChild("VisualActor").IsOk())
 			CParamNode::LoadXMLString(out, "<Entity><VisualActor><SilhouetteDisplay>false</SilhouetteDisplay></VisualActor></Entity>");
 
-		// Corpses should remain visible in fog-of-war
-		CParamNode::LoadXMLString(out, "<Entity><Visibility><RetainInFog>true</RetainInFog><AlwaysVisible>false</AlwaysVisible><Preview>false</Preview></Visibility></Entity>");
+		// Corpses should remain visible in fog-of-war (for the owner only)
+		CParamNode::LoadXMLString(out, "<Entity><Visibility><Corpse>true</Corpse></Visibility></Entity>");
 	}
 }
 
@@ -512,7 +511,12 @@ void CTemplateLoader::CopyFoundationSubset(CParamNode& out, const CParamNode& in
 	// Foundations should be visible themselves in fog-of-war if their base template is,
 	// but shouldn't have any vision range
 	if (out.GetChild("Entity").GetChild("Vision").IsOk())
+	{
 		CParamNode::LoadXMLString(out, "<Entity><Vision><Range>0</Range></Vision></Entity>");
+		// Foundations should not have special vision capabilities either
+		if (out.GetChild("Entity").GetChild("Vision").GetChild("RevealShore").IsOk())
+			CParamNode::LoadXMLString(out, "<Entity><Vision><RevealShore>false</RevealShore></Vision></Entity>");
+	}
 }
 
 void CTemplateLoader::CopyConstructionSubset(CParamNode& out, const CParamNode& in)
@@ -533,21 +537,27 @@ void CTemplateLoader::CopyResourceSubset(CParamNode& out, const CParamNode& in)
 {
 	// Currently used for animals which die and leave a gatherable corpse.
 	// Mostly serves to filter out components like Vision, UnitAI, etc.
+	// Don't emit sound as our samples only apply to living animals.
 	std::set<std::string> permittedComponentTypes;
 	permittedComponentTypes.insert("Ownership");
 	permittedComponentTypes.insert("Position");
 	permittedComponentTypes.insert("VisualActor");
 	permittedComponentTypes.insert("Identity");
-	permittedComponentTypes.insert("Obstruction");
 	permittedComponentTypes.insert("Minimap");
 	permittedComponentTypes.insert("ResourceSupply");
 	permittedComponentTypes.insert("Selectable");
 	permittedComponentTypes.insert("Footprint");
 	permittedComponentTypes.insert("StatusBars");
 	permittedComponentTypes.insert("OverlayRenderer");
-	permittedComponentTypes.insert("Sound");
 	permittedComponentTypes.insert("AIProxy");
 
 	CParamNode::LoadXMLString(out, "<Entity/>");
 	out.CopyFilteredChildrenOfChild(in, "Entity", permittedComponentTypes);
+	
+	// When dying, resources lose the unitMotion component
+	// This causes them to have no clearance. Since unit obstructions no longer have a radius,
+	// this makes them unreachable in some cases (see #3530).
+	// Instead, create a static, unblocking (see #3530 for why) static obstruction.
+	// TODO: this should probably be generalized as a parameter on entity death or something.
+	CParamNode::LoadXMLString(out, "<Entity><Obstruction><Active>true</Active><BlockMovement>false</BlockMovement><BlockPathfinding>false</BlockPathfinding><BlockFoundation>false</BlockFoundation><BlockConstruction>false</BlockConstruction><DisableBlockMovement>false</DisableBlockMovement><DisableBlockPathfinding>false</DisableBlockPathfinding><Static width=\"2.0\" depth=\"2.0\"/></Obstruction></Entity>");
 }
