@@ -1,4 +1,4 @@
-/* Copyright (C) 2014 Wildfire Games.
+/* Copyright (C) 2015 Wildfire Games.
  * This file is part of 0 A.D.
  *
  * 0 A.D. is free software: you can redistribute it and/or modify
@@ -260,7 +260,7 @@ void OverlayRenderer::PrepareForRendering()
 	PROFILE3("prepare overlays");
 
 	// This is where we should do something like sort the overlays by
-	// colour/sprite/etc for more efficient rendering
+	// color/sprite/etc for more efficient rendering
 
 	for (size_t i = 0; i < m->texlines.size(); ++i)
 	{
@@ -607,7 +607,7 @@ void OverlayRenderer::RenderForegroundOverlays(const CCamera& viewCamera)
 		shader = tech->GetShader();
 	}
 
-	float uvs[8] = { 0,0, 1,0, 1,1, 0,1 };
+	float uvs[8] = { 0,1, 1,1, 1,0, 0,0 };
 	
 	if (g_Renderer.GetRenderPath() == CRenderer::RP_SHADER)
 		shader->TexCoordPointer(GL_TEXTURE0, 2, GL_FLOAT, sizeof(float)*2, &uvs[0]);
@@ -622,6 +622,8 @@ void OverlayRenderer::RenderForegroundOverlays(const CCamera& viewCamera)
 			shader->BindTexture(str_baseTex, sprite->m_Texture);
 		else
 			sprite->m_Texture->Bind();
+		
+		shader->Uniform(str_colorMul, sprite->m_Color);
 
 		CVector3D pos[4] = {
 			sprite->m_Position + right*sprite->m_X0 + up*sprite->m_Y0,
