@@ -1,4 +1,4 @@
-/* Copyright (C) 2012 Wildfire Games.
+/* Copyright (C) 2015 Wildfire Games.
  * This file is part of 0 A.D.
  *
  * 0 A.D. is free software: you can redistribute it and/or modify
@@ -24,8 +24,8 @@
 #include "lib/ogl.h"
 #include "maths/MathUtil.h"
 #include "ps/Game.h"
-#include "ps/Overlay.h"
 #include "ps/Profile.h"
+#include "ps/Shapes.h"
 #include "ps/World.h"
 #include "renderer/Renderer.h"
 #include "renderer/TerrainRenderer.h"
@@ -33,32 +33,6 @@
 
 #include <algorithm>
 
-// Handy things for STL:
-
-/// Functor for sorting pairs, using the &lt;-ordering of their second values.
-struct compare2nd
-{
-	template<typename S, typename T> bool operator()(const std::pair<S, T>& a, const std::pair<S, T>& b) const
-	{
-		return a.second < b.second;
-	}
-};
-
-/// Functor for comparing the firsts of pairs to a specified value.
-template<typename S> struct equal1st
-{
-	const S& val;
-	equal1st(const S& val) : val(val) {}
-	template <typename T> bool operator()(const std::pair<S, T>& a) const
-	{
-		return a.first == val;
-	}
-private:
-	const equal1st& operator=(const equal1st& rhs);
-};
-
-//////////////////////////////////////////////////////////////////////////
-
 // Global overlay list management:
 
 static std::vector<std::pair<ITerrainOverlay*, int> > g_TerrainOverlayList;
@@ -66,19 +40,21 @@ static std::vector<std::pair<ITerrainOverlay*, int> > g_TerrainOverlayList;
 ITerrainOverlay::ITerrainOverlay(int priority)
 {
 	// Add to global list of overlays
-	g_TerrainOverlayList.push_back(std::make_pair(this, priority));
+	g_TerrainOverlayList.emplace_back(this, priority);
 	// Sort by overlays by priority. Do stable sort so that adding/removing
 	// overlays doesn't randomly disturb all the existing ones (which would
 	// be noticeable if they have the same priority and overlap).
 	std::stable_sort(g_TerrainOverlayList.begin(), g_TerrainOverlayList.end(),
-		compare2nd());
+		[](const std::pair<ITerrainOverlay*, int>& a, const std::pair<ITerrainOverlay*, int>& b) {
+			return a.second < b.second;
+		});
 }
 
 ITerrainOverlay::~ITerrainOverlay()
 {
 	std::vector<std::pair<ITerrainOverlay*, int> >::iterator newEnd =
 		std::remove_if(g_TerrainOverlayList.begin(), g_TerrainOverlayList.end(),
-			equal1st<ITerrainOverlay*>(this));
+			[this](const std::pair<ITerrainOverlay*, int>& a) { return a.first == this; });
 	g_TerrainOverlayList.erase(newEnd, g_TerrainOverlayList.end());
 }
 
@@ -180,12 +156,12 @@ void TerrainOverlay::RenderBeforeWater()
 #endif
 }
 
-void TerrainOverlay::RenderTile(const CColor& colour, bool draw_hidden)
+void TerrainOverlay::RenderTile(const CColor& color, bool draw_hidden)
 {
-	RenderTile(colour, draw_hidden, m_i, m_j);
+	RenderTile(color, draw_hidden, m_i, m_j);
 }
 
-void TerrainOverlay::RenderTile(const CColor& colour, bool draw_hidden, ssize_t i, ssize_t j)
+void TerrainOverlay::RenderTile(const CColor& color, bool draw_hidden, ssize_t i, ssize_t j)
 {
 	// TODO: if this is unpleasantly slow, make it much more efficient
 	// (e.g. buffering data and making a single draw call? or at least
@@ -210,7 +186,7 @@ void TerrainOverlay::RenderTile(const CColor& colour, bool draw_hidden, ssize_t
 
 	CVector3D pos;
 	glBegin(GL_TRIANGLES);
-		glColor4fv(colour.FloatArray());
+		glColor4fv(color.FloatArray());
 		if (m_Terrain->GetTriangulationDir(i, j))
 		{
 			m_Terrain->CalcPosition(i,   j,   pos); glVertex3fv(pos.GetFloatArray());
@@ -236,12 +212,12 @@ void TerrainOverlay::RenderTile(const CColor& colour, bool draw_hidden, ssize_t
 #endif
 }
 
-void TerrainOverlay::RenderTileOutline(const CColor& colour, int line_width, bool draw_hidden)
+void TerrainOverlay::RenderTileOutline(const CColor& color, int line_width, bool draw_hidden)
 {
-	RenderTileOutline(colour, line_width, draw_hidden, m_i, m_j);
+	RenderTileOutline(color, line_width, draw_hidden, m_i, m_j);
 }
 
-void TerrainOverlay::RenderTileOutline(const CColor& colour, int line_width, bool draw_hidden, ssize_t i, ssize_t j)
+void TerrainOverlay::RenderTileOutline(const CColor& color, int line_width, bool draw_hidden, ssize_t i, ssize_t j)
 {
 	if (draw_hidden)
 	{
@@ -265,7 +241,7 @@ void TerrainOverlay::RenderTileOutline(const CColor& colour, int line_width, boo
 
 	CVector3D pos;
 	glBegin(GL_QUADS);
-		glColor4fv(colour.FloatArray());
+		glColor4fv(color.FloatArray());
 		m_Terrain->CalcPosition(i,   j,   pos); glVertex3fv(pos.GetFloatArray());
 		m_Terrain->CalcPosition(i+1, j,   pos); glVertex3fv(pos.GetFloatArray());
 		m_Terrain->CalcPosition(i+1, j+1, pos); glVertex3fv(pos.GetFloatArray());
@@ -331,3 +307,41 @@ void TerrainTextureOverlay::RenderAfterWater(int cullGroup)
 
 	g_Renderer.GetTerrainRenderer().RenderTerrainOverlayTexture(cullGroup, matrix);
 }
+
+SColor4ub TerrainTextureOverlay::GetColor(size_t idx, u8 alpha) const
+{
+	static u8 colors[][3] = {
+		{ 255, 0, 0 },
+		{ 0, 255, 0 },
+		{ 0, 0, 255 },
+		{ 255, 255, 0 },
+		{ 255, 0, 255 },
+		{ 0, 255, 255 },
+		{ 255, 255, 255 },
+
+		{ 127, 0, 0 },
+		{ 0, 127, 0 },
+		{ 0, 0, 127 },
+		{ 127, 127, 0 },
+		{ 127, 0, 127 },
+		{ 0, 127, 127 },
+		{ 127, 127, 127},
+
+		{ 255, 127, 0 },
+		{ 127, 255, 0 },
+		{ 255, 0, 127 },
+		{ 127, 0, 255},
+		{ 0, 255, 127 },
+		{ 0, 127, 255},
+		{ 255, 127, 127},
+		{ 127, 255, 127},
+		{ 127, 127, 255},
+
+		{ 127, 255, 255 },
+		{ 255, 127, 255 },
+		{ 255, 255, 127 },
+	};
+
+	size_t c = idx % ARRAY_SIZE(colors);
+	return SColor4ub(colors[c][0], colors[c][1], colors[c][2], alpha);
+}
