@@ -281,7 +281,7 @@ void TerrainRenderer::RenderTerrain(int cullGroup)
 	const CLightEnv& lightEnv = g_Renderer.GetLightEnv();
 
 	pglClientActiveTextureARB(GL_TEXTURE0);
-	glEnableClientState(GL_COLOR_ARRAY); // diffuse lighting colours
+	glEnableClientState(GL_COLOR_ARRAY); // diffuse lighting colors
 
 	// The vertex color is scaled by 0.5 to permit overbrightness without clamping.
 	// We therefore need to draw clamp((texture*lighting)*2.0), where 'texture'
@@ -394,6 +394,7 @@ void TerrainRenderer::RenderTerrainOverlayTexture(int cullGroup, CMatrix3D& text
 	glEnable(GL_BLEND);
 	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
 	glDepthMask(0);
+	glDisable(GL_DEPTH_TEST);
 
 	glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
 
@@ -433,6 +434,7 @@ void TerrainRenderer::RenderTerrainOverlayTexture(int cullGroup, CMatrix3D& text
 	glMatrixMode(GL_MODELVIEW);
 
 	glDepthMask(1);
+	glEnable(GL_DEPTH_TEST);
 	glDisable(GL_BLEND);
 	glDisableClientState(GL_COLOR_ARRAY);
 	glDisableClientState(GL_VERTEX_ARRAY);
