@@ -1,4 +1,4 @@
-/* Copyright (C) 2014 Wildfire Games.
+/* Copyright (C) 2015 Wildfire Games.
  * This file is part of 0 A.D.
  *
  * 0 A.D. is free software: you can redistribute it and/or modify
@@ -144,10 +144,8 @@ WaterManager::~WaterManager()
 	// Cleanup if the caller messed up
 	UnloadWaterTextures();
 
-	// TODO: when c++11 is around, use lambdas or something because short Korea is best Korea.
-	for (size_t i = 0; i < m_ShoreWaves.size(); ++i)
+	for (WaveObject* const& obj : m_ShoreWaves)
 	{
-		WaveObject* obj = m_ShoreWaves[i];
 		if (obj->m_VBvertices)
 			g_VBMan.Release(obj->m_VBvertices);
 		delete obj;
@@ -438,10 +436,9 @@ void WaterManager::RecomputeDistanceHeightmap()
 	// This is currently upsampled by a factor of 2 to get more precision
 	// This could be refined to only be done near the coast itself, but it's probably not necessary.
 	
-	float level = SideSize;
 	for (size_t z = 0; z < SideSize; ++z)
 	{
-		level = SideSize;
+		float level = SideSize;
 		for (size_t x = 0; x < SideSize; ++x)
 			m_DistanceHeightmap[z*SideSize + x] = terrain->GetExactGroundLevel(x*2, z*2) >= m_WaterHeight ? level = 0.f : ++level;
 		level = SideSize;
@@ -459,7 +456,7 @@ void WaterManager::RecomputeDistanceHeightmap()
 	}
 	for (size_t x = 0; x < SideSize; ++x)
 	{
-		level = SideSize;
+		float level = SideSize;
 		for (size_t z = 0; z < SideSize; ++z)
 		{
 			if (terrain->GetExactGroundLevel(x*2, z*2) >= m_WaterHeight)
@@ -496,10 +493,8 @@ void WaterManager::CreateWaveMeshes()
 	size_t SideSize = m_MapSize*2;
 	CTerrain* terrain = g_Game->GetWorld()->GetTerrain();
 
-	// TODO: when c++11 is around, use lambdas or something because short Korea is best Korea.
-	for (size_t i = 0; i < m_ShoreWaves.size(); ++i)
+	for (WaveObject* const& obj : m_ShoreWaves)
 	{
-		WaveObject* obj = m_ShoreWaves[i];
 		if (obj->m_VBvertices)
 			g_VBMan.Release(obj->m_VBvertices);
 		delete obj;
@@ -519,7 +514,7 @@ void WaterManager::CreateWaveMeshes()
 	std::set<int> CoastalPointsSet;
 	for (size_t z = 1; z < SideSize-1; ++z)
 		for (size_t x = 1; x < SideSize-1; ++x)
-			if (abs(m_DistanceHeightmap[z*SideSize + x]-1.0f) < 0.2f)
+			if (fabs(m_DistanceHeightmap[z*SideSize + x]-1.0f) < 0.2f)
 				CoastalPointsSet.insert(z*SideSize + x);
 	
 	// Second step: create chains out of those coastal points.
@@ -913,11 +908,11 @@ void WaterManager::RenderWaves(const CFrustum& frustrum)
 
 	for (size_t a = 0; a < m_ShoreWaves.size(); ++a)
 	{
-		if (!frustrum.IsBoxVisible(CVector3D(0,0,0), m_ShoreWaves[a]->m_AABB))
+		if (!frustrum.IsBoxVisible(m_ShoreWaves[a]->m_AABB))
 			continue;
 		
 		CVertexBuffer::VBChunk* VBchunk = m_ShoreWaves[a]->m_VBvertices;
-		SWavesVertex *base=(SWavesVertex *)VBchunk->m_Owner->Bind();
+		SWavesVertex* base = (SWavesVertex*)VBchunk->m_Owner->Bind();
 		
 		// setup data pointers
 		GLsizei stride = sizeof(SWavesVertex);
