@@ -28,12 +28,11 @@
 namespace JSI_Renderer
 {
 	std::string GetRenderPath(ScriptInterface::CxPrivate* pCxPrivate);
-	void SetRenderPath(ScriptInterface::CxPrivate* pCxPrivate, std::string name);
+	void SetRenderPath(ScriptInterface::CxPrivate* pCxPrivate, const std::string& name);
 
 	DECLARE_BOOLEAN_SCRIPT_SETTING(Shadows);
 	DECLARE_BOOLEAN_SCRIPT_SETTING(ShadowPCF);
 	DECLARE_BOOLEAN_SCRIPT_SETTING(Particles);
-	DECLARE_BOOLEAN_SCRIPT_SETTING(GenTangents);
 	DECLARE_BOOLEAN_SCRIPT_SETTING(PreferGLSL);
 	DECLARE_BOOLEAN_SCRIPT_SETTING(WaterUgly);
 	DECLARE_BOOLEAN_SCRIPT_SETTING(WaterFancyEffects);
