@@ -1,4 +1,4 @@
-/* Copyright (C) 2014 Wildfire Games.
+/* Copyright (C) 2016 Wildfire Games.
  * This file is part of 0 A.D.
  *
  * 0 A.D. is free software: you can redistribute it and/or modify
@@ -19,7 +19,6 @@
 
 #include "ScriptInterface.h"
 #include "ScriptRuntime.h"
-// #include "DebuggingServer.h" // JS debugger temporarily disabled during the SpiderMonkey upgrade (check trac ticket #2348 for details)
 #include "ScriptStats.h"
 
 #include "lib/debug.h"
@@ -116,7 +115,8 @@ void ErrorReporter(JSContext* cx, const char* message, JSErrorReport* report)
 
 		JS::RootedValue rval(cx);
 		const char dumpStack[] = "this.stack.trimRight().replace(/^/mg, '  ')"; // indent each line
-		if (JS_EvaluateScript(cx, excnObj, dumpStack, ARRAY_SIZE(dumpStack)-1, "(eval)", 1, &rval))
+		JS::CompileOptions opts(cx);
+		if (JS::Evaluate(cx, excnObj, opts.setFileAndLine("(eval)", 1), dumpStack, ARRAY_SIZE(dumpStack)-1, &rval))
 		{
 			std::string stackTrace;
 			if (ScriptInterface::FromJSVal(cx, rval, stackTrace))
@@ -318,7 +318,7 @@ ScriptInterface_impl::ScriptInterface_impl(const char* nativeScopeName, const sh
 	JS_SetParallelIonCompilationEnabled(m_runtime->m_rt, true);
 
 	// For GC debugging:
-	// JS_SetGCZeal(m_cx, 2);
+	// JS_SetGCZeal(m_cx, 2, JS_DEFAULT_ZEAL_FREQ);
 
 	JS_SetContextPrivate(m_cx, NULL);
 
@@ -378,13 +378,6 @@ ScriptInterface_impl::~ScriptInterface_impl()
 	JS_DestroyContext(m_cx);
 }
 
-// Inside ScriptInterface.cpp directly to emphasize that it should only be used/needed internally and not from other code.
-template<> bool ScriptInterface::FromJSVal<ScriptInterface::HandleWrapper>(JSContext* UNUSED(cx), JS::HandleValue v, ScriptInterface::HandleWrapper& out)
-{
-	out.set(v);
-	return true;
-}
-
 void ScriptInterface_impl::Register(const char* name, JSNative fptr, uint nargs)
 {
 	JSAutoRequest rq(m_cx);
@@ -402,16 +395,6 @@ ScriptInterface::ScriptInterface(const char* nativeScopeName, const char* debugN
 			g_ScriptStatsTable->Add(this, debugName);
 	}
 	
-	// JS debugger temporarily disabled during the SpiderMonkey upgrade (check trac ticket #2348 for details)
-	/*
-	if (g_JSDebuggerEnabled && g_DebuggingServer != NULL)
-	{
-		if(!JS_SetDebugMode(GetContext(), true))
-			LOGERROR("Failed to set Spidermonkey to debug mode!");
-		else
-			g_DebuggingServer->RegisterScriptinterface(debugName, this);
-	} */
-
 	m_CxPrivate.pScriptInterface = this;
 	JS_SetContextPrivate(m->m_cx, (void*)&m_CxPrivate);
 }
@@ -423,11 +406,6 @@ ScriptInterface::~ScriptInterface()
 		if (g_ScriptStatsTable)
 			g_ScriptStatsTable->Remove(this);
 	}
-	
-	// Unregister from the Debugger class
-	// JS debugger temporarily disabled during the SpiderMonkey upgrade (check trac ticket #2348 for details)
-	//if (g_JSDebuggerEnabled && g_DebuggingServer != NULL)
-	//	g_DebuggingServer->UnRegisterScriptinterface(this);
 }
 
 void ScriptInterface::ShutDown()
@@ -448,7 +426,9 @@ ScriptInterface::CxPrivate* ScriptInterface::GetScriptInterfaceAndCBData(JSConte
 
 JS::Value ScriptInterface::GetCachedValue(CACHED_VAL valueIdentifier)
 {
-	return m->m_ScriptValCache[valueIdentifier].get();
+	std::map<ScriptInterface::CACHED_VAL, DefPersistentRooted<JS::Value>>::iterator it = m->m_ScriptValCache.find(valueIdentifier);
+	ENSURE(it != m->m_ScriptValCache.end());
+	return it->second.get();
 }
 
 
@@ -461,14 +441,12 @@ bool ScriptInterface::LoadGlobalScripts()
 	// Load and execute *.js in the global scripts directory
 	VfsPaths pathnames;
 	vfs::GetPathnames(g_VFS, L"globalscripts/", L"*.js", pathnames);
-	for (VfsPaths::iterator it = pathnames.begin(); it != pathnames.end(); ++it)
-	{
-		if (!LoadGlobalScriptFile(*it))
+	for (const VfsPath& path : pathnames)
+		if (!LoadGlobalScriptFile(path))
 		{
-			LOGERROR("LoadGlobalScripts: Failed to load script %s", it->string8());
+			LOGERROR("LoadGlobalScripts: Failed to load script %s", path.string8());
 			return false;
 		}
-	}
 
 	JSAutoRequest rq(m->m_cx);
 	JS::RootedValue proto(m->m_cx);
@@ -565,9 +543,9 @@ void ScriptInterface::DefineCustomObjectType(JSClass *clasp, JSNative constructo
 	m_CustomObjectTypes[typeName] = std::move(type);
 }
 
-JSObject* ScriptInterface::CreateCustomObject(const std::string & typeName)
+JSObject* ScriptInterface::CreateCustomObject(const std::string& typeName) const
 {
-	std::map < std::string, CustomType > ::iterator it = m_CustomObjectTypes.find(typeName);
+	std::map<std::string, CustomType>::const_iterator it = m_CustomObjectTypes.find(typeName);
 
 	if (it == m_CustomObjectTypes.end())
 		throw PSERROR_Scripting_TypeDoesNotExist();
@@ -665,7 +643,7 @@ bool ScriptInterface::SetProperty_(JS::HandleValue obj, const wchar_t* name, JS:
 	JS::RootedObject object(m->m_cx, &obj.toObject());
 
 	utf16string name16(name, name + wcslen(name));
-	if (! JS_DefineUCProperty(m->m_cx, object, reinterpret_cast<const jschar*>(name16.c_str()), name16.length(), value, NULL, NULL, attrs))
+	if (!JS_DefineUCProperty(m->m_cx, object, reinterpret_cast<const char16_t*>(name16.c_str()), name16.length(), value, NULL, NULL, attrs))
 		return false;
 	return true;
 }
@@ -768,19 +746,17 @@ bool ScriptInterface::EnumeratePropertyNamesWithPrefix(JS::HandleValue objVal, c
 		return true; // reached the end of the prototype chain
 	
 	JS::RootedObject obj(m->m_cx, &objVal.toObject());
-	JS::RootedObject it(m->m_cx, JS_NewPropertyIterator(m->m_cx, obj));
-	if (!it)
+	JS::AutoIdArray props(m->m_cx, JS_Enumerate(m->m_cx, obj));
+	if (!props)
 		return false;
 
-	while (true)
+	for (size_t i = 0; i < props.length(); ++i)
 	{
-		JS::RootedId idp(m->m_cx);
+		JS::RootedId id(m->m_cx, props[i]);
 		JS::RootedValue val(m->m_cx);
-		if (! JS_NextProperty(m->m_cx, it, idp.address()) || ! JS_IdToValue(m->m_cx, idp, &val))
+		if (!JS_IdToValue(m->m_cx, id, &val))
 			return false;
 
-		if (val.isUndefined())
-			break; // end of iteration
 		if (!val.isString())
 			continue; // ignore integer properties
 
@@ -793,7 +769,7 @@ bool ScriptInterface::EnumeratePropertyNamesWithPrefix(JS::HandleValue objVal, c
 		if(0 == strcmp(&buf[0], prefix))
 		{
 			size_t len;
-			const jschar* chars = JS_GetStringCharsAndLength(m->m_cx, name, &len);
+			const char16_t* chars = JS_GetStringCharsAndLength(m->m_cx, name, &len);
 			out.push_back(std::string(chars, chars+len));
 		}
 	}
@@ -843,7 +819,7 @@ bool ScriptInterface::LoadScript(const VfsPath& filename, const std::string& cod
 	uint lineNo = 1;
 	// CompileOptions does not copy the contents of the filename string pointer.
 	// Passing a temporary string there will cause undefined behaviour, so we create a separate string to avoid the temporary.
-	std::string filenameStr(utf8_from_wstring(filename.string()));
+	std::string filenameStr = filename.string8();
 
 	JS::CompileOptions options(m->m_cx);
 	options.setFileAndLine(filenameStr.c_str(), lineNo);
@@ -851,7 +827,7 @@ bool ScriptInterface::LoadScript(const VfsPath& filename, const std::string& cod
 
 	JS::RootedFunction func(m->m_cx,
 	JS_CompileUCFunction(m->m_cx, global, NULL, 0, NULL,
-			reinterpret_cast<const jschar*> (codeUtf16.c_str()), (uint)(codeUtf16.length()), options)
+			reinterpret_cast<const char16_t*>(codeUtf16.c_str()), (uint)(codeUtf16.length()), options)
 	);
 	if (!func)
 		return false;
@@ -871,11 +847,15 @@ bool ScriptInterface::LoadGlobalScript(const VfsPath& filename, const std::wstri
 	JS::RootedObject global(m->m_cx, m->m_glob);
 	utf16string codeUtf16(code.begin(), code.end());
 	uint lineNo = 1;
+	// CompileOptions does not copy the contents of the filename string pointer.
+	// Passing a temporary string there will cause undefined behaviour, so we create a separate string to avoid the temporary.
+	std::string filenameStr = filename.string8();
 
 	JS::RootedValue rval(m->m_cx);
-	return JS_EvaluateUCScript(m->m_cx, global,
-			reinterpret_cast<const jschar*> (codeUtf16.c_str()), (uint)(codeUtf16.length()),
-			utf8_from_wstring(filename.string()).c_str(), lineNo, &rval);
+	JS::CompileOptions opts(m->m_cx);
+	opts.setFileAndLine(filenameStr.c_str(), lineNo);
+	return JS::Evaluate(m->m_cx, global, opts,
+			reinterpret_cast<const char16_t*>(codeUtf16.c_str()), (uint)(codeUtf16.length()), &rval);
 }
 
 bool ScriptInterface::LoadGlobalScriptFile(const VfsPath& path)
@@ -902,11 +882,15 @@ bool ScriptInterface::LoadGlobalScriptFile(const VfsPath& path)
 
 	utf16string codeUtf16(code.begin(), code.end());
 	uint lineNo = 1;
+	// CompileOptions does not copy the contents of the filename string pointer.
+	// Passing a temporary string there will cause undefined behaviour, so we create a separate string to avoid the temporary.
+	std::string filenameStr = path.string8();
 
 	JS::RootedValue rval(m->m_cx);
-	return JS_EvaluateUCScript(m->m_cx, global,
-			reinterpret_cast<const jschar*> (codeUtf16.c_str()), (uint)(codeUtf16.length()),
-			utf8_from_wstring(path.string()).c_str(), lineNo, &rval);
+	JS::CompileOptions opts(m->m_cx);
+	opts.setFileAndLine(filenameStr.c_str(), lineNo);
+	return JS::Evaluate(m->m_cx, global, opts,
+			reinterpret_cast<const char16_t*>(codeUtf16.c_str()), (uint)(codeUtf16.length()), &rval);
 }
 
 bool ScriptInterface::Eval(const char* code)
@@ -922,7 +906,9 @@ bool ScriptInterface::Eval_(const char* code, JS::MutableHandleValue rval)
 	JS::RootedObject global(m->m_cx, m->m_glob);
 	utf16string codeUtf16(code, code+strlen(code));
 	
-	return JS_EvaluateUCScript(m->m_cx, global, (const jschar*)codeUtf16.c_str(), (uint)codeUtf16.length(), "(eval)", 1, rval);
+	JS::CompileOptions opts(m->m_cx);
+	opts.setFileAndLine("(eval)", 1);
+	return JS::Evaluate(m->m_cx, global, opts, reinterpret_cast<const char16_t*>(codeUtf16.c_str()), (uint)codeUtf16.length(), rval);
 }
 
 bool ScriptInterface::Eval_(const wchar_t* code, JS::MutableHandleValue rval)
@@ -931,7 +917,9 @@ bool ScriptInterface::Eval_(const wchar_t* code, JS::MutableHandleValue rval)
 	JS::RootedObject global(m->m_cx, m->m_glob);
 	utf16string codeUtf16(code, code+wcslen(code));
 
-	return JS_EvaluateUCScript(m->m_cx, global, (const jschar*)codeUtf16.c_str(), (uint)codeUtf16.length(), "(eval)", 1, rval);
+	JS::CompileOptions opts(m->m_cx);
+	opts.setFileAndLine("(eval)", 1);
+	return JS::Evaluate(m->m_cx, global, opts, reinterpret_cast<const char16_t*>(codeUtf16.c_str()), (uint)codeUtf16.length(), rval);
 }
 
 bool ScriptInterface::ParseJSON(const std::string& string_utf8, JS::MutableHandleValue out)
@@ -939,7 +927,7 @@ bool ScriptInterface::ParseJSON(const std::string& string_utf8, JS::MutableHandl
 	JSAutoRequest rq(m->m_cx);
 	std::wstring attrsW = wstring_from_utf8(string_utf8);
  	utf16string string(attrsW.begin(), attrsW.end());
-	if (JS_ParseJSON(m->m_cx, reinterpret_cast<const jschar*>(string.c_str()), (u32)string.size(), out))
+	if (JS_ParseJSON(m->m_cx, reinterpret_cast<const char16_t*>(string.c_str()), (u32)string.size(), out))
 		return true;
 
 	LOGERROR("JS_ParseJSON failed!");
@@ -992,7 +980,7 @@ void ScriptInterface::ReadJSONFile(const VfsPath& path, JS::MutableHandleValue o
 
 struct Stringifier
 {
-	static bool callback(const jschar* buf, u32 len, void* data)
+	static bool callback(const char16_t* buf, u32 len, void* data)
 	{
 		utf16string str(buf, buf+len);
 		std::wstring strw(str.begin(), str.end());
@@ -1091,16 +1079,6 @@ void* ScriptInterface::GetPrivate(JS::HandleObject obj)
 	return JS_GetPrivate(obj);
 }
 
-void ScriptInterface::DumpHeap()
-{
-#if MOZJS_DEBUG_ABI
-	JS_DumpHeap(GetJSRuntime(), stderr, NULL, JSTRACE_OBJECT, NULL, (size_t)-1, NULL);
-#endif
-	fprintf(stderr, "# Bytes allocated: %u\n", JS_GetGCParameter(GetJSRuntime(), JSGC_BYTES));
-	JS_GC(GetJSRuntime());
-	fprintf(stderr, "# Bytes allocated after GC: %u\n", JS_GetGCParameter(GetJSRuntime(), JSGC_BYTES));
-}
-
 void ScriptInterface::MaybeGC()
 {
 	JS_MaybeGC(m->m_cx);
