@@ -1,4 +1,4 @@
-/* Copyright (C) 2014 Wildfire Games.
+/* Copyright (C) 2016 Wildfire Games.
  * This file is part of 0 A.D.
  *
  * 0 A.D. is free software: you can redistribute it and/or modify
@@ -22,6 +22,7 @@
 
 #include "lib/file/vfs/vfs_path.h"
 
+#include "maths/Fixed.h"
 #include "ScriptTypes.h"
 #include "ScriptVal.h"
 #include "ps/Errors.h"
@@ -60,7 +61,6 @@ class ScriptRuntime;
 
 extern shared_ptr<ScriptRuntime> g_ScriptRuntime;
 
-class CDebuggingServer;
 
 /**
  * Abstraction around a SpiderMonkey JSContext.
@@ -162,7 +162,7 @@ public:
 	template<typename T0, typename T1, typename T2>
 	bool CallFunctionVoid(JS::HandleValue val, const char* name, const T0& a0, const T1& a1, const T2& a2);
 
-	JSObject* CreateCustomObject(const std::string & typeName);
+	JSObject* CreateCustomObject(const std::string & typeName) const;
 	void DefineCustomObjectType(JSClass *clasp, JSNative constructor, uint minArgs, JSPropertySpec *ps, JSFunctionSpec *fs, JSPropertySpec *static_ps, JSFunctionSpec *static_fs);
 
 	jsval GetGlobalObject();
@@ -313,11 +313,6 @@ public:
 	 */
 	template<typename T> static void ToJSVal(JSContext* cx, JS::MutableHandleValue ret, T const& val);
 
-	/**
-	 * Dump some memory heap debugging information to stderr.
-	 */
-	void DumpHeap();
-
 	/**
 	 * MaybeGC tries to determine whether garbage collection in cx's runtime would free up enough memory to be worth the amount of time it would take.
 	 * This calls JS_MaybeGC directly, which does not do incremental GC. Usually you should prefer MaybeIncrementalRuntimeGC.
@@ -381,6 +376,14 @@ public:
 	{
 		AssignOrToJSVal(cx, handle, a);
 	}
+	
+	/**
+	 * Converts |val| to T if needed or just returns it if it's a handle.
+	 * This is meant for use in other templates where we want to use the same code for JS::HandleValue and
+	 * other types.
+	 */
+	template <typename T>
+	static T AssignOrFromJSVal(JSContext* cx, const JS::HandleValue& val, bool& ret);
 
 private:
 	
@@ -426,7 +429,7 @@ private:
 	
 	// Take care to keep this declaration before heap rooted members. Destructors of heap rooted
 	// members have to be called before the runtime destructor.
-	std::auto_ptr<ScriptInterface_impl> m;
+	std::unique_ptr<ScriptInterface_impl> m;
 	
 	boost::rand48* m_rng;
 	std::map<std::string, CustomType> m_CustomObjectTypes;
@@ -482,6 +485,21 @@ inline void ScriptInterface::AssignOrToJSValUnrooted<JS::Value>(JSContext* UNUSE
 	handle.set(a);
 }
 
+template<typename T>
+inline T ScriptInterface::AssignOrFromJSVal(JSContext* cx, const JS::HandleValue& val, bool& ret)
+{
+	T retVal;
+	ret = FromJSVal(cx, val, retVal);
+	return retVal;
+}
+
+template<>
+inline JS::HandleValue ScriptInterface::AssignOrFromJSVal<JS::HandleValue>(JSContext* UNUSED(cx), const JS::HandleValue& val, bool& ret)
+{
+	ret = true;
+	return val;
+}
+
 template<typename T0>
 bool ScriptInterface::CallFunctionVoid(JS::HandleValue val, const char* name, const T0& a0)
 {
