@@ -1,4 +1,4 @@
-/* Copyright (C) 2010 Wildfire Games.
+/* Copyright (C) 2016 Wildfire Games.
  * This file is part of 0 A.D.
  *
  * 0 A.D. is free software: you can redistribute it and/or modify
@@ -20,8 +20,11 @@
 #include "scriptinterface/ScriptInterface.h"
 
 #include "maths/Fixed.h"
+#include "maths/FixedVector2D.h"
+#include "maths/FixedVector3D.h"
 #include "maths/MathUtil.h"
 #include "ps/CLogger.h"
+#include "ps/Filesystem.h"
 
 #include "jsapi.h"
 
@@ -31,6 +34,7 @@ class TestScriptConversions : public CxxTest::TestSuite
 	void convert_to(const T& value, const std::string& expected)
 	{
 		ScriptInterface script("Test", "Test", g_ScriptRuntime);
+		TS_ASSERT(script.LoadGlobalScripts());
 		JSContext* cx = script.GetContext();
 		JSAutoRequest rq(cx);
 
@@ -50,6 +54,7 @@ class TestScriptConversions : public CxxTest::TestSuite
 	void roundtrip(const T& value, const char* expected)
 	{
 		ScriptInterface script("Test", "Test", g_ScriptRuntime);
+		TS_ASSERT(script.LoadGlobalScripts());
 		JSContext* cx = script.GetContext();
 		JSAutoRequest rq(cx);
 
@@ -68,7 +73,44 @@ class TestScriptConversions : public CxxTest::TestSuite
 		TS_ASSERT_EQUALS(value, v2);
 	}
 
+	template <typename T>
+	void call_prototype_function(const T& u, const T& v, const std::string& func, const std::string& expected)
+	{
+		ScriptInterface script("Test", "Test", g_ScriptRuntime);
+		TS_ASSERT(script.LoadGlobalScripts());
+		JSContext* cx = script.GetContext();
+		JSAutoRequest rq(cx);
+
+		JS::RootedValue v1(cx);
+		ScriptInterface::ToJSVal(cx, &v1, v);
+		JS::RootedValue u1(cx);
+		ScriptInterface::ToJSVal(cx, &u1, u);
+
+		T r;
+		JS::RootedValue r1(cx);
+
+		TS_ASSERT(script.CallFunction(u1, func.c_str(), v1, r));
+		ScriptInterface::ToJSVal(cx, &r1, r);
+
+		std::string source;
+		JS::RootedValue global(cx, script.GetGlobalObject());
+		TS_ASSERT(script.CallFunction(global, "uneval", r1, source));
+
+		TS_ASSERT_STR_EQUALS(source, expected);
+	}
+
 public:
+	void setUp()
+	{
+		g_VFS = CreateVfs(20 * MiB);
+		TS_ASSERT_OK(g_VFS->Mount(L"", DataDir()/"mods"/"_test.sim", VFS_MOUNT_MUST_EXIST));
+	}
+
+	void tearDown()
+	{
+		g_VFS.reset();
+	}
+
 	void test_roundtrip()
 	{
 		roundtrip<bool>(true, "true");
@@ -101,23 +143,31 @@ public:
 
 		std::string s1 = "test";
 		s1[1] = '\0';
+		std::string s2 = "тест";
+		s2[2] = s2[3] = '\0';
 
 		std::wstring w1 = L"test";
 		w1[1] = '\0';
+		std::wstring w2 = L"тест";
+		w2[1] = '\0';
 
 		roundtrip<std::string>("", "\"\"");
 		roundtrip<std::string>("test", "\"test\"");
+		roundtrip<std::string>("тест", "\"\\xD1\\x82\\xD0\\xB5\\xD1\\x81\\xD1\\x82\"");
 		roundtrip<std::string>(s1, "\"t\\x00st\"");
-		// TODO: should test non-ASCII strings
+		roundtrip<std::string>(s2, "\"\\xD1\\x82\\x00\\x00\\xD1\\x81\\xD1\\x82\"");
 
 		roundtrip<std::wstring>(L"", "\"\"");
 		roundtrip<std::wstring>(L"test", "\"test\"");
+		// Windows has two byte wchar_t. We test for this explicitly since we can catch more possible issues this way.
+		roundtrip<std::wstring>(L"тест", sizeof(wchar_t) == 2 ? "\"\\xD1\\u201A\\xD0\\xB5\\xD1\\x81\\xD1\\u201A\"" : "\"\\u0442\\u0435\\u0441\\u0442\"");
 		roundtrip<std::wstring>(w1, "\"t\\x00st\"");
-		// TODO: should test non-ASCII strings
+		roundtrip<std::wstring>(w2, sizeof(wchar_t) == 2 ? "\"\\xD1\\x00\\xD0\\xB5\\xD1\\x81\\xD1\\u201A\"" : "\"\\u0442\\x00\\u0441\\u0442\"");
 
 		convert_to<const char*>("", "\"\"");
 		convert_to<const char*>("test", "\"test\"");
 		convert_to<const char*>(s1.c_str(), "\"t\"");
+		convert_to<const char*>(s2.c_str(), "\"\\xD1\\x82\"");
 
 		roundtrip<fixed>(fixed::FromInt(0), "0");
 		roundtrip<fixed>(fixed::FromInt(123), "123");
@@ -171,12 +221,10 @@ public:
 		TS_ASSERT(isnan(f));
 	}
 
-	// TODO: test vectors
+	// NOTE: fixed and vector conversions are defined in simulation2/scripting/EngineScriptConversions.cpp
 
 	void test_fixed()
 	{
-		// NOTE: fixed conversions are defined in simulation2/scripting/EngineScriptConversions.cpp
-
 		fixed f;
 
 		f.SetInternalValue(10590283);
@@ -191,4 +239,22 @@ public:
 		f.SetInternalValue(2000000001);
 		roundtrip<fixed>(f, "30517.57814025879");
 	}
+
+	void test_vector2d()
+	{
+		CFixedVector2D v(fixed::Zero(), fixed::Pi());
+		roundtrip<CFixedVector2D>(v, "({x:0, y:3.1415863037109375})");
+
+		CFixedVector2D u(fixed::FromInt(1), fixed::Zero());
+		call_prototype_function<CFixedVector2D>(u, v, "add", "({x:1, y:3.1415863037109375})");
+	}
+
+	void test_vector3d()
+	{
+		CFixedVector3D v(fixed::Zero(), fixed::Pi(), fixed::FromInt(1));
+		roundtrip<CFixedVector3D>(v, "({x:0, y:3.1415863037109375, z:1})");
+
+		CFixedVector3D u(fixed::Pi(), fixed::Zero(), fixed::FromInt(2));
+		call_prototype_function<CFixedVector3D>(u, v, "add", "({x:3.1415863037109375, y:3.1415863037109375, z:3})");
+	}
 };
