@@ -1,4 +1,4 @@
-/* Copyright (C) 2014 Wildfire Games.
+/* Copyright (C) 2015 Wildfire Games.
  * This file is part of 0 A.D.
  *
  * 0 A.D. is free software: you can redistribute it and/or modify
@@ -29,6 +29,8 @@
 
 #include "maths/Vector3D.h"
 
+#include "ps/CStr.h"
+
 #define DEFAULT_MESSAGE_IMPL(name) \
 	virtual int GetType() const { return MT_##name; } \
 	virtual const char* GetScriptHandlerName() const { return "On" #name; } \
@@ -458,13 +460,13 @@ class CMessagePathResult : public CMessage
 public:
 	DEFAULT_MESSAGE_IMPL(PathResult)
 
-	CMessagePathResult(u32 ticket, const ICmpPathfinder::Path& path) :
+	CMessagePathResult(u32 ticket, const WaypointPath& path) :
 		ticket(ticket), path(path)
 	{
 	}
 
 	u32 ticket;
-	ICmpPathfinder::Path path;
+	WaypointPath path;
 };
 
 /**
@@ -538,4 +540,27 @@ public:
 	}
 };
 
+/**
+* Cinematics events
+*/
+
+class CMessageCinemaPathEnded : public CMessage
+{
+public:
+	DEFAULT_MESSAGE_IMPL(CinemaPathEnded)
+
+	CMessageCinemaPathEnded(CStrW name) :
+	name(name)
+	{
+	}
+
+	CStrW name;
+};
+
+class CMessageCinemaQueueEnded : public CMessage
+{
+public:
+	DEFAULT_MESSAGE_IMPL(CinemaQueueEnded)
+};
+
 #endif // INCLUDED_MESSAGETYPES
