@@ -1,4 +1,4 @@
-/* Copyright (C) 2014 Wildfire Games.
+/* Copyright (C) 2016 Wildfire Games.
  * This file is part of 0 A.D.
  *
  * 0 A.D. is free software: you can redistribute it and/or modify
@@ -41,15 +41,11 @@
 #include "ps/Loader.h"
 #include "ps/Profile.h"
 #include "ps/Pyrogenesis.h"
+#include "ps/Util.h"
 #include "ps/XML/Xeromyces.h"
 
 #include <iomanip>
 
-#if MSC_VERSION
-#include <process.h>
-#define getpid _getpid // use the non-deprecated function name
-#endif
-
 static std::string Hexify(const std::string& s) // TODO: shouldn't duplicate this function in so many places
 {
 	std::stringstream str;
@@ -79,6 +75,11 @@ public:
 			CFG_GET_VAL("ooslog", m_EnableOOSLog);
 			CFG_GET_VAL("serializationtest", m_EnableSerializationTest);
 		}
+
+		m_OOSLogPath = getDateIndexSubdirectory(psLogDir() / "oos_logs");
+
+		if (m_EnableOOSLog)
+			debug_printf("Writing ooslogs to %s\n", m_OOSLogPath.string8().c_str());
 	}
 
 	~CSimulation2Impl()
@@ -103,7 +104,7 @@ public:
 
 	static bool LoadDefaultScripts(CComponentManager& componentManager, std::set<VfsPath>* loadedScripts);
 	static bool LoadScripts(CComponentManager& componentManager, std::set<VfsPath>* loadedScripts, const VfsPath& path);
-	static bool LoadTriggerScripts(CComponentManager& componentManager, JS::HandleValue mapSettings);
+	static bool LoadTriggerScripts(CComponentManager& componentManager, JS::HandleValue mapSettings, std::set<VfsPath>* loadedScripts);
 	Status ReloadChangedFile(const VfsPath& path);
 
 	static Status ReloadChangedFileCB(void* param, const VfsPath& path)
@@ -132,7 +133,7 @@ public:
 	uint32_t m_TurnNumber;
 
 	bool m_EnableOOSLog;
-
+	OsPath m_OOSLogPath;
 
 	// Functions and data for the serialization test mode: (see Update() for relevant comments)
 
@@ -156,15 +157,16 @@ public:
 	{
 		JSContext* cxOld = oldScript.GetContext();
 		JSAutoRequest rqOld(cxOld);
-		
+
 		std::vector<SimulationCommand> newCommands;
 		newCommands.reserve(commands.size());
-		for (size_t i = 0; i < commands.size(); ++i)
+		for (const SimulationCommand& command : commands)
 		{
 			JSContext* cxNew = newScript.GetContext();
 			JSAutoRequest rqNew(cxNew);
-			JS::RootedValue tmpCommand(cxNew, newScript.CloneValueFromOtherContext(oldScript, commands[i].data));
-			SimulationCommand cmd(commands[i].player, cxNew, tmpCommand);
+			JS::RootedValue tmpCommand(cxNew, newScript.CloneValueFromOtherContext(oldScript, command.data));
+			newScript.FreezeObject(tmpCommand, true);
+			SimulationCommand cmd(command.player, cxNew, tmpCommand);
 			newCommands.emplace_back(std::move(cmd));
 		}
 		return newCommands;
@@ -187,28 +189,33 @@ bool CSimulation2Impl::LoadScripts(CComponentManager& componentManager, std::set
 		return false;
 
 	bool ok = true;
-	for (VfsPaths::iterator it = pathnames.begin(); it != pathnames.end(); ++it)
+	for (const VfsPath& path : pathnames)
 	{
-		VfsPath filename = *it;
 		if (loadedScripts)
-			loadedScripts->insert(filename);
-		LOGMESSAGE("Loading simulation script '%s'", filename.string8());
-		if (!componentManager.LoadScript(filename))
+			loadedScripts->insert(path);
+		LOGMESSAGE("Loading simulation script '%s'", path.string8());
+		if (!componentManager.LoadScript(path))
 			ok = false;
 	}
 	return ok;
 }
 
-bool CSimulation2Impl::LoadTriggerScripts(CComponentManager& componentManager, JS::HandleValue mapSettings)
+bool CSimulation2Impl::LoadTriggerScripts(CComponentManager& componentManager, JS::HandleValue mapSettings, std::set<VfsPath>* loadedScripts)
 {
 	bool ok = true;
 	if (componentManager.GetScriptInterface().HasProperty(mapSettings, "TriggerScripts"))
 	{
 		std::vector<std::string> scriptNames;
 		componentManager.GetScriptInterface().GetProperty(mapSettings, "TriggerScripts", scriptNames);
-		for (u32 i = 0; i < scriptNames.size(); ++i)
+		for (const std::string& triggerScript : scriptNames)
 		{
-			std::string scriptName = "maps/" + scriptNames[i];
+			std::string scriptName = "maps/" + triggerScript;
+			if (loadedScripts)
+			{
+				if (loadedScripts->find(scriptName) != loadedScripts->end())
+					continue;
+				loadedScripts->insert(scriptName);
+			}
 			LOGMESSAGE("Loading trigger script '%s'", scriptName.c_str());
 			if (!componentManager.LoadScript(scriptName.data()))
 				ok = false;
@@ -289,8 +296,8 @@ void CSimulation2Impl::ReportSerializationFailure(
 	SerializationTestState* primaryStateBefore, SerializationTestState* primaryStateAfter,
 	SerializationTestState* secondaryStateBefore, SerializationTestState* secondaryStateAfter)
 {
-	OsPath path = psLogDir() / "oos_log";
-	CreateDirectories(path, 0700);
+	const OsPath path = getDateIndexSubdirectory(psLogDir() / "serializationtest");
+	debug_printf("Writing serializationtest-data to %s\n", path.string8().c_str());
 
 	// Clean up obsolete files from previous runs
 	wunlink(path / "hash.before.a");
@@ -361,24 +368,25 @@ void CSimulation2Impl::Update(int turnLength, const std::vector<SimulationComman
 		secondaryContext.m_Terrain = &secondaryTerrain;
 		CComponentManager secondaryComponentManager(secondaryContext, scriptInterface.GetRuntime());
 		secondaryComponentManager.LoadComponentTypes();
-		ENSURE(LoadDefaultScripts(secondaryComponentManager, NULL));
+		std::set<VfsPath> secondaryLoadedScripts;
+		ENSURE(LoadDefaultScripts(secondaryComponentManager, &secondaryLoadedScripts));
 		ResetComponentState(secondaryComponentManager, false, false);
 
 		// Load the trigger scripts after we have loaded the simulation.
 		{
 			JSContext* cx2 = secondaryComponentManager.GetScriptInterface().GetContext();
 			JSAutoRequest rq2(cx2);
-			JS::RootedValue mapSettingsCloned(cx2, 
+			JS::RootedValue mapSettingsCloned(cx2,
 				secondaryComponentManager.GetScriptInterface().CloneValueFromOtherContext(
 					scriptInterface, m_MapSettings));
-			ENSURE(LoadTriggerScripts(secondaryComponentManager, mapSettingsCloned));
+			ENSURE(LoadTriggerScripts(secondaryComponentManager, mapSettingsCloned, &secondaryLoadedScripts));
 		}
 
 		// Load the map into the secondary simulation
 
 		LDR_BeginRegistering();
 		CMapReader* mapReader = new CMapReader; // automatically deletes itself
-			
+
 		std::string mapType;
 		scriptInterface.GetProperty(m_InitAttributes, "mapType", mapType);
 		if (mapType == "random")
@@ -438,9 +446,6 @@ void CSimulation2Impl::Update(int turnLength, const std::vector<SimulationComman
 		}
 	}
 
-//	if (m_TurnNumber == 0)
-//		m_ComponentManager.GetScriptInterface().DumpHeap();
-
 	// Run the GC occasionally
 	// No delay because a lot of garbage accumulates in one turn and in non-visual replays there are
 	// much more turns in the same time than in normal games.
@@ -479,7 +484,10 @@ void CSimulation2Impl::UpdateComponents(CSimContext& simContext, fixed turnLengt
 
 	CmpPtr<ICmpPathfinder> cmpPathfinder(simContext, SYSTEM_ENTITY);
 	if (cmpPathfinder)
+	{
+		cmpPathfinder->UpdateGrid();
 		cmpPathfinder->FinishAsyncRequests();
+	}
 
 	// Push AI commands onto the queue before we use them
 	CmpPtr<ICmpAIManager> cmpAIManager(simContext, SYSTEM_ENTITY);
@@ -542,12 +550,9 @@ void CSimulation2Impl::DumpState()
 {
 	PROFILE("DumpState");
 
-	std::stringstream pid;
-	pid << getpid();
 	std::stringstream name;\
 	name << std::setw(5) << std::setfill('0') << m_TurnNumber << ".txt";
-	OsPath path = psLogDir() / "sim_log" / pid.str() / name.str();
-	CreateDirectories(path.Parent(), 0700);
+	const OsPath path = m_OOSLogPath / name.str();
 	std::ofstream file (OsString(path).c_str(), std::ofstream::out | std::ofstream::trunc);
 
 	file << "State hash: " << std::hex;
@@ -581,7 +586,11 @@ CSimulation2::~CSimulation2()
 
 void CSimulation2::EnableOOSLog()
 {
+	if (m->m_EnableOOSLog)
+		return;
+
 	m->m_EnableOOSLog = true;
+	debug_printf("Writing ooslogs to %s\n", m->m_OOSLogPath.string8().c_str());
 }
 
 void CSimulation2::EnableSerializationTest()
@@ -589,6 +598,11 @@ void CSimulation2::EnableSerializationTest()
 	m->m_EnableSerializationTest = true;
 }
 
+void CSimulation2::SetCurrentDisplayedPlayer(int playerID)
+{
+	m->m_SimContext.SetCurrentDisplayedPlayer(playerID);
+}
+
 entity_id_t CSimulation2::AddEntity(const std::wstring& templateName)
 {
 	return m->m_ComponentManager.AddEntity(templateName, m->m_ComponentManager.AllocateNewEntity());
@@ -639,7 +653,7 @@ const CSimulation2::InterfaceListUnordered& CSimulation2::GetEntitiesWithInterfa
 	return m->m_ComponentManager.GetEntitiesWithInterfaceUnordered(iid);
 }
 
-const CSimContext& CSimulation2::GetSimContext() const
+CSimContext& CSimulation2::GetSimContext() const
 {
 	return m->m_SimContext;
 }
@@ -756,9 +770,9 @@ void CSimulation2::LoadMapSettings()
 {
 	JSContext* cx = GetScriptInterface().GetContext();
 	JSAutoRequest rq(cx);
-	
+
 	JS::RootedValue global(cx, GetScriptInterface().GetGlobalObject());
-	
+
 	// Initialize here instead of in Update()
 	GetScriptInterface().CallFunctionVoid(global, "LoadMapSettings", m->m_MapSettings);
 
@@ -766,7 +780,7 @@ void CSimulation2::LoadMapSettings()
 		GetScriptInterface().LoadScript(L"map startup script", m->m_StartupScript);
 
 	// Load the trigger scripts after we have loaded the simulation and the map.
-	m->LoadTriggerScripts(m->m_ComponentManager, m->m_MapSettings);
+	m->LoadTriggerScripts(m->m_ComponentManager, m->m_MapSettings, &m->m_LoadedScripts);
 }
 
 int CSimulation2::ProgressiveLoad()
@@ -823,14 +837,14 @@ static std::vector<std::string> GetJSONData(const VfsPath& path)
 	}
 
 	std::vector<std::string> data;
-	for (VfsPaths::const_iterator it = pathnames.begin(); it != pathnames.end(); ++it)
+	for (const VfsPath& p : pathnames)
 	{
 		// Load JSON file
 		CVFSFile file;
-		PSRETURN ret = file.Load(g_VFS, *it);
+		PSRETURN ret = file.Load(g_VFS, p);
 		if (ret != PSRETURN_OK)
 		{
-			LOGERROR("GetJSONData: Failed to load file '%s': %s", path.string8(), GetErrorString(ret));
+			LOGERROR("GetJSONData: Failed to load file '%s': %s", p.string8(), GetErrorString(ret));
 			continue;
 		}
 
@@ -847,7 +861,7 @@ std::vector<std::string> CSimulation2::GetRMSData()
 
 std::vector<std::string> CSimulation2::GetCivData()
 {
-	return GetJSONData(L"civs/");
+	return GetJSONData(L"simulation/data/civs/");
 }
 
 static std::string ReadJSON(const VfsPath& path)
@@ -872,12 +886,12 @@ static std::string ReadJSON(const VfsPath& path)
 
 std::string CSimulation2::GetPlayerDefaults()
 {
-	return ReadJSON(L"simulation/data/player_defaults.json");
+	return ReadJSON(L"simulation/data/settings/player_defaults.json");
 }
 
 std::string CSimulation2::GetMapSizes()
 {
-	return ReadJSON(L"simulation/data/map_sizes.json");
+	return ReadJSON(L"simulation/data/settings/map_sizes.json");
 }
 
 std::string CSimulation2::GetAIData()
@@ -886,11 +900,11 @@ std::string CSimulation2::GetAIData()
 	JSContext* cx = scriptInterface.GetContext();
 	JSAutoRequest rq(cx);
 	JS::RootedValue aiData(cx, ICmpAIManager::GetAIs(scriptInterface));
-	
+
 	// Build single JSON string with array of AI data
 	JS::RootedValue ais(cx);
 	if (!scriptInterface.Eval("({})", &ais) || !scriptInterface.SetProperty(ais, "AIData", aiData))
 		return std::string();
-	
+
 	return scriptInterface.StringifyJSON(&ais);
 }
