@@ -1,4 +1,4 @@
-/* Copyright (C) 2014 Wildfire Games.
+/* Copyright (C) 2016 Wildfire Games.
  * This file is part of 0 A.D.
  *
  * 0 A.D. is free software: you can redistribute it and/or modify
@@ -36,7 +36,7 @@
 #include "simulation2/components/ICmpTemplateManager.h"
 #include "simulation2/components/ICmpTechnologyTemplateManager.h"
 #include "simulation2/components/ICmpTerritoryManager.h"
-#include "simulation2/helpers/Grid.h"
+#include "simulation2/helpers/LongPathfinder.h"
 #include "simulation2/serialization/DebugSerializer.h"
 #include "simulation2/serialization/StdDeserializer.h"
 #include "simulation2/serialization/StdSerializer.h"
@@ -77,7 +77,7 @@ private:
 	{
 		NONCOPYABLE(CAIPlayer);
 	public:
-		CAIPlayer(CAIWorker& worker, const std::wstring& aiName, player_id_t player, uint8_t difficulty,
+		CAIPlayer(CAIWorker& worker, const std::wstring& aiName, player_id_t player, u8 difficulty,
 				shared_ptr<ScriptInterface> scriptInterface) :
 			m_Worker(worker), m_AIName(aiName), m_Player(player), m_Difficulty(difficulty), 
 			m_ScriptInterface(scriptInterface), m_Obj(scriptInterface->GetJSRuntime())
@@ -89,7 +89,7 @@ private:
 			// LoadScripts will only load each script once even though we call it for each player
 			if (!m_Worker.LoadScripts(m_AIName))
 				return false;
-				
+
 			JSContext* cx = m_ScriptInterface->GetContext();
 			JSAutoRequest rq(cx);
 
@@ -179,13 +179,13 @@ private:
 		CAIWorker& m_Worker;
 		std::wstring m_AIName;
 		player_id_t m_Player;
-		uint8_t m_Difficulty;
+		u8 m_Difficulty;
 		bool m_UseSharedComponent;
-		
+
 		// Take care to keep this declaration before heap rooted members. Destructors of heap rooted
 		// members have to be called before the runtime destructor.
 		shared_ptr<ScriptInterface> m_ScriptInterface;
-		
+
 		JS::PersistentRootedValue m_Obj;
 		std::vector<shared_ptr<ScriptInterface::StructuredClone> > m_Commands;
 	};
@@ -221,9 +221,11 @@ public:
 
 		m_ScriptInterface->RegisterFunction<void, int, JS::HandleValue, CAIWorker::PostCommand>("PostCommand");
 		m_ScriptInterface->RegisterFunction<void, std::wstring, CAIWorker::IncludeModule>("IncludeModule");
-		m_ScriptInterface->RegisterFunction<void, CAIWorker::DumpHeap>("DumpHeap");
 		m_ScriptInterface->RegisterFunction<void, CAIWorker::ForceGC>("ForceGC");
-		
+
+		m_ScriptInterface->RegisterFunction<JS::Value, JS::HandleValue, JS::HandleValue, pass_class_t, CAIWorker::ComputePath>("ComputePath");
+		m_ScriptInterface->RegisterFunction<JS::Value, pass_class_t, CAIWorker::GetConnectivityGrid>("GetConnectivityGrid");
+
 		m_ScriptInterface->RegisterFunction<void, std::wstring, std::vector<u32>, u32, u32, u32, CAIWorker::DumpImage>("DumpImage");
 	}
 
@@ -231,7 +233,7 @@ public:
 	{
 		JS_RemoveExtraGCRootsTracer(m_ScriptInterface->GetJSRuntime(), Trace, this);
 	}
-	
+
 	bool LoadScripts(const std::wstring& moduleName)
 	{
 		// Ignore modules that are already loaded
@@ -249,19 +251,19 @@ public:
 			return false;
 		}
 
-		for (VfsPaths::iterator it = pathnames.begin(); it != pathnames.end(); ++it)
+		for (const VfsPath& path : pathnames)
 		{
-			if (!m_ScriptInterface->LoadGlobalScriptFile(*it))
+			if (!m_ScriptInterface->LoadGlobalScriptFile(path))
 			{
-				LOGERROR("Failed to load script %s", it->string8());
+				LOGERROR("Failed to load script %s", path.string8());
 				return false;
 			}
 		}
 
 		return true;
 	}
-	
-	static void IncludeModule(ScriptInterface::CxPrivate* pCxPrivate, std::wstring name)
+
+	static void IncludeModule(ScriptInterface::CxPrivate* pCxPrivate, const std::wstring& name)
 	{
 		ENSURE(pCxPrivate->pCBData);
 		CAIWorker* self = static_cast<CAIWorker*> (pCxPrivate->pCBData);
@@ -274,7 +276,7 @@ public:
 		CAIWorker* self = static_cast<CAIWorker*> (pCxPrivate->pCBData);
 		self->PostCommand(playerid, cmd);
 	}
-	
+
 	void PostCommand(int playerid, JS::HandleValue cmd)
 	{
 		for (size_t i=0; i<m_Players.size(); i++)
@@ -285,43 +287,80 @@ public:
 				return;
 			}
 		}
-		
+
 		LOGERROR("Invalid playerid in PostCommand!");	
 	}
-	// The next two ought to be implmeneted someday but for now as it returns "null" it can't
-	static void DumpHeap(ScriptInterface::CxPrivate* pCxPrivate)
+
+	static JS::Value ComputePath(ScriptInterface::CxPrivate* pCxPrivate,
+		JS::HandleValue position, JS::HandleValue goal, pass_class_t passClass)
 	{
-		pCxPrivate->pScriptInterface->DumpHeap();
+		ENSURE(pCxPrivate->pCBData);
+		CAIWorker* self = static_cast<CAIWorker*> (pCxPrivate->pCBData);
+		JSContext* cx(self->m_ScriptInterface->GetContext());
+
+		CFixedVector2D pos, goalPos;
+		std::vector<CFixedVector2D> waypoints;
+		JS::RootedValue retVal(cx);
+
+		self->m_ScriptInterface->FromJSVal<CFixedVector2D>(cx, position, pos);
+		self->m_ScriptInterface->FromJSVal<CFixedVector2D>(cx, goal, goalPos);
+
+		self->ComputePath(pos, goalPos, passClass, waypoints);
+		self->m_ScriptInterface->ToJSVal<std::vector<CFixedVector2D> >(cx, &retVal, waypoints);
+
+		return retVal;
 	}
+
+	void ComputePath(const CFixedVector2D& pos, const CFixedVector2D& goal, pass_class_t passClass, std::vector<CFixedVector2D>& waypoints)
+	{
+		WaypointPath ret;
+		PathGoal pathGoal = { PathGoal::POINT, goal.X, goal.Y };
+		m_LongPathfinder.ComputePath(pos.X, pos.Y, pathGoal, passClass, ret);
+
+		for (Waypoint& wp : ret.m_Waypoints)
+			waypoints.emplace_back(wp.x, wp.z);
+	}
+
+	static JS::Value GetConnectivityGrid(ScriptInterface::CxPrivate* pCxPrivate, pass_class_t passClass)
+	{
+		ENSURE(pCxPrivate->pCBData);
+		CAIWorker* self = static_cast<CAIWorker*> (pCxPrivate->pCBData);
+		JSContext* cx(self->m_ScriptInterface->GetContext());
+
+		JS::RootedValue retVal(cx);
+		self->m_ScriptInterface->ToJSVal<Grid<u16> >(cx, &retVal, self->m_LongPathfinder.GetConnectivityGrid(passClass));
+		return retVal;
+	}
+
 	static void ForceGC(ScriptInterface::CxPrivate* pCxPrivate)
 	{
 		PROFILE3("AI compute GC");
 		JS_GC(pCxPrivate->pScriptInterface->GetJSRuntime());
 	}
-	
+
 	/**
 	 * Debug function for AI scripts to dump 2D array data (e.g. terrain tile weights).
 	 */
-	static void DumpImage(ScriptInterface::CxPrivate* UNUSED(pCxPrivate), std::wstring name, std::vector<u32> data, u32 w, u32 h, u32 max)
+	static void DumpImage(ScriptInterface::CxPrivate* UNUSED(pCxPrivate), const std::wstring& name, const std::vector<u32>& data, u32 w, u32 h, u32 max)
 	{
 		// TODO: this is totally not threadsafe.
 		VfsPath filename = L"screenshots/aidump/" + name;
-		
+
 		if (data.size() != w*h)
 		{
 			debug_warn(L"DumpImage: data size doesn't match w*h");
 			return;
 		}
-		
+
 		if (max == 0)
 		{
 			debug_warn(L"DumpImage: max must not be 0");
 			return;
 		}
-		
+
 		const size_t bpp = 8;
 		int flags = TEX_BOTTOM_UP|TEX_GREY;
-		
+
 		const size_t img_size = w * h * bpp/8;
 		const size_t hdr_size = tex_hdr_size(filename);
 		shared_ptr<u8> buf;
@@ -329,15 +368,15 @@ public:
 		Tex t;
 		if (t.wrap(w, h, bpp, flags, buf, hdr_size) < 0)
 			return;
-		
+
 		u8* img = buf.get() + hdr_size;
 		for (size_t i = 0; i < data.size(); ++i)
 			img[i] = (u8)((data[i] * 255) / max);
-		
+
 		tex_write(&t, filename);
 	}
 
-	void SetRNGSeed(uint32_t seed)
+	void SetRNGSeed(u32 seed)
 	{
 		m_RNG.seed(seed);
 	}
@@ -350,10 +389,10 @@ public:
 		// we don't need to load it.
 		if (!m_HasSharedComponent)
 			return false;
-		
+
 		// reset the value so it can be used to determine if we actually initialized it.
 		m_HasSharedComponent = false;
-		
+
 		if (LoadScripts(L"common-api"))
 			m_HasSharedComponent = true;
 		else
@@ -361,7 +400,7 @@ public:
 
 		// mainly here for the error messages
 		OsPath path = L"simulation/ai/common-api/";
-		
+
 		// Constructor name is SharedScript, it's in the module API3
 		// TODO: Hardcoding this is bad, we need a smarter way. 
 		JS::RootedValue AIModule(cx);
@@ -372,31 +411,31 @@ public:
 			LOGERROR("Failed to create shared AI component: %s: can't find module '%s'", path.string8(), "API3");
 			return false;
 		}
-		
+
 		if (!m_ScriptInterface->GetProperty(AIModule, "SharedScript", &ctor)
 		    || ctor.isUndefined())
 		{
 			LOGERROR("Failed to create shared AI component: %s: can't find constructor '%s'", path.string8(), "SharedScript");
 			return false;
 		}
-		
+
 		// Set up the data to pass as the constructor argument
 		JS::RootedValue settings(cx);
 		m_ScriptInterface->Eval(L"({})", &settings);
 		JS::RootedValue playersID(cx);
 		m_ScriptInterface->Eval(L"({})", &playersID);
-		
+
 		for (size_t i = 0; i < m_Players.size(); ++i)
 		{
 			JS::RootedValue val(cx);
 			m_ScriptInterface->ToJSVal(cx, &val, m_Players[i]->m_Player);
 			m_ScriptInterface->SetPropertyInt(playersID, i, val, true);
 		}
-		
+
 		m_ScriptInterface->SetProperty(settings, "players", playersID);
 		ENSURE(m_HasLoadedEntityTemplates);
 		m_ScriptInterface->SetProperty(settings, "templates", m_EntityTemplates, false);
-		
+
 		if (hasTechs)
 		{
 			m_ScriptInterface->SetProperty(settings, "techTemplates", m_TechTemplates, false);
@@ -408,26 +447,26 @@ public:
 			m_ScriptInterface->Eval("({})", &fakeTech);
 			m_ScriptInterface->SetProperty(settings, "techTemplates", fakeTech, false);
 		}
-		
+
 		JS::AutoValueVector argv(cx);
 		argv.append(settings);
 		m_ScriptInterface->CallConstructor(ctor, argv, &m_SharedAIObj);
-		
+
 		if (m_SharedAIObj.get().isNull())
 		{
 			LOGERROR("Failed to create shared AI component: %s: error calling constructor '%s'", path.string8(), "SharedScript");
 			return false;
 		}
-		
+
 		return true;
 	}
 
-	bool AddPlayer(const std::wstring& aiName, player_id_t player, uint8_t difficulty)
+	bool AddPlayer(const std::wstring& aiName, player_id_t player, u8 difficulty)
 	{
 		shared_ptr<CAIPlayer> ai(new CAIPlayer(*this, aiName, player, difficulty, m_ScriptInterface));
 		if (!ai->Initialise())
 			return false;
-		
+
 		// this will be set to true if we need to load the shared Component.
 		if (!m_HasSharedComponent)
 			m_HasSharedComponent = ai->m_UseSharedComponent;
@@ -437,44 +476,58 @@ public:
 		return true;
 	}
 
-	bool RunGamestateInit(const shared_ptr<ScriptInterface::StructuredClone>& gameState, const Grid<u16>& passabilityMap, const Grid<u8>& territoryMap)
+	bool RunGamestateInit(const shared_ptr<ScriptInterface::StructuredClone>& gameState, const Grid<NavcellData>& passabilityMap, const Grid<u8>& territoryMap, 
+		const std::map<std::string, pass_class_t>& nonPathfindingPassClassMasks, const std::map<std::string, pass_class_t>& pathfindingPassClassMasks)
 	{
 		// this will be run last by InitGame.Js, passing the full game representation.
 		// For now it will run for the shared Component.
 		// This is NOT run during deserialization.
 		JSContext* cx = m_ScriptInterface->GetContext();
 		JSAutoRequest rq(cx);
-		
+
 		JS::RootedValue state(cx);
 		m_ScriptInterface->ReadStructuredClone(gameState, &state);
 		ScriptInterface::ToJSVal(cx, &m_PassabilityMapVal, passabilityMap);
 		ScriptInterface::ToJSVal(cx, &m_TerritoryMapVal, territoryMap);
-		
+
+		m_PassabilityMap = passabilityMap;
+		m_NonPathfindingPassClasses = nonPathfindingPassClassMasks;
+		m_PathfindingPassClasses = pathfindingPassClassMasks;
+
+		m_LongPathfinder.Reload(&m_PassabilityMap, nonPathfindingPassClassMasks, pathfindingPassClassMasks);
+
 		if (m_HasSharedComponent)
 		{
 			m_ScriptInterface->SetProperty(state, "passabilityMap", m_PassabilityMapVal, true);
 			m_ScriptInterface->SetProperty(state, "territoryMap", m_TerritoryMapVal, true);
 			m_ScriptInterface->CallFunctionVoid(m_SharedAIObj, "init", state);
-			
+
 			for (size_t i = 0; i < m_Players.size(); ++i)
 			{
 				if (m_HasSharedComponent && m_Players[i]->m_UseSharedComponent)
 					m_Players[i]->InitAI(state, m_SharedAIObj);
 			}
 		}
-		
+
 		return true;
 	}
-	void StartComputation(const shared_ptr<ScriptInterface::StructuredClone>& gameState, const Grid<u16>& passabilityMap, const Grid<u8>& territoryMap, bool territoryMapDirty)
+	void StartComputation(const shared_ptr<ScriptInterface::StructuredClone>& gameState, 
+		const Grid<NavcellData>& passabilityMap, const GridUpdateInformation& dirtinessInformations,
+		const Grid<u8>& territoryMap, bool territoryMapDirty, 
+		const std::map<std::string, pass_class_t>& nonPathfindingPassClassMasks, const std::map<std::string, pass_class_t>& pathfindingPassClassMasks)
 	{
 		ENSURE(m_CommandsComputed);
 
 		m_GameState = gameState;
 		JSContext* cx = m_ScriptInterface->GetContext();
 
-		if (passabilityMap.m_DirtyID != m_PassabilityMap.m_DirtyID)
+		if (dirtinessInformations.dirty)
 		{
 			m_PassabilityMap = passabilityMap;
+			if (dirtinessInformations.globallyDirty)
+				m_LongPathfinder.Reload(&m_PassabilityMap, nonPathfindingPassClassMasks, pathfindingPassClassMasks);
+			else
+				m_LongPathfinder.Update(&m_PassabilityMap, dirtinessInformations.dirtinessGrid);
 			ScriptInterface::ToJSVal(cx, &m_PassabilityMapVal, m_PassabilityMap);
 		}
 
@@ -498,8 +551,8 @@ public:
 
 	void GetCommands(std::vector<SCommandSets>& commands)
 	{
- 		WaitToFinishComputation();
-		
+		WaitToFinishComputation();
+
 		commands.clear();
 		commands.resize(m_Players.size());
 		for (size_t i = 0; i < m_Players.size(); ++i)
@@ -513,7 +566,7 @@ public:
 	{
 		m_ScriptInterface->ReadStructuredClone(techTemplates, &m_TechTemplates);
 	}
-	
+
 	void LoadEntityTemplates(const std::vector<std::pair<std::string, const CParamNode*> >& templates)
 	{
 		JSContext* cx = m_ScriptInterface->GetContext();
@@ -558,21 +611,18 @@ public:
 	{
 		JSContext* cx = m_ScriptInterface->GetContext();
 		JSAutoRequest rq(cx);
-		
+
 		std::stringstream rngStream;
 		rngStream << m_RNG;
 		serializer.StringASCII("rng", rngStream.str(), 0, 32);
 
 		serializer.NumberU32_Unbounded("turn", m_TurnNum);
 
-		serializer.NumberU32_Unbounded("num ais", (u32)m_Players.size());
-
 		serializer.Bool("useSharedScript", m_HasSharedComponent);
 		if (m_HasSharedComponent)
 		{
 			JS::RootedValue sharedData(cx);
-			JS::RootedValue tmpSharedAIObj(cx, m_SharedAIObj.get()); // TODO: Check if this temporary root can be removed after SpiderMonkey 31 upgrade
-			if (!m_ScriptInterface->CallFunction(tmpSharedAIObj, "Serialize", &sharedData))
+			if (!m_ScriptInterface->CallFunction(m_SharedAIObj, "Serialize", &sharedData))
 				LOGERROR("AI shared script Serialize call failed");
 			serializer.ScriptVal("sharedData", &sharedData);
 		}
@@ -581,7 +631,7 @@ public:
 			serializer.String("name", m_Players[i]->m_AIName, 1, 256);
 			serializer.NumberI32_Unbounded("player", m_Players[i]->m_Player);
 			serializer.NumberU8_Unbounded("difficulty", m_Players[i]->m_Difficulty);
-			
+
 			serializer.NumberU32_Unbounded("num commands", (u32)m_Players[i]->m_Commands.size());
 			for (size_t j = 0; j < m_Players[i]->m_Commands.size(); ++j)
 			{
@@ -590,27 +640,34 @@ public:
 				serializer.ScriptVal("command", &val);
 			}
 
-			JS::RootedValue tmpPlayerObj(cx, m_Players[i]->m_Obj.get()); // TODO: Check if this temporary root can be removed after SpiderMonkey 31 upgrade 
-			bool hasCustomSerialize = m_ScriptInterface->HasProperty(tmpPlayerObj, "Serialize");
+			bool hasCustomSerialize = m_ScriptInterface->HasProperty(m_Players[i]->m_Obj, "Serialize");
 			if (hasCustomSerialize)
 			{
 				JS::RootedValue scriptData(cx);
-				if (!m_ScriptInterface->CallFunction(tmpPlayerObj, "Serialize", &scriptData))
+				if (!m_ScriptInterface->CallFunction(m_Players[i]->m_Obj, "Serialize", &scriptData))
 					LOGERROR("AI script Serialize call failed");
 				serializer.ScriptVal("data", &scriptData);
 			}
 			else
 			{
-				serializer.ScriptVal("data", &tmpPlayerObj);
+				serializer.ScriptVal("data", &m_Players[i]->m_Obj);
 			}
 		}
+
+		// AI pathfinder
+		SerializeMap<SerializeString, SerializeU16_Unbounded>()(serializer, "non pathfinding pass classes", m_NonPathfindingPassClasses);
+		SerializeMap<SerializeString, SerializeU16_Unbounded>()(serializer, "pathfinding pass classes", m_PathfindingPassClasses);
+		serializer.NumberU16_Unbounded("pathfinder grid w", m_PassabilityMap.m_W);
+		serializer.NumberU16_Unbounded("pathfinder grid h", m_PassabilityMap.m_H);
+		serializer.RawBytes("pathfinder grid data", (const u8*)m_PassabilityMap.m_Data, 
+			m_PassabilityMap.m_W*m_PassabilityMap.m_H*sizeof(NavcellData));
 	}
 
-	void Deserialize(std::istream& stream)
+	void Deserialize(std::istream& stream, u32 numAis)
 	{
 		JSContext* cx = m_ScriptInterface->GetContext();
 		JSAutoRequest rq(cx);
-		
+
 		ENSURE(m_CommandsComputed); // deserializing while we're still actively computing would be bad
 
 		CStdDeserializer deserializer(*m_ScriptInterface, stream);
@@ -626,17 +683,14 @@ public:
 
 		deserializer.NumberU32_Unbounded("turn", m_TurnNum);
 
-		uint32_t numAis;
-		deserializer.NumberU32_Unbounded("num ais", numAis);
-
 		deserializer.Bool("useSharedScript", m_HasSharedComponent);
-		TryLoadSharedComponent(false);
 		if (m_HasSharedComponent)
 		{
+			TryLoadSharedComponent(false);
+
 			JS::RootedValue sharedData(cx);
-			JS::RootedValue tmpSharedAIObj(cx, m_SharedAIObj.get()); // TODO: Check if this temporary root can be removed after SpiderMonkey 31
 			deserializer.ScriptVal("sharedData", &sharedData);
-			if (!m_ScriptInterface->CallFunctionVoid(tmpSharedAIObj, "Deserialize", sharedData))
+			if (!m_ScriptInterface->CallFunctionVoid(m_SharedAIObj, "Deserialize", sharedData))
 				LOGERROR("AI shared script Deserialize call failed");
 		}
 
@@ -644,14 +698,14 @@ public:
 		{
 			std::wstring name;
 			player_id_t player;
-			uint8_t difficulty;
+			u8 difficulty;
 			deserializer.String("name", name, 1, 256);
 			deserializer.NumberI32_Unbounded("player", player);
 			deserializer.NumberU8_Unbounded("difficulty",difficulty);
 			if (!AddPlayer(name, player, difficulty))
 				throw PSERROR_Deserialize_ScriptError();
 
-			uint32_t numCommands;
+			u32 numCommands;
 			deserializer.NumberU32_Unbounded("num commands", numCommands);
 			m_Players.back()->m_Commands.reserve(numCommands);
 			for (size_t j = 0; j < numCommands; ++j)
@@ -660,11 +714,11 @@ public:
 				deserializer.ScriptVal("command", &val);
 				m_Players.back()->m_Commands.push_back(m_ScriptInterface->WriteStructuredClone(val));
 			}
-			
+
 			// TODO: this is yucky but necessary while the AIs are sharing data between contexts;
-			//	ideally a new (de)serializer instance would be created for each player
-			//	so they would have a single, consistent script context to use and serializable
-			//	prototypes could be stored in their ScriptInterface
+			// ideally a new (de)serializer instance would be created for each player
+			// so they would have a single, consistent script context to use and serializable
+			// prototypes could be stored in their ScriptInterface
 			deserializer.SetSerializablePrototypes(m_DeserializablePrototypes);
 
 			bool hasCustomDeserialize = m_ScriptInterface->HasProperty(m_Players.back()->m_Obj, "Deserialize");
@@ -687,8 +741,18 @@ public:
 				deserializer.ScriptVal("data", &m_Players.back()->m_Obj);
 			}
 		}
+
+		// AI pathfinder
+		SerializeMap<SerializeString, SerializeU16_Unbounded>()(deserializer, "non pathfinding pass classes", m_NonPathfindingPassClasses);
+		SerializeMap<SerializeString, SerializeU16_Unbounded>()(deserializer, "pathfinding pass classes", m_PathfindingPassClasses);
+		u16 mapW, mapH;
+		deserializer.NumberU16_Unbounded("pathfinder grid w", mapW);
+		deserializer.NumberU16_Unbounded("pathfinder grid h", mapH);
+		m_PassabilityMap = Grid<NavcellData>(mapW, mapH);
+		deserializer.RawBytes("pathfinder grid data", (u8*)m_PassabilityMap.m_Data, mapW*mapH*sizeof(NavcellData));
+		m_LongPathfinder.Reload(&m_PassabilityMap, m_NonPathfindingPassClasses, m_PathfindingPassClasses);
 	}
-	
+
 	int getPlayerSize()
 	{
 		return m_Players.size();
@@ -717,9 +781,9 @@ private:
 
 	void TraceMember(JSTracer *trc)
 	{
-		for (auto& prototypes : m_DeserializablePrototypes)
-			JS_CallHeapObjectTracer(trc, &prototypes.second, "CAIWorker::m_DeserializablePrototypes");
-		for (auto& metadata : m_PlayerMetadata)
+		for (std::pair<const std::wstring, JS::Heap<JSObject*>>& prototype : m_DeserializablePrototypes)
+			JS_CallHeapObjectTracer(trc, &prototype.second, "CAIWorker::m_DeserializablePrototypes");
+		for (std::pair<const VfsPath, JS::Heap<JS::Value>>& metadata : m_PlayerMetadata)
 			JS_CallHeapValueTracer(trc, &metadata.second, "CAIWorker::m_PlayerMetadata");
 	}
 
@@ -736,7 +800,7 @@ private:
 	}
 
 	void PerformComputation()
-	{			
+	{
 		// Deserialize the game state, to pass to the AI's HandleMessage
 		JSContext* cx = m_ScriptInterface->GetContext();
 		JSAutoRequest rq(cx);
@@ -758,10 +822,9 @@ private:
 		if (m_HasSharedComponent)
 		{
 			PROFILE3("AI run shared component");
-			JS::RootedValue tmpSharedAIObj(cx, m_SharedAIObj.get()); // TODO: Check if this temporary root can be removed after SpiderMonkey 31
-			m_ScriptInterface->CallFunctionVoid(tmpSharedAIObj, "onUpdate", state);
+			m_ScriptInterface->CallFunctionVoid(m_SharedAIObj, "onUpdate", state);
 		}
-		
+
 		for (size_t i = 0; i < m_Players.size(); ++i)
 		{
 			PROFILE3("AI script");
@@ -778,7 +841,7 @@ private:
 	// Take care to keep this declaration before heap rooted members. Destructors of heap rooted
 	// members have to be called before the runtime destructor.
 	shared_ptr<ScriptRuntime> m_ScriptRuntime;
-	
+
 	shared_ptr<ScriptInterface> m_ScriptInterface;
 	boost::rand48 m_RNG;
 	u32 m_TurnNum;
@@ -793,15 +856,19 @@ private:
 	bool m_HasSharedComponent;
 	JS::PersistentRootedValue m_SharedAIObj;
 	std::vector<SCommandSets> m_Commands;
-	
+
 	std::set<std::wstring> m_LoadedModules;
-	
+
 	shared_ptr<ScriptInterface::StructuredClone> m_GameState;
-	Grid<u16> m_PassabilityMap;
+	Grid<NavcellData> m_PassabilityMap;
 	JS::PersistentRootedValue m_PassabilityMapVal;
 	Grid<u8> m_TerritoryMap;
 	JS::PersistentRootedValue m_TerritoryMapVal;
 
+	std::map<std::string, pass_class_t> m_NonPathfindingPassClasses;
+	std::map<std::string, pass_class_t> m_PathfindingPassClasses;
+	LongPathfinder m_LongPathfinder;
+
 	bool m_CommandsComputed;
 
 	shared_ptr<ObjectIdCache<std::wstring> > m_SerializablePrototypes;
@@ -841,6 +908,8 @@ public:
 
 	virtual void Serialize(ISerializer& serialize)
 	{
+		serialize.NumberU32_Unbounded("num ais", m_Worker.getPlayerSize());
+
 		// Because the AI worker uses its own ScriptInterface, we can't use the
 		// ISerializer (which was initialised with the simulation ScriptInterface)
 		// directly. So we'll just grab the ISerializer's stream and write to it
@@ -853,10 +922,13 @@ public:
 	{
 		Init(paramNode);
 
-		ForceLoadEntityTemplates();
+		u32 numAis;
+		deserialize.NumberU32_Unbounded("num ais", numAis);
+		if (numAis > 0)
+			ForceLoadEntityTemplates();
+
+		m_Worker.Deserialize(deserialize.GetStream(), numAis);
 
-		m_Worker.Deserialize(deserialize.GetStream());
-		
 		m_JustDeserialized = true;
 	}
 
@@ -883,7 +955,7 @@ public:
 		}
 	}
 
-	virtual void AddPlayer(std::wstring id, player_id_t player, uint8_t difficulty)
+	virtual void AddPlayer(const std::wstring& id, player_id_t player, u8 difficulty)
 	{
 		m_Worker.AddPlayer(id, player, difficulty);
 
@@ -894,8 +966,8 @@ public:
 		if (cmpRangeManager)
 			cmpRangeManager->SetLosRevealAll(player, true);
 	}
-	
-	virtual void SetRNGSeed(uint32_t seed)
+
+	virtual void SetRNGSeed(u32 seed)
 	{
 		m_Worker.SetRNGSeed(seed);
 	}
@@ -905,15 +977,15 @@ public:
 		ScriptInterface& scriptInterface = GetSimContext().GetScriptInterface();
 		JSContext* cx = scriptInterface.GetContext();
 		JSAutoRequest rq(cx);
-		
+
 		// load the technology templates
 		CmpPtr<ICmpTechnologyTemplateManager> cmpTechTemplateManager(GetSystemEntity());
 		ENSURE(cmpTechTemplateManager);
-		
+
 		// Get the game state from AIInterface
 		JS::RootedValue techTemplates(cx);
 		cmpTechTemplateManager->GetAllTechs(&techTemplates);
-		
+
 		m_Worker.RegisterTechTemplates(scriptInterface.WriteStructuredClone(techTemplates));
 		m_Worker.TryLoadSharedComponent(true);
 	}
@@ -923,22 +995,22 @@ public:
 		ScriptInterface& scriptInterface = GetSimContext().GetScriptInterface();
 		JSContext* cx = scriptInterface.GetContext();
 		JSAutoRequest rq(cx);
-		
+
 		CmpPtr<ICmpAIInterface> cmpAIInterface(GetSystemEntity());
 		ENSURE(cmpAIInterface);
-		
+
 		// Get the game state from AIInterface
 		// We flush events from the initialization so we get a clean state now.
 		JS::RootedValue state(cx);
 		cmpAIInterface->GetFullRepresentation(&state, true);
 
 		// Get the passability data
-		Grid<u16> dummyGrid;
-		const Grid<u16>* passabilityMap = &dummyGrid;
+		Grid<NavcellData> dummyGrid;
+		const Grid<NavcellData>* passabilityMap = &dummyGrid;
 		CmpPtr<ICmpPathfinder> cmpPathfinder(GetSystemEntity());
 		if (cmpPathfinder)
 			passabilityMap = &cmpPathfinder->GetPassabilityGrid();
-		
+
 		// Get the territory data
 		//	Since getting the territory grid can trigger a recalculation, we check NeedUpdate first
 		Grid<u8> dummyGrid2;
@@ -948,10 +1020,13 @@ public:
 		{
 			territoryMap = &cmpTerritoryManager->GetTerritoryGrid();
 		}
-		
-		LoadPathfinderClasses(state);
 
-		m_Worker.RunGamestateInit(scriptInterface.WriteStructuredClone(state), *passabilityMap, *territoryMap);
+		LoadPathfinderClasses(state);
+		std::map<std::string, pass_class_t> nonPathfindingPassClassMasks, pathfindingPassClassMasks;
+		if (cmpPathfinder)
+			cmpPathfinder->GetPassabilityClasses(nonPathfindingPassClassMasks, pathfindingPassClassMasks);
+
+		m_Worker.RunGamestateInit(scriptInterface.WriteStructuredClone(state), *passabilityMap, *territoryMap, nonPathfindingPassClassMasks, pathfindingPassClassMasks);
 	}
 
 	virtual void StartComputation()
@@ -966,7 +1041,7 @@ public:
 
 		if (m_Worker.getPlayerSize() == 0)
 			return;
-		
+
 		CmpPtr<ICmpAIInterface> cmpAIInterface(GetSystemEntity());
 		ENSURE(cmpAIInterface);
 
@@ -978,14 +1053,16 @@ public:
 			cmpAIInterface->GetRepresentation(&state);
 
 		// Get the passability data
-		Grid<u16> dummyGrid;
-		const Grid<u16>* passabilityMap = &dummyGrid;
+		Grid<NavcellData> dummyGrid;
+		const Grid<NavcellData>* passabilityMap = &dummyGrid;
 		CmpPtr<ICmpPathfinder> cmpPathfinder(GetSystemEntity());
 		if (cmpPathfinder)
 			passabilityMap = &cmpPathfinder->GetPassabilityGrid();
 
+		GridUpdateInformation dirtinessInformations = cmpPathfinder->GetDirtinessData();
+
 		// Get the territory data
-		//	Since getting the territory grid can trigger a recalculation, we check NeedUpdate first
+		// Since getting the territory grid can trigger a recalculation, we check NeedUpdate first
 		bool territoryMapDirty = false;
 		Grid<u8> dummyGrid2;
 		const Grid<u8>* territoryMap = &dummyGrid2;
@@ -997,9 +1074,15 @@ public:
 		}
 
 		LoadPathfinderClasses(state);
+		std::map<std::string, pass_class_t> nonPathfindingPassClassMasks, pathfindingPassClassMasks;
+		if (cmpPathfinder)
+			cmpPathfinder->GetPassabilityClasses(nonPathfindingPassClassMasks, pathfindingPassClassMasks);
+
+		m_Worker.StartComputation(scriptInterface.WriteStructuredClone(state), 
+			*passabilityMap, dirtinessInformations,
+			*territoryMap, territoryMapDirty,
+			nonPathfindingPassClassMasks, pathfindingPassClassMasks);
 
-		m_Worker.StartComputation(scriptInterface.WriteStructuredClone(state), *passabilityMap, *territoryMap, territoryMapDirty);
-		
 		m_JustDeserialized = false;
 	}
 
@@ -1011,12 +1094,12 @@ public:
 		CmpPtr<ICmpCommandQueue> cmpCommandQueue(GetSystemEntity());
 		if (!cmpCommandQueue)
 			return;
-		
+
 		ScriptInterface& scriptInterface = GetSimContext().GetScriptInterface();
 		JSContext* cx = scriptInterface.GetContext();
 		JSAutoRequest rq(cx);
 		JS::RootedValue clonedCommandVal(cx);
-		
+
 		for (size_t i = 0; i < commands.size(); ++i)
 		{
 			for (size_t j = 0; j < commands[i].commands.size(); ++j)
@@ -1056,7 +1139,7 @@ private:
 
 		const CParamNode* node = cmpTemplateManager->GetTemplateWithoutValidation(m_TemplateNames[m_TemplateLoadedIdx]);
 		if (node)
-			m_Templates.push_back(std::make_pair(m_TemplateNames[m_TemplateLoadedIdx], node));
+			m_Templates.emplace_back(m_TemplateNames[m_TemplateLoadedIdx], node);
 
 		m_TemplateLoadedIdx++;
 
@@ -1085,10 +1168,11 @@ private:
 		JSAutoRequest rq(cx);
 
 		JS::RootedValue classesVal(cx);
-		scriptInterface.Eval("({ pathfinderObstruction: 1, foundationObstruction: 2 })", &classesVal);
+		scriptInterface.Eval("({})", &classesVal);
 
-		std::map<std::string, ICmpPathfinder::pass_class_t> classes = cmpPathfinder->GetPassabilityClasses();
-		for (std::map<std::string, ICmpPathfinder::pass_class_t>::iterator it = classes.begin(); it != classes.end(); ++it)
+		std::map<std::string, pass_class_t> classes;
+		cmpPathfinder->GetPassabilityClasses(classes);
+		for (std::map<std::string, pass_class_t>::iterator it = classes.begin(); it != classes.end(); ++it)
 			scriptInterface.SetProperty(classesVal, it->first.c_str(), it->second, true);
 
 		scriptInterface.SetProperty(state, "passabilityClasses", classesVal, true);
