@@ -1,4 +1,4 @@
-/* Copyright (C) 2013 Wildfire Games.
+/* Copyright (C) 2015 Wildfire Games.
  * This file is part of 0 A.D.
  *
  * 0 A.D. is free software: you can redistribute it and/or modify
@@ -59,15 +59,21 @@ public:
 			"<choice>"
 				"<element name='Square' a:help='Set the footprint to a square of the given size'>"
 					"<attribute name='width' a:help='Size of the footprint along the left/right direction (in metres)'>"
-						"<ref name='positiveDecimal'/>"
+						"<data type='decimal'>"
+							"<param name='minExclusive'>0.0</param>"
+						"</data>"
 					"</attribute>"
 					"<attribute name='depth' a:help='Size of the footprint along the front/back direction (in metres)'>"
-						"<ref name='positiveDecimal'/>"
+						"<data type='decimal'>"
+							"<param name='minExclusive'>0.0</param>"
+						"</data>"
 					"</attribute>"
 				"</element>"
 				"<element name='Circle' a:help='Set the footprint to a circle of the given size'>"
 					"<attribute name='radius' a:help='Radius of the footprint (in metres)'>"
-						"<ref name='positiveDecimal'/>"
+						"<data type='decimal'>"
+							"<param name='minExclusive'>0.0</param>"
+						"</data>"
 					"</attribute>"
 				"</element>"
 			"</choice>"
@@ -154,7 +160,7 @@ public:
 		if (!cmpUnitMotion)
 			return error;
 
-		ICmpPathfinder::pass_class_t spawnedPass = cmpUnitMotion->GetPassabilityClass();
+		pass_class_t spawnedPass = cmpUnitMotion->GetPassabilityClass();
 		CmpPtr<ICmpPathfinder> cmpPathfinder(GetSystemEntity());
 		if (!cmpPathfinder)
 			return error;
@@ -285,7 +291,7 @@ public:
 		if (!cmpUnitMotion)
 			return error;
 
-		ICmpPathfinder::pass_class_t spawnedPass = cmpUnitMotion->GetPassabilityClass();
+		pass_class_t spawnedPass = cmpUnitMotion->GetPassabilityClass();
 		CmpPtr<ICmpPathfinder> cmpPathfinder(GetSystemEntity());
 		if (!cmpPathfinder)
 			return error;
@@ -294,7 +300,7 @@ public:
 		CmpPtr<ICmpUnitMotion> cmpEntityMotion(GetEntityHandle());
 		if (!cmpEntityMotion)
 			return error;
-		ICmpPathfinder::pass_class_t entityPass = cmpEntityMotion->GetPassabilityClass();
+		pass_class_t entityPass = cmpEntityMotion->GetPassabilityClass();
 		
 		CFixedVector2D initialPos = cmpPosition->GetPosition2D();
 		entity_angle_t initialAngle = cmpPosition->GetRotation().Y;
