@@ -1,4 +1,4 @@
-/* Copyright (C) 2013 Wildfire Games.
+/* Copyright (C) 2015 Wildfire Games.
  * This file is part of 0 A.D.
  *
  * 0 A.D. is free software: you can redistribute it and/or modify
@@ -23,7 +23,7 @@
 #include "ps/CLogger.h"
 #include "simulation2/MessageTypes.h"
 #include "simulation2/components/ICmpObstructionManager.h"
-#include "simulation2/components/ICmpPosition.h"
+#include "simulation2/components/ICmpUnitMotion.h"
 #include "simulation2/serialization/SerializeTemplates.h"
 
 #define MAX(x,y) x>y ? x : y
@@ -58,6 +58,7 @@ public:
 	entity_pos_t m_Size0; // radius or width
 	entity_pos_t m_Size1; // radius or depth
 	flags_t m_TemplateFlags;
+	entity_pos_t m_Clearance;
 
 	typedef struct {
 		entity_pos_t dx, dz;
@@ -112,16 +113,18 @@ public:
 			"<choice>"
 				"<element name='Static'>"
 					"<attribute name='width'>"
-						"<ref name='positiveDecimal'/>"
+						"<data type='decimal'>"
+							"<param name='minInclusive'>1.5</param>"
+						"</data>"
 					"</attribute>"
 					"<attribute name='depth'>"
-						"<ref name='positiveDecimal'/>"
+						"<data type='decimal'>"
+							"<param name='minInclusive'>1.5</param>"
+						"</data>"
 					"</attribute>"
 				"</element>"
 				"<element name='Unit'>"
-					"<attribute name='radius'>"
-						"<ref name='positiveDecimal'/>"
-					"</attribute>"
+					"<empty/>"
 				"</element>"
 				"<element name='Obstructions'>"
 					"<zeroOrMore>"
@@ -138,10 +141,14 @@ public:
 								"</attribute>"
 							"</optional>"
 							"<attribute name='width'>"
-								"<ref name='positiveDecimal'/>"
+								"<data type='decimal'>"
+									"<param name='minInclusive'>1.5</param>"
+								"</data>"
 							"</attribute>"
 							"<attribute name='depth'>"
-								"<ref name='positiveDecimal'/>"
+								"<data type='decimal'>"
+									"<param name='minInclusive'>1.5</param>"
+								"</data>"
 							"</attribute>"
 						"</element>"
 					"</zeroOrMore>"
@@ -177,6 +184,9 @@ public:
 
 	virtual void Init(const CParamNode& paramNode)
 	{
+		// The minimum obstruction size is the navcell size * sqrt(2)
+		// This is enforced in the schema as a minimum of 1.5
+		fixed minObstruction = (Pathfinding::NAVCELL_SIZE.Square() * 2).Sqrt();
 		m_TemplateFlags = 0;
 		if (paramNode.GetChild("BlockMovement").ToBool())
 			m_TemplateFlags |= ICmpObstructionManager::FLAG_BLOCK_MOVEMENT;
@@ -196,13 +206,18 @@ public:
 		if (paramNode.GetChild("Unit").IsOk())
 		{
 			m_Type = UNIT;
-			m_Size0 = m_Size1 = paramNode.GetChild("Unit").GetChild("@radius").ToFixed();
+
+			CmpPtr<ICmpUnitMotion> cmpUnitMotion(GetEntityHandle());
+			if (cmpUnitMotion)
+				m_Clearance = cmpUnitMotion->GetUnitClearance();
 		}
 		else if (paramNode.GetChild("Static").IsOk())
 		{
 			m_Type = STATIC;
 			m_Size0 = paramNode.GetChild("Static").GetChild("@width").ToFixed();
 			m_Size1 = paramNode.GetChild("Static").GetChild("@depth").ToFixed();
+			ENSURE(m_Size0 > minObstruction);
+			ENSURE(m_Size1 > minObstruction);
 		}
 		else
 		{
@@ -215,6 +230,8 @@ public:
 				Shape b;
 				b.size0 = it->second.GetChild("@width").ToFixed();
 				b.size1 = it->second.GetChild("@depth").ToFixed();
+				ENSURE(b.size0 > minObstruction);
+				ENSURE(b.size1 > minObstruction);
 				b.dx = it->second.GetChild("@x").ToFixed();
 				b.dz = it->second.GetChild("@z").ToFixed();
 				b.da = entity_angle_t::FromInt(0);
@@ -264,6 +281,8 @@ public:
 		serialize.NumberU8_Unbounded("flags", m_Flags);
 		if (m_Type == CLUSTER)
 			SerializeVector<SerializeTag>()(serialize, "cluster tags", m_ClusterTags);
+		if (m_Type == UNIT)
+			serialize.NumberFixed_Unbounded("clearance", m_Clearance);
 	}
 
 	virtual void Serialize(ISerializer& serialize)
@@ -300,7 +319,7 @@ public:
 			{
 				cmpObstructionManager->MoveShape(m_Tag, data.x, data.z, data.a);
 
-				if(m_Type == CLUSTER)
+				if (m_Type == CLUSTER)
 				{
 					for (size_t i = 0; i < m_Shapes.size(); ++i)
 					{
@@ -319,7 +338,7 @@ public:
 						data.x, data.z, data.a, m_Size0, m_Size1, m_Flags, m_ControlGroup, m_ControlGroup2);
 				else if (m_Type == UNIT)
 					m_Tag = cmpObstructionManager->AddUnitShape(GetEntityId(),
-						data.x, data.z, m_Size0, (flags_t)(m_Flags | (m_Moving ? ICmpObstructionManager::FLAG_MOVING : 0)), m_ControlGroup);
+						data.x, data.z, m_Clearance, (flags_t)(m_Flags | (m_Moving ? ICmpObstructionManager::FLAG_MOVING : 0)), m_ControlGroup);
 				else
 					AddClusterShapes(data.x, data.x, data.a);
 			}
@@ -376,7 +395,7 @@ public:
 					pos.X, pos.Y, cmpPosition->GetRotation().Y, m_Size0, m_Size1, m_Flags, m_ControlGroup, m_ControlGroup2);
 			else if (m_Type == UNIT)
 				m_Tag = cmpObstructionManager->AddUnitShape(GetEntityId(),
-					pos.X, pos.Y, m_Size0, (flags_t)(m_Flags | (m_Moving ? ICmpObstructionManager::FLAG_MOVING : 0)), m_ControlGroup);
+					pos.X, pos.Y, m_Clearance, (flags_t)(m_Flags | (m_Moving ? ICmpObstructionManager::FLAG_MOVING : 0)), m_ControlGroup);
 			else 
 				AddClusterShapes(pos.X, pos.Y, cmpPosition->GetRotation().Y);
 		}
@@ -470,10 +489,8 @@ public:
 			pos = cmpPosition->GetPreviousPosition2D();
 		else
 			pos = cmpPosition->GetPosition2D();
-		if (m_Type == STATIC)
-			out = cmpObstructionManager->GetStaticShapeObstruction(pos.X, pos.Y, cmpPosition->GetRotation().Y, m_Size0, m_Size1);
-		else if (m_Type == UNIT)
-			out = cmpObstructionManager->GetUnitShapeObstruction(pos.X, pos.Y, m_Size0);
+		if (m_Type == UNIT)
+			out = cmpObstructionManager->GetUnitShapeObstruction(pos.X, pos.Y, m_Clearance);
 		else
 			out = cmpObstructionManager->GetStaticShapeObstruction(pos.X, pos.Y, cmpPosition->GetRotation().Y, m_Size0, m_Size1);
 		return true;
@@ -482,11 +499,25 @@ public:
 	virtual entity_pos_t GetUnitRadius()
 	{
 		if (m_Type == UNIT)
-			return m_Size0;
+			return m_Clearance;
 		else
 			return entity_pos_t::Zero();
 	}
 
+	virtual entity_pos_t GetSize()
+	{
+		if (m_Type == UNIT)
+			return m_Clearance;
+		else
+			return CFixedVector2D(m_Size0 / 2, m_Size1 / 2).Length();
+	}
+
+	virtual void SetUnitClearance(const entity_pos_t& clearance)
+	{
+		if (m_Type == UNIT)
+			m_Clearance = clearance;
+	}
+
 	virtual bool IsControlPersistent()
 	{
 		return m_ControlPersist;
@@ -520,7 +551,7 @@ public:
 		}
 
 		// Get passability class
-		ICmpPathfinder::pass_class_t passClass = cmpPathfinder->GetPassabilityClass(className);
+		pass_class_t passClass = cmpPathfinder->GetPassabilityClass(className);
 
 		// Ignore collisions within the same control group, or with other non-foundation-blocking shapes.
 		// Note that, since the control group for each entity defaults to the entity's ID, this is typically 
@@ -529,7 +560,7 @@ public:
 			ICmpObstructionManager::FLAG_BLOCK_FOUNDATION);
 
 		if (m_Type == UNIT)
-			return cmpPathfinder->CheckUnitPlacement(filter, pos.X, pos.Y, m_Size0, passClass, onlyCenterPoint);
+			return cmpPathfinder->CheckUnitPlacement(filter, pos.X, pos.Y, m_Clearance, passClass, onlyCenterPoint);
 		else
 			return cmpPathfinder->CheckBuildingPlacement(filter, pos.X, pos.Y, cmpPosition->GetRotation().Y, m_Size0, m_Size1, GetEntityId(), passClass, onlyCenterPoint);
 	}
@@ -561,7 +592,7 @@ public:
 			ICmpObstructionManager::FLAG_BLOCK_FOUNDATION);
 
 		if (m_Type == UNIT)
-			return !cmpObstructionManager->TestUnitShape(filter, pos.X, pos.Y, m_Size0, NULL);
+			return !cmpObstructionManager->TestUnitShape(filter, pos.X, pos.Y, m_Clearance, NULL);
 		else
 			return !cmpObstructionManager->TestStaticShape(filter, pos.X, pos.Y, cmpPosition->GetRotation().Y, m_Size0, m_Size1, NULL );
 	} 
@@ -570,26 +601,10 @@ public:
 	{
 		std::vector<entity_id_t> ret;
 
-		CmpPtr<ICmpPosition> cmpPosition(GetEntityHandle());
-		if (!cmpPosition)
-			return ret; // error
-
-		if (!cmpPosition->IsInWorld())
-			return ret; // no obstruction
-
-		CFixedVector2D pos = cmpPosition->GetPosition2D();
-
 		CmpPtr<ICmpObstructionManager> cmpObstructionManager(GetSystemEntity());
 		if (!cmpObstructionManager)
 			return ret; // error
 
-		// required precondition to use SkipControlGroupsRequireFlagObstructionFilter
-		if (m_ControlGroup == INVALID_ENTITY)
-		{
-			LOGERROR("[CmpObstruction] Cannot test for unit or structure obstructions; primary control group must be valid");
-			return ret;
-		}
-
 		flags_t flags = 0;
 		bool invertMatch = false;
 
@@ -613,18 +628,16 @@ public:
 		}
 
 		// Ignore collisions within the same control group, or with other shapes that don't match the filter.
-		// Note that, since the control group for each entity defaults to the entity's ID, this is typically 
+		// Note that, since the control group for each entity defaults to the entity's ID, this is typically
 		// equivalent to only ignoring the entity's own shape and other shapes that don't match the filter.
-		SkipControlGroupsRequireFlagObstructionFilter filter(invertMatch,
-				m_ControlGroup, m_ControlGroup2, flags);
+		SkipControlGroupsRequireFlagObstructionFilter filter(invertMatch, m_ControlGroup, m_ControlGroup2, flags);
+
+		ICmpObstructionManager::ObstructionSquare square;
+		if (!GetObstructionSquare(square))
+			return ret; // error
+
+		cmpObstructionManager->GetUnitsOnObstruction(square, ret, filter);
 
-		if (m_Type == STATIC)
-			cmpObstructionManager->TestStaticShape(filter, pos.X, pos.Y, cmpPosition->GetRotation().Y, m_Size0, m_Size1, &ret);
-		else if (m_Type == UNIT)
-			cmpObstructionManager->TestUnitShape(filter, pos.X, pos.Y, m_Size0, &ret);
-		else
-			cmpObstructionManager->TestStaticShape(filter, pos.X, pos.Y, cmpPosition->GetRotation().Y, m_Size0, m_Size1, &ret);
-	
 		return ret;
 	}
 
