@@ -1,4 +1,4 @@
-/* Copyright (C) 2012 Wildfire Games.
+/* Copyright (C) 2015 Wildfire Games.
  * This file is part of 0 A.D.
  *
  * 0 A.D. is free software: you can redistribute it and/or modify
@@ -27,6 +27,8 @@
 #include "graphics/TextureManager.h"
 #include "renderer/Renderer.h"
 
+#include "ps/CLogger.h"
+
 class CCmpOverlayRenderer : public ICmpOverlayRenderer
 {
 public:
@@ -112,8 +114,12 @@ public:
 		UpdateMessageSubscriptions();
 	}
 
-	virtual void AddSprite(VfsPath textureName, CFixedVector2D corner0, CFixedVector2D corner1, CFixedVector3D position)
+	virtual void AddSprite(const VfsPath& textureName, const CFixedVector2D& corner0, const CFixedVector2D& corner1, const CFixedVector3D& position, const std::string& color)
 	{
+		CColor colorObj(1.0f, 1.0f, 1.0f, 1.0f);
+		if (!colorObj.ParseString(color, 1))
+			LOGERROR("OverlayRenderer: Error parsing '%s'", color);
+
 		CTextureProperties textureProps(textureName);
 
 		SOverlaySprite sprite;
@@ -122,6 +128,7 @@ public:
 		sprite.m_Y0 = corner0.Y.ToFloat();
 		sprite.m_X1 = corner1.X.ToFloat();
 		sprite.m_Y1 = corner1.Y.ToFloat();
+		sprite.m_Color = colorObj;
 
 		m_Sprites.push_back(sprite);
 		m_SpriteOffsets.push_back(CVector3D(position));
@@ -159,7 +166,7 @@ public:
 
 	void RenderSubmit(SceneCollector& collector)
 	{
-		if (!m_Enabled)
+		if (!m_Enabled || !ICmpOverlayRenderer::m_OverrideVisible)
 			return;
 
 		for (size_t i = 0; i < m_Sprites.size(); ++i)
