@@ -1,4 +1,4 @@
-/* Copyright (C) 2014 Wildfire Games.
+/* Copyright (C) 2016 Wildfire Games.
  * This file is part of 0 A.D.
  *
  * 0 A.D. is free software: you can redistribute it and/or modify
@@ -297,7 +297,7 @@ public:
 		return &m_Turrets;
 	}
 
-	virtual void SetTurretParent(entity_id_t id, CFixedVector3D offset)
+	virtual void SetTurretParent(entity_id_t id, const CFixedVector3D& offset)
 	{
 		if (m_TurretParent != INVALID_ENTITY)
 		{
@@ -695,7 +695,7 @@ public:
 			}
 		}
 
-		float y = baseY + m_Y.ToFloat() + Interpolate(m_LastYDifference.ToFloat(), 0.f, frameOffset);
+		float y = baseY + m_Y.ToFloat() + Interpolate(-1 * m_LastYDifference.ToFloat(), 0.f, frameOffset);
 
 		CMatrix3D m;
 
@@ -827,6 +827,7 @@ public:
 			}
 			else if (m_Territory != INVALID_PLAYER)
 			{
+				m_Territory = INVALID_PLAYER;
 				CMessageTerritoryPositionChanged msg(GetEntityId(), m_Territory);
 				GetSimContext().GetComponentManager().PostMessage(GetEntityId(), msg);
 			}
