@@ -1,4 +1,4 @@
-/* Copyright (C) 2015 Wildfire Games.
+/* Copyright (C) 2016 Wildfire Games.
  * This file is part of 0 A.D.
  *
  * 0 A.D. is free software: you can redistribute it and/or modify
@@ -27,6 +27,7 @@
 #include "simulation2/components/ICmpMirage.h"
 #include "simulation2/components/ICmpOwnership.h"
 #include "simulation2/components/ICmpPosition.h"
+#include "simulation2/components/ICmpObstructionManager.h"
 #include "simulation2/components/ICmpTerritoryManager.h"
 #include "simulation2/components/ICmpVisibility.h"
 #include "simulation2/components/ICmpVision.h"
@@ -38,10 +39,8 @@
 #include "graphics/Terrain.h"
 #include "lib/timer.h"
 #include "ps/CLogger.h"
-#include "ps/Overlay.h"
 #include "ps/Profile.h"
 #include "renderer/Scene.h"
-#include "lib/ps_stl.h"
 
 #define LOS_TILES_RATIO 8
 #define DEBUG_RANGE_MANAGER_BOUNDS 0
@@ -98,18 +97,15 @@ static u32 CalcSharedLosMask(std::vector<player_id_t> players)
 }
 
 /**
- * Returns shared 1-bit mask for given list of players.
+ * Add a player to mask, which is a 1-bit mask representing a list of players.
+ * Returns true if the mask is modified.
  */
-static u32 CalcSharedDirtyVisibilityMask(std::vector<player_id_t> players)
+static bool AddPlayerSharedDirtyVisibilityMask(u16& mask, player_id_t player)
 {
-	u16 playerMask = 0;
-	for (size_t i = 0; i < players.size(); i++)
-	{
-		if (players[i] > 0 && players[i] <= 16)
-			playerMask |= (0x1 << (players[i] - 1));
-	}
-
-	return playerMask;
+	u16 oldMask = mask;
+	if (player > 0 && player <= 16)
+		mask |= (0x1 << (player - 1));
+	return oldMask != mask;
 }
 
 /**
@@ -174,10 +170,11 @@ static std::map<entity_id_t, EntityParabolicRangeOutline> ParabolicRangesOutline
  */
 struct EntityData
 {
-	EntityData() : visibilities(0), retainInFog(0), owner(-1), inWorld(0), flags(1), scriptedVisibility(0) { }
+	EntityData() : visibilities(0), size(0), retainInFog(0), owner(-1), inWorld(0), flags(1), scriptedVisibility(0) { }
 	entity_pos_t x, z;
 	entity_pos_t visionRange;
 	u32 visibilities; // 2-bit visibility, per player
+	u32 size;
 	u8 retainInFog; // boolean
 	i8 owner;
 	u8 inWorld; // boolean
@@ -185,7 +182,7 @@ struct EntityData
 	u8 scriptedVisibility; // boolean, see ComputeLosVisibility
 };
 
-cassert(sizeof(EntityData) == 24);
+cassert(sizeof(EntityData) == 28);
 
 /**
  * Serialization helper template for Query
@@ -238,10 +235,12 @@ struct SerializeEntityData
 		serialize.NumberFixed_Unbounded("z", value.z);
 		serialize.NumberFixed_Unbounded("vision", value.visionRange);
 		serialize.NumberU32_Unbounded("visibilities", value.visibilities);
+		serialize.NumberU32_Unbounded("size", value.size);
 		serialize.NumberU8("retain in fog", value.retainInFog, 0, 1);
 		serialize.NumberI8_Unbounded("owner", value.owner);
 		serialize.NumberU8("in world", value.inWorld, 0, 1);
 		serialize.NumberU8_Unbounded("flags", value.flags);
+		serialize.NumberU8_Unbounded("scripted visibility", value.scriptedVisibility);
 	}
 };
 
@@ -305,6 +304,10 @@ public:
 	bool m_DebugOverlayDirty;
 	std::vector<SOverlayLine> m_DebugOverlayLines;
 
+	// Deserialization flag. A lot of different functions are called by Deserialize()
+	// and we don't want to pass isDeserializing bool arguments to all of them...
+	bool m_Deserializing;
+
 	// World bounds (entities are expected to be within this range)
 	entity_pos_t m_WorldX0;
 	entity_pos_t m_WorldZ0;
@@ -316,7 +319,7 @@ public:
 	std::map<tag_t, Query> m_Queries;
 	EntityMap<EntityData> m_EntityData;
 
-	SpatialSubdivision m_Subdivision; // spatial index of m_EntityData
+	FastSpatialSubdivision m_Subdivision; // spatial index of m_EntityData
 
 	// LOS state:
 	static const player_id_t MAX_LOS_PLAYER_ID = 16;
@@ -325,7 +328,7 @@ public:
 	bool m_LosCircular;
 	i32 m_TerrainVerticesPerSide;
 	
-	// Cache for visibility tracking (not serialized)
+	// Cache for visibility tracking
 	i32 m_LosTilesPerSide;
 	bool m_GlobalVisibilityUpdate;
 	std::vector<u8> m_GlobalPlayerVisibilityUpdate;
@@ -369,6 +372,7 @@ public:
 		m_DebugOverlayEnabled = false;
 		m_DebugOverlayDirty = true;
 
+		m_Deserializing = false;
 		m_WorldX0 = m_WorldZ0 = m_WorldX1 = m_WorldZ1 = entity_pos_t::Zero();
 
 		// Initialise with bogus values (these will get replaced when
@@ -382,6 +386,9 @@ public:
 		m_SharedLosMasks.resize(MAX_LOS_PLAYER_ID+2,0);
 		m_SharedDirtyVisibilityMasks.resize(MAX_LOS_PLAYER_ID + 2, 0);
 
+		m_GlobalVisibilityUpdate = true;
+		m_GlobalPlayerVisibilityUpdate.resize(MAX_LOS_PLAYER_ID);
+
 		m_LosCircular = false;
 		m_TerrainVerticesPerSide = 0;
 	}
@@ -406,6 +413,9 @@ public:
 		serialize.Bool("los circular", m_LosCircular);
 		serialize.NumberI32_Unbounded("terrain verts per side", m_TerrainVerticesPerSide);
 
+		serialize.Bool("global visibility update", m_GlobalVisibilityUpdate);
+		SerializeVector<SerializeU8_Unbounded>()(serialize, "global player visibility update", m_GlobalPlayerVisibilityUpdate);
+		SerializeVector<SerializeU16_Unbounded>()(serialize, "dirty visibility", m_DirtyVisibility);
 		SerializeVector<SerializeU32_Unbounded>()(serialize, "modified entities", m_ModifiedEntities);
 
 		// We don't serialize m_Subdivision, m_LosPlayerCounts or m_LosTiles
@@ -429,7 +439,9 @@ public:
 		SerializeCommon(deserialize);
 
 		// Reinitialise subdivisions and LOS data
-		ResetDerivedData(true);
+		m_Deserializing = true;
+		ResetDerivedData();
+		m_Deserializing = false;
 	}
 
 	virtual void HandleMessage(const CMessage& msg, bool UNUSED(global))
@@ -463,6 +475,11 @@ public:
 			if (cmpVisibility)
 				entdata.retainInFog = (cmpVisibility->GetRetainInFog() ? 1 : 0);
 
+			// Store the size
+			CmpPtr<ICmpObstruction> cmpObstruction(GetSimContext(), ent);
+			if (cmpObstruction)
+				entdata.size = cmpObstruction->GetSize().ToInt_RoundToInfinity();
+
 			// Remember this entity
 			m_EntityData.insert(ent, entdata);
 			break;
@@ -484,7 +501,7 @@ public:
 				{
 					CFixedVector2D from(it->second.x, it->second.z);
 					CFixedVector2D to(msgData.x, msgData.z);
-					m_Subdivision.Move(ent, from, to);
+					m_Subdivision.Move(ent, from, to, it->second.size);
 					LosMove(it->second.owner, it->second.visionRange, from, to);
 					i32 oldLosTile = PosToLosTilesHelper(it->second.x, it->second.z);
 					i32 newLosTile = PosToLosTilesHelper(msgData.x, msgData.z);
@@ -497,7 +514,7 @@ public:
 				else
 				{
 					CFixedVector2D to(msgData.x, msgData.z);
-					m_Subdivision.Add(ent, to);
+					m_Subdivision.Add(ent, to, it->second.size);
 					LosAdd(it->second.owner, it->second.visionRange, to);
 					AddToTile(PosToLosTilesHelper(msgData.x, msgData.z), ent);
 				}
@@ -511,7 +528,7 @@ public:
 				if (it->second.inWorld)
 				{
 					CFixedVector2D from(it->second.x, it->second.z);
-					m_Subdivision.Remove(ent, from);
+					m_Subdivision.Remove(ent, from, it->second.size);
 					LosRemove(it->second.owner, it->second.visionRange, from);
 					RemoveFromTile(PosToLosTilesHelper(it->second.x, it->second.z), ent);
 				}
@@ -561,7 +578,7 @@ public:
 
 			if (it->second.inWorld)
 			{
-				m_Subdivision.Remove(ent, CFixedVector2D(it->second.x, it->second.z));
+				m_Subdivision.Remove(ent, CFixedVector2D(it->second.x, it->second.z), it->second.size);
 				RemoveFromTile(PosToLosTilesHelper(it->second.x, it->second.z), ent);
 			}
 
@@ -628,7 +645,7 @@ public:
 		m_WorldZ1 = z1;
 		m_TerrainVerticesPerSide = (i32)vertices;
 
-		ResetDerivedData(false);
+		ResetDerivedData();
 	}
 
 	virtual void Verify()
@@ -642,10 +659,12 @@ public:
 
 		std::vector<std::vector<u16> > oldPlayerCounts = m_LosPlayerCounts;
 		std::vector<u32> oldStateRevealed = m_LosStateRevealed;
-		SpatialSubdivision oldSubdivision = m_Subdivision;
+		FastSpatialSubdivision oldSubdivision = m_Subdivision;
 		std::vector<std::set<entity_id_t> > oldLosTiles = m_LosTiles;
 
-		ResetDerivedData(true);
+		m_Deserializing = true;
+		ResetDerivedData();
+		m_Deserializing = false;
 
 		if (oldPlayerCounts != m_LosPlayerCounts)
 		{
@@ -673,13 +692,13 @@ public:
 			debug_warn(L"inconsistent los tiles");
 	}
 
-	SpatialSubdivision* GetSubdivision()
+	FastSpatialSubdivision* GetSubdivision()
 	{
-		return & m_Subdivision;
+		return &m_Subdivision;
 	}
 
 	// Reinitialise subdivisions and LOS data, based on entity data
-	void ResetDerivedData(bool skipLosState)
+	void ResetDerivedData()
 	{
 		ENSURE(m_WorldX0.IsZero() && m_WorldZ0.IsZero()); // don't bother implementing non-zero offsets yet
 		ResetSubdivisions(m_WorldX1, m_WorldZ1);
@@ -690,7 +709,7 @@ public:
 		m_LosPlayerCounts.resize(MAX_LOS_PLAYER_ID+1);
 		m_ExploredVertices.clear();
 		m_ExploredVertices.resize(MAX_LOS_PLAYER_ID+1, 0);
-		if (skipLosState)
+		if (m_Deserializing)
 		{
 			// recalc current exploration stats.
 			for (i32 j = 0; j < m_TerrainVerticesPerSide; j++)
@@ -712,14 +731,16 @@ public:
 		}
 		m_LosStateRevealed.clear();
 		m_LosStateRevealed.resize(m_TerrainVerticesPerSide*m_TerrainVerticesPerSide);
-		
-		m_DirtyVisibility.clear();
-		m_DirtyVisibility.resize(m_LosTilesPerSide*m_LosTilesPerSide);
+
+		if (!m_Deserializing)
+		{
+			m_DirtyVisibility.clear();
+			m_DirtyVisibility.resize(m_LosTilesPerSide*m_LosTilesPerSide);
+		}
+		ENSURE(m_DirtyVisibility.size() == (size_t)(m_LosTilesPerSide*m_LosTilesPerSide));
+
 		m_LosTiles.clear();
 		m_LosTiles.resize(m_LosTilesPerSide*m_LosTilesPerSide);
-		m_GlobalVisibilityUpdate = true;
-		m_GlobalPlayerVisibilityUpdate.clear();
-		m_GlobalPlayerVisibilityUpdate.resize(MAX_LOS_PLAYER_ID);
 
 		for (EntityMap<EntityData>::const_iterator it = m_EntityData.begin(); it != m_EntityData.end(); ++it)
 		{
@@ -746,20 +767,18 @@ public:
 
 	void ResetSubdivisions(entity_pos_t x1, entity_pos_t z1)
 	{
-		// Use 8x8 tile subdivisions
-		// (TODO: find the optimal number instead of blindly guessing)
-		m_Subdivision.Reset(x1, z1, entity_pos_t::FromInt(8*TERRAIN_TILE_SIZE));
+		m_Subdivision.Reset(x1, z1);
 
 		for (EntityMap<EntityData>::const_iterator it = m_EntityData.begin(); it != m_EntityData.end(); ++it)
 		{
 			if (it->second.inWorld)
-				m_Subdivision.Add(it->first, CFixedVector2D(it->second.x, it->second.z));
+				m_Subdivision.Add(it->first, CFixedVector2D(it->second.x, it->second.z), it->second.size);
 		}
 	}
 
 	virtual tag_t CreateActiveQuery(entity_id_t source,
 		entity_pos_t minRange, entity_pos_t maxRange,
-		std::vector<int> owners, int requiredInterface, u8 flags)
+		const std::vector<int>& owners, int requiredInterface, u8 flags)
 	{
 		tag_t id = m_QueryNext++;
 		m_Queries[id] = ConstructQuery(source, minRange, maxRange, owners, requiredInterface, flags);
@@ -769,7 +788,7 @@ public:
 
 	virtual tag_t CreateActiveParabolicQuery(entity_id_t source,
 		entity_pos_t minRange, entity_pos_t maxRange, entity_pos_t elevationBonus,
-		std::vector<int> owners, int requiredInterface, u8 flags)
+		const std::vector<int>& owners, int requiredInterface, u8 flags)
 	{
 		tag_t id = m_QueryNext++;
 		m_Queries[id] = ConstructParabolicQuery(source, minRange, maxRange, elevationBonus, owners, requiredInterface, flags);
@@ -814,9 +833,22 @@ public:
 		q.enabled = false;
 	}
 
-	virtual std::vector<entity_id_t> ExecuteQueryAroundPos(CFixedVector2D pos,
+	virtual bool IsActiveQueryEnabled(tag_t tag)
+	{
+		std::map<tag_t, Query>::const_iterator it = m_Queries.find(tag);
+		if (it == m_Queries.end())
+		{
+			LOGERROR("CCmpRangeManager: IsActiveQueryEnabled called with invalid tag %u", tag);
+			return false;
+		}
+
+		const Query& q = it->second;
+		return q.enabled;
+	}
+
+	virtual std::vector<entity_id_t> ExecuteQueryAroundPos(const CFixedVector2D& pos,
 		entity_pos_t minRange, entity_pos_t maxRange,
-		std::vector<int> owners, int requiredInterface)
+		const std::vector<int>& owners, int requiredInterface)
 	{
 		Query q = ConstructQuery(INVALID_ENTITY, minRange, maxRange, owners, requiredInterface, GetEntityFlagMask("normal"));
 		std::vector<entity_id_t> r;
@@ -830,7 +862,7 @@ public:
 
 	virtual std::vector<entity_id_t> ExecuteQuery(entity_id_t source,
 		entity_pos_t minRange, entity_pos_t maxRange,
-		std::vector<int> owners, int requiredInterface)
+		const std::vector<int>& owners, int requiredInterface)
 	{
 		PROFILE("ExecuteQuery");
 
@@ -891,9 +923,17 @@ public:
 
 	virtual std::vector<entity_id_t> GetEntitiesByPlayer(player_id_t player)
 	{
-		std::vector<entity_id_t> entities;
+		return GetEntitiesByMask(CalcOwnerMask(player));
+	}
 
-		u32 ownerMask = CalcOwnerMask(player);
+	virtual std::vector<entity_id_t> GetNonGaiaEntities()
+	{
+		return GetEntitiesByMask(((1 << MAX_LOS_PLAYER_ID) - 1) << 1);
+	}
+
+	std::vector<entity_id_t> GetEntitiesByMask(u32 ownerMask)
+	{
+		std::vector<entity_id_t> entities;
 
 		for (EntityMap<EntityData>::const_iterator it = m_EntityData.begin(); it != m_EntityData.end(); ++it)
 		{
@@ -1056,6 +1096,7 @@ public:
 
 				r.push_back(it->first);
 			}
+			std::sort(r.begin(), r.end());
 		}
 		// check a regular range (i.e. not the entire world, and not parabolic)
 		else
@@ -1086,11 +1127,13 @@ public:
 
 				r.push_back(it->first);
 			}
+			std::sort(r.begin(), r.end());
 		}
 	}
 
-	virtual entity_pos_t GetElevationAdaptedRange(CFixedVector3D pos, CFixedVector3D rot, entity_pos_t range, entity_pos_t elevationBonus, entity_pos_t angle)
+	virtual entity_pos_t GetElevationAdaptedRange(const CFixedVector3D& pos1, const CFixedVector3D& rot, entity_pos_t range, entity_pos_t elevationBonus, entity_pos_t angle)
 	{
+		CFixedVector3D pos(pos1);
 		entity_pos_t r = entity_pos_t::Zero() ;
 
 		pos.Y += elevationBonus;
@@ -1217,6 +1260,9 @@ public:
 		for (size_t i = 0; i < owners.size(); ++i)
 			q.ownersMask |= CalcOwnerMask(owners[i]);
 
+		if (q.ownersMask == 0)
+			LOGWARNING("CCmpRangeManager: No owners in query for entity %u", source);
+
 		q.interface = requiredInterface;
 		q.flagsMask = flagsMask;
 
@@ -1238,10 +1284,10 @@ public:
 	{
 		if (!m_DebugOverlayEnabled)
 			return;
-		static CColor disabledRingColour(1, 0, 0, 1);	// red
-		static CColor enabledRingColour(0, 1, 0, 1);	// green
-		static CColor subdivColour(0, 0, 1, 1);			// blue
-		static CColor rayColour(1, 1, 0, 0.2f);
+		static CColor disabledRingColor(1, 0, 0, 1);	// red
+		static CColor enabledRingColor(0, 1, 0, 1);	// green
+		static CColor subdivColor(0, 0, 1, 1);			// blue
+		static CColor rayColor(1, 1, 0, 0.2f);
 
 		if (m_DebugOverlayDirty)
 		{
@@ -1260,7 +1306,7 @@ public:
 				if (!q.parabolic)
 				{
 					m_DebugOverlayLines.push_back(SOverlayLine());
-					m_DebugOverlayLines.back().m_Color = (q.enabled ? enabledRingColour : disabledRingColour);
+					m_DebugOverlayLines.back().m_Color = (q.enabled ? enabledRingColor : disabledRingColor);
 					SimRender::ConstructCircleOnGround(GetSimContext(), pos.X.ToFloat(), pos.Y.ToFloat(), q.maxRange.ToFloat(), m_DebugOverlayLines.back(), true);
 				}
 				else
@@ -1306,7 +1352,7 @@ public:
 						ParabolicRangesOutlines[q.source.GetId()] = e;
 					}
 
-					CColor thiscolor = q.enabled ? enabledRingColour : disabledRingColour;
+					CColor thiscolor = q.enabled ? enabledRingColor : disabledRingColor;
 
 					// draw the outline (piece by piece)
 					for (size_t i = 3; i < coords.size(); i += 2)
@@ -1343,21 +1389,20 @@ public:
 					coords.push_back(targetPos.Y.ToFloat());
 
 					m_DebugOverlayLines.push_back(SOverlayLine());
-					m_DebugOverlayLines.back().m_Color = rayColour;
+					m_DebugOverlayLines.back().m_Color = rayColor;
 					SimRender::ConstructLineOnGround(GetSimContext(), coords, m_DebugOverlayLines.back(), true);
 				}
 			}
 
 			// render subdivision grid
-			float divSize = m_Subdivision.GetDivisionSize().ToFloat();
-			int width = m_Subdivision.GetWidth();
-			int height = m_Subdivision.GetHeight();
-			for (int x = 0; x < width; ++x)
+			float divSize = m_Subdivision.GetDivisionSize();
+			int size = m_Subdivision.GetWidth();
+			for (int x = 0; x < size; ++x)
 			{
-				for (int y = 0; y < height; ++y)
+				for (int y = 0; y < size; ++y)
 				{
 					m_DebugOverlayLines.push_back(SOverlayLine());
-					m_DebugOverlayLines.back().m_Color = subdivColour;
+					m_DebugOverlayLines.back().m_Color = subdivColor;
 
 					float xpos = x*divSize + divSize/2;
 					float zpos = y*divSize + divSize/2;
@@ -1373,7 +1418,7 @@ public:
 			collector.Submit(&m_DebugOverlayLines[i]);
 	}
 
-	virtual u8 GetEntityFlagMask(std::string identifier)
+	virtual u8 GetEntityFlagMask(const std::string& identifier)
 	{
 		if (identifier == "normal")
 			return 1;
@@ -1384,7 +1429,7 @@ public:
 		return 0;
 	}
 
-	virtual void SetEntityFlag(entity_id_t ent, std::string identifier, bool value)
+	virtual void SetEntityFlag(entity_id_t ent, const std::string& identifier, bool value)
 	{
 		EntityMap<EntityData>::iterator it = m_EntityData.find(ent);
 
@@ -1602,14 +1647,14 @@ public:
 	void UpdateVisibilityData()
 	{
 		PROFILE("UpdateVisibilityData");
-		
+
 		for (i32 n = 0; n < m_LosTilesPerSide * m_LosTilesPerSide; ++n)
 		{
 			for (player_id_t player = 1; player < MAX_LOS_PLAYER_ID + 1; ++player)
 			{
 				if (IsVisibilityDirty(m_DirtyVisibility[n], player) || m_GlobalPlayerVisibilityUpdate[player-1] == 1 || m_GlobalVisibilityUpdate)
 				{
-					for (auto& ent : m_LosTiles[n])
+					for (const entity_id_t& ent : m_LosTiles[n])
 						UpdateVisibility(ent, player);
 				}
 			}
@@ -1617,16 +1662,21 @@ public:
 		}
 
 		std::fill(m_GlobalPlayerVisibilityUpdate.begin(), m_GlobalPlayerVisibilityUpdate.end(), 0);
-
-		// Don't bother updating modified entities if the update was global
-		if (!m_GlobalVisibilityUpdate)
-		{
-			for (auto& ent : m_ModifiedEntities)
-				UpdateVisibility(ent);
-		}
-
-		m_ModifiedEntities.clear();
 		m_GlobalVisibilityUpdate = false;
+
+		// Calling UpdateVisibility can modify m_ModifiedEntities, so be careful:
+		// infinite loops could be triggered by feedback between entities and their mirages.
+		std::map<entity_id_t, u8> attempts;
+		while (!m_ModifiedEntities.empty())
+		{
+			entity_id_t ent = m_ModifiedEntities.back();
+			m_ModifiedEntities.pop_back();
+
+			++attempts[ent];
+			ENSURE(attempts[ent] < 100 && "Infinite loop in UpdateVisibilityData");
+
+			UpdateVisibility(ent);
+		}
 	}
 
 	virtual void RequestVisibilityUpdate(entity_id_t ent)
@@ -1646,7 +1696,7 @@ public:
 
 		if (oldVis == newVis)
 			return;
-		
+
 		itEnts->second.visibilities = (itEnts->second.visibilities & ~(0x3 << 2 * (player - 1))) | (newVis << 2 * (player - 1));
 
 		CMessageVisibilityChanged msg(player, ent, oldVis, newVis);
@@ -1654,7 +1704,7 @@ public:
 	}
 
 	void UpdateVisibility(entity_id_t ent)
-	{		
+	{
 		for (player_id_t player = 1; player < MAX_LOS_PLAYER_ID + 1; ++player)
 			UpdateVisibility(ent, player);
 	}
@@ -1690,7 +1740,7 @@ public:
 	{
 		m_LosCircular = enabled;
 
-		ResetDerivedData(false);
+		ResetDerivedData();
 	}
 
 	virtual bool GetLosCircular()
@@ -1698,14 +1748,18 @@ public:
 		return m_LosCircular;
 	}
 
-	virtual void SetSharedLos(player_id_t player, std::vector<player_id_t> players)
+	virtual void SetSharedLos(player_id_t player, const std::vector<player_id_t>& players)
 	{
 		m_SharedLosMasks[player] = CalcSharedLosMask(players);
 
-		u16 oldMask = m_SharedDirtyVisibilityMasks[player];
-		m_SharedDirtyVisibilityMasks[player] = CalcSharedDirtyVisibilityMask(players);
+		// From now on, units belonging to any of 'players' can trigger visibility updates for 'player'.
+		bool modified = false;
 
-		if (oldMask != m_SharedDirtyVisibilityMasks[player] && player <= m_GlobalPlayerVisibilityUpdate.size())
+		for (player_id_t partner : players)
+			if (AddPlayerSharedDirtyVisibilityMask(m_SharedDirtyVisibilityMasks[partner], player))
+				modified = true;
+
+		if (modified && (size_t)player <= m_GlobalPlayerVisibilityUpdate.size())
 			m_GlobalPlayerVisibilityUpdate[player-1] = 1;
 	}
 
@@ -1733,13 +1787,25 @@ public:
 
 	virtual void ExploreTerritories()
 	{
+		PROFILE3("ExploreTerritories");
+
 		CmpPtr<ICmpTerritoryManager> cmpTerritoryManager(GetSystemEntity());
-
 		const Grid<u8>& grid = cmpTerritoryManager->GetTerritoryGrid();
-		ENSURE(grid.m_W == m_TerrainVerticesPerSide-1 && grid.m_H == m_TerrainVerticesPerSide-1);
 
-		// For each tile, if it is owned by a valid player then update the LOS
-		// for every vertex around that tile, to mark them as explored
+		// Territory data is stored per territory-tile (typically a multiple of terrain-tiles).
+		// LOS data is stored per terrain-tile vertex.
+
+		// For each territory-tile, if it is owned by a valid player then update the LOS
+		// for every vertex inside/around that tile, to mark them as explored.
+
+		// Currently this code doesn't support territory-tiles smaller than terrain-tiles
+		// (it will get scale==0 and break), or a non-integer multiple, so check that first
+		cassert(ICmpTerritoryManager::NAVCELLS_PER_TERRITORY_TILE >= Pathfinding::NAVCELLS_PER_TILE);
+		cassert(ICmpTerritoryManager::NAVCELLS_PER_TERRITORY_TILE % Pathfinding::NAVCELLS_PER_TILE == 0);
+
+		int scale = ICmpTerritoryManager::NAVCELLS_PER_TERRITORY_TILE / Pathfinding::NAVCELLS_PER_TILE;
+
+		ENSURE(grid.m_W*scale == m_TerrainVerticesPerSide-1 && grid.m_H*scale == m_TerrainVerticesPerSide-1);
 
 		for (u16 j = 0; j < grid.m_H; ++j)
 		{
@@ -1748,15 +1814,19 @@ public:
 				u8 p = grid.get(i, j) & ICmpTerritoryManager::TERRITORY_PLAYER_MASK;
 				if (p > 0 && p <= MAX_LOS_PLAYER_ID)
 				{
-					u32 &explored = m_ExploredVertices.at(p);
-					explored += !(m_LosState[i + j*m_TerrainVerticesPerSide] & (LOS_EXPLORED << (2*(p-1))));
-					m_LosState[i + j*m_TerrainVerticesPerSide] |= (LOS_EXPLORED << (2*(p-1)));
-					explored += !(m_LosState[i+1 + j*m_TerrainVerticesPerSide] & (LOS_EXPLORED << (2*(p-1))));
-					m_LosState[i+1 + j*m_TerrainVerticesPerSide] |= (LOS_EXPLORED << (2*(p-1)));
-					explored += !(m_LosState[i + (j+1)*m_TerrainVerticesPerSide] & (LOS_EXPLORED << (2*(p-1))));
-					m_LosState[i + (j+1)*m_TerrainVerticesPerSide] |= (LOS_EXPLORED << (2*(p-1)));
-					explored += !(m_LosState[i+1 + (j+1)*m_TerrainVerticesPerSide] & (LOS_EXPLORED << (2*(p-1))));
-					m_LosState[i+1 + (j+1)*m_TerrainVerticesPerSide] |= (LOS_EXPLORED << (2*(p-1)));
+					u32& explored = m_ExploredVertices.at(p);
+					for (int dj = 0; dj <= scale; ++dj)
+					{
+						for (int di = 0; di <= scale; ++di)
+						{
+							u32& losState = m_LosState[(i*scale+di) + (j*scale+dj)*m_TerrainVerticesPerSide];
+							if (!(losState & (LOS_EXPLORED << (2*(p-1)))))
+							{
+								explored++;
+								losState |= (LOS_EXPLORED << (2*(p-1)));
+							}
+						}
+					}
 				}
 			}
 		}
@@ -1806,13 +1876,46 @@ public:
 		}
 	}
 
+	virtual void RevealShore(player_id_t p, bool enable)
+	{
+		if (p <= 0 || p > MAX_LOS_PLAYER_ID)
+			return;
+
+		// Maximum distance to the shore
+		const u16 maxdist = 10;
+
+		CmpPtr<ICmpPathfinder> cmpPathfinder(GetSystemEntity());
+		const Grid<u16>& shoreGrid = cmpPathfinder->ComputeShoreGrid(true);
+		ENSURE(shoreGrid.m_W == m_TerrainVerticesPerSide-1 && shoreGrid.m_H == m_TerrainVerticesPerSide-1);
+
+		std::vector<u16>& counts = m_LosPlayerCounts.at(p);
+		ENSURE(!counts.empty());
+		u16* countsData = &counts[0];
+
+		for (u16 j = 0; j < shoreGrid.m_H; ++j)
+		{
+			for (u16 i = 0; i < shoreGrid.m_W; ++i)
+			{
+				u16 shoredist = shoreGrid.get(i, j);
+				if (shoredist > maxdist)
+					continue;
+
+				// Maybe we could be more clever and don't add dummy strips of one tile
+				if (enable)
+					LosAddStripHelper(p, i, i, j, countsData);
+				else
+					LosRemoveStripHelper(p, i, i, j, countsData);
+			}
+		}
+	}
+
 	/**
 	 * Returns whether the given vertex is outside the normal bounds of the world
 	 * (i.e. outside the range of a circular map)
 	 */
 	inline bool LosIsOffWorld(ssize_t i, ssize_t j)
 	{
-		// WARNING: CCmpObstructionManager::Rasterise needs to be kept in sync with this
+		// WARNING: CCmpPathfinder::UpdateGrid needs to be kept in sync with this
 		const ssize_t edgeSize = 3; // number of vertexes around the edge that will be off-world
 
 		if (m_LosCircular)
@@ -1860,23 +1963,7 @@ public:
 					m_LosState[idx] |= ((LOS_VISIBLE | LOS_EXPLORED) << (2*(owner-1)));
 				}
 
-				// Mark the LoS tiles around the updated vertex
-				// 1: left-up, 2: right-up, 3: left-down, 4: right-down
-				int n1 = ((j-1)/LOS_TILES_RATIO)*m_LosTilesPerSide + (i-1)/LOS_TILES_RATIO;
-				int n2 = ((j-1)/LOS_TILES_RATIO)*m_LosTilesPerSide + i/LOS_TILES_RATIO;
-				int n3 = (j/LOS_TILES_RATIO)*m_LosTilesPerSide + (i-1)/LOS_TILES_RATIO;
-				int n4 = (j/LOS_TILES_RATIO)*m_LosTilesPerSide + i/LOS_TILES_RATIO;
-
-				u16 sharedDirtyVisibilityMask = m_SharedDirtyVisibilityMasks[owner];
-
-				if (j > 0 && i > 0)
-					m_DirtyVisibility[n1] |= sharedDirtyVisibilityMask;
-				if (n2 != n1 && j > 0 && i < m_TerrainVerticesPerSide)
-					m_DirtyVisibility[n2] |= sharedDirtyVisibilityMask;
-				if (n3 != n1 && j < m_TerrainVerticesPerSide && i > 0)
-					m_DirtyVisibility[n3] |= sharedDirtyVisibilityMask;
-				if (n4 != n1 && j < m_TerrainVerticesPerSide && i < m_TerrainVerticesPerSide)
-					m_DirtyVisibility[n4] |= sharedDirtyVisibilityMask;
+				MarkVisibilityDirtyAroundTile(owner, i, j);
 			}
 
 			ASSERT(counts[idx] < 65535);
@@ -1906,28 +1993,36 @@ public:
 				m_LosState[idx] &= ~(LOS_VISIBLE << (2*(owner-1)));
 
 				i32 i = i0 + idx - idx0;
-
-				// Mark the LoS tiles around the updated vertex
-				// 1: left-up, 2: right-up, 3: left-down, 4: right-down
-				int n1 = ((j-1)/LOS_TILES_RATIO)*m_LosTilesPerSide + (i-1)/LOS_TILES_RATIO;
-				int n2 = ((j-1)/LOS_TILES_RATIO)*m_LosTilesPerSide + i/LOS_TILES_RATIO;
-				int n3 = (j/LOS_TILES_RATIO)*m_LosTilesPerSide + (i-1)/LOS_TILES_RATIO;
-				int n4 = (j/LOS_TILES_RATIO)*m_LosTilesPerSide + i/LOS_TILES_RATIO;
-
-				u16 sharedDirtyVisibilityMask = m_SharedDirtyVisibilityMasks[owner];
-
-				if (j > 0 && i > 0)
-					m_DirtyVisibility[n1] |= sharedDirtyVisibilityMask;
-				if (n2 != n1 && j > 0 && i < m_TerrainVerticesPerSide)
-					m_DirtyVisibility[n2] |= sharedDirtyVisibilityMask;
-				if (n3 != n1 && j < m_TerrainVerticesPerSide && i > 0)
-					m_DirtyVisibility[n3] |= sharedDirtyVisibilityMask;
-				if (n4 != n1 && j < m_TerrainVerticesPerSide && i < m_TerrainVerticesPerSide)
-					m_DirtyVisibility[n4] |= sharedDirtyVisibilityMask;
+				MarkVisibilityDirtyAroundTile(owner, i, j);
 			}
 		}
 	}
 
+	inline void MarkVisibilityDirtyAroundTile(u8 owner, i32 i, i32 j)
+	{
+		// If we're still in the deserializing process, we must not modify m_DirtyVisibility
+		if (m_Deserializing)
+			return;
+
+		// Mark the LoS tiles around the updated vertex
+		// 1: left-up, 2: right-up, 3: left-down, 4: right-down
+		int n1 = ((j-1)/LOS_TILES_RATIO)*m_LosTilesPerSide + (i-1)/LOS_TILES_RATIO;
+		int n2 = ((j-1)/LOS_TILES_RATIO)*m_LosTilesPerSide + i/LOS_TILES_RATIO;
+		int n3 = (j/LOS_TILES_RATIO)*m_LosTilesPerSide + (i-1)/LOS_TILES_RATIO;
+		int n4 = (j/LOS_TILES_RATIO)*m_LosTilesPerSide + i/LOS_TILES_RATIO;
+
+		u16 sharedDirtyVisibilityMask = m_SharedDirtyVisibilityMasks[owner];
+
+		if (j > 0 && i > 0)
+			m_DirtyVisibility[n1] |= sharedDirtyVisibilityMask;
+		if (n2 != n1 && j > 0 && i < m_TerrainVerticesPerSide)
+			m_DirtyVisibility[n2] |= sharedDirtyVisibilityMask;
+		if (n3 != n1 && j < m_TerrainVerticesPerSide && i > 0)
+			m_DirtyVisibility[n3] |= sharedDirtyVisibilityMask;
+		if (n4 != n1 && j < m_TerrainVerticesPerSide && i < m_TerrainVerticesPerSide)
+			m_DirtyVisibility[n4] |= sharedDirtyVisibilityMask;
+	}
+
 	/**
 	 * Update the LOS state of tiles within a given circular range,
 	 * either adding or removing visibility depending on the template parameter.
@@ -2189,10 +2284,10 @@ public:
 		return m_ExploredVertices.at((u8)player) * 100 / m_TotalInworldVertices;
 	}
 
-	virtual u8 GetUnionPercentMapExplored(std::vector<player_id_t> players)
+	virtual u8 GetUnionPercentMapExplored(const std::vector<player_id_t>& players)
 	{
 		u32 exploredVertices = 0;
-		std::vector<player_id_t>::iterator playerIt;
+		std::vector<player_id_t>::const_iterator playerIt;
 
 		for (i32 j = 0; j < m_TerrainVerticesPerSide; j++)
 		{
