@@ -1,4 +1,4 @@
-/* Copyright (C) 2014 Wildfire Games.
+/* Copyright (C) 2015 Wildfire Games.
  * This file is part of 0 A.D.
  *
  * 0 A.D. is free software: you can redistribute it and/or modify
@@ -274,7 +274,7 @@ public:
 		for (size_t i = 0; i < m_Units.size(); ++i)
 		{
 			SUnit& unit = m_Units[i];
-			if (!unit.actor)
+			if (!unit.actor || !unit.inWorld)
 				continue;
 			if (unit.sweptBounds.RayIntersect(origin, dir))
 				candidates.push_back(&unit);
@@ -319,7 +319,7 @@ public:
 				
 				center = selectionBox.m_Center;
 			}
-			outEntities.push_back(std::make_pair(unit.entity, center));
+			outEntities.emplace_back(unit.entity, center);
 		}
 	}
 };
@@ -430,7 +430,7 @@ void CCmpUnitRenderer::RenderSubmit(SceneCollector& collector, const CFrustum& f
 			unit.lastTransformFrame = m_FrameNumber;
 		}
 
-		if (culling && !frustum.IsBoxVisible(CVector3D(0, 0, 0), unitModel.GetWorldBoundsRec()))
+		if (culling && !frustum.IsBoxVisible(unitModel.GetWorldBoundsRec()))
 			continue;
 
 		collector.SubmitRecursive(&unitModel);
