@@ -67,7 +67,7 @@ private:
 	bool m_IsFoundationActor;
 	CUnit* m_Unit;
 
-	fixed m_R, m_G, m_B; // shading colour
+	fixed m_R, m_G, m_B; // shading color
 
 	std::map<std::string, std::string> m_AnimOverride;
 
@@ -146,21 +146,31 @@ public:
 						"</element>"
 						"<element name='Box' a:help='Sets the selection shape to a box of specified dimensions'>"
 							"<attribute name='width'>"
-								"<ref name='positiveDecimal' />"
+								"<data type='decimal'>"
+									"<param name='minExclusive'>0.0</param>"
+								"</data>"
 							"</attribute>"
 							"<attribute name='height'>"
-								"<ref name='positiveDecimal' />"
+								"<data type='decimal'>"
+									"<param name='minExclusive'>0.0</param>"
+								"</data>"
 							"</attribute>"
 							"<attribute name='depth'>"
-								"<ref name='positiveDecimal' />"
+								"<data type='decimal'>"
+									"<param name='minExclusive'>0.0</param>"
+								"</data>"
 							"</attribute>"
 						"</element>"
 						"<element name='Cylinder' a:help='Sets the selection shape to a cylinder of specified dimensions'>"
 							"<attribute name='radius'>"
-								"<ref name='positiveDecimal' />"
+								"<data type='decimal'>"
+									"<param name='minExclusive'>0.0</param>"
+								"</data>"
 							"</attribute>"
 							"<attribute name='height'>"
-								"<ref name='positiveDecimal' />"
+								"<data type='decimal'>"
+									"<param name='minExclusive'>0.0</param>"
+								"</data>"
 							"</attribute>"
 						"</element>"
 					"</choice>"
@@ -400,7 +410,7 @@ public:
 		return CVector3D();
 	}
 
-	virtual void SelectAnimation(std::string name, bool once, fixed speed, std::wstring soundgroup)
+	virtual void SelectAnimation(const std::string& name, bool once, fixed speed, const std::wstring& soundgroup)
 	{
 		m_AnimRunThreshold = fixed::Zero();
 		m_AnimName = name;
@@ -418,12 +428,12 @@ public:
 		}
 	}
 
-	virtual void ReplaceMoveAnimation(std::string name, std::string replace)
+	virtual void ReplaceMoveAnimation(const std::string& name, const std::string& replace)
 	{
 		m_AnimOverride[name] = replace;
 	}
 
-	virtual void ResetMoveAnimation(std::string name)
+	virtual void ResetMoveAnimation(const std::string& name)
 	{
 		std::map<std::string, std::string>::const_iterator it = m_AnimOverride.find(name);
 		if (it != m_AnimOverride.end())
@@ -470,7 +480,7 @@ public:
 		}
 	}
 
-	virtual void SetShadingColour(fixed r, fixed g, fixed b, fixed a)
+	virtual void SetShadingColor(fixed r, fixed g, fixed b, fixed a)
 	{
 		m_R = r;
 		m_G = g;
@@ -484,7 +494,7 @@ public:
 		}
 	}
 
-	virtual void SetVariable(std::string name, float value)
+	virtual void SetVariable(const std::string& name, float value)
 	{
 		if (m_Unit)
 		{
