@@ -1,4 +1,4 @@
-/* Copyright (C) 2012 Wildfire Games.
+/* Copyright (C) 2015 Wildfire Games.
  * This file is part of 0 A.D.
  *
  * 0 A.D. is free software: you can redistribute it and/or modify
@@ -20,8 +20,7 @@
 
 #include "simulation2/system/Interface.h"
 
-#include "simulation2/helpers/Grid.h"
-#include "simulation2/helpers/Position.h"
+#include "simulation2/helpers/Pathfinding.h"
 
 #include "maths/FixedVector2D.h"
 
@@ -48,7 +47,7 @@ class IObstructionTestFilter;
  * Units can be marked as either moving or stationary, which simply determines whether
  * certain filters include or exclude them.
  *
- * The @c Rasterise function approximates the current set of shapes onto a 2D grid,
+ * The @c Rasterize function approximates the current set of shapes onto a 2D grid,
  * for use with tile-based pathfinding.
  */
 class ICmpObstructionManager : public IComponent
@@ -113,15 +112,15 @@ public:
 	 * 
 	 * @param ent entity ID associated with this shape (or INVALID_ENTITY if none)
 	 * @param x,z coordinates of center, in world space
-	 * @param r radius of circle or half the unit's width/height
+	 * @param clearance pathfinding clearance of the unit (works as a radius)
 	 * @param flags a set of EFlags values
 	 * @param group control group (typically the owner entity, or a formation controller entity
 	 *	- units ignore collisions with others in the same group)
 	 * @return a valid tag for manipulating the shape
 	 * @see UnitShape
 	 */
-	virtual tag_t AddUnitShape(entity_id_t ent, entity_pos_t x, entity_pos_t z, entity_angle_t r, flags_t flags,
-		entity_id_t group) = 0;
+	virtual tag_t AddUnitShape(entity_id_t ent, entity_pos_t x, entity_pos_t z, entity_pos_t clearance,
+		flags_t flags, entity_id_t group) = 0;
 
 	/**
 	 * Adjust the position and angle of an existing shape.
@@ -169,9 +168,10 @@ public:
 	 * @param x1 X coordinate of line's second point
 	 * @param z1 Z coordinate of line's second point
 	 * @param r radius (half width) of line
+	 * @param relaxClearanceForUnits whether unit-unit collisions should be more permissive.
 	 * @return true if there is a collision
 	 */
-	virtual bool TestLine(const IObstructionTestFilter& filter, entity_pos_t x0, entity_pos_t z0, entity_pos_t x1, entity_pos_t z1, entity_pos_t r) = 0;
+	virtual bool TestLine(const IObstructionTestFilter& filter, entity_pos_t x0, entity_pos_t z0, entity_pos_t x1, entity_pos_t z1, entity_pos_t r, bool relaxClearanceForUnits) = 0;
 
 	/**
 	 * Collision test a static square shape against the current set of shapes.
@@ -195,36 +195,28 @@ public:
 	 * @param filter filter to restrict the shapes that are being tested against
 	 * @param x X coordinate of shape's center
 	 * @param z Z coordinate of shape's center
-	 * @param r radius of the shape (half the unit's width/height)
+	 * @param clearance clearance of the shape's unit
 	 * @param out if non-NULL, all colliding shapes' entities will be added to this list
 	 * 
 	 * @return true if there is a collision
 	 */
 	virtual bool TestUnitShape(const IObstructionTestFilter& filter,
-		entity_pos_t x, entity_pos_t z, entity_pos_t r,
+		entity_pos_t x, entity_pos_t z, entity_pos_t clearance,
 		std::vector<entity_id_t>* out) = 0;
 
 	/**
-	 * Bit-flags for Rasterise.
+	 * Convert the current set of shapes onto a navcell grid, for all passability classes contained in @p passClasses.
+	 * If @p fullUpdate is false, the function will only go through dirty shapes.
+	 * Shapes are expanded by the @p passClasses clearances, by ORing their masks onto the @p grid.
 	 */
-	enum TileObstruction
-	{
-		TILE_OBSTRUCTED_PATHFINDING = (1 << 0), // set if the tile pathfinder should consider this tile blocked
-		TILE_OBSTRUCTED_FOUNDATION = (1 << 1), // set if the AI foundation placement algorithm should consider this tile blocked
-		TILE_OUTOFBOUNDS = (1 << 2) // set if this tile is outside the world boundaries
-	};
+	virtual void Rasterize(Grid<NavcellData>& grid, const std::vector<PathfinderPassability>& passClasses, bool fullUpdate) = 0;
 
 	/**
-	 * Convert the current set of shapes onto a grid.
-	 * Tiles that are intersected by a pathfind-blocking shape will have TILE_OBSTRUCTED_PATHFINDING set;
-	 * tiles that are intersected by a foundation-blocking shape will also have TILE_OBSTRUCTED_FOUNDATION;
-	 * tiles that are outside the world bounds will also have TILE_OUTOFBOUNDS;
-	 * others will be set to 0.
-	 * This is very cheap if the grid has been rasterised before and the set of shapes has not changed.
-	 * @param grid the grid to be updated
-	 * @return true if any changes were made to the grid, false if it was already up-to-date
+	 * Gets dirtiness information and resets it afterwards. Then it's the role of CCmpPathfinder
+	 * to pass the information to other components if needed. (AIs, etc.)
+	 * The return value is false if an update is unnecessary.
 	 */
-	virtual bool Rasterise(Grid<u8>& grid) = 0;
+	virtual void UpdateInformations(GridUpdateInformation& informations) = 0;
 
 	/**
 	 * Standard representation for all types of shapes, for use with geometry processing code.
@@ -246,14 +238,18 @@ public:
 	 * @param squares output list of obstructions
 	 */
 	virtual void GetObstructionsInRange(const IObstructionTestFilter& filter, entity_pos_t x0, entity_pos_t z0, entity_pos_t x1, entity_pos_t z1, std::vector<ObstructionSquare>& squares) = 0;
+	virtual void GetStaticObstructionsInRange(const IObstructionTestFilter& filter, entity_pos_t x0, entity_pos_t z0, entity_pos_t x1, entity_pos_t z1, std::vector<ObstructionSquare>& squares) = 0;
+	virtual void GetUnitObstructionsInRange(const IObstructionTestFilter& filter, entity_pos_t x0, entity_pos_t z0, entity_pos_t x1, entity_pos_t z1, std::vector<ObstructionSquare>& squares) = 0;
 
 	/**
-	 * Find a single obstruction that blocks a unit at the given point with the given radius.
-	 * Static obstructions (buildings) are more important than unit obstructions, and
-	 * obstructions that cover the given point are more important than those that only cover
-	 * the point expanded by the radius.
+	 * Returns the entity IDs of all unit shapes that intersect the given
+	 * obstruction square, filtering out using the given filter.
+	 * @param square the Obstruction squre we want to compare with.
+	 * @param out output list of obstructions
+	 * @param filter filter for the obstructing units
+	 * @param strict whether to be strict in the check or more permissive (ie rasterize more or less). Default false.
 	 */
-	virtual bool FindMostImportantObstruction(const IObstructionTestFilter& filter, entity_pos_t x, entity_pos_t z, entity_pos_t r, ObstructionSquare& square) = 0;
+	virtual void GetUnitsOnObstruction(const ObstructionSquare& square, std::vector<entity_id_t>& out, const IObstructionTestFilter& filter, bool strict = false) = 0;
 
 	/**
 	 * Get the obstruction square representing the given shape.
@@ -261,7 +257,7 @@ public:
 	 */
 	virtual ObstructionSquare GetObstruction(tag_t tag) = 0;
 
-	virtual ObstructionSquare GetUnitShapeObstruction(entity_pos_t x, entity_pos_t z, entity_pos_t r) = 0;
+	virtual ObstructionSquare GetUnitShapeObstruction(entity_pos_t x, entity_pos_t z, entity_pos_t clearance) = 0;
 
 	virtual ObstructionSquare GetStaticShapeObstruction(entity_pos_t x, entity_pos_t z, entity_angle_t a, entity_pos_t w, entity_pos_t h) = 0;
 
@@ -270,6 +266,8 @@ public:
 	 */
 	virtual void SetPassabilityCircular(bool enabled) = 0;
 
+	virtual bool GetPassabilityCircular() const = 0;
+
 	/**
 	 * Toggle the rendering of debug info.
 	 */
@@ -344,10 +342,13 @@ public:
 	{
 		if (group == m_Group || (group2 != INVALID_ENTITY && group2 == m_Group))
 			return false;
+
 		if (!(flags & ICmpObstructionManager::FLAG_BLOCK_MOVEMENT))
 			return false;
+
 		if ((flags & ICmpObstructionManager::FLAG_MOVING) && !m_AvoidMoving)
 			return false;
+
 		return true;
 	}
 };
