@@ -21,10 +21,11 @@
 
 #include "simulation2/system/InterfaceScripted.h"
 
-#include "ps/Overlay.h"
+#include "ps/Shapes.h"
 
 BEGIN_INTERFACE_WRAPPER(Selectable)
 DEFINE_INTERFACE_METHOD_2("SetSelectionHighlight", void, ICmpSelectable, SetSelectionHighlight, CColor, bool)
 END_INTERFACE_WRAPPER(Selectable)
 
 bool ICmpSelectable::ms_EnableDebugOverlays = false;
+bool ICmpSelectable::m_OverrideVisible = true;
