@@ -1,4 +1,4 @@
-/* Copyright (C) 2011 Wildfire Games.
+/* Copyright (C) 2015 Wildfire Games.
  * This file is part of 0 A.D.
  *
  * 0 A.D. is free software: you can redistribute it and/or modify
@@ -40,7 +40,7 @@ class MockPosition : public ICmpPosition
 public:
 	DEFAULT_MOCK_COMPONENT()
 
-	virtual void SetTurretParent(entity_id_t UNUSED(id), CFixedVector3D UNUSED(pos)) {}
+	virtual void SetTurretParent(entity_id_t UNUSED(id), const CFixedVector3D& UNUSED(pos)) {}
 	virtual entity_id_t GetTurretParent() {return INVALID_ENTITY;}
 	virtual void UpdateTurretPosition() {}
 	virtual std::set<entity_id_t>* GetTurrets() { return NULL; }
@@ -89,7 +89,7 @@ public:
 	{
 		ComponentTestHelper test(g_ScriptRuntime);
 
-		ICmpRangeManager* cmp = test.Add<ICmpRangeManager>(CID_RangeManager, "");
+		ICmpRangeManager* cmp = test.Add<ICmpRangeManager>(CID_RangeManager, "", SYSTEM_ENTITY);
 
 		MockVision vision;
 		test.AddMock(100, IID_Vision, vision);
