@@ -1,4 +1,4 @@
-/* Copyright (C) 2014 Wildfire Games.
+/* Copyright (C) 2015 Wildfire Games.
  * This file is part of 0 A.D.
  *
  * 0 A.D. is free software: you can redistribute it and/or modify
@@ -32,6 +32,7 @@
  - @ref messages
  - @ref component-creation
  - @ref schema
+ - @ref system-components
 - @ref allowing-js-interfaces
 - @ref defining-js-components
 - @ref defining-js-interfaces
@@ -237,7 +238,7 @@ Deinit();
 @endcode
 
 The order of <code>Init</code>/<code>Deserialize</code>/<code>Deinit</code> between entities is mostly undefined,
-so they must not rely on other entities or components already existing; @em except that the SYSTEM_ENTITY is
+so they must not rely on other entities or components already existing; @em except that the @c SYSTEM_ENTITY is
 created before anything else and therefore may be used, and that the components for a single entity will be
 processed in the order determined by TypeList.h.
 
@@ -297,12 +298,39 @@ The content will typically be one of:
 - <code>&lt;text/></code>
 - <code>&lt;data type='boolean'/></code>
 - <code>&lt;data type='decimal'/></code>
+- <code>&lt;data type='integer'/></code>
 - <code>&lt;data type='nonNegativeInteger'/></code>
 - <code>&lt;data type='positiveInteger'/></code>
+- <code>&lt;ref name='decimal'/></code>
 - <code>&lt;ref name='nonNegativeDecimal'/></code>
 - <code>&lt;ref name='positiveDecimal'/></code>
 
-(The last two are slightly different since they're not standard data types.)
+
+The <code>&lt;data&gt;</code> elements are native elements, while the <code>&lt;ref&gt;</code> elements are elements added for our engine. These non-native elements allow the definition of an operation that depends on the parent template. Possible operations are "add" and "mul", and can be applied as the example below.
+
+Say the parent template is
+@code 
+<Entity> 
+  <Example> 
+    <Name>Semi-Humanoids</Name> 
+    <Height>9000</Height> 
+    <Eyes/> 
+  </Example> 
+  <!-- ... other components ... --> 
+</Entity> 
+@endcode 
+and the child template appears like
+@code 
+<Entity> 
+  <Example> 
+    <Name>Barney</Name> 
+    <Height op="add">5</Height> 
+    <Eyes/> 
+  </Example> 
+  <!-- ... other components ... --> 
+</Entity> 
+@endcode 
+then Barney would have a height of 9005. 
 
 Elements can be wrapped in <code>&lt;optional></code>.
 Groups of elements can be wrapped in <code>&lt;choice></code> to allow only one of them.
@@ -315,6 +343,19 @@ For early development of a new component, you can set the schema to <code>&lt;re
 If you don't define @c GetSchema, then the default is <code>&lt;empty/></code> (i.e. there must be no elements).
 
 
+@subsection system-components System components
+
+System components are global singleton components of the @c SYSTEM_ENTITY.
+These are added to it in @c CComponentManager::AddSystemComponents, and
+are passed an empty @c paramNode on @c Init.
+
+JS system components can be registered using:
+
+@code
+Engine.RegisterSystemComponentType(IID_ExampleSystem, "ExampleSystem", ExampleSystem);
+@endcode
+
+
 @section allowing-js-interfaces Allowing interfaces to be implemented in JS
 
 If we want to allow both C++ and JS implementations of @c ICmpExample,
