@@ -1,4 +1,4 @@
-/* Copyright (C) 2012 Wildfire Games.
+/* Copyright (C) 2015 Wildfire Games.
  * This file is part of 0 A.D.
  *
  * 0 A.D. is free software: you can redistribute it and/or modify
@@ -67,8 +67,11 @@ void SimRender::ConstructLineOnGround(const CSimContext& context, const std::vec
 	}
 }
 
-void SimRender::ConstructCircleOnGround(const CSimContext& context, float x, float z, float radius,
-		SOverlayLine& overlay, bool floating, float heightOffset)
+static void ConstructCircleOrClosedArc(
+	const CSimContext& context, float x, float z, float radius,
+	bool isCircle,
+	float start, float end,
+	SOverlayLine& overlay, bool floating, float heightOffset)
 {
 	overlay.m_Coords.clear();
 
@@ -85,20 +88,56 @@ void SimRender::ConstructCircleOnGround(const CSimContext& context, float x, flo
 	}
 
 	// Adapt the circle resolution to look reasonable for small and largeish radiuses
-	size_t numPoints = clamp((size_t)(radius*4.0f), (size_t)12, (size_t)48);
+	size_t numPoints = clamp((size_t)(radius*(end-start)), (size_t)12, (size_t)48);
 
-	overlay.m_Coords.reserve((numPoints + 1) * 3);
+	if (!isCircle)
+		overlay.m_Coords.reserve((numPoints + 1 + 2) * 3);
+	else
+		overlay.m_Coords.reserve((numPoints + 1) * 3);
+
+	float cy;
+	if (!isCircle)
+	{
+		// Start at the center point
+		cy = std::max(water, cmpTerrain->GetExactGroundLevel(x, z)) + heightOffset;
+		overlay.m_Coords.push_back(x);
+		overlay.m_Coords.push_back(cy);
+		overlay.m_Coords.push_back(z);
+	}
 
 	for (size_t i = 0; i <= numPoints; ++i) // use '<=' so it's a closed loop
 	{
-		float a = (float)i * 2 * (float)M_PI / (float)numPoints;
-		float px = x + radius * sinf(a);
-		float pz = z + radius * cosf(a);
+		float a = start + (float)i * (end - start) / (float)numPoints;
+		float px = x + radius * cosf(a);
+		float pz = z + radius * sinf(a);
 		float py = std::max(water, cmpTerrain->GetExactGroundLevel(px, pz)) + heightOffset;
 		overlay.m_Coords.push_back(px);
 		overlay.m_Coords.push_back(py);
 		overlay.m_Coords.push_back(pz);
 	}
+
+	if (!isCircle)
+	{
+		// Return to the center point
+		overlay.m_Coords.push_back(x);
+		overlay.m_Coords.push_back(cy);
+		overlay.m_Coords.push_back(z);
+	}
+}
+
+void SimRender::ConstructCircleOnGround(
+	const CSimContext& context, float x, float z, float radius,
+	SOverlayLine& overlay, bool floating, float heightOffset)
+{
+	ConstructCircleOrClosedArc(context, x, z, radius, true, 0.0f, 2.0f*(float)M_PI, overlay, floating, heightOffset);
+}
+
+void SimRender::ConstructClosedArcOnGround(
+	const CSimContext& context, float x, float z, float radius,
+	float start, float end,
+	SOverlayLine& overlay, bool floating, float heightOffset)
+{
+	ConstructCircleOrClosedArc(context, x, z, radius, false, start, end, overlay, floating, heightOffset);
 }
 
 // This method splits up a straight line into a number of line segments each having a length ~= TERRAIN_TILE_SIZE
@@ -112,10 +151,10 @@ static void SplitLine(std::vector<std::pair<float, float> >& coords, float x1, f
 		float yPieceLength = (y1 - y2) / (float)pieces;
 		for (size_t i = 1; i <= (pieces - 1); ++i)
 		{
-			coords.push_back(std::make_pair(x1 - (xPieceLength * (float)i), y1 - (yPieceLength * (float)i)));
+			coords.emplace_back(x1 - (xPieceLength * (float)i), y1 - (yPieceLength * (float)i));
 		}
 	}
-	coords.push_back(std::make_pair(x2, y2));
+	coords.emplace_back(x2, y2);
 }
 
 void SimRender::ConstructSquareOnGround(const CSimContext& context, float x, float z, float w, float h, float a,
@@ -143,7 +182,7 @@ void SimRender::ConstructSquareOnGround(const CSimContext& context, float x, flo
 	// Add the first vertex, since SplitLine will be adding only the second end-point of the each line to
 	// the coordinates list. We don't have to worry about the other lines, since the end-point of one line
 	// will be the starting point of the next
-	coords.push_back(std::make_pair(x - w/2*c + h/2*s, z + w/2*s + h/2*c));
+	coords.emplace_back(x - w/2*c + h/2*s, z + w/2*s + h/2*c);
 
 	SplitLine(coords, x - w/2*c + h/2*s, z + w/2*s + h/2*c, x - w/2*c - h/2*s, z + w/2*s - h/2*c);
 	SplitLine(coords, x - w/2*c - h/2*s, z + w/2*s - h/2*c, x + w/2*c - h/2*s, z - w/2*s - h/2*c);
