@@ -18,7 +18,7 @@
 #ifndef INCLUDED_SIMULATIONCOMMAND
 #define INCLUDED_SIMULATIONCOMMAND
 
-#include "scriptinterface/ScriptVal.h"
+#include "scriptinterface/ScriptInterface.h"
 #include "simulation2/helpers/Player.h"
 
 /**
@@ -26,8 +26,27 @@
  */
 struct SimulationCommand
 {
+	SimulationCommand(player_id_t player, JSContext* cx, JS::HandleValue val)
+		: player(player), data(cx, val)
+	{
+	}
+
+	SimulationCommand(SimulationCommand&& cmd)
+		: player(cmd.player), data(cmd.data)
+	{
+	}
+
+	// std::vector::insert requires the move assignment operator at compilation time,
+	// but apparently never uses it (it uses the move constructor).
+	SimulationCommand& operator=(SimulationCommand&& other)
+	{
+		this->player = other.player;
+		this->data.set(other.data);
+		return *this;
+	}
+
 	player_id_t player;
-	CScriptValRooted data;
+	JS::PersistentRootedValue data;
 };
 
 #endif // INCLUDED_SIMULATIONCOMMAND
