@@ -149,9 +149,9 @@ void CDebugSerializer::PutString(const char* name, const std::string& value)
 
 void CDebugSerializer::PutScriptVal(const char* name, JS::MutableHandleValue value)
 {
-	std::wstring source = m_ScriptInterface.ToString(value, true);
+	std::string source = m_ScriptInterface.ToString(value, true);
 
-	m_Stream << INDENT << name << ": " << utf8_from_wstring(source) << "\n";
+	m_Stream << INDENT << name << ": " << source << "\n";
 }
 
 void CDebugSerializer::PutRaw(const char* name, const u8* data, size_t len)
