@@ -1,4 +1,4 @@
-/* Copyright (C) 2013 Wildfire Games.
+/* Copyright (C) 2016 Wildfire Games.
  * This file is part of 0 A.D.
  *
  * 0 A.D. is free software: you can redistribute it and/or modify
@@ -55,10 +55,10 @@ void CStdDeserializer::Trace(JSTracer *trc, void *data)
 
 void CStdDeserializer::TraceMember(JSTracer *trc)
 {
-	for (size_t i=0; i<m_ScriptBackrefs.size(); i++)
+	for (size_t i=0; i<m_ScriptBackrefs.size(); ++i)
 		JS_CallHeapObjectTracer(trc, &m_ScriptBackrefs[i], "StdDeserializer::m_ScriptBackrefs");
 
-	for (auto& proto : m_SerializablePrototypes)
+	for (std::pair<const std::wstring, JS::Heap<JSObject*>>& proto : m_SerializablePrototypes)
 		JS_CallHeapObjectTracer(trc, &proto.second, "StdDeserializer::m_SerializablePrototypes");
 }
 
@@ -81,8 +81,14 @@ void CStdDeserializer::Get(const char* name, u8* data, size_t len)
 	UNUSED2(name);
 #endif
 	m_Stream.read((char*)data, (std::streamsize)len);
-	if (!m_Stream.good()) // hit eof before len, or other errors
-		throw PSERROR_Deserialize_ReadFailed();
+	if (!m_Stream.good())
+	{
+		// hit eof before len, or other errors
+		// NOTE: older libc++ versions incorrectly set eofbit on the last char; test gcount as a workaround
+		// see https://llvm.org/bugs/show_bug.cgi?id=9335
+		if (m_Stream.bad() || m_Stream.fail() || (m_Stream.eof() && m_Stream.gcount() != (std::streamsize)len))
+			throw PSERROR_Deserialize_ReadFailed();
+	}
 }
 
 std::istream& CStdDeserializer::GetStream()
@@ -196,8 +202,8 @@ jsval CStdDeserializer::ReadScriptVal(const char* UNUSED(name), JS::HandleObject
 			if (hasCustomDeserialize)
 			{
 				JS::RootedValue serialize(cx);
-				if (!JS_LookupProperty(cx, obj, "Serialize", &serialize))
-					throw PSERROR_Serialize_ScriptError("JS_LookupProperty failed");
+				if (!JS_GetProperty(cx, obj, "Serialize", &serialize))
+					throw PSERROR_Serialize_ScriptError("JS_GetProperty failed");
 				bool hasNullSerialize = hasCustomSerialize && serialize.isNull();
 
 				// If Serialize is null, we'll still call Deserialize but with undefined argument
@@ -228,7 +234,7 @@ jsval CStdDeserializer::ReadScriptVal(const char* UNUSED(name), JS::HandleObject
 			ReadStringUTF16("prop name", propname);
 			JS::RootedValue propval(cx, ReadScriptVal("prop value", JS::NullPtr()));
 
-			if (!JS_SetUCProperty(cx, obj, (const jschar*)propname.data(), propname.length(), propval))
+			if (!JS_SetUCProperty(cx, obj, (const char16_t*)propname.data(), propname.length(), propval))
 				throw PSERROR_Deserialize_ScriptError();
 		}
 
@@ -281,7 +287,7 @@ jsval CStdDeserializer::ReadScriptVal(const char* UNUSED(name), JS::HandleObject
 		if (!JS_GetClassObject(cx, JSProto_Number, &ctorobj))
 			throw PSERROR_Deserialize_ScriptError("JS_GetClassObject failed");
 
-		JS::RootedObject obj(cx, JS_New(cx, ctorobj, val));
+		JS::RootedObject obj(cx, JS_New(cx, ctorobj, JS::HandleValueArray(val)));
 		if (!obj)
 			throw PSERROR_Deserialize_ScriptError("JS_New failed");
 		AddScriptBackref(obj);
@@ -299,7 +305,7 @@ jsval CStdDeserializer::ReadScriptVal(const char* UNUSED(name), JS::HandleObject
 		if (!JS_GetClassObject(cx, JSProto_String, &ctorobj))
 			throw PSERROR_Deserialize_ScriptError("JS_GetClassObject failed");
 
-		JS::RootedObject obj(cx, JS_New(cx, ctorobj, val));
+		JS::RootedObject obj(cx, JS_New(cx, ctorobj, JS::HandleValueArray(val)));
 		if (!obj)
 			throw PSERROR_Deserialize_ScriptError("JS_New failed");
 		AddScriptBackref(obj);
@@ -315,7 +321,7 @@ jsval CStdDeserializer::ReadScriptVal(const char* UNUSED(name), JS::HandleObject
 		if (!JS_GetClassObject(cx, JSProto_Boolean, &ctorobj))
 			throw PSERROR_Deserialize_ScriptError("JS_GetClassObject failed");
 
-		JS::RootedObject obj(cx, JS_New(cx, ctorobj, val));
+		JS::RootedObject obj(cx, JS_New(cx, ctorobj, JS::HandleValueArray(val)));
 		if (!obj)
 			throw PSERROR_Deserialize_ScriptError("JS_New failed");
 		AddScriptBackref(obj);
@@ -390,8 +396,8 @@ jsval CStdDeserializer::ReadScriptVal(const char* UNUSED(name), JS::HandleObject
 #if BYTE_ORDER != LITTLE_ENDIAN
 #error TODO: need to convert JS ArrayBuffer data from little-endian
 #endif
-		void* contents = NULL;
-		contents = JS_AllocateArrayBufferContents(cx, length);
+		void* contents = malloc(length);
+		ENSURE(contents);
 		RawBytes("buffer data", (u8*)contents, length);
 		JS::RootedObject bufferObj(cx, JS_NewArrayBufferWithContents(cx, length, contents));
 		AddScriptBackref(bufferObj);
@@ -404,6 +410,9 @@ jsval CStdDeserializer::ReadScriptVal(const char* UNUSED(name), JS::HandleObject
 		NumberU32_Unbounded("map size", mapSize);
 		JS::RootedValue mapVal(cx);
 		m_ScriptInterface.Eval("(new Map())", &mapVal);
+
+		// To match the serializer order, we reserve the map's backref tag here
+		u32 mapTag = ReserveScriptBackref();
 		
 		for (u32 i=0; i<mapSize; ++i)
 		{
@@ -412,9 +421,28 @@ jsval CStdDeserializer::ReadScriptVal(const char* UNUSED(name), JS::HandleObject
 			m_ScriptInterface.CallFunctionVoid(mapVal, "set", key, value);
 		}
 		JS::RootedObject mapObj(cx, &mapVal.toObject());
-		AddScriptBackref(mapObj);
+		SetReservedScriptBackref(mapTag, mapObj);
 		return mapVal;
 	}
+	case SCRIPT_TYPE_OBJECT_SET:
+	{
+		u32 setSize;
+		NumberU32_Unbounded("set size", setSize);
+		JS::RootedValue setVal(cx);
+		m_ScriptInterface.Eval("(new Set())", &setVal);
+
+		// To match the serializer order, we reserve the set's backref tag here
+		u32 setTag = ReserveScriptBackref();
+
+		for (u32 i=0; i<setSize; ++i)
+		{
+			JS::RootedValue value(cx, ReadScriptVal("set value", JS::NullPtr()));
+			m_ScriptInterface.CallFunctionVoid(setVal, "add", value);
+		}
+		JS::RootedObject setObj(cx, &setVal.toObject());
+		SetReservedScriptBackref(setTag, setObj);
+		return setVal;
+	}
 	default:
 		throw PSERROR_Deserialize_OutOfBounds();
 	}
@@ -438,7 +466,7 @@ void CStdDeserializer::ScriptString(const char* name, JS::MutableHandleString ou
 #error TODO: probably need to convert JS strings from little-endian
 #endif
 
-	out.set(JS_NewUCStringCopyN(m_ScriptInterface.GetContext(), (const jschar*)str.data(), str.length()));
+	out.set(JS_NewUCStringCopyN(m_ScriptInterface.GetContext(), (const char16_t*)str.data(), str.length()));
 	if (!out)
 		throw PSERROR_Deserialize_ScriptError("JS_NewUCStringCopyN failed");
 }
