@@ -1,4 +1,4 @@
-/* Copyright (C) 2014 Wildfire Games.
+/* Copyright (C) 2015 Wildfire Games.
  * This file is part of 0 A.D.
  *
  * 0 A.D. is free software: you can redistribute it and/or modify
@@ -42,10 +42,10 @@ void CParamNode::LoadXML(CParamNode& ret, const XMBFile& xmb, const wchar_t* sou
 	ret.ApplyLayer(xmb, xmb.GetRoot(), sourceIdentifier);
 }
 
-void CParamNode::LoadXML(CParamNode& ret, const VfsPath& path)
+void CParamNode::LoadXML(CParamNode& ret, const VfsPath& path, const std::string& validatorName)
 {
 	CXeromyces xero;
-	PSRETURN ok = xero.Load(g_VFS, path);
+	PSRETURN ok = xero.Load(g_VFS, path, validatorName);
 	if (ok != PSRETURN_OK)
 		return; // (Xeromyces already logged an error)
 
@@ -76,7 +76,13 @@ void CParamNode::ApplyLayer(const XMBFile& xmb, const XMBElement& element, const
 	// Look for special attributes
 	int at_disable = xmb.GetAttributeID("disable");
 	int at_replace = xmb.GetAttributeID("replace");
+	int at_op = xmb.GetAttributeID("op");
 	int at_datatype = xmb.GetAttributeID("datatype");
+	enum op {
+		INVALID,
+		ADD,
+		MUL
+	} op = INVALID;
 	bool replacing = false;
 	{
 		XERO_ITER_ATTR(element, attr)
@@ -91,6 +97,15 @@ void CParamNode::ApplyLayer(const XMBFile& xmb, const XMBElement& element, const
 				m_Childs.erase(name);
 				replacing = true;
 			}
+			else if (attr.Name == at_op)
+			{
+				if (std::wstring(attr.Value.begin(), attr.Value.end()) == L"add") 
+					op = ADD; 
+				else if (std::wstring(attr.Value.begin(), attr.Value.end()) == L"mul") 
+					op = MUL; 
+				else 
+					LOGWARNING("Invalid op '%ls'", attr.Value); 
+			}
 		}
 	}
 	{
@@ -137,6 +152,22 @@ void CParamNode::ApplyLayer(const XMBFile& xmb, const XMBElement& element, const
 
 	// Add this element as a child node
 	CParamNode& node = m_Childs[name];
+	if (op != INVALID) 
+	{ 
+		// TODO: Support parsing of data types other than fixed; log warnings in other cases
+		fixed oldval = node.ToFixed();
+		fixed mod = fixed::FromString(CStrW(value));
+		switch (op) 
+		{ 
+		case ADD: 
+			node.m_Value = (oldval + mod).ToString().FromUTF8(); 
+			break; 
+		case MUL: 
+			node.m_Value = (oldval.Multiply(mod)).ToString().FromUTF8(); 
+			break; 
+		} 
+		hasSetValue = true; 
+	} 
 	if (!hasSetValue)
 		node.m_Value = value;
 
@@ -150,7 +181,8 @@ void CParamNode::ApplyLayer(const XMBFile& xmb, const XMBElement& element, const
 	XERO_ITER_ATTR(element, attr)
 	{
 		// Skip special attributes
-		if (attr.Name == at_replace) continue;
+		if (attr.Name == at_replace || attr.Name == at_op)
+			continue;
 		// Add any others
 		std::string attrName = xmb.GetAttributeString(attr.Name);
 		node.m_Childs["@" + attrName].m_Value = attr.Value.FromUTF8();
@@ -331,7 +363,7 @@ void CParamNode::ConstructJSVal(JSContext* cx, JS::MutableHandleValue ret) const
 
 		// Just a string
 		utf16string text(m_Value.begin(), m_Value.end());
-		JS::RootedString str(cx, JS_InternUCStringN(cx, reinterpret_cast<const jschar*>(text.data()), text.length()));
+		JS::RootedString str(cx, JS_InternUCStringN(cx, reinterpret_cast<const char16_t*>(text.data()), text.length()));
 		if (str)
 		{
 			ret.setString(str);
@@ -366,7 +398,7 @@ void CParamNode::ConstructJSVal(JSContext* cx, JS::MutableHandleValue ret) const
 	if (!m_Value.empty())
 	{
 		utf16string text(m_Value.begin(), m_Value.end());
-		JS::RootedString str(cx, JS_InternUCStringN(cx, reinterpret_cast<const jschar*>(text.data()), text.length()));
+		JS::RootedString str(cx, JS_InternUCStringN(cx, reinterpret_cast<const char16_t*>(text.data()), text.length()));
 		if (!str)
 		{
 			ret.setUndefined();
