@@ -1,4 +1,4 @@
-/* Copyright (C) 2013 Wildfire Games.
+/* Copyright (C) 2015 Wildfire Games.
  * This file is part of 0 A.D.
  *
  * 0 A.D. is free software: you can redistribute it and/or modify
@@ -167,7 +167,8 @@ public:
 		deserialize.NumberI32("z", n, 0, 65535);
 		TS_ASSERT_EQUALS(n, 12345);
 
-		TS_ASSERT(stream.good());
+		// NOTE: Don't use good() here - it fails due to a bug in older libc++ versions
+		TS_ASSERT(!stream.bad() && !stream.fail());
 		TS_ASSERT_EQUALS(stream.peek(), EOF);
 	}
 
@@ -233,7 +234,8 @@ public:
 		deserialize.RawBytes("raw bytes", cbuf, 6);
 		TS_ASSERT_SAME_DATA(cbuf, (const u8*)"\0\1\2\3\x0f\x10\x42", 7);
 
-		TS_ASSERT(stream.good());
+		// NOTE: Don't use good() here - it fails due to a bug in older libc++ versions
+		TS_ASSERT(!stream.bad() && !stream.fail());
 		TS_ASSERT_EQUALS(stream.peek(), EOF);
 	}
 
@@ -248,7 +250,30 @@ public:
 
 		TS_ASSERT_EQUALS(serialize.GetHashLength(), (size_t)16);
 		TS_ASSERT_SAME_DATA(serialize.ComputeHash(), "\xa0\x3a\xe5\x3e\x9b\xd7\xfb\x11\x88\x35\xc6\xfb\xb9\x94\xa9\x72", 16);
-		// echo -en "\x85\xff\xff\xff\xd2\x04\x00\x00\x39\x30\x00\x00" | openssl md5 | perl -pe 's/(..)/\\x$1/g'
+		// echo -en "\x85\xff\xff\xff\xd2\x04\x00\x00\x39\x30\x00\x00" | openssl md5 -binary | xxd -p | perl -pe 's/(..)/\\x$1/g'
+	}
+
+	void test_Hash_stream()
+	{
+		ScriptInterface script("Test", "Test", g_ScriptRuntime);
+		CHashSerializer hashSerialize(script);
+
+		hashSerialize.NumberI32_Unbounded("x", -123);
+		hashSerialize.NumberU32_Unbounded("y", 1234);
+		hashSerialize.NumberI32("z", 12345, 0, 65535);
+
+		ISerializer& serialize = hashSerialize;
+
+		{
+			CStdSerializer streamSerialize(script, serialize.GetStream());
+			streamSerialize.NumberI32_Unbounded("x2", -456);
+			streamSerialize.NumberU32_Unbounded("y2", 5678);
+			streamSerialize.NumberI32("z2", 45678, 0, 65535);
+		}
+
+		TS_ASSERT_EQUALS(hashSerialize.GetHashLength(), (size_t)16);
+		TS_ASSERT_SAME_DATA(hashSerialize.ComputeHash(), "\x5c\xff\x33\xd1\x72\xdd\x6d\x77\xa8\xd4\xa1\xf6\x84\xcc\xaa\x10", 16);
+		// echo -en "\x85\xff\xff\xff\xd2\x04\x00\x00\x39\x30\x00\x00\x38\xfe\xff\xff\x2e\x16\x00\x00\x6e\xb2\x00\x00" | openssl md5 -binary | xxd -p | perl -pe 's/(..)/\\x$1/g'
 	}
 
 	void test_bounds()
@@ -264,83 +289,7 @@ public:
 
 	// TODO: test exceptions more thoroughly
 
-	void test_script_basic()
-	{
-		ScriptInterface script("Test", "Test", g_ScriptRuntime);
-		JSContext* cx = script.GetContext();
-		JSAutoRequest rq(cx);
-		
-		JS::RootedValue obj(cx);
-		TS_ASSERT(script.Eval("({'x': 123, 'y': [1, 1.5, '2', 'test', undefined, null, true, false]})", &obj));
-
-		{
-			std::stringstream stream;
-			CDebugSerializer serialize(script, stream);
-			serialize.ScriptVal("script", &obj);
-			TS_ASSERT_STR_EQUALS(stream.str(),
-					"script: {\n"
-					"  \"x\": 123,\n"
-					"  \"y\": [\n"
-					"    1,\n"
-					"    1.5,\n"
-					"    \"2\",\n"
-					"    \"test\",\n"
-					"    null,\n"
-					"    null,\n"
-					"    true,\n"
-					"    false\n"
-					"  ]\n"
-					"}\n");
-		}
-
-		{
-			std::stringstream stream;
-			CStdSerializer serialize(script, stream);
-
-			serialize.ScriptVal("script", &obj);
-
-			TS_ASSERT_STREAM(stream, 119,
-					"\x03" // SCRIPT_TYPE_OBJECT
-					"\x02\0\0\0" // num props
-					"\x01\0\0\0" "x\0" // "x"
-					"\x05" // SCRIPT_TYPE_INT
-					"\x7b\0\0\0" // 123
-					"\x01\0\0\0" "y\0" // "y"
-					"\x02" // SCRIPT_TYPE_ARRAY
-					"\x08\0\0\0" // array length
-					"\x08\0\0\0" // num props
-					"\x01\0\0\0" "0\0" // "0"
-					"\x05" "\x01\0\0\0" // SCRIPT_TYPE_INT 1
-					"\x01\0\0\0" "1\0" // "1"
-					"\x06" "\0\0\0\0\0\0\xf8\x3f" // SCRIPT_TYPE_DOUBLE 1.5
-					"\x01\0\0\0" "2\0" // "2"
-					"\x04" "\x01\0\0\0" "2\0" // SCRIPT_TYPE_STRING "2"
-					"\x01\0\0\0" "3\0" // "3"
-					"\x04" "\x04\0\0\0" "t\0e\0s\0t\0" // SCRIPT_TYPE_STRING "test"
-					"\x01\0\0\0" "4\0" // "4"
-					"\x00" // SCRIPT_TYPE_VOID
-					"\x01\0\0\0" "5\0" // "5"
-					"\x01" // SCRIPT_TYPE_NULL
-					"\x01\0\0\0" "6\0" // "6"
-					"\x07" "\x01" // SCRIPT_TYPE_BOOLEAN true
-					"\x01\0\0\0" "7\0" // "7"
-					"\x07" "\x00" // SCRIPT_TYPE_BOOLEAN false
-			);
-
-			CStdDeserializer deserialize(script, stream);
-
-			JS::RootedValue newobj(cx);
-			deserialize.ScriptVal("script", &newobj);
-			TS_ASSERT(stream.good());
-			TS_ASSERT_EQUALS(stream.peek(), EOF);
-
-			std::string source;
-			TS_ASSERT(script.CallFunction(newobj, "toSource", source));
-			TS_ASSERT_STR_EQUALS(source, "({x:123, y:[1, 1.5, \"2\", \"test\", (void 0), null, true, false]})");
-		}
-	}
-
-	void helper_script_roundtrip(const char* msg, const char* input, const char* expected, size_t expstreamlen = 0, const char* expstream = NULL)
+	void helper_script_roundtrip(const char* msg, const char* input, const char* expected, size_t expstreamlen = 0, const char* expstream = NULL, const char* debug = NULL)
 	{
 		ScriptInterface script("Test", "Test", g_ScriptRuntime);
 		JSContext* cx = script.GetContext();
@@ -349,6 +298,14 @@ public:
 		JS::RootedValue obj(cx);
 		TSM_ASSERT(msg, script.Eval(input, &obj));
 
+		if (debug)
+		{
+			std::stringstream dbgstream;
+			CDebugSerializer serialize(script, dbgstream);
+			serialize.ScriptVal("script", &obj);
+			TS_ASSERT_STR_EQUALS(dbgstream.str(), debug);
+		}
+
 		std::stringstream stream;
 		CStdSerializer serialize(script, stream);
 
@@ -363,7 +320,8 @@ public:
 
 		JS::RootedValue newobj(cx);
 		deserialize.ScriptVal("script", &newobj);
-		TSM_ASSERT(msg, stream.good());
+		// NOTE: Don't use good() here - it fails due to a bug in older libc++ versions
+		TSM_ASSERT(msg, !stream.bad() && !stream.fail());
 		TSM_ASSERT_EQUALS(msg, stream.peek(), EOF);
 
 		std::string source;
@@ -371,6 +329,56 @@ public:
 		TS_ASSERT_STR_EQUALS(source, expected);
 	}
 
+	void test_script_basic()
+	{
+		helper_script_roundtrip("Object",
+			"({'x': 123, 'y': [1, 1.5, '2', 'test', undefined, null, true, false]})",
+		/* expected: */
+			"({x:123, y:[1, 1.5, \"2\", \"test\", (void 0), null, true, false]})",
+		/* expected stream: */
+			119,
+			"\x03" // SCRIPT_TYPE_OBJECT
+			"\x02\0\0\0" // num props
+			"\x01\0\0\0" "x\0" // "x"
+			"\x05" // SCRIPT_TYPE_INT
+			"\x7b\0\0\0" // 123
+			"\x01\0\0\0" "y\0" // "y"
+			"\x02" // SCRIPT_TYPE_ARRAY
+			"\x08\0\0\0" // array length
+			"\x08\0\0\0" // num props
+			"\x01\0\0\0" "0\0" // "0"
+			"\x05" "\x01\0\0\0" // SCRIPT_TYPE_INT 1
+			"\x01\0\0\0" "1\0" // "1"
+			"\x06" "\0\0\0\0\0\0\xf8\x3f" // SCRIPT_TYPE_DOUBLE 1.5
+			"\x01\0\0\0" "2\0" // "2"
+			"\x04" "\x01\0\0\0" "2\0" // SCRIPT_TYPE_STRING "2"
+			"\x01\0\0\0" "3\0" // "3"
+			"\x04" "\x04\0\0\0" "t\0e\0s\0t\0" // SCRIPT_TYPE_STRING "test"
+			"\x01\0\0\0" "4\0" // "4"
+			"\x00" // SCRIPT_TYPE_VOID
+			"\x01\0\0\0" "5\0" // "5"
+			"\x01" // SCRIPT_TYPE_NULL
+			"\x01\0\0\0" "6\0" // "6"
+			"\x07" "\x01" // SCRIPT_TYPE_BOOLEAN true
+			"\x01\0\0\0" "7\0" // "7"
+			"\x07" "\x00", // SCRIPT_TYPE_BOOLEAN false
+		/* expected debug: */
+			"script: {\n"
+			"  \"x\": 123,\n"
+			"  \"y\": [\n"
+			"    1,\n"
+			"    1.5,\n"
+			"    \"2\",\n"
+			"    \"test\",\n"
+			"    null,\n"
+			"    null,\n"
+			"    true,\n"
+			"    false\n"
+			"  ]\n"
+			"}\n"
+		);
+	}
+
 	void test_script_unicode()
 	{
 		helper_script_roundtrip("unicode", "({"
@@ -403,6 +411,11 @@ public:
 		helper_script_roundtrip("Boolean with props", "var b=new Boolean('true'); b.foo='bar'; b", "(new Boolean(true))");
 	}
 
+	void test_script_objects_properties()
+	{
+		helper_script_roundtrip("Object with null in prop name", "({\"foo\\0bar\":1})", "({\'foo\\x00bar\':1})");
+	}
+
 	void test_script_typed_arrays_simple()
 	{
 		helper_script_roundtrip("Int8Array",
@@ -490,10 +503,10 @@ public:
 	void test_script_typed_arrays_complex()
 	{
 		helper_script_roundtrip("ArrayBuffer with Int16Array",
-			"var buf=new ArrayBuffer(16);"
-			"var int16=Int16Array(buf);"
-			"for(var i=0; i<int16.length; ++i)"
-			"  int16[i]=(i+1)*8192;"
+			"var buf = new ArrayBuffer(16);"
+			"var int16 = new Int16Array(buf);"
+			"for(var i = 0; i < int16.length; ++i)"
+			"  int16[i] = (i+1)*8192;"
 			"int16",
 		/* expected: */
 			"({0:8192, 1:16384, 2:24576, 3:-32768, 4:-24576, 5:-16384, 6:-8192, 7:0})"
@@ -501,7 +514,7 @@ public:
 
 		helper_script_roundtrip("ArrayBuffer with Int16Array and Uint32Array",
 			"var buf = new ArrayBuffer(16);"
-			"var int16 = Int16Array(buf);"
+			"var int16 = new Int16Array(buf);"
 			"for(var i=0; i < int16.length; ++i)"
 			"  int16[i] = (i+1)*32768;"
 			"var uint32 = new Uint32Array(buf);"
@@ -515,16 +528,16 @@ public:
 
 		helper_script_roundtrip("ArrayBuffer with complex structure",
 			"var buf=new ArrayBuffer(16);" // 16 bytes
-			"var chunk1=Int8Array(buf, 0, 4);" // 4 bytes
-			"var chunk2=Uint16Array(buf, 4, 2);" // 4 bytes
-			"var chunk3=Int32Array(buf, 8, 2);" // 8 bytes
-			"for(var i=0; i<chunk1.length; ++i)"
-			"  chunk1[i]=255;"
-			"for(var i=0; i<chunk2.length; ++i)"
-			"  chunk2[i]=65535;"
-			"for(var i=0; i<chunk3.length; ++i)"
-			"  chunk3[i]=4294967295;"
-			"var bytes = Uint8Array(buf);"
+			"var chunk1 = new Int8Array(buf, 0, 4);" // 4 bytes
+			"var chunk2 = new Uint16Array(buf, 4, 2);" // 4 bytes
+			"var chunk3 = new Int32Array(buf, 8, 2);" // 8 bytes
+			"for(var i = 0; i < chunk1.length; ++i)"
+			"  chunk1[i] = 255;"
+			"for(var i = 0; i < chunk2.length; ++i)"
+			"  chunk2[i] = 65535;"
+			"for(var i = 0; i < chunk3.length; ++i)"
+			"  chunk3[i] = 4294967295;"
+			"var bytes = new Uint8Array(buf);"
 			"({'struct':[chunk1, chunk2, chunk3], 'bytes':bytes})",
 		/* expected: */ "({"
 				"struct:[{0:-1, 1:-1, 2:-1, 3:-1}, {0:65535, 1:65535}, {0:-1, 1:-1}], "
@@ -533,6 +546,168 @@ public:
 		);
 	}
 
+	void test_script_map()
+	{
+		helper_script_roundtrip("Map - empty",
+			"(new Map())",
+		/* expected: */
+			"({})",
+		/* expected stream: */
+			5,
+			"\x0f" // SCRIPT_TYPE_MAP
+			"\x00\x00\x00\x00" // size
+		);
+
+		helper_script_roundtrip("Map with elements and property",
+			"var a = new Map(); a.set(12, 16); a.set(\"bar\", \"baz\"); a.foo = 27; a",
+		/* expected: */
+			"({})",
+		/* expected stream: */
+			37,
+			"\x0f" // SCRIPT_TYPE_MAP
+			"\x02\0\0\0" // size
+
+			"\x05" // SCRIPT_TYPE_INT
+			"\x0C\0\0\0" // 12
+			"\x05" // SCRIPT_TYPE_INT
+			"\x10\0\0\0" // 16
+
+			"\x04" // SCRIPT_TYPE_STRING
+			"\x03\0\0\0" "b\0a\0r\0" // "bar"
+			"\x04" // SCRIPT_TYPE_STRING
+			"\x03\0\0\0" "b\0a\0z\0" // "baz"
+			// NOTE: We drop properties on Maps when serializing
+		);
+	}
+
+	void test_script_set()
+	{
+		helper_script_roundtrip("Set - empty",
+			"(new Set())",
+		/* expected: */
+			"({})",
+		/* expected stream: */
+			5,
+			"\x10" // SCRIPT_TYPE_SET
+			"\x00\x00\x00\x00" // size
+		);
+
+		helper_script_roundtrip("Set with elements and property", 
+			"var a = new Set(); a.add(12); a.add(\"bar\"); a.foo = 27; a",
+		/* expected: */
+			"({})",
+		/* expected stream: */
+			21,
+			"\x10" // SCRIPT_TYPE_SET
+			"\x02\0\0\0" // size
+
+			"\x05" // SCRIPT_TYPE_INT
+			"\x0C\0\0\0" // 12
+
+			"\x04" // SCRIPT_TYPE_STRING
+			"\x03\0\0\0" "b\0a\0r\0" // "bar"
+			// NOTE: We drop properties on Sets when serializing
+		);
+	}
+
+	void test_script_map_nested()
+	{
+		helper_script_roundtrip("Nested maps",
+			"var a = new Map();"
+			"a.set(\"Builder/Range\", new Map());"
+			"a.get(\"Builder/Range\").set(\"155\", { \"add\": 0, \"multiply\": 1});"
+			"a.get(\"Builder/Range\").set(\"2300\", { \"add\": 0, \"multiply\": 1.1});"
+			"a.get(\"Builder/Range\").set(\"159\", { \"add\": 0, \"multiply\": 1});"
+			"a",
+		/* expected: */
+			"({})",
+		/* expected stream: */
+			215,
+			"\x0f" // SCRIPT_TYPE_MAP
+			"\x01\0\0\0" // size
+
+			"\x04" // SCRIPT_TYPE_STRING
+			"\x0d\0\0\0" "B\0u\0i\0l\0d\0e\0r\0/\0R\0a\0n\0g\0e\0" // "Builder/Range"
+			"\x0f" // SCRIPT_TYPE_MAP
+			"\x03\0\0\0" // size
+
+				"\x04" // SCRIPT_TYPE_STRING
+				"\x03\0\0\0" "1\0005\0005\000" // "155"
+				"\x03" // SCRIPT_TYPE_OBJECT
+				"\x02\0\0\0" // num props
+					"\x03\0\0\0" "a\0d\0d\0" // "add"
+					"\x05" // SCRIPT_TYPE_INT
+					"\0\0\0\0" // 0
+					"\x08\0\0\0" "m\0u\0l\0t\0i\0p\0l\0y\0" // "multiply"
+					"\x05" // SCRIPT_TYPE_INT
+					"\x01\0\0\0" // 1
+
+				"\x04" // SCRIPT_TYPE_STRING
+				"\x04\0\0\0" "2\0003\0000\0000\000" // "2300"
+				"\x03" // SCRIPT_TYPE_OBJECT
+				"\x02\0\0\0" // num props
+					"\x03\0\0\0" "a\0d\0d\0" // "add"
+					"\x05" // SCRIPT_TYPE_INT
+					"\0\0\0\0" // 0
+					"\x08\0\0\0" "m\0u\0l\0t\0i\0p\0l\0y\0" // "multiply"
+					"\x06" // SCRIPT_TYPE_DOUBLE
+					"\x9a\x99\x99\x99\x99\x99\xf1\x3f" // 1.1
+
+				"\x04" // SCRIPT_TYPE_STRING
+				"\x03\0\0\0" "1\0005\0009\000" // "159"
+				"\x03" // SCRIPT_TYPE_OBJECT
+				"\x02\0\0\0" // num props
+					"\x03\0\0\0" "a\0d\0d\0" // "add"
+					"\x05" // SCRIPT_TYPE_INT
+					"\0\0\0\0" // 0
+					"\x08\0\0\0" "m\0u\0l\0t\0i\0p\0l\0y\0" // "multiply"
+					"\x05" // SCRIPT_TYPE_INT
+					"\x01\0\0\0" // 1
+		);
+
+		helper_script_roundtrip("Nested maps using backrefs",
+			"var a = new Map(); var b = new Map(); b.set(1, a); b.set(2, a); b",
+		/* expected: */
+			"({})",
+		/* expected stream: */
+			25,
+			"\x0f" // SCRIPT_TYPE_MAP
+			"\x02\0\0\0" // size
+
+			"\x05" // SCRIPT_TYPE_INT
+			"\x01\0\0\0" // 1
+			"\x0f" // SCRIPT_TYPE_MAP
+			"\0\0\0\0" // size
+
+			"\x05" // SCRIPT_TYPE_INT
+			"\x02\0\0\0" // 2
+			"\x08" // SCRIPT_TYPE_BACKREF
+			"\x02\0\0\0" // ref. to object #2, i.e. "a", with #1 being "b"
+		);
+	}
+
+	void test_script_set_nested()
+	{
+		helper_script_roundtrip("Nested sets using backrefs",
+			"var a = new Set(); var b = new Set(); a.add(b); a.add({\"bar\": b}); a",
+		/* expected: */
+			"({})",
+		/* expected stream: */
+			30,
+			"\x10" // SCRIPT_TYPE_SET
+			"\x02\0\0\0" // size
+
+			"\x10" // SCRIPT_TYPE_SET
+			"\x00\0\0\0" // size
+
+			"\x03" // SCRIPT_TYPE_OBJECT
+			"\x01\0\0\0" // num props
+				"\x03\0\0\0" "b\0a\0r\0" // "bar"
+				"\x08" // SCRIPT_TYPE_BACKREF
+				"\x02\0\0\0" // ref to object #2, i.e. "b", with #1 being "a"
+		);
+	}
+
 	// TODO: prototype objects
 
 	void test_script_nonfinite()
@@ -625,7 +800,8 @@ public:
 
 			JS::RootedValue newobj(cx);
 			deserialize.ScriptVal("script", &newobj);
-			TS_ASSERT(stream.good());
+			// NOTE: Don't use good() here - it fails due to a bug in older libc++ versions
+			TS_ASSERT(!stream.bad() && !stream.fail());
 			TS_ASSERT_EQUALS(stream.peek(), EOF);
 
 			if (i == 0)
