@@ -1,4 +1,4 @@
-/* Copyright (C) 2012 Wildfire Games.
+/* Copyright (C) 2015 Wildfire Games.
  * This file is part of 0 A.D.
  *
  * 0 A.D. is free software: you can redistribute it and/or modify
@@ -19,33 +19,29 @@
 
 #include "OggData.h"
 
-
 #if CONFIG2_AUDIO
 
+#include "ps/CLogger.h"
 #include "ps/CLogger.h"
 #include "ps/Filesystem.h"
 #include "soundmanager/SoundManager.h"
-#include "ps/CLogger.h"
 
 COggData::COggData()
+	: m_Format(0), m_Frequency(0), m_OneShot(false), m_BuffersUsed(0)
 {
-	m_OneShot = false;
-	m_Format = 0;
-	m_Frequency = 0;
-	m_BuffersUsed = 0;
 }
 
 COggData::~COggData()
 {
-	AL_CHECK
+	AL_CHECK;
 	if (ogg)
 		ogg->Close();
 
-	AL_CHECK
+	AL_CHECK;
 	if ( m_BuffersUsed > 0 )
-		alDeleteBuffers(m_BuffersUsed, &m_Buffer[0] );
+		alDeleteBuffers(m_BuffersUsed, &m_Buffer[0]);
 
-	AL_CHECK
+	AL_CHECK;
 	m_BuffersUsed = 0;
 }
 
@@ -57,48 +53,47 @@ void COggData::SetFormatAndFreq(int form, ALsizei freq)
 
 bool COggData::IsStereo()
 {
-	return ( m_Format == AL_FORMAT_STEREO16 );
+	return m_Format == AL_FORMAT_STEREO16;
 }
 
 bool COggData::InitOggFile(const VfsPath& itemPath)
 {
-  if ( CSoundManager* sndManager = (CSoundManager*)g_SoundManager )
-  {
-		int buffersToStart = sndManager->GetBufferCount();	
-		if ( OpenOggNonstream( g_VFS, itemPath, ogg) == INFO::OK )
-		{
-			m_FileFinished = false;
+	CSoundManager* sndManager = (CSoundManager*)g_SoundManager;
+	if (!sndManager)
+		return false;
 
-			SetFormatAndFreq(ogg->Format(), ogg->SamplingRate() );
-			SetFileName( itemPath );
-		
-			AL_CHECK
-			
-			alGenBuffers(buffersToStart, m_Buffer);
-			
-			if(alGetError() != AL_NO_ERROR) 
-			{
-				LOGERROR( L"- Error creating initial buffer !!\n");
-				return false;
-			}
-			else
-			{
-				m_BuffersUsed = FetchDataIntoBuffer(buffersToStart, m_Buffer);
-				if (m_FileFinished)
-				{
-					m_OneShot = true;
-					if (m_BuffersUsed < buffersToStart)
-					{
-						alDeleteBuffers(buffersToStart - m_BuffersUsed, &m_Buffer[m_BuffersUsed]);
-					}
-				}
-				AL_CHECK
-			}
-			return true;
-	  }
+	int buffersToStart = sndManager->GetBufferCount();	
+	if (OpenOggNonstream(g_VFS, itemPath, ogg) != INFO::OK)
+		return false;
+
+	m_FileFinished = false;
+
+	SetFormatAndFreq(ogg->Format(), ogg->SamplingRate());
+	SetFileName(itemPath);
+
+	AL_CHECK;
+	
+	alGenBuffers(buffersToStart, m_Buffer);
+	
+	ALenum err = alGetError();
+	if (err != AL_NO_ERROR) 
+	{
+		LOGERROR("Failed to create initial buffer. OpenAL error: %s\n", alGetString(err));
 		return false;
 	}
-	return false;
+
+	m_BuffersUsed = FetchDataIntoBuffer(buffersToStart, m_Buffer);
+	if (m_FileFinished)
+	{
+		m_OneShot = true;
+		if (m_BuffersUsed < buffersToStart)
+		{
+			alDeleteBuffers(buffersToStart - m_BuffersUsed, &m_Buffer[m_BuffersUsed]);
+		}
+	}
+	AL_CHECK;
+
+	return true;
 }
 
 ALsizei COggData::GetBufferCount()
@@ -124,28 +119,28 @@ bool COggData::IsOneShot()
 
 int COggData::FetchDataIntoBuffer(int count, ALuint* buffers)
 {
-  if ( CSoundManager* sndManager = (CSoundManager*)g_SoundManager )
-  {
-		long bufferSize = sndManager->GetBufferSize();
-		
-		u8* pcmout = new u8[bufferSize + 5000];
-		int buffersWritten = 0;
-		
-		for (int i = 0; (i < count) && !m_FileFinished; i++)
+	CSoundManager* sndManager = (CSoundManager*)g_SoundManager;
+	if (!sndManager)
+		return 0;
+
+	long bufferSize = sndManager->GetBufferSize();
+	
+	u8* pcmout = new u8[bufferSize + 5000];
+	int buffersWritten = 0;
+	
+	for (int i = 0; i < count && !m_FileFinished; ++i)
+	{
+		memset(pcmout, 0, bufferSize + 5000);
+		Status totalRet = ogg->GetNextChunk(pcmout, bufferSize);
+		m_FileFinished = ogg->atFileEOF();
+		if (totalRet > 0)
 		{
-			memset( pcmout, 0, bufferSize + 5000 );
-			Status totalRet = ogg->GetNextChunk( pcmout, bufferSize);
-			m_FileFinished = ogg->atFileEOF();
-			if (totalRet > 0)
-			{
-				buffersWritten++;
-				alBufferData(buffers[i], m_Format, pcmout, (ALsizei)totalRet, (int)m_Frequency);
-			}
+			buffersWritten++;
+			alBufferData(buffers[i], m_Format, pcmout, (ALsizei)totalRet, (int)m_Frequency);
 		}
-		delete[] pcmout;
-		return buffersWritten;
 	}
-	return 0;
+	delete[] pcmout;
+	return buffersWritten;
 }
 
 
@@ -160,4 +155,3 @@ ALuint* COggData::GetBufferPtr()
 }
 
 #endif // CONFIG2_AUDIO
-
