@@ -123,7 +123,7 @@ OsPath DataDir()
 
 namespace
 {
-	void script_TS_FAIL(ScriptInterface::CxPrivate* UNUSED(pCxPrivate), std::wstring msg)
+	void script_TS_FAIL(ScriptInterface::CxPrivate* UNUSED(pCxPrivate), const std::wstring& msg)
 	{
 		TS_FAIL(msg);
 	}
