@@ -61,6 +61,7 @@ class PlayerInfo(Base):
 	metalUsed = Column(Integer)
 	vegetarianFoodGathered = Column(Integer)
 	treasuresCollected = Column(Integer)
+	lootCollected = Column(Integer)
 	tributesSent = Column(Integer)
 	tributesReceived = Column(Integer)
 	totalUnitsTrained = Column(Integer)
@@ -87,6 +88,9 @@ class PlayerInfo(Base):
 	shipUnitsTrained = Column(Integer)
 	shipUnitsLost = Column(Integer)
 	enemyShipUnitsKilled = Column(Integer)
+	traderUnitsTrained = Column(Integer)
+	traderUnitsLost = Column(Integer)
+	enemyTraderUnitsKilled = Column(Integer)
 	totalBuildingsConstructed = Column(Integer)
 	totalBuildingsLost = Column(Integer)
 	enemytotalBuildingsDestroyed = Column(Integer)
