@@ -55,7 +55,7 @@ class LeaderboardList():
       stats['highestRating'] = str(player.first().highest_rating)
 
     playerID = player.first().id
-    players = db.query(Player).order_by(Player.rating.desc()).all()
+    players = db.query(Player).filter(Player.rating != -1).order_by(Player.rating.desc()).all()
 
     for rank, user in enumerate(players):
       if (user.jid.lower() == JID.lower()):
@@ -122,13 +122,14 @@ class LeaderboardList():
     totalScoreStats = {'economyScore', 'militaryScore', 'totalScore'}
     resourceStats = {'foodGathered', 'foodUsed', 'woodGathered', 'woodUsed', 
             'stoneGathered', 'stoneUsed', 'metalGathered', 'metalUsed', 'vegetarianFoodGathered',
-            'treasuresCollected', 'tributesSent', 'tributesReceived'}
+            'treasuresCollected', 'lootCollected', 'tributesSent', 'tributesReceived'}
     unitsStats = {'totalUnitsTrained', 'totalUnitsLost', 'enemytotalUnitsKilled', 'infantryUnitsTrained',
             'infantryUnitsLost', 'enemyInfantryUnitsKilled', 'workerUnitsTrained', 'workerUnitsLost',
             'enemyWorkerUnitsKilled', 'femaleUnitsTrained', 'femaleUnitsLost', 'enemyFemaleUnitsKilled',
             'cavalryUnitsTrained', 'cavalryUnitsLost', 'enemyCavalryUnitsKilled', 'championUnitsTrained',
             'championUnitsLost', 'enemyChampionUnitsKilled', 'heroUnitsTrained', 'heroUnitsLost',
-            'enemyHeroUnitsKilled', 'shipUnitsTrained', 'shipUnitsLost', 'enemyShipUnitsKilled'}
+            'enemyHeroUnitsKilled', 'shipUnitsTrained', 'shipUnitsLost', 'enemyShipUnitsKilled', 'traderUnitsTrained',
+            'traderUnitsLost', 'enemyTraderUnitsKilled'}
     buildingsStats = {'totalBuildingsConstructed', 'totalBuildingsLost', 'enemytotalBuildingsDestroyed',
             'civCentreBuildingsConstructed', 'civCentreBuildingsLost', 'enemyCivCentreBuildingsDestroyed',
             'houseBuildingsConstructed', 'houseBuildingsLost', 'enemyHouseBuildingsDestroyed',
@@ -247,11 +248,8 @@ class LeaderboardList():
         JIDs for sending.
     """
     board = {}
-    players = db.query(Player).order_by(Player.rating.desc()).limit(100).all()
+    players = db.query(Player).filter(Player.rating != -1).order_by(Player.rating.desc()).limit(100).all()
     for rank, player in enumerate(players):
-      # Don't send uninitialized ratings.
-      if player.rating == -1:
-        continue
       board[player.jid] = {'name': '@'.join(player.jid.split('@')[:-1]), 'rating': str(player.rating)}
     return board
   def getRatingList(self, nicks):
@@ -262,8 +260,8 @@ class LeaderboardList():
     JID to nick conveniently.
     """
     ratinglist = {}
-    for JID in nicks.keys():
-      players = db.query(Player).filter(Player.jid.ilike(str(JID)))
+    for JID in list(nicks):
+      players = db.query(Player.jid, Player.rating).filter(Player.jid.ilike(str(JID)))
       if players.first():
         if players.first().rating == -1:
           ratinglist[nicks[JID]] = {'name': nicks[JID], 'rating': ''}
@@ -489,6 +487,7 @@ class XpartaMuPP(sleekxmpp.ClientXMPP):
     self.room = room
     self.nick = nick
 
+    self.ratingListUpdate = False
     # Game collection
     self.gameList = GameList()
 
@@ -544,20 +543,20 @@ class XpartaMuPP(sleekxmpp.ClientXMPP):
     """
     Process presence stanza from a chat room.
     """
+    self.ratingListUpdate = True
     if presence['muc']['nick'] != self.nick:
       # If it doesn't already exist, store player JID mapped to their nick.
       if str(presence['muc']['jid']) not in self.nicks:
         self.nicks[str(presence['muc']['jid'])] = presence['muc']['nick']
       # Check the jid isn't already in the lobby.
-      if str(presence['muc']['jid']) != self.lastLeft:
-        # Send Gamelist to new player.
-        self.sendGameList(presence['muc']['jid'])
-        # Following two calls make sqlalchemy complain about using objects in the
-        #  incorrect thread. TODO: Figure out how to fix this.
-        # Send Leaderboard to new player.
-        #self.sendBoardList(presence['muc']['jid'])
-        # Register on leaderboard.
-        #self.leaderboard.getOrCreatePlayer(presence['muc']['jid'])
+      # Send Gamelist to new player.
+      self.sendGameList(presence['muc']['jid'])
+      # Following two calls make sqlalchemy complain about using objects in the
+      #  incorrect thread. TODO: Figure out how to fix this.
+      # Send Leaderboard to new player.
+      #self.sendBoardList(presence['muc']['jid'])
+      # Register on leaderboard.
+      #self.leaderboard.getOrCreatePlayer(presence['muc']['jid'])
       logging.debug("Client '%s' connected with a nick of '%s'." %(presence['muc']['jid'], presence['muc']['nick']))
 
   def muc_offline(self, presence):
@@ -615,13 +614,6 @@ class XpartaMuPP(sleekxmpp.ClientXMPP):
           except:
             traceback.print_exc()
             logging.error("Failed to process leaderboardlist request from %s" % iq['from'].bare)
-        elif command == 'getratinglist':
-          try:
-            self.leaderboard.getOrCreatePlayer(iq['from'])
-            self.sendRatingList(iq['from'])
-          except:
-            traceback.print_exc()
-            logging.error("Failed to process ratinglist request from %s" % iq['from'].bare)
         else:
           logging.error("Failed to process boardlist request from %s" % iq['from'].bare)
       elif 'profile' in iq.plugins:
@@ -689,6 +681,9 @@ class XpartaMuPP(sleekxmpp.ClientXMPP):
           logging.error("Failed to update game statistics for %s" % iq['from'].bare)
     else:
        logging.error("Failed to process stanza type '%s' received from %s" % iq['type'], iq['from'].bare)
+    if self.ratingListUpdate == True:
+      self.sendRatingList()
+      self.ratingListUpdate = False
 
   def sendGameList(self, to = ""):
     """
@@ -698,7 +693,7 @@ class XpartaMuPP(sleekxmpp.ClientXMPP):
     """
     games = self.gameList.getAllGames()
     if to == "":
-      for JID in self.nicks.keys():
+      for JID in list(self.nicks):
         stz = GameListXmppPlugin()
 
         ## Pull games and add each to the stanza        
@@ -757,7 +752,7 @@ class XpartaMuPP(sleekxmpp.ClientXMPP):
     stz.addCommand('boardlist')
     iq.setPayload(stz)
     if to == "":    
-      for JID in self.nicks.keys():
+      for JID in list(self.nicks):
         ## Set additional IQ attributes
         iq['to'] = JID
         ## Try sending the stanza
@@ -777,7 +772,7 @@ class XpartaMuPP(sleekxmpp.ClientXMPP):
         iq.send(block=False, now=True)
       except:
         logging.error("Failed to send leaderboard list")
-        
+
   def sendRatingList(self, to = ""):
     """
       Send the rating list.
@@ -794,7 +789,7 @@ class XpartaMuPP(sleekxmpp.ClientXMPP):
     stz.addCommand('ratinglist')
     iq.setPayload(stz)
     if to == "":    
-      for JID in self.nicks.keys():
+      for JID in list(self.nicks):
         ## Set additional IQ attributes
         iq['to'] = JID
         ## Try sending the stanza
@@ -825,7 +820,7 @@ class XpartaMuPP(sleekxmpp.ClientXMPP):
 
     online = False;
     ## Pull stats and add it to the stanza
-    for JID in self.nicks.keys():
+    for JID in list(self.nicks):
       if self.nicks[JID] == player:
         stats = self.leaderboard.getProfile(JID)
         online = True
