@@ -1,4 +1,4 @@
-/* Copyright (C) 2011 Wildfire Games.
+/* Copyright (C) 2015 Wildfire Games.
  * This file is part of 0 A.D.
  *
  * 0 A.D. is free software: you can redistribute it and/or modify
@@ -46,6 +46,8 @@ public:
 	void OnQuit(wxCommandEvent& event);
 	void OnUndo(wxCommandEvent& event);
 	void OnRedo(wxCommandEvent& event);
+    void OnCopy(wxCommandEvent& event);
+    void OnPaste(wxCommandEvent& event);
 
 	void OnWireframe(wxCommandEvent& event);
 	void OnMessageTrace(wxCommandEvent& event);
@@ -55,6 +57,9 @@ public:
 	void OnCameraReset(wxCommandEvent& event);
 	void OnRenderPath(wxCommandEvent& event);
 	void OnDumpState(wxCommandEvent& event);
+    void OnSelectedObjectsChange(const std::vector<AtlasMessage::ObjectID>& selectedObjects);
+
+    void OnMenuOpen(wxMenuEvent& event);
 
 	bool OpenFile(const wxString& name, const wxString& filename);
 
@@ -71,6 +76,8 @@ public:
 
 	void SelectPage(const wxString& classname) { m_SectionLayout.SelectPage(classname); }
 
+	bool DiscardChangesDialog();
+
 private:
 
 	ToolManager m_ToolManager;
@@ -86,6 +93,8 @@ private:
 	wxString m_OpenFilename;
 	FileHistory m_FileHistory;
 
+	wxIcon m_Icon;
+
 	DECLARE_EVENT_TABLE();
 };
 
