@@ -141,8 +141,11 @@ void MapSettingsControl::CreateWidgets()
 
 	sizer->AddSpacer(5);
 
+	// TODO: replace by filenames in binaries/data/mods/public/simulation/data/settings/victory_conditions/
 	wxArrayString gameTypes;
 	gameTypes.Add(_T("conquest"));
+	gameTypes.Add(_T("conquest_structures"));
+	gameTypes.Add(_T("conquest_units"));
 	gameTypes.Add(_T("wonder"));
 	gameTypes.Add(_T("endless"));
 
@@ -370,7 +373,7 @@ void MapSidebar::OnFirstDisplay()
 	qrySizes.Post();
 	AtObj sizes = AtlasObject::LoadFromJSON(*qrySizes.sizes);
 	wxChoice* sizeChoice = wxDynamicCast(FindWindow(ID_RandomSize), wxChoice);
-	for (AtIter s = sizes["Sizes"]["item"]; s.defined(); ++s)
+	for (AtIter s = sizes["Data"]["item"]; s.defined(); ++s)
 	{
 		long tiles = 0;
 		wxString(s["Tiles"]).ToLong(&tiles);
