@@ -1,4 +1,4 @@
-/* Copyright (C) 2014 Wildfire Games.
+/* Copyright (C) 2015 Wildfire Games.
  * This file is part of 0 A.D.
  *
  * 0 A.D. is free software: you can redistribute it and/or modify
@@ -71,6 +71,7 @@ public:
 
 	void OnFirstDisplay();
 	void ShowActorViewer(bool show);
+	void OnSelectedObjectsChange(const std::vector<AtlasMessage::ObjectID>& selectedObjects);
 
 private:
 	void OnViewerSetting(wxCommandEvent& evt);
@@ -88,9 +89,9 @@ private:
 	int m_ViewerPropPointsMode; // 0 disabled, 1 for point markers, 2 for point markers + axes
 
 	wxPanel* m_ViewerPanel;
+	wxScrolledWindow* m_TemplateNames;
 
 	ObjectSidebarImpl* p;
-
 	DECLARE_EVENT_TABLE();
 };
 
@@ -389,7 +390,10 @@ ObjectBottomBar::ObjectBottomBar(
 		viewerButtonsLeft->Add(Tooltipped(new wxButton(m_ViewerPanel, ID_ViewerWireframe,   _("Wireframe")),      _("Toggle wireframe / solid rendering")), wxSizerFlags().Expand());
 		viewerButtonsLeft->Add(Tooltipped(new wxButton(m_ViewerPanel, ID_ViewerMove,        _("Move")),           _("Toggle movement along ground when playing walk/run animations")), wxSizerFlags().Expand());
 		viewerButtonsLeft->Add(Tooltipped(new wxButton(m_ViewerPanel, ID_ViewerGround,      _("Ground")),         _("Toggle the ground plane")), wxSizerFlags().Expand());
-		viewerButtonsLeft->Add(Tooltipped(new wxButton(m_ViewerPanel, ID_ViewerWater,       _("Water")),          _("Toggle the water plane")), wxSizerFlags().Expand());
+		// TODO: disabled until http://trac.wildfiregames.com/ticket/2692 is fixed
+		wxButton* waterButton = new wxButton(m_ViewerPanel, ID_ViewerWater, _("Water"));
+		waterButton->Enable(false);
+		viewerButtonsLeft->Add(Tooltipped(waterButton, _("Toggle the water plane")), wxSizerFlags().Expand());
 		viewerButtonsLeft->Add(Tooltipped(new wxButton(m_ViewerPanel, ID_ViewerShadows,     _("Shadows")),        _("Toggle shadow rendering")), wxSizerFlags().Expand());
 		viewerButtonsLeft->Add(Tooltipped(new wxButton(m_ViewerPanel, ID_ViewerPolyCount,   _("Poly count")),     _("Toggle polygon-count statistics - turn off ground and shadows for more useful data")), wxSizerFlags().Expand());
 
@@ -462,10 +466,68 @@ ObjectBottomBar::ObjectBottomBar(
 	mainSizer->Add(playerVariationSizer, wxSizerFlags().Expand());
 
 	// ----------------------------------------------------------------------------------
+	// --- display template name
+	wxSizer* displaySizer = new wxStaticBoxSizer(wxVERTICAL, this, _("Selected entities"));
+	m_TemplateNames = new wxScrolledWindow(this);
+	m_TemplateNames->SetMinSize(wxSize(250, -1));
+	m_TemplateNames->SetScrollRate(0, 5);
+	wxSizer* scrollwindowSizer = new wxBoxSizer(wxVERTICAL);
+	m_TemplateNames->SetSizer(scrollwindowSizer);
+	displaySizer->Add(m_TemplateNames, wxSizerFlags().Proportion(1).Expand());
+	m_TemplateNames->Layout();
+	mainSizer->AddSpacer(3);
+	mainSizer->Add(displaySizer, wxSizerFlags().Proportion(1).Expand());
+
+	g_SelectedObjects.RegisterObserver(0, &ObjectBottomBar::OnSelectedObjectsChange, this);
 
 	SetSizer(mainSizer);
 }
 
+static wxControl* CreateTemplateNameObject(wxWindow* parent, const std::string templateName, int counterTemplate)
+{
+	wxString idTemplate(wxString::FromUTF8(templateName.c_str()));
+	if (counterTemplate > 1)
+		idTemplate.Append(wxString::Format(wxT(" (%i)"), counterTemplate));
+
+	wxStaticText* templateNameObject = new wxStaticText(parent, wxID_ANY, idTemplate);
+	return templateNameObject;
+}
+
+void ObjectBottomBar::OnSelectedObjectsChange(const std::vector<AtlasMessage::ObjectID>& selectedObjects)
+{
+	Freeze();
+	wxSizer* sizer = m_TemplateNames->GetSizer();
+	sizer->Clear(true);
+
+	AtlasMessage::qGetSelectedObjectsTemplateNames objectTemplatesName(selectedObjects);
+	objectTemplatesName.Post();
+	std::vector<std::string> names = *objectTemplatesName.names;
+
+	int counterTemplate = 0;
+	std::string lastTemplateName = "";
+	for (std::vector<std::string>::const_iterator it = names.begin(); it != names.end(); ++it)
+	{
+		if (lastTemplateName == "")
+			lastTemplateName = (*it);
+
+		if (lastTemplateName == (*it))
+		{
+			++counterTemplate;
+			continue;
+		}
+
+		sizer->Add(CreateTemplateNameObject(m_TemplateNames, lastTemplateName, counterTemplate), wxSizerFlags().Align(wxALIGN_LEFT));
+
+		lastTemplateName = (*it);
+		counterTemplate = 1;
+	}
+	// Add the remaining template
+	sizer->Add(CreateTemplateNameObject(m_TemplateNames, lastTemplateName, counterTemplate), wxSizerFlags().Align(wxALIGN_LEFT));
+
+	Thaw();
+	sizer->FitInside(m_TemplateNames);
+}
+
 void ObjectBottomBar::OnFirstDisplay()
 {
 	// We use messages here because the simulation is not init'd otherwise (causing a crash)
