@@ -22,7 +22,7 @@
 
 #include "../GameLoop.h"
 #include "../View.h"
-#include "graphics/CinemaTrack.h"
+#include "graphics/CinemaManager.h"
 #include "graphics/GameView.h"
 #include "gui/GUIManager.h"
 #include "gui/GUI.h"
@@ -56,9 +56,6 @@ MESSAGEHANDLER(Screenshot)
 
 QUERYHANDLER(CinemaRecord)
 {
-	CCinemaManager* manager = g_Game->GetView()->GetCinema();
-	manager->SetCurrentPath(*msg->path, false, false);
-
 	const int w = msg->width, h = msg->height;
 
 	{
@@ -82,7 +79,6 @@ QUERYHANDLER(CinemaRecord)
 	{
 		AtlasView::GetView_Game()->Update(1.f / msg->framerate);
 
-		manager->MoveToPointAt((float)frame/msg->framerate);
 		Render();
 		Atlas_GLSwapBuffers((void*)g_AtlasGameLoop->glCanvas);
 
@@ -196,12 +192,7 @@ MESSAGEHANDLER(GuiKeyEvent)
 {
 	SDL_Event_ ev = { { 0 } };
 	ev.ev.type = msg->pressed ? SDL_KEYDOWN : SDL_KEYUP;
-#if SDL_VERSION_ATLEAST(2, 0, 0)
 	ev.ev.key.keysym.sym = (SDL_Keycode)(int)msg->sdlkey;
-#else
-	ev.ev.key.keysym.sym = (SDLKey)(int)msg->sdlkey;
-	ev.ev.key.keysym.unicode = msg->unichar;
-#endif
 	in_dispatch_event(&ev);
 }
 
@@ -213,12 +204,7 @@ MESSAGEHANDLER(GuiCharEvent)
 
 	SDL_Event_ ev = { { 0 } };
 	ev.ev.type = SDL_KEYDOWN;
-#if SDL_VERSION_ATLEAST(2, 0, 0)
 	ev.ev.key.keysym.sym = (SDL_Keycode)(int)msg->sdlkey;
-#else
-	ev.ev.key.keysym.sym = (SDLKey)(int)msg->sdlkey;
-	ev.ev.key.keysym.unicode = msg->unichar;
-#endif
 	in_dispatch_event(&ev);
 
 	ev.ev.type = SDL_KEYUP;
